/*
   Quero Toolbar
   http://www.quero.at/
   Copyright 2013 Viktor Krammer

   This file is part of Quero Toolbar.

   Quero Toolbar is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Quero Toolbar is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Quero Toolbar.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
   Portions of the source code:

   punycode.c from RFC 3492
   http://www.nicemice.net/idn/
   Adam M. Costello
   http://www.nicemice.net/amc/
*/

#include "StdAfx.h"
#include "idna.h"
#include "QDebug.h"
#include "QToolbar.h"
#include <strsafe.h>

extern bool g_IDNSupport;

// Unicode codepoint and mapping tables used for Nameprep

const struct UnicodeInterval UCCharSets[NUCCHARSETS]={
{0x00,0x002D,0x002D}, // Hyphen
{0x00,0x0030,0x0039}, // Digits
{0x00,0x0061,0x007A}, // Small Latin Letters
{0x00,0x0080,0x00FF}, // Latin-1 Supplement 
{0x00,0x0100,0x017F}, // Latin Extended-A 
{0x00,0x0180,0x024F}, // Latin Extended-B 
{0x00,0x0250,0x02AF}, // IPA Extensions 
{0x00,0x02B0,0x02FF}, // Spacing Modifier Letters 
{0x00,0x0300,0x036F}, // Combining Diacritical Marks 
{0x00,0x0370,0x03FF}, // Greek and Coptic 
{0x00,0x0400,0x04FF}, // Cyrillic 
{0x00,0x0500,0x052F}, // Cyrillic Supplement 
{0x00,0x0530,0x058F}, // Armenian 
{0x00,0x0590,0x05FF}, // Hebrew 
{0x00,0x0600,0x06FF}, // Arabic 
{0x00,0x0700,0x074F}, // Syriac 
{0x00,0x0750,0x077F}, // Arabic Supplement 
{0x00,0x0780,0x07BF}, // Thaana 
{0x00,0x07B0,0x08FF}, // N'Ko 
{0x00,0x0800,0x085F}, // Avestan and Pahlavi 
{0x00,0x0860,0x087F}, // Mandaic 
{0x00,0x0880,0x08AF}, // Samaritan 
{0x00,0x0900,0x097F}, // Devanaghari 
{0x00,0x0980,0x09FF}, // Bengali 
{0x00,0x0A00,0x0A7F}, // Gumurkhi 
{0x00,0x0A80,0x0AFF}, // Gujarati 
{0x00,0x0B00,0x0B7F}, // Oriya 
{0x00,0x0B80,0x0BFF}, // Tamil 
{0x00,0x0C00,0x0C7F}, // Telugu 
{0x00,0x0C80,0x0CFF}, // Kannada 
{0x00,0x0D00,0x0D7F}, // Malayalam 
{0x00,0x0D80,0x0DFF}, // Sinhala 
{0x00,0x0E00,0x0E7F}, // Thai 
{0x00,0x0E80,0x0EFF}, // Lao 
{0x00,0x0F00,0x0FFF}, // Tibetan 
{0x00,0x1000,0x109F}, // Myanmar 
{0x00,0x10A0,0x10FF}, // Georgian 
{0x00,0x1100,0x11FF}, // Hangeul Jamo 
{0x00,0x1200,0x137F}, // Ethiopic 
{0x00,0x1380,0x139F}, // Ethiopic Extended 
{0x00,0x13A0,0x13FF}, // Cherokee 
{0x00,0x1400,0x167F}, // Unified Canadian Aboriginal Syllabics 
{0x00,0x1680,0x169F}, // Ogham 
{0x00,0x16A0,0x16FF}, // Runic 
{0x00,0x1700,0x171F}, // Tagalog 
{0x00,0x1720,0x173F}, // Hanunoo 
{0x00,0x1740,0x175F}, // Buhid 
{0x00,0x1760,0x177F}, // Tagbanwa 
{0x00,0x1780,0x17FF}, // Khmer 
{0x00,0x1800,0x18AF}, // Mongolian 
{0x00,0x18B0,0x18FF}, // Cham 
{0x00,0x1900,0x194F}, // Limbu 
{0x00,0x1950,0x197F}, // Tai Le 
{0x00,0x1980,0x19DF}, // New Tai Lue 
{0x00,0x19E0,0x19FF}, // Khmer Symbols 
{0x00,0x1A00,0x1A1F}, // Buginese 
{0x00,0x1A20,0x1A5F}, // Batak 
{0x00,0x1A80,0x1AEF}, // Lanna (Old Tai Lue) 
{0x00,0x1B00,0x1B7F}, // Balinese 
{0x00,0x1B80,0x1BDF}, // Viet Thai 
{0x00,0x1C00,0x1C5F}, // Meitei/Manipuri 
{0x00,0x1C80,0x1CCF}, // Lepcha 
{0x00,0x1CD0,0x1CFF}, // Kayah Li 
{0x00,0x1D00,0x1D7F}, // Phonetic Extensions 
{0x00,0x1D80,0x1DBF}, // Phonetic Extensions Supplement 
{0x00,0x1DC0,0x1DFF}, // Combining Diacritical Marks Supplement 
{0x00,0x1E00,0x1EFF}, // Latin Extended Additional 
{0x00,0x1F00,0x1FFF}, // Greek Extended 
{0x00,0x2000,0x206F}, // General Punctuation 
{0x00,0x2070,0x209F}, // Superscripts and Subscripts 
{0x00,0x20A0,0x20CF}, // Currency Symbols 
{0x00,0x20D0,0x20FF}, // Combining Diacritical Marks for Symbols 
{0x00,0x2100,0x214F}, // Letterlike Symbols 
{0x00,0x2150,0x218F}, // Number Forms 
{0x00,0x2190,0x21FF}, // Arrows 
{0x00,0x2200,0x22FF}, // Mathematical Operators 
{0x00,0x2300,0x23FF}, // Miscellaneous Technical 
{0x00,0x2400,0x243F}, // Control Pictures 
{0x00,0x2440,0x245F}, // Optical Character Recognition 
{0x00,0x2460,0x24FF}, // Enclosed Alphanumerics 
{0x00,0x2500,0x257F}, // Box Drawing 
{0x00,0x2580,0x259F}, // Block Elements 
{0x00,0x25A0,0x25FF}, // Geometric Shapes 
{0x00,0x2600,0x26FF}, // Miscellaneous Symbols 
{0x00,0x2700,0x27BF}, // Dingbats 
{0x00,0x27C0,0x27EF}, // Miscellaneous Mathematical Symbols-A 
{0x00,0x27F0,0x27FF}, // Supplemental Arrows-A 
{0x00,0x2800,0x28FF}, // Braille Patterns 
{0x00,0x2900,0x297F}, // Supplemental Arrows-B 
{0x00,0x2980,0x29FF}, // Miscellaneous Mathematical Symbols-B 
{0x00,0x2A00,0x2AFF}, // Supplemental Mathematical Operators 
{0x00,0x2B00,0x2BFF}, // Miscellaneous Symbols and Arrows
{0x00,0x2C00,0x2C5F}, // Glagolitic 
{0x00,0x2C60,0x2CFF}, // Latin Extended C 
{0x00,0x2C80,0x2C8F}, // Coptic 
{0x00,0x2D00,0x2D2F}, // Georgian Supplement 
{0x00,0x2D30,0x2D7F}, // Tifinagh 
{0x00,0x2CD0,0x2CFF}, // Old Hungarian 
{0x00,0x2D80,0x2DDF}, // Ethiopic Extended 
{0x00,0x2DE0,0x2DFF}, // Ol Chiki 
{0x00,0x2E00,0x2E7F}, // Supplemental Punctuation 
{0x00,0x2E80,0x2EFF}, // CJK Radicals Supplement 
{0x00,0x2F00,0x2FDF}, // Kangxi Radicals 
{0x00,0x2FF0,0x2FFF}, // Ideographic Description Characters 
{0x00,0x3000,0x303F}, // CJK Symbols and Punctuation 
{0x00,0x3040,0x309F}, // Hiragana 
{0x00,0x30A0,0x30FF}, // Katakana 
{0x00,0x3100,0x312F}, // Bopomofo 
{0x00,0x3130,0x318F}, // Hangeul Compatibility Jamo 
{0x00,0x3190,0x319F}, // Kanbun 
{0x00,0x31A0,0x31BF}, // Bopomofo Extended 
{0x00,0x31C0,0x31EF}, // CJK Basic Strokes 
{0x00,0x31F0,0x31FF}, // Katakana Phonetic Extensions 
{0x00,0x3200,0x32FF}, // Enclosed CJK Letters and Months 
{0x00,0x3300,0x33FF}, // CJK Compatibility 
{0x00,0x3400,0x4DBF}, // CJK Unified Ideographs Extension A 
{0x00,0x4DC0,0x4DFF}, // Yijing Hexagram Symbols 
{0x00,0x4E00,0x9FFF}, // CJK Unified Ideographs 
{0x00,0xA000,0xA48F}, // Yi Syllables 
{0x00,0xA4C0,0xA6FF}, // Yi Extensions 
{0x00,0xA490,0xA4CF}, // Yi Radicals 
{0x00,0xA700,0xA71F}, // Modifier Tone Letters 
{0x00,0xA720,0xA77F}, // Pollard Phonetic 
{0x00,0xA780,0xA7FF}, // Grantha 
{0x00,0xA800,0xA82F}, // Syloti Nagri 
{0x00,0xA840,0xA87F}, // Phags-pa 
{0x00,0xA880,0xA8DF}, // Newari 
{0x00,0xA900,0xA95F}, // Chakma 
{0x00,0xA900,0xA95F}, // Javanese
{0x00,0xAA00,0xAA3F}, // Varang Kshiti 
{0x00,0xAA40,0xAA6F}, // Sorang Sompeng 
{0x00,0xAA80,0xAADF}, // Siddham 
{0x00,0xAB00,0xAB5F}, // Saurashtra 
{0x00,0xAB80,0xABBF}, // Pahawh Hmong 
{0x00,0xAC00,0xD7AF}, // Hangeul Syllables 
{0x00,0xD800,0xDFFF}, // Surrogates 
{0x00,0xE000,0xF8FF}, // Private Use Area 
{0x00,0xF900,0xFAFF}, // CJK Compatibility Ideographs 
{0x00,0xFB00,0xFB4F}, // Alphabetic Presentation Forms 
{0x00,0xFB50,0xFDFF}, // Arabic Presentation Forms-A 
{0x00,0xFE00,0xFE0F}, // Variation Selectors 
{0x00,0xFE10,0xFE1F}, // Vertical Form
{0x00,0xFE20,0xFE2F}, // Combining Half Marks 
{0x00,0xFE30,0xFE4F}, // CJK Compatibility Forms 
{0x00,0xFE50,0xFE6F}, // Small Form Variants 
{0x00,0xFE70,0xFEFF}, // Arabic Presentation Forms-B 
{0x00,0xFF00,0xFFEF}, // Halfwidth and Fullwidth Forms 
{0x00,0xFFF0,0xFFFF}, // Specials 
{0x01,0x0000,0x007F}, // Linear B Syllabary
{0x01,0x0080,0x00FF}, // Linear B Ideograms
{0x01,0x0100,0x013F}, // Aegean Numbers
{0x01,0x0140,0x018F}, // Ancient Greek Numbers
{0x01,0x0300,0x032F}, // Old Italic
{0x01,0x0330,0x034F}, // Gothic
{0x01,0x0350,0x037F}, // Old Permic
{0x01,0x0380,0x039F}, // Ugaritic
{0x01,0x03A0,0x03DF}, // Old Persian
{0x01,0x0400,0x044F}, // Deseret
{0x01,0x0450,0x047F}, // Shavian
{0x01,0x0480,0x04AF}, // Osmanya
{0x01,0x0500,0x051F}, // Lycian
{0x01,0x0520,0x053F}, // Iberian
{0x01,0x057F,0x05BF}, // Elbasan
{0x01,0x05C0,0x05DF}, // Buthakukye
{0x01,0x0600,0x077F}, // Linear A
{0x01,0x0780,0x07BF}, // Cypro Minoan
{0x01,0x0800,0x083F}, // Cypriot Syllabary
{0x01,0x0840,0x087F}, // Meroitic
{0x01,0x0880,0x08AF}, // Manichaean
{0x01,0x08B0,0x08CF}, // Balti
{0x01,0x08D0,0x08FF}, // Yezidi
{0x01,0x0900,0x091F}, // Phoenician
{0x01,0x0920,0x093F}, // Aramaic
{0x01,0x0940,0x095F}, // South Arabian
{0x01,0x0960,0x097F}, // North Arabic
{0x01,0x0980,0x099F}, // Palmyrene
{0x01,0x09A0,0x09BF}, // Nabataean
{0x01,0x0A00,0x0A5F}, // Kharoshthi
{0x01,0x0A60,0x0A7F}, // Hatran
{0x01,0x0A80,0x0ABF}, // Orkhon
{0x01,0x0B00,0x0B1F}, // Lydian
{0x01,0x0B20,0x0B3F}, // Carian
{0x01,0x0B40,0x0B5F}, // Numidian
{0x01,0x0B60,0x0B7F}, // Elymaic
{0x01,0x0B80,0x0BDF}, // Uigur
{0x01,0x0C00,0x0DFF}, // Proto Elamite
{0x01,0x0E00,0x0E6F}, // Byblos
{0x01,0x1000,0x104F}, // Brahmi
{0x01,0x1080,0x10BF}, // Pyu
{0x01,0x1100,0x113F}, // Soyombo
{0x01,0x1140,0x117F}, // Ahorn
{0x01,0x1180,0x11DF}, // Turkestani
{0x01,0x1200,0x125F}, // Tulu
{0x01,0x1280,0x12DF}, // Rejang
{0x01,0x1300,0x13DF}, // Landa
{0x01,0x1380,0x13DF}, // Modi
{0x01,0x1400,0x145F}, // Chalukya (Box Headed)
{0x01,0x1480,0x14DF}, // Chola
{0x01,0x1500,0x155F}, // Satavahana
{0x01,0x1580,0x15DF}, // Takri
{0x01,0x1600,0x165F}, // Kaithi
{0x01,0x1800,0x18FF}, // Vai
{0x01,0x1900,0x1AFF}, // Barnum
{0x01,0x1B00,0x1BBF}, // Mende
{0x01,0x1BC0,0x1BEF}, // Bassa
{0x01,0x1D00,0x1D2F}, // Chinook
{0x01,0x1E00,0x1FFF}, // Shorthands
{0x01,0x2000,0x23FF}, // Cuneiform
{0x01,0x2400,0x247F}, // Cuneiform Numbers
{0x01,0x2480,0x24FF}, // Archaic Cuneiform Extensions 1
{0x01,0x2800,0x28AF}, // Hittite Hieroglyphs/Luvian
{0x01,0x2900,0x2A8F}, // Indus
{0x01,0x2B00,0x2E4F}, // Rongorongo
{0x01,0x3000,0x307F}, // Tengwar
{0x01,0x3080,0x30FF}, // Cirth
{0x01,0x3200,0x35FF}, // Blissymbols
{0x01,0x3600,0x36FF}, // Blissymbol Extensions
{0x01,0x4000,0x42FF}, // Basic Egyptian Hieroglyphs
{0x01,0x4400,0x56FF}, // Egyptian Hieroglyohs Extended
{0x01,0x5700,0x63FF}, // Egyptian Hieroglyohs Extended A
{0x01,0x6400,0x68FF}, // Mayan Hieroglyphs
{0x01,0x6C00,0x6FFF}, // Aztec Pictograms
{0x01,0x7000,0x89FF}, // Tangut Ideographs
{0x01,0x8C00,0x8D7F}, // Kitan Small Script
{0x01,0x8D80,0x9FFF}, // Kitan Large Script
{0x01,0xA000,0xA5FF}, // Jurchin
{0x01,0xA800,0xAAFF}, // Naxi Geba
{0x01,0xA8C0,0xAFFF}, // Naxi Tomba
{0x01,0xB000,0xB5FF}, // Nushu
{0x01,0xD000,0xD0FF}, // Byzantine Musical Symbols
{0x01,0xD100,0xD1FF}, // Musical Symbols
{0x01,0xD200,0xD24F}, // Ancient Greek Musical Notation
{0x01,0xD300,0xD35F}, // Tai Xuan Jing Symbols
{0x01,0xD360,0xD37F}, // Rod Numerals
{0x01,0xD400,0xD7FF}, // Mathematical Alphanumeric Symbols
{0x01,0xD800,0xDBFF}, // Sutton Sign Writing
{0x02,0x0000,0xA6DF}, // CJK Unified Ideographs Extension B
{0x02,0xA700,0xF7FF}, // Unified Ideographs Extensions C1
{0x02,0xF800,0xFA1F}, // CJK Compatibility Ideographs Supplement
{0x0E,0x0000,0x007F}, // Tags
{0x0E,0x0100,0x01EF}, // Variation Selectors Supplement
{0x0F,0x0000,0xFFFF}, // Supplementary Private Use Area A
{0x10,0x0000,0xFFFF}  // Supplementary Private Use Area B
};

const TCHAR *UCCharSetNames[NUCCHARSETS_NAMES]=
{
L"Hyphen", // 0x002D-0x002D
L"Digits", // 0x0030-0x0039
L"Latin", // 0x0061-0x007A
L"Latin-1", // 0x0080-0x00FF
L"Latin-A", // 0x0100-0x017F
L"Latin-B", // 0x0180-0x024F
L"IPA", // 0x0250-0x02AF
L"Spacing Modifier", // 0x02B0-0x02FF
L"Diacritics", // 0x0300-0x036F
L"Greek/Coptic", // 0x0370-0x03FF
L"Cyrillic", // 0x0400-0x04FF
L"Cyrillic Sup", // Supplement // 0x0500-0x052F
L"Armenian", // 0x0530-0x058F
L"Hebrew", // 0x0590-0x05FF
L"Arabic", // 0x0600-0x06FF
L"Syriac", // 0x0700-0x074F
L"Arabic Sup", // 0x0750-0x077F
L"Thaana", // 0x0780-0x07BF
L"N'Ko", // 0x07B0-0x08FF
L"Avestan/Pahlavi", // 0x0800-0x085F
L"Mandaic", // 0x0860-0x087F
L"Samaritan", // 0x0880-0x08AF
L"Devanaghari", // 0x0900-0x097F
L"Bengali", // 0x0980-0x09FF
L"Gumurkhi", // 0x0A00-0x0A7F
L"Gujarati", // 0x0A80-0x0AFF
L"Oriya", // 0x0B00-0x0B7F
L"Tamil", // 0x0B80-0x0BFF
L"Telugu", // 0x0C00-0x0C7F
L"Kannada", // 0x0C80-0x0CFF
L"Malayalam", // 0x0D00-0x0D7F
L"Sinhala", // 0x0D80-0x0DFF
L"Thai", // 0x0E00-0x0E7F
L"Lao", // 0x0E80-0x0EFF
L"Tibetan", // 0x0F00-0x0FFF
L"Myanmar", // 0x1000-0x109F
L"Georgian", // 0x10A0-0x10FF
L"Hangul", // 0x1100-0x11FF
L"Ethiopic", // 0x1200-0x137F
L"Ethiopic Ext", // 0x1380-0x139F
L"Cherokee", // 0x13A0-0x13FF
L"Canadian Aboriginal", // 0x1400-0x167F
L"Ogham", // 0x1680-0x169F
L"Runic", // 0x16A0-0x16FF
L"Tagalog", // 0x1700-0x171F
L"Hanunoo", // 0x1720-0x173F
L"Buhid", // 0x1740-0x175F
L"Tagbanwa", // 0x1760-0x177F
L"Khmer", // 0x1780-0x17FF
L"Mongolian", // 0x1800-0x18AF
L"Cham", // 0x18B0-0x18FF
L"Limbu", // 0x1900-0x194F
L"Tai Le", // 0x1950-0x197F
L"New Tai Lue", // 0x1980-0x19DF
L"Khmer", // 0x19E0-0x19FF
L"Buginese", // 0x1A00-0x1A1F
L"Batak", // 0x1A20-0x1A5F
L"Lanna", // 0x1A80-0x1AEF
L"Balinese", // 0x1B00-0x1B7F
L"Viet Thai", // 0x1B80-0x1BDF
L"Meitei/Manipuri", // 0x1C00-0x1C5F
L"Lepcha", // 0x1C80-0x1CCF
L"Kayah Li", // 0x1CD0-0x1CFF
L"Phonetic", // 0x1D00-0x1D7F
L"Phonetic Sup", // 0x1D80-0x1DBF
L"Diacritics Sup", // 0x1DC0-0x1DFF
L"Latin Ext", // 0x1E00-0x1EFF
L"Greek Ext", // 0x1F00-0x1FFF
L"Punctuation", // 0x2000-0x206F
L"Super/Subscripts", // 0x2070-0x209F
L"Currency", // 0x20A0-0x20CF
L"Diacritics for Symbols", // 0x20D0-0x20FF
L"Letterlike", // 0x2100-0x214F
L"Number Forms", // 0x2150-0x218F
L"Arrows", // 0x2190-0x21FF
L"Mathematical", // 0x2200-0x22FF
L"Technical", // 0x2300-0x23FF
L"Control Pictures", // 0x2400-0x243F
L"OCR", // 0x2440-0x245F
L"Enclosed Alphanum", // 0x2460-0x24FF
L"Box Drawing", // 0x2500-0x257F
L"Block Elements", // 0x2580-0x259F
L"Geometric Shapes", // 0x25A0-0x25FF
L"Symbols", // 0x2600-0x26FF
L"Dingbats", // 0x2700-0x27BF
L"Mathematical-A", // 0x27C0-0x27EF
L"Arrows-A", // 0x27F0-0x27FF
L"Braille", // 0x2800-0x28FF
L"Arrows-B", // 0x2900-0x297F
L"Mathematical-B", // 0x2980-0x29FF
L"Mathematical Sup", // 0x2A00-0x2AFF
L"Symbols/Arrows", // 0x2B00-0x2BFF
L"Glagolitic", // 0x2C00-0x2C5F
L"Latin-C", // 0x2C60-0x2CFF
L"Coptic", // 0x2C80-0x2C8F
L"Georgian Sup", // 0x2D00-0x2D2F
L"Tifinagh", // 0x2D30-0x2D7F
L"Old Hungarian", // 0x2CD0-0x2CFF
L"Ethiopic Ext", // 0x2D80-0x2DDF
L"Ol Chiki", // 0x2DE0-0x2DFF
L"Punctuation Sup", // 0x2E00-0x2E7F
L"CJK Radicals", // 0x2E80-0x2EFF
L"Kangxi", // 0x2F00-0x2FDF
L"Ideographic Description", // 0x2FF0-0x2FFF
L"CJK Symbols/Punctuation", // 0x3000-0x303F
L"Hiragana", // 0x3040-0x309F
L"Katakana", // 0x30A0-0x30FF
L"Bopomofo", // 0x3100-0x312F
L"Hangul Compat", // 0x3130-0x318F
L"Kanbun", // 0x3190-0x319F
L"Bopomofo Ext", // 0x31A0-0x31BF
L"CJK Strokes", // 0x31C0-0x31EF
L"Katakana Phonetic", // 0x31F0-0x31FF
L"CJK enclosed", // 0x3200-0x32FF
L"CJK Compat", // 0x3300-0x33FF
L"CJK Ext A", // 0x3400-0x4DBF
L"Yijing", // 0x4DC0-0x4DFF
L"CJK", // 0x4E00-0x9FFF
L"Yi", // 0xA000-0xA48F
L"Yi Ext", // 0xA4C0-0xA6FF
L"Yi Radicals", // 0xA490-0xA4CF
L"Modifier Tone", // 0xA700-0xA71F
L"Pollard", // 0xA720-0xA77F
L"Grantha", // 0xA780-0xA7FF
L"Syloti Nagri", // 0xA800-0xA82F
L"Phags-pa", // 0xA840-0xA87F
L"Newari", // 0xA880-0xA8DF
L"Chakma", // 0xA900-0xA95F
L"Javanese", // 0xA900-0xA95F
L"Varang Kshiti", // 0xAA00-0xAA3F
L"Sorang Sompeng", // 0xAA40-0xAA6F
L"Siddham", // 0xAA80-0xAADF
L"Saurashtra", // 0xAB00-0xAB5F
L"Pahawh Hmong", // 0xAB80-0xABBF
L"Hangul Syl", // 0xAC00-0xD7AF
L"Surrogates", // Surrogates // 0xD800-0xDFFF
L"Private", // Private Use Area // 0xE000-0xF8FF
L"CJK Compat Ideographs", // 0xF900-0xFAFF
L"Presentation Forms", // 0xFB00-0xFB4F
L"Arabic-A", // 0xFB50-0xFDFF
L"Variation", // 0xFE00-0xFE0F
L"Vertical Forms", // 0xFE10-0xFE1F
L"Half Marks", // 0xFE20-0xFE2F
L"CJK Compat Forms", // 0xFE30-0xFE4F
L"Variants", // 0xFE50-0xFE6F
L"Arabic-B", // 0xFE70-0xFEFF
L"Half/Fullwidth", // 0xFF00-0xFFEF
L"Specials", // 0xFFF0-0xFFFF
L"Linear B Syllabary", // 0x10000-0x1007F
L"Linear B Ideograms", // 0x10080-0x100FF
L"Aegean Numbers", // 0x10100-0x1013F
L"Ancient Greek Numbers", // 0x10140-0x1018F
L"Old Italic", // 0x10300-0x1032F
L"Gothic", // 0x10330-0x1034F
L"Old Permic", // 0x10350-0x1037F
L"Ugaritic", // 0x10380-0x1039F
L"Old Persian", // 0x103A0-0x103DF
L"Deseret", // 0x10400-0x1044F
L"Shavian", // 0x10450-0x1047F
L"Osmanya", // 0x10480-0x104AF
L"Lycian", // 0x10500-0x1051F
L"Iberian", // 0x10520-0x1053F
L"Elbasan", // 0x1057F-0x105BF
L"Buthakukye", // 0x105C0-0x105DF
L"Linear A", // 0x10600-0x1077F
L"Cypro Minoan", // 0x10780-0x107BF
L"Cypriot Syllabary", // 0x10800-0x1083F
L"Meroitic", // 0x10840-0x1087F
L"Manichaean", // 0x10880-0x108AF
L"Balti", // 0x108B0-0x108CF
L"Yezidi", // 0x108D0-0x108FF
L"Phoenician", // 0x10900-0x1091F
L"Aramaic", // 0x10920-0x1093F
L"South Arabian", // 0x10940-0x1095F
L"North Arabic", // 0x10960-0x1097F
L"Palmyrene", // 0x10980-0x1099F
L"Nabataean", // 0x109A0-0x109BF
L"Kharoshthi", // 0x10A00-0x10A5F
L"Hatran", // 0x10A60-0x10A7F
L"Orkhon", // 0x10A80-0x10ABF
L"Lydian", // 0x10B00-0x10B1F
L"Carian", // 0x10B20-0x10B3F
L"Numidian", // 0x10B40-0x10B5F
L"Elymaic", // 0x10B60-0x10B7F
L"Uigur", // 0x10B80-0x10BDF
L"Proto Elamite", // 0x10C00-0x10DFF
L"Byblos", // 0x10E00-0x10E6F
L"Brahmi", // 0x11000-0x1104F
L"Pyu", // 0x11080-0x110BF
L"Soyombo", // 0x11100-0x1113F
L"Ahorn", // 0x11140-0x1117F
L"Turkestani", // 0x11180-0x111DF
L"Tulu", // 0x11200-0x1125F
L"Rejang", // 0x11280-0x112DF
L"Landa", // 0x11300-0x113DF
L"Modi", // 0x11380-0x113DF
L"Chalukya", // 0x11400-0x1145F
L"Chola", // 0x11480-0x114DF
L"Satavahana", // 0x11500-0x1155F
L"Takri", // 0x11580-0x115DF
L"Kaithi", // 0x11600-0x1165F
L"Vai", // 0x11800-0x118FF
L"Barnum", // 0x11900-0x11AFF
L"Mende", // 0x11B00-0x11BBF
L"Bassa", // 0x11BC0-0x11BEF
L"Chinook", // 0x11D00-0x11D2F
L"Shorthands", // 0x11E00-0x11FFF
L"Cuneiform", // 0x12000-0x123FF
L"Cuneiform Numbers", // 0x12400-0x1247F
L"Archaic Cuneiform Ext", // 0x12480-0x124FF
L"Hittite Hieroglyphs/Luvian", // 0x12800-0x128AF
L"Indus", // 0x12900-0x12A8F
L"Rongorongo", // 0x12B00-0x12E4F
L"Tengwar", // 0x13000-0x1307F
L"Cirth", // 0x13080-0x130FF
L"Blissymbols", // 0x13200-0x135FF
L"Blissymbol Ext", // 0x13600-0x136FF
L"Egyptian Hieroglyphs", // 0x14000-0x142FF
L"Egyptian Hieroglyphs Ext", // 0x14400-0x156FF
L"Egyptian Hieroglyphs Ext A", // 0x15700-0x163FF
L"Mayan Hieroglyphs", // 0x16400-0x168FF
L"Aztec Pictograms", // 0x16C00-0x16FFF
L"Tangut Ideographs", // 0x17000-0x189FF
L"Kitan Small", // 0x18C00-0x18D7F
L"Kitan Large", // 0x18D80-0x19FFF
L"Jurchin", // 0x1A000-0x1A5FF
L"Naxi Geba", // 0x1A800-0x1AAFF
L"Naxi Tomba", // 0x1A8C0-0x1AFFF
L"Nushu", // 0x1B000-0x1B5FF
L"Byzantine Musical", // 0x1D000-0x1D0FF
L"Musical", // 0x1D100-0x1D1FF
L"Ancient Greek Musical", // 0x1D200-0x1D24F
L"Tai Xuan Jing", // 0x1D300-0x1D35F
L"Rod Numerals", // 0x1D360-0x1D37F
L"Mathematical Alphanum", // 0x1D400-0x1D7FF
L"Sutton Sign Writing", // 0x1D800-0x1DBFF
L"CJK Ext B", // 0x20000-0x2A6DF
L"CJK Ext C1", // 0x2A700-0x2F7FF
L"CJK Compat Ideographs Sup", // 0x2F800-0x2FA1F
L"Tags", // 0xE0000-0xE007F
L"Variation Sup", // 0xE0100-0xE01EF
L"Private", // 0xF0000-0xFFFFF
L"Private", // 0x100000-0x10FFFF
L"ASCII",
L"Unknown"
};

const struct UnicodeInterval UCIgnored[NUCIGNORED]={ // Mapped to nothing
{0x00,0x00AD,0x00AD}, // SOFT HYPHEN
{0x00,0x1806,0x1806}, // MONGOLIAN TODO SOFT HYPHEN
{0x00,0x2060,0x2060}, // WORD JOINER
{0x00,0x034F,0x034F}, // COMBINING GRAPHEME JOINER
{0x00,0x180B,0x180B}, // MONGOLIAN FREE VARIATION SELECTOR ONE
{0x00,0x180C,0x180C}, // MONGOLIAN FREE VARIATION SELECTOR TWO
{0x00,0x180D,0x180D}, // MONGOLIAN FREE VARIATION SELECTOR THREE
{0x00,0x200B,0x200B}, // ZERO WIDTH SPACE
{0x00,0x200C,0x200C}, // ZERO WIDTH NON-JOINER
{0x00,0x200D,0x200D}, // ZERO WIDTH JOINER
{0x00,0xFE00,0xFE0F}, // VARIATION SELECTOR-1 to -16
{0x00,0xFEFF,0xFEFF}, // ZERO WIDTH NO-BREAK SPACE
};

const struct UnicodeMapping UCMapping[NUCMAPPING]={
{0x00,0x0041,{0x0061,0x0000,0x0000,0x0000}},
{0x00,0x0042,{0x0062,0x0000,0x0000,0x0000}},
{0x00,0x0043,{0x0063,0x0000,0x0000,0x0000}},
{0x00,0x0044,{0x0064,0x0000,0x0000,0x0000}},
{0x00,0x0045,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0x0046,{0x0066,0x0000,0x0000,0x0000}},
{0x00,0x0047,{0x0067,0x0000,0x0000,0x0000}},
{0x00,0x0048,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0x0049,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x004A,{0x006A,0x0000,0x0000,0x0000}},
{0x00,0x004B,{0x006B,0x0000,0x0000,0x0000}},
{0x00,0x004C,{0x006C,0x0000,0x0000,0x0000}},
{0x00,0x004D,{0x006D,0x0000,0x0000,0x0000}},
{0x00,0x004E,{0x006E,0x0000,0x0000,0x0000}},
{0x00,0x004F,{0x006F,0x0000,0x0000,0x0000}},
{0x00,0x0050,{0x0070,0x0000,0x0000,0x0000}},
{0x00,0x0051,{0x0071,0x0000,0x0000,0x0000}},
{0x00,0x0052,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0x0053,{0x0073,0x0000,0x0000,0x0000}},
{0x00,0x0054,{0x0074,0x0000,0x0000,0x0000}},
{0x00,0x0055,{0x0075,0x0000,0x0000,0x0000}},
{0x00,0x0056,{0x0076,0x0000,0x0000,0x0000}},
{0x00,0x0057,{0x0077,0x0000,0x0000,0x0000}},
{0x00,0x0058,{0x0078,0x0000,0x0000,0x0000}},
{0x00,0x0059,{0x0079,0x0000,0x0000,0x0000}},
{0x00,0x005A,{0x007A,0x0000,0x0000,0x0000}},
{0x00,0x00B5,{0x03BC,0x0000,0x0000,0x0000}},
{0x00,0x00C0,{0x00E0,0x0000,0x0000,0x0000}},
{0x00,0x00C1,{0x00E1,0x0000,0x0000,0x0000}},
{0x00,0x00C2,{0x00E2,0x0000,0x0000,0x0000}},
{0x00,0x00C3,{0x00E3,0x0000,0x0000,0x0000}},
{0x00,0x00C4,{0x00E4,0x0000,0x0000,0x0000}},
{0x00,0x00C5,{0x00E5,0x0000,0x0000,0x0000}},
{0x00,0x00C6,{0x00E6,0x0000,0x0000,0x0000}},
{0x00,0x00C7,{0x00E7,0x0000,0x0000,0x0000}},
{0x00,0x00C8,{0x00E8,0x0000,0x0000,0x0000}},
{0x00,0x00C9,{0x00E9,0x0000,0x0000,0x0000}},
{0x00,0x00CA,{0x00EA,0x0000,0x0000,0x0000}},
{0x00,0x00CB,{0x00EB,0x0000,0x0000,0x0000}},
{0x00,0x00CC,{0x00EC,0x0000,0x0000,0x0000}},
{0x00,0x00CD,{0x00ED,0x0000,0x0000,0x0000}},
{0x00,0x00CE,{0x00EE,0x0000,0x0000,0x0000}},
{0x00,0x00CF,{0x00EF,0x0000,0x0000,0x0000}},
{0x00,0x00D0,{0x00F0,0x0000,0x0000,0x0000}},
{0x00,0x00D1,{0x00F1,0x0000,0x0000,0x0000}},
{0x00,0x00D2,{0x00F2,0x0000,0x0000,0x0000}},
{0x00,0x00D3,{0x00F3,0x0000,0x0000,0x0000}},
{0x00,0x00D4,{0x00F4,0x0000,0x0000,0x0000}},
{0x00,0x00D5,{0x00F5,0x0000,0x0000,0x0000}},
{0x00,0x00D6,{0x00F6,0x0000,0x0000,0x0000}},
{0x00,0x00D8,{0x00F8,0x0000,0x0000,0x0000}},
{0x00,0x00D9,{0x00F9,0x0000,0x0000,0x0000}},
{0x00,0x00DA,{0x00FA,0x0000,0x0000,0x0000}},
{0x00,0x00DB,{0x00FB,0x0000,0x0000,0x0000}},
{0x00,0x00DC,{0x00FC,0x0000,0x0000,0x0000}},
{0x00,0x00DD,{0x00FD,0x0000,0x0000,0x0000}},
{0x00,0x00DE,{0x00FE,0x0000,0x0000,0x0000}},
//{0x00,0x00DF,{0x0073,0x0073,0x0000,0x0000}}, // German small letter sharp s can be registered by DENIC
{0x00,0x0100,{0x0101,0x0000,0x0000,0x0000}},
{0x00,0x0102,{0x0103,0x0000,0x0000,0x0000}},
{0x00,0x0104,{0x0105,0x0000,0x0000,0x0000}},
{0x00,0x0106,{0x0107,0x0000,0x0000,0x0000}},
{0x00,0x0108,{0x0109,0x0000,0x0000,0x0000}},
{0x00,0x010A,{0x010B,0x0000,0x0000,0x0000}},
{0x00,0x010C,{0x010D,0x0000,0x0000,0x0000}},
{0x00,0x010E,{0x010F,0x0000,0x0000,0x0000}},
{0x00,0x0110,{0x0111,0x0000,0x0000,0x0000}},
{0x00,0x0112,{0x0113,0x0000,0x0000,0x0000}},
{0x00,0x0114,{0x0115,0x0000,0x0000,0x0000}},
{0x00,0x0116,{0x0117,0x0000,0x0000,0x0000}},
{0x00,0x0118,{0x0119,0x0000,0x0000,0x0000}},
{0x00,0x011A,{0x011B,0x0000,0x0000,0x0000}},
{0x00,0x011C,{0x011D,0x0000,0x0000,0x0000}},
{0x00,0x011E,{0x011F,0x0000,0x0000,0x0000}},
{0x00,0x0120,{0x0121,0x0000,0x0000,0x0000}},
{0x00,0x0122,{0x0123,0x0000,0x0000,0x0000}},
{0x00,0x0124,{0x0125,0x0000,0x0000,0x0000}},
{0x00,0x0126,{0x0127,0x0000,0x0000,0x0000}},
{0x00,0x0128,{0x0129,0x0000,0x0000,0x0000}},
{0x00,0x012A,{0x012B,0x0000,0x0000,0x0000}},
{0x00,0x012C,{0x012D,0x0000,0x0000,0x0000}},
{0x00,0x012E,{0x012F,0x0000,0x0000,0x0000}},
{0x00,0x0130,{0x0069,0x0307,0x0000,0x0000}},
{0x00,0x0132,{0x0133,0x0000,0x0000,0x0000}},
{0x00,0x0134,{0x0135,0x0000,0x0000,0x0000}},
{0x00,0x0136,{0x0137,0x0000,0x0000,0x0000}},
{0x00,0x0139,{0x013A,0x0000,0x0000,0x0000}},
{0x00,0x013B,{0x013C,0x0000,0x0000,0x0000}},
{0x00,0x013D,{0x013E,0x0000,0x0000,0x0000}},
{0x00,0x013F,{0x0140,0x0000,0x0000,0x0000}},
{0x00,0x0141,{0x0142,0x0000,0x0000,0x0000}},
{0x00,0x0143,{0x0144,0x0000,0x0000,0x0000}},
{0x00,0x0145,{0x0146,0x0000,0x0000,0x0000}},
{0x00,0x0147,{0x0148,0x0000,0x0000,0x0000}},
{0x00,0x0149,{0x02BC,0x006E,0x0000,0x0000}},
{0x00,0x014A,{0x014B,0x0000,0x0000,0x0000}},
{0x00,0x014C,{0x014D,0x0000,0x0000,0x0000}},
{0x00,0x014E,{0x014F,0x0000,0x0000,0x0000}},
{0x00,0x0150,{0x0151,0x0000,0x0000,0x0000}},
{0x00,0x0152,{0x0153,0x0000,0x0000,0x0000}},
{0x00,0x0154,{0x0155,0x0000,0x0000,0x0000}},
{0x00,0x0156,{0x0157,0x0000,0x0000,0x0000}},
{0x00,0x0158,{0x0159,0x0000,0x0000,0x0000}},
{0x00,0x015A,{0x015B,0x0000,0x0000,0x0000}},
{0x00,0x015C,{0x015D,0x0000,0x0000,0x0000}},
{0x00,0x015E,{0x015F,0x0000,0x0000,0x0000}},
{0x00,0x0160,{0x0161,0x0000,0x0000,0x0000}},
{0x00,0x0162,{0x0163,0x0000,0x0000,0x0000}},
{0x00,0x0164,{0x0165,0x0000,0x0000,0x0000}},
{0x00,0x0166,{0x0167,0x0000,0x0000,0x0000}},
{0x00,0x0168,{0x0169,0x0000,0x0000,0x0000}},
{0x00,0x016A,{0x016B,0x0000,0x0000,0x0000}},
{0x00,0x016C,{0x016D,0x0000,0x0000,0x0000}},
{0x00,0x016E,{0x016F,0x0000,0x0000,0x0000}},
{0x00,0x0170,{0x0171,0x0000,0x0000,0x0000}},
{0x00,0x0172,{0x0173,0x0000,0x0000,0x0000}},
{0x00,0x0174,{0x0175,0x0000,0x0000,0x0000}},
{0x00,0x0176,{0x0177,0x0000,0x0000,0x0000}},
{0x00,0x0178,{0x00FF,0x0000,0x0000,0x0000}},
{0x00,0x0179,{0x017A,0x0000,0x0000,0x0000}},
{0x00,0x017B,{0x017C,0x0000,0x0000,0x0000}},
{0x00,0x017D,{0x017E,0x0000,0x0000,0x0000}},
{0x00,0x017F,{0x0073,0x0000,0x0000,0x0000}},
{0x00,0x0181,{0x0253,0x0000,0x0000,0x0000}},
{0x00,0x0182,{0x0183,0x0000,0x0000,0x0000}},
{0x00,0x0184,{0x0185,0x0000,0x0000,0x0000}},
{0x00,0x0186,{0x0254,0x0000,0x0000,0x0000}},
{0x00,0x0187,{0x0188,0x0000,0x0000,0x0000}},
{0x00,0x0189,{0x0256,0x0000,0x0000,0x0000}},
{0x00,0x018A,{0x0257,0x0000,0x0000,0x0000}},
{0x00,0x018B,{0x018C,0x0000,0x0000,0x0000}},
{0x00,0x018E,{0x01DD,0x0000,0x0000,0x0000}},
{0x00,0x018F,{0x0259,0x0000,0x0000,0x0000}},
{0x00,0x0190,{0x025B,0x0000,0x0000,0x0000}},
{0x00,0x0191,{0x0192,0x0000,0x0000,0x0000}},
{0x00,0x0193,{0x0260,0x0000,0x0000,0x0000}},
{0x00,0x0194,{0x0263,0x0000,0x0000,0x0000}},
{0x00,0x0196,{0x0269,0x0000,0x0000,0x0000}},
{0x00,0x0197,{0x0268,0x0000,0x0000,0x0000}},
{0x00,0x0198,{0x0199,0x0000,0x0000,0x0000}},
{0x00,0x019C,{0x026F,0x0000,0x0000,0x0000}},
{0x00,0x019D,{0x0272,0x0000,0x0000,0x0000}},
{0x00,0x019F,{0x0275,0x0000,0x0000,0x0000}},
{0x00,0x01A0,{0x01A1,0x0000,0x0000,0x0000}},
{0x00,0x01A2,{0x01A3,0x0000,0x0000,0x0000}},
{0x00,0x01A4,{0x01A5,0x0000,0x0000,0x0000}},
{0x00,0x01A6,{0x0280,0x0000,0x0000,0x0000}},
{0x00,0x01A7,{0x01A8,0x0000,0x0000,0x0000}},
{0x00,0x01A9,{0x0283,0x0000,0x0000,0x0000}},
{0x00,0x01AC,{0x01AD,0x0000,0x0000,0x0000}},
{0x00,0x01AE,{0x0288,0x0000,0x0000,0x0000}},
{0x00,0x01AF,{0x01B0,0x0000,0x0000,0x0000}},
{0x00,0x01B1,{0x028A,0x0000,0x0000,0x0000}},
{0x00,0x01B2,{0x028B,0x0000,0x0000,0x0000}},
{0x00,0x01B3,{0x01B4,0x0000,0x0000,0x0000}},
{0x00,0x01B5,{0x01B6,0x0000,0x0000,0x0000}},
{0x00,0x01B7,{0x0292,0x0000,0x0000,0x0000}},
{0x00,0x01B8,{0x01B9,0x0000,0x0000,0x0000}},
{0x00,0x01BC,{0x01BD,0x0000,0x0000,0x0000}},
{0x00,0x01C4,{0x01C6,0x0000,0x0000,0x0000}},
{0x00,0x01C5,{0x01C6,0x0000,0x0000,0x0000}},
{0x00,0x01C7,{0x01C9,0x0000,0x0000,0x0000}},
{0x00,0x01C8,{0x01C9,0x0000,0x0000,0x0000}},
{0x00,0x01CA,{0x01CC,0x0000,0x0000,0x0000}},
{0x00,0x01CB,{0x01CC,0x0000,0x0000,0x0000}},
{0x00,0x01CD,{0x01CE,0x0000,0x0000,0x0000}},
{0x00,0x01CF,{0x01D0,0x0000,0x0000,0x0000}},
{0x00,0x01D1,{0x01D2,0x0000,0x0000,0x0000}},
{0x00,0x01D3,{0x01D4,0x0000,0x0000,0x0000}},
{0x00,0x01D5,{0x01D6,0x0000,0x0000,0x0000}},
{0x00,0x01D7,{0x01D8,0x0000,0x0000,0x0000}},
{0x00,0x01D9,{0x01DA,0x0000,0x0000,0x0000}},
{0x00,0x01DB,{0x01DC,0x0000,0x0000,0x0000}},
{0x00,0x01DE,{0x01DF,0x0000,0x0000,0x0000}},
{0x00,0x01E0,{0x01E1,0x0000,0x0000,0x0000}},
{0x00,0x01E2,{0x01E3,0x0000,0x0000,0x0000}},
{0x00,0x01E4,{0x01E5,0x0000,0x0000,0x0000}},
{0x00,0x01E6,{0x01E7,0x0000,0x0000,0x0000}},
{0x00,0x01E8,{0x01E9,0x0000,0x0000,0x0000}},
{0x00,0x01EA,{0x01EB,0x0000,0x0000,0x0000}},
{0x00,0x01EC,{0x01ED,0x0000,0x0000,0x0000}},
{0x00,0x01EE,{0x01EF,0x0000,0x0000,0x0000}},
{0x00,0x01F0,{0x006A,0x030C,0x0000,0x0000}},
{0x00,0x01F1,{0x01F3,0x0000,0x0000,0x0000}},
{0x00,0x01F2,{0x01F3,0x0000,0x0000,0x0000}},
{0x00,0x01F4,{0x01F5,0x0000,0x0000,0x0000}},
{0x00,0x01F6,{0x0195,0x0000,0x0000,0x0000}},
{0x00,0x01F7,{0x01BF,0x0000,0x0000,0x0000}},
{0x00,0x01F8,{0x01F9,0x0000,0x0000,0x0000}},
{0x00,0x01FA,{0x01FB,0x0000,0x0000,0x0000}},
{0x00,0x01FC,{0x01FD,0x0000,0x0000,0x0000}},
{0x00,0x01FE,{0x01FF,0x0000,0x0000,0x0000}},
{0x00,0x0200,{0x0201,0x0000,0x0000,0x0000}},
{0x00,0x0202,{0x0203,0x0000,0x0000,0x0000}},
{0x00,0x0204,{0x0205,0x0000,0x0000,0x0000}},
{0x00,0x0206,{0x0207,0x0000,0x0000,0x0000}},
{0x00,0x0208,{0x0209,0x0000,0x0000,0x0000}},
{0x00,0x020A,{0x020B,0x0000,0x0000,0x0000}},
{0x00,0x020C,{0x020D,0x0000,0x0000,0x0000}},
{0x00,0x020E,{0x020F,0x0000,0x0000,0x0000}},
{0x00,0x0210,{0x0211,0x0000,0x0000,0x0000}},
{0x00,0x0212,{0x0213,0x0000,0x0000,0x0000}},
{0x00,0x0214,{0x0215,0x0000,0x0000,0x0000}},
{0x00,0x0216,{0x0217,0x0000,0x0000,0x0000}},
{0x00,0x0218,{0x0219,0x0000,0x0000,0x0000}},
{0x00,0x021A,{0x021B,0x0000,0x0000,0x0000}},
{0x00,0x021C,{0x021D,0x0000,0x0000,0x0000}},
{0x00,0x021E,{0x021F,0x0000,0x0000,0x0000}},
{0x00,0x0220,{0x019E,0x0000,0x0000,0x0000}},
{0x00,0x0222,{0x0223,0x0000,0x0000,0x0000}},
{0x00,0x0224,{0x0225,0x0000,0x0000,0x0000}},
{0x00,0x0226,{0x0227,0x0000,0x0000,0x0000}},
{0x00,0x0228,{0x0229,0x0000,0x0000,0x0000}},
{0x00,0x022A,{0x022B,0x0000,0x0000,0x0000}},
{0x00,0x022C,{0x022D,0x0000,0x0000,0x0000}},
{0x00,0x022E,{0x022F,0x0000,0x0000,0x0000}},
{0x00,0x0230,{0x0231,0x0000,0x0000,0x0000}},
{0x00,0x0232,{0x0233,0x0000,0x0000,0x0000}},
{0x00,0x0345,{0x03B9,0x0000,0x0000,0x0000}},
{0x00,0x037A,{0x0020,0x03B9,0x0000,0x0000}},
{0x00,0x0386,{0x03AC,0x0000,0x0000,0x0000}},
{0x00,0x0388,{0x03AD,0x0000,0x0000,0x0000}},
{0x00,0x0389,{0x03AE,0x0000,0x0000,0x0000}},
{0x00,0x038A,{0x03AF,0x0000,0x0000,0x0000}},
{0x00,0x038C,{0x03CC,0x0000,0x0000,0x0000}},
{0x00,0x038E,{0x03CD,0x0000,0x0000,0x0000}},
{0x00,0x038F,{0x03CE,0x0000,0x0000,0x0000}},
{0x00,0x0390,{0x03B9,0x0308,0x0301,0x0000}},
{0x00,0x0391,{0x03B1,0x0000,0x0000,0x0000}},
{0x00,0x0392,{0x03B2,0x0000,0x0000,0x0000}},
{0x00,0x0393,{0x03B3,0x0000,0x0000,0x0000}},
{0x00,0x0394,{0x03B4,0x0000,0x0000,0x0000}},
{0x00,0x0395,{0x03B5,0x0000,0x0000,0x0000}},
{0x00,0x0396,{0x03B6,0x0000,0x0000,0x0000}},
{0x00,0x0397,{0x03B7,0x0000,0x0000,0x0000}},
{0x00,0x0398,{0x03B8,0x0000,0x0000,0x0000}},
{0x00,0x0399,{0x03B9,0x0000,0x0000,0x0000}},
{0x00,0x039A,{0x03BA,0x0000,0x0000,0x0000}},
{0x00,0x039B,{0x03BB,0x0000,0x0000,0x0000}},
{0x00,0x039C,{0x03BC,0x0000,0x0000,0x0000}},
{0x00,0x039D,{0x03BD,0x0000,0x0000,0x0000}},
{0x00,0x039E,{0x03BE,0x0000,0x0000,0x0000}},
{0x00,0x039F,{0x03BF,0x0000,0x0000,0x0000}},
{0x00,0x03A0,{0x03C0,0x0000,0x0000,0x0000}},
{0x00,0x03A1,{0x03C1,0x0000,0x0000,0x0000}},
{0x00,0x03A3,{0x03C3,0x0000,0x0000,0x0000}},
{0x00,0x03A4,{0x03C4,0x0000,0x0000,0x0000}},
{0x00,0x03A5,{0x03C5,0x0000,0x0000,0x0000}},
{0x00,0x03A6,{0x03C6,0x0000,0x0000,0x0000}},
{0x00,0x03A7,{0x03C7,0x0000,0x0000,0x0000}},
{0x00,0x03A8,{0x03C8,0x0000,0x0000,0x0000}},
{0x00,0x03A9,{0x03C9,0x0000,0x0000,0x0000}},
{0x00,0x03AA,{0x03CA,0x0000,0x0000,0x0000}},
{0x00,0x03AB,{0x03CB,0x0000,0x0000,0x0000}},
{0x00,0x03B0,{0x03C5,0x0308,0x0301,0x0000}},
{0x00,0x03C2,{0x03C3,0x0000,0x0000,0x0000}},
{0x00,0x03D0,{0x03B2,0x0000,0x0000,0x0000}},
{0x00,0x03D1,{0x03B8,0x0000,0x0000,0x0000}},
{0x00,0x03D2,{0x03C5,0x0000,0x0000,0x0000}},
{0x00,0x03D3,{0x03CD,0x0000,0x0000,0x0000}},
{0x00,0x03D4,{0x03CB,0x0000,0x0000,0x0000}},
{0x00,0x03D5,{0x03C6,0x0000,0x0000,0x0000}},
{0x00,0x03D6,{0x03C0,0x0000,0x0000,0x0000}},
{0x00,0x03D8,{0x03D9,0x0000,0x0000,0x0000}},
{0x00,0x03DA,{0x03DB,0x0000,0x0000,0x0000}},
{0x00,0x03DC,{0x03DD,0x0000,0x0000,0x0000}},
{0x00,0x03DE,{0x03DF,0x0000,0x0000,0x0000}},
{0x00,0x03E0,{0x03E1,0x0000,0x0000,0x0000}},
{0x00,0x03E2,{0x03E3,0x0000,0x0000,0x0000}},
{0x00,0x03E4,{0x03E5,0x0000,0x0000,0x0000}},
{0x00,0x03E6,{0x03E7,0x0000,0x0000,0x0000}},
{0x00,0x03E8,{0x03E9,0x0000,0x0000,0x0000}},
{0x00,0x03EA,{0x03EB,0x0000,0x0000,0x0000}},
{0x00,0x03EC,{0x03ED,0x0000,0x0000,0x0000}},
{0x00,0x03EE,{0x03EF,0x0000,0x0000,0x0000}},
{0x00,0x03F0,{0x03BA,0x0000,0x0000,0x0000}},
{0x00,0x03F1,{0x03C1,0x0000,0x0000,0x0000}},
{0x00,0x03F2,{0x03C3,0x0000,0x0000,0x0000}},
{0x00,0x03F4,{0x03B8,0x0000,0x0000,0x0000}},
{0x00,0x03F5,{0x03B5,0x0000,0x0000,0x0000}},
{0x00,0x0400,{0x0450,0x0000,0x0000,0x0000}},
{0x00,0x0401,{0x0451,0x0000,0x0000,0x0000}},
{0x00,0x0402,{0x0452,0x0000,0x0000,0x0000}},
{0x00,0x0403,{0x0453,0x0000,0x0000,0x0000}},
{0x00,0x0404,{0x0454,0x0000,0x0000,0x0000}},
{0x00,0x0405,{0x0455,0x0000,0x0000,0x0000}},
{0x00,0x0406,{0x0456,0x0000,0x0000,0x0000}},
{0x00,0x0407,{0x0457,0x0000,0x0000,0x0000}},
{0x00,0x0408,{0x0458,0x0000,0x0000,0x0000}},
{0x00,0x0409,{0x0459,0x0000,0x0000,0x0000}},
{0x00,0x040A,{0x045A,0x0000,0x0000,0x0000}},
{0x00,0x040B,{0x045B,0x0000,0x0000,0x0000}},
{0x00,0x040C,{0x045C,0x0000,0x0000,0x0000}},
{0x00,0x040D,{0x045D,0x0000,0x0000,0x0000}},
{0x00,0x040E,{0x045E,0x0000,0x0000,0x0000}},
{0x00,0x040F,{0x045F,0x0000,0x0000,0x0000}},
{0x00,0x0410,{0x0430,0x0000,0x0000,0x0000}},
{0x00,0x0411,{0x0431,0x0000,0x0000,0x0000}},
{0x00,0x0412,{0x0432,0x0000,0x0000,0x0000}},
{0x00,0x0413,{0x0433,0x0000,0x0000,0x0000}},
{0x00,0x0414,{0x0434,0x0000,0x0000,0x0000}},
{0x00,0x0415,{0x0435,0x0000,0x0000,0x0000}},
{0x00,0x0416,{0x0436,0x0000,0x0000,0x0000}},
{0x00,0x0417,{0x0437,0x0000,0x0000,0x0000}},
{0x00,0x0418,{0x0438,0x0000,0x0000,0x0000}},
{0x00,0x0419,{0x0439,0x0000,0x0000,0x0000}},
{0x00,0x041A,{0x043A,0x0000,0x0000,0x0000}},
{0x00,0x041B,{0x043B,0x0000,0x0000,0x0000}},
{0x00,0x041C,{0x043C,0x0000,0x0000,0x0000}},
{0x00,0x041D,{0x043D,0x0000,0x0000,0x0000}},
{0x00,0x041E,{0x043E,0x0000,0x0000,0x0000}},
{0x00,0x041F,{0x043F,0x0000,0x0000,0x0000}},
{0x00,0x0420,{0x0440,0x0000,0x0000,0x0000}},
{0x00,0x0421,{0x0441,0x0000,0x0000,0x0000}},
{0x00,0x0422,{0x0442,0x0000,0x0000,0x0000}},
{0x00,0x0423,{0x0443,0x0000,0x0000,0x0000}},
{0x00,0x0424,{0x0444,0x0000,0x0000,0x0000}},
{0x00,0x0425,{0x0445,0x0000,0x0000,0x0000}},
{0x00,0x0426,{0x0446,0x0000,0x0000,0x0000}},
{0x00,0x0427,{0x0447,0x0000,0x0000,0x0000}},
{0x00,0x0428,{0x0448,0x0000,0x0000,0x0000}},
{0x00,0x0429,{0x0449,0x0000,0x0000,0x0000}},
{0x00,0x042A,{0x044A,0x0000,0x0000,0x0000}},
{0x00,0x042B,{0x044B,0x0000,0x0000,0x0000}},
{0x00,0x042C,{0x044C,0x0000,0x0000,0x0000}},
{0x00,0x042D,{0x044D,0x0000,0x0000,0x0000}},
{0x00,0x042E,{0x044E,0x0000,0x0000,0x0000}},
{0x00,0x042F,{0x044F,0x0000,0x0000,0x0000}},
{0x00,0x0460,{0x0461,0x0000,0x0000,0x0000}},
{0x00,0x0462,{0x0463,0x0000,0x0000,0x0000}},
{0x00,0x0464,{0x0465,0x0000,0x0000,0x0000}},
{0x00,0x0466,{0x0467,0x0000,0x0000,0x0000}},
{0x00,0x0468,{0x0469,0x0000,0x0000,0x0000}},
{0x00,0x046A,{0x046B,0x0000,0x0000,0x0000}},
{0x00,0x046C,{0x046D,0x0000,0x0000,0x0000}},
{0x00,0x046E,{0x046F,0x0000,0x0000,0x0000}},
{0x00,0x0470,{0x0471,0x0000,0x0000,0x0000}},
{0x00,0x0472,{0x0473,0x0000,0x0000,0x0000}},
{0x00,0x0474,{0x0475,0x0000,0x0000,0x0000}},
{0x00,0x0476,{0x0477,0x0000,0x0000,0x0000}},
{0x00,0x0478,{0x0479,0x0000,0x0000,0x0000}},
{0x00,0x047A,{0x047B,0x0000,0x0000,0x0000}},
{0x00,0x047C,{0x047D,0x0000,0x0000,0x0000}},
{0x00,0x047E,{0x047F,0x0000,0x0000,0x0000}},
{0x00,0x0480,{0x0481,0x0000,0x0000,0x0000}},
{0x00,0x048A,{0x048B,0x0000,0x0000,0x0000}},
{0x00,0x048C,{0x048D,0x0000,0x0000,0x0000}},
{0x00,0x048E,{0x048F,0x0000,0x0000,0x0000}},
{0x00,0x0490,{0x0491,0x0000,0x0000,0x0000}},
{0x00,0x0492,{0x0493,0x0000,0x0000,0x0000}},
{0x00,0x0494,{0x0495,0x0000,0x0000,0x0000}},
{0x00,0x0496,{0x0497,0x0000,0x0000,0x0000}},
{0x00,0x0498,{0x0499,0x0000,0x0000,0x0000}},
{0x00,0x049A,{0x049B,0x0000,0x0000,0x0000}},
{0x00,0x049C,{0x049D,0x0000,0x0000,0x0000}},
{0x00,0x049E,{0x049F,0x0000,0x0000,0x0000}},
{0x00,0x04A0,{0x04A1,0x0000,0x0000,0x0000}},
{0x00,0x04A2,{0x04A3,0x0000,0x0000,0x0000}},
{0x00,0x04A4,{0x04A5,0x0000,0x0000,0x0000}},
{0x00,0x04A6,{0x04A7,0x0000,0x0000,0x0000}},
{0x00,0x04A8,{0x04A9,0x0000,0x0000,0x0000}},
{0x00,0x04AA,{0x04AB,0x0000,0x0000,0x0000}},
{0x00,0x04AC,{0x04AD,0x0000,0x0000,0x0000}},
{0x00,0x04AE,{0x04AF,0x0000,0x0000,0x0000}},
{0x00,0x04B0,{0x04B1,0x0000,0x0000,0x0000}},
{0x00,0x04B2,{0x04B3,0x0000,0x0000,0x0000}},
{0x00,0x04B4,{0x04B5,0x0000,0x0000,0x0000}},
{0x00,0x04B6,{0x04B7,0x0000,0x0000,0x0000}},
{0x00,0x04B8,{0x04B9,0x0000,0x0000,0x0000}},
{0x00,0x04BA,{0x04BB,0x0000,0x0000,0x0000}},
{0x00,0x04BC,{0x04BD,0x0000,0x0000,0x0000}},
{0x00,0x04BE,{0x04BF,0x0000,0x0000,0x0000}},
{0x00,0x04C1,{0x04C2,0x0000,0x0000,0x0000}},
{0x00,0x04C3,{0x04C4,0x0000,0x0000,0x0000}},
{0x00,0x04C5,{0x04C6,0x0000,0x0000,0x0000}},
{0x00,0x04C7,{0x04C8,0x0000,0x0000,0x0000}},
{0x00,0x04C9,{0x04CA,0x0000,0x0000,0x0000}},
{0x00,0x04CB,{0x04CC,0x0000,0x0000,0x0000}},
{0x00,0x04CD,{0x04CE,0x0000,0x0000,0x0000}},
{0x00,0x04D0,{0x04D1,0x0000,0x0000,0x0000}},
{0x00,0x04D2,{0x04D3,0x0000,0x0000,0x0000}},
{0x00,0x04D4,{0x04D5,0x0000,0x0000,0x0000}},
{0x00,0x04D6,{0x04D7,0x0000,0x0000,0x0000}},
{0x00,0x04D8,{0x04D9,0x0000,0x0000,0x0000}},
{0x00,0x04DA,{0x04DB,0x0000,0x0000,0x0000}},
{0x00,0x04DC,{0x04DD,0x0000,0x0000,0x0000}},
{0x00,0x04DE,{0x04DF,0x0000,0x0000,0x0000}},
{0x00,0x04E0,{0x04E1,0x0000,0x0000,0x0000}},
{0x00,0x04E2,{0x04E3,0x0000,0x0000,0x0000}},
{0x00,0x04E4,{0x04E5,0x0000,0x0000,0x0000}},
{0x00,0x04E6,{0x04E7,0x0000,0x0000,0x0000}},
{0x00,0x04E8,{0x04E9,0x0000,0x0000,0x0000}},
{0x00,0x04EA,{0x04EB,0x0000,0x0000,0x0000}},
{0x00,0x04EC,{0x04ED,0x0000,0x0000,0x0000}},
{0x00,0x04EE,{0x04EF,0x0000,0x0000,0x0000}},
{0x00,0x04F0,{0x04F1,0x0000,0x0000,0x0000}},
{0x00,0x04F2,{0x04F3,0x0000,0x0000,0x0000}},
{0x00,0x04F4,{0x04F5,0x0000,0x0000,0x0000}},
{0x00,0x04F8,{0x04F9,0x0000,0x0000,0x0000}},
{0x00,0x0500,{0x0501,0x0000,0x0000,0x0000}},
{0x00,0x0502,{0x0503,0x0000,0x0000,0x0000}},
{0x00,0x0504,{0x0505,0x0000,0x0000,0x0000}},
{0x00,0x0506,{0x0507,0x0000,0x0000,0x0000}},
{0x00,0x0508,{0x0509,0x0000,0x0000,0x0000}},
{0x00,0x050A,{0x050B,0x0000,0x0000,0x0000}},
{0x00,0x050C,{0x050D,0x0000,0x0000,0x0000}},
{0x00,0x050E,{0x050F,0x0000,0x0000,0x0000}},
{0x00,0x0531,{0x0561,0x0000,0x0000,0x0000}},
{0x00,0x0532,{0x0562,0x0000,0x0000,0x0000}},
{0x00,0x0533,{0x0563,0x0000,0x0000,0x0000}},
{0x00,0x0534,{0x0564,0x0000,0x0000,0x0000}},
{0x00,0x0535,{0x0565,0x0000,0x0000,0x0000}},
{0x00,0x0536,{0x0566,0x0000,0x0000,0x0000}},
{0x00,0x0537,{0x0567,0x0000,0x0000,0x0000}},
{0x00,0x0538,{0x0568,0x0000,0x0000,0x0000}},
{0x00,0x0539,{0x0569,0x0000,0x0000,0x0000}},
{0x00,0x053A,{0x056A,0x0000,0x0000,0x0000}},
{0x00,0x053B,{0x056B,0x0000,0x0000,0x0000}},
{0x00,0x053C,{0x056C,0x0000,0x0000,0x0000}},
{0x00,0x053D,{0x056D,0x0000,0x0000,0x0000}},
{0x00,0x053E,{0x056E,0x0000,0x0000,0x0000}},
{0x00,0x053F,{0x056F,0x0000,0x0000,0x0000}},
{0x00,0x0540,{0x0570,0x0000,0x0000,0x0000}},
{0x00,0x0541,{0x0571,0x0000,0x0000,0x0000}},
{0x00,0x0542,{0x0572,0x0000,0x0000,0x0000}},
{0x00,0x0543,{0x0573,0x0000,0x0000,0x0000}},
{0x00,0x0544,{0x0574,0x0000,0x0000,0x0000}},
{0x00,0x0545,{0x0575,0x0000,0x0000,0x0000}},
{0x00,0x0546,{0x0576,0x0000,0x0000,0x0000}},
{0x00,0x0547,{0x0577,0x0000,0x0000,0x0000}},
{0x00,0x0548,{0x0578,0x0000,0x0000,0x0000}},
{0x00,0x0549,{0x0579,0x0000,0x0000,0x0000}},
{0x00,0x054A,{0x057A,0x0000,0x0000,0x0000}},
{0x00,0x054B,{0x057B,0x0000,0x0000,0x0000}},
{0x00,0x054C,{0x057C,0x0000,0x0000,0x0000}},
{0x00,0x054D,{0x057D,0x0000,0x0000,0x0000}},
{0x00,0x054E,{0x057E,0x0000,0x0000,0x0000}},
{0x00,0x054F,{0x057F,0x0000,0x0000,0x0000}},
{0x00,0x0550,{0x0580,0x0000,0x0000,0x0000}},
{0x00,0x0551,{0x0581,0x0000,0x0000,0x0000}},
{0x00,0x0552,{0x0582,0x0000,0x0000,0x0000}},
{0x00,0x0553,{0x0583,0x0000,0x0000,0x0000}},
{0x00,0x0554,{0x0584,0x0000,0x0000,0x0000}},
{0x00,0x0555,{0x0585,0x0000,0x0000,0x0000}},
{0x00,0x0556,{0x0586,0x0000,0x0000,0x0000}},
{0x00,0x0587,{0x0565,0x0582,0x0000,0x0000}},
{0x00,0x1E00,{0x1E01,0x0000,0x0000,0x0000}},
{0x00,0x1E02,{0x1E03,0x0000,0x0000,0x0000}},
{0x00,0x1E04,{0x1E05,0x0000,0x0000,0x0000}},
{0x00,0x1E06,{0x1E07,0x0000,0x0000,0x0000}},
{0x00,0x1E08,{0x1E09,0x0000,0x0000,0x0000}},
{0x00,0x1E0A,{0x1E0B,0x0000,0x0000,0x0000}},
{0x00,0x1E0C,{0x1E0D,0x0000,0x0000,0x0000}},
{0x00,0x1E0E,{0x1E0F,0x0000,0x0000,0x0000}},
{0x00,0x1E10,{0x1E11,0x0000,0x0000,0x0000}},
{0x00,0x1E12,{0x1E13,0x0000,0x0000,0x0000}},
{0x00,0x1E14,{0x1E15,0x0000,0x0000,0x0000}},
{0x00,0x1E16,{0x1E17,0x0000,0x0000,0x0000}},
{0x00,0x1E18,{0x1E19,0x0000,0x0000,0x0000}},
{0x00,0x1E1A,{0x1E1B,0x0000,0x0000,0x0000}},
{0x00,0x1E1C,{0x1E1D,0x0000,0x0000,0x0000}},
{0x00,0x1E1E,{0x1E1F,0x0000,0x0000,0x0000}},
{0x00,0x1E20,{0x1E21,0x0000,0x0000,0x0000}},
{0x00,0x1E22,{0x1E23,0x0000,0x0000,0x0000}},
{0x00,0x1E24,{0x1E25,0x0000,0x0000,0x0000}},
{0x00,0x1E26,{0x1E27,0x0000,0x0000,0x0000}},
{0x00,0x1E28,{0x1E29,0x0000,0x0000,0x0000}},
{0x00,0x1E2A,{0x1E2B,0x0000,0x0000,0x0000}},
{0x00,0x1E2C,{0x1E2D,0x0000,0x0000,0x0000}},
{0x00,0x1E2E,{0x1E2F,0x0000,0x0000,0x0000}},
{0x00,0x1E30,{0x1E31,0x0000,0x0000,0x0000}},
{0x00,0x1E32,{0x1E33,0x0000,0x0000,0x0000}},
{0x00,0x1E34,{0x1E35,0x0000,0x0000,0x0000}},
{0x00,0x1E36,{0x1E37,0x0000,0x0000,0x0000}},
{0x00,0x1E38,{0x1E39,0x0000,0x0000,0x0000}},
{0x00,0x1E3A,{0x1E3B,0x0000,0x0000,0x0000}},
{0x00,0x1E3C,{0x1E3D,0x0000,0x0000,0x0000}},
{0x00,0x1E3E,{0x1E3F,0x0000,0x0000,0x0000}},
{0x00,0x1E40,{0x1E41,0x0000,0x0000,0x0000}},
{0x00,0x1E42,{0x1E43,0x0000,0x0000,0x0000}},
{0x00,0x1E44,{0x1E45,0x0000,0x0000,0x0000}},
{0x00,0x1E46,{0x1E47,0x0000,0x0000,0x0000}},
{0x00,0x1E48,{0x1E49,0x0000,0x0000,0x0000}},
{0x00,0x1E4A,{0x1E4B,0x0000,0x0000,0x0000}},
{0x00,0x1E4C,{0x1E4D,0x0000,0x0000,0x0000}},
{0x00,0x1E4E,{0x1E4F,0x0000,0x0000,0x0000}},
{0x00,0x1E50,{0x1E51,0x0000,0x0000,0x0000}},
{0x00,0x1E52,{0x1E53,0x0000,0x0000,0x0000}},
{0x00,0x1E54,{0x1E55,0x0000,0x0000,0x0000}},
{0x00,0x1E56,{0x1E57,0x0000,0x0000,0x0000}},
{0x00,0x1E58,{0x1E59,0x0000,0x0000,0x0000}},
{0x00,0x1E5A,{0x1E5B,0x0000,0x0000,0x0000}},
{0x00,0x1E5C,{0x1E5D,0x0000,0x0000,0x0000}},
{0x00,0x1E5E,{0x1E5F,0x0000,0x0000,0x0000}},
{0x00,0x1E60,{0x1E61,0x0000,0x0000,0x0000}},
{0x00,0x1E62,{0x1E63,0x0000,0x0000,0x0000}},
{0x00,0x1E64,{0x1E65,0x0000,0x0000,0x0000}},
{0x00,0x1E66,{0x1E67,0x0000,0x0000,0x0000}},
{0x00,0x1E68,{0x1E69,0x0000,0x0000,0x0000}},
{0x00,0x1E6A,{0x1E6B,0x0000,0x0000,0x0000}},
{0x00,0x1E6C,{0x1E6D,0x0000,0x0000,0x0000}},
{0x00,0x1E6E,{0x1E6F,0x0000,0x0000,0x0000}},
{0x00,0x1E70,{0x1E71,0x0000,0x0000,0x0000}},
{0x00,0x1E72,{0x1E73,0x0000,0x0000,0x0000}},
{0x00,0x1E74,{0x1E75,0x0000,0x0000,0x0000}},
{0x00,0x1E76,{0x1E77,0x0000,0x0000,0x0000}},
{0x00,0x1E78,{0x1E79,0x0000,0x0000,0x0000}},
{0x00,0x1E7A,{0x1E7B,0x0000,0x0000,0x0000}},
{0x00,0x1E7C,{0x1E7D,0x0000,0x0000,0x0000}},
{0x00,0x1E7E,{0x1E7F,0x0000,0x0000,0x0000}},
{0x00,0x1E80,{0x1E81,0x0000,0x0000,0x0000}},
{0x00,0x1E82,{0x1E83,0x0000,0x0000,0x0000}},
{0x00,0x1E84,{0x1E85,0x0000,0x0000,0x0000}},
{0x00,0x1E86,{0x1E87,0x0000,0x0000,0x0000}},
{0x00,0x1E88,{0x1E89,0x0000,0x0000,0x0000}},
{0x00,0x1E8A,{0x1E8B,0x0000,0x0000,0x0000}},
{0x00,0x1E8C,{0x1E8D,0x0000,0x0000,0x0000}},
{0x00,0x1E8E,{0x1E8F,0x0000,0x0000,0x0000}},
{0x00,0x1E90,{0x1E91,0x0000,0x0000,0x0000}},
{0x00,0x1E92,{0x1E93,0x0000,0x0000,0x0000}},
{0x00,0x1E94,{0x1E95,0x0000,0x0000,0x0000}},
{0x00,0x1E96,{0x0068,0x0331,0x0000,0x0000}},
{0x00,0x1E97,{0x0074,0x0308,0x0000,0x0000}},
{0x00,0x1E98,{0x0077,0x030A,0x0000,0x0000}},
{0x00,0x1E99,{0x0079,0x030A,0x0000,0x0000}},
{0x00,0x1E9A,{0x0061,0x02BE,0x0000,0x0000}},
{0x00,0x1E9B,{0x1E61,0x0000,0x0000,0x0000}},
{0x00,0x1EA0,{0x1EA1,0x0000,0x0000,0x0000}},
{0x00,0x1EA2,{0x1EA3,0x0000,0x0000,0x0000}},
{0x00,0x1EA4,{0x1EA5,0x0000,0x0000,0x0000}},
{0x00,0x1EA6,{0x1EA7,0x0000,0x0000,0x0000}},
{0x00,0x1EA8,{0x1EA9,0x0000,0x0000,0x0000}},
{0x00,0x1EAA,{0x1EAB,0x0000,0x0000,0x0000}},
{0x00,0x1EAC,{0x1EAD,0x0000,0x0000,0x0000}},
{0x00,0x1EAE,{0x1EAF,0x0000,0x0000,0x0000}},
{0x00,0x1EB0,{0x1EB1,0x0000,0x0000,0x0000}},
{0x00,0x1EB2,{0x1EB3,0x0000,0x0000,0x0000}},
{0x00,0x1EB4,{0x1EB5,0x0000,0x0000,0x0000}},
{0x00,0x1EB6,{0x1EB7,0x0000,0x0000,0x0000}},
{0x00,0x1EB8,{0x1EB9,0x0000,0x0000,0x0000}},
{0x00,0x1EBA,{0x1EBB,0x0000,0x0000,0x0000}},
{0x00,0x1EBC,{0x1EBD,0x0000,0x0000,0x0000}},
{0x00,0x1EBE,{0x1EBF,0x0000,0x0000,0x0000}},
{0x00,0x1EC0,{0x1EC1,0x0000,0x0000,0x0000}},
{0x00,0x1EC2,{0x1EC3,0x0000,0x0000,0x0000}},
{0x00,0x1EC4,{0x1EC5,0x0000,0x0000,0x0000}},
{0x00,0x1EC6,{0x1EC7,0x0000,0x0000,0x0000}},
{0x00,0x1EC8,{0x1EC9,0x0000,0x0000,0x0000}},
{0x00,0x1ECA,{0x1ECB,0x0000,0x0000,0x0000}},
{0x00,0x1ECC,{0x1ECD,0x0000,0x0000,0x0000}},
{0x00,0x1ECE,{0x1ECF,0x0000,0x0000,0x0000}},
{0x00,0x1ED0,{0x1ED1,0x0000,0x0000,0x0000}},
{0x00,0x1ED2,{0x1ED3,0x0000,0x0000,0x0000}},
{0x00,0x1ED4,{0x1ED5,0x0000,0x0000,0x0000}},
{0x00,0x1ED6,{0x1ED7,0x0000,0x0000,0x0000}},
{0x00,0x1ED8,{0x1ED9,0x0000,0x0000,0x0000}},
{0x00,0x1EDA,{0x1EDB,0x0000,0x0000,0x0000}},
{0x00,0x1EDC,{0x1EDD,0x0000,0x0000,0x0000}},
{0x00,0x1EDE,{0x1EDF,0x0000,0x0000,0x0000}},
{0x00,0x1EE0,{0x1EE1,0x0000,0x0000,0x0000}},
{0x00,0x1EE2,{0x1EE3,0x0000,0x0000,0x0000}},
{0x00,0x1EE4,{0x1EE5,0x0000,0x0000,0x0000}},
{0x00,0x1EE6,{0x1EE7,0x0000,0x0000,0x0000}},
{0x00,0x1EE8,{0x1EE9,0x0000,0x0000,0x0000}},
{0x00,0x1EEA,{0x1EEB,0x0000,0x0000,0x0000}},
{0x00,0x1EEC,{0x1EED,0x0000,0x0000,0x0000}},
{0x00,0x1EEE,{0x1EEF,0x0000,0x0000,0x0000}},
{0x00,0x1EF0,{0x1EF1,0x0000,0x0000,0x0000}},
{0x00,0x1EF2,{0x1EF3,0x0000,0x0000,0x0000}},
{0x00,0x1EF4,{0x1EF5,0x0000,0x0000,0x0000}},
{0x00,0x1EF6,{0x1EF7,0x0000,0x0000,0x0000}},
{0x00,0x1EF8,{0x1EF9,0x0000,0x0000,0x0000}},
{0x00,0x1F08,{0x1F00,0x0000,0x0000,0x0000}},
{0x00,0x1F09,{0x1F01,0x0000,0x0000,0x0000}},
{0x00,0x1F0A,{0x1F02,0x0000,0x0000,0x0000}},
{0x00,0x1F0B,{0x1F03,0x0000,0x0000,0x0000}},
{0x00,0x1F0C,{0x1F04,0x0000,0x0000,0x0000}},
{0x00,0x1F0D,{0x1F05,0x0000,0x0000,0x0000}},
{0x00,0x1F0E,{0x1F06,0x0000,0x0000,0x0000}},
{0x00,0x1F0F,{0x1F07,0x0000,0x0000,0x0000}},
{0x00,0x1F18,{0x1F10,0x0000,0x0000,0x0000}},
{0x00,0x1F19,{0x1F11,0x0000,0x0000,0x0000}},
{0x00,0x1F1A,{0x1F12,0x0000,0x0000,0x0000}},
{0x00,0x1F1B,{0x1F13,0x0000,0x0000,0x0000}},
{0x00,0x1F1C,{0x1F14,0x0000,0x0000,0x0000}},
{0x00,0x1F1D,{0x1F15,0x0000,0x0000,0x0000}},
{0x00,0x1F28,{0x1F20,0x0000,0x0000,0x0000}},
{0x00,0x1F29,{0x1F21,0x0000,0x0000,0x0000}},
{0x00,0x1F2A,{0x1F22,0x0000,0x0000,0x0000}},
{0x00,0x1F2B,{0x1F23,0x0000,0x0000,0x0000}},
{0x00,0x1F2C,{0x1F24,0x0000,0x0000,0x0000}},
{0x00,0x1F2D,{0x1F25,0x0000,0x0000,0x0000}},
{0x00,0x1F2E,{0x1F26,0x0000,0x0000,0x0000}},
{0x00,0x1F2F,{0x1F27,0x0000,0x0000,0x0000}},
{0x00,0x1F38,{0x1F30,0x0000,0x0000,0x0000}},
{0x00,0x1F39,{0x1F31,0x0000,0x0000,0x0000}},
{0x00,0x1F3A,{0x1F32,0x0000,0x0000,0x0000}},
{0x00,0x1F3B,{0x1F33,0x0000,0x0000,0x0000}},
{0x00,0x1F3C,{0x1F34,0x0000,0x0000,0x0000}},
{0x00,0x1F3D,{0x1F35,0x0000,0x0000,0x0000}},
{0x00,0x1F3E,{0x1F36,0x0000,0x0000,0x0000}},
{0x00,0x1F3F,{0x1F37,0x0000,0x0000,0x0000}},
{0x00,0x1F48,{0x1F40,0x0000,0x0000,0x0000}},
{0x00,0x1F49,{0x1F41,0x0000,0x0000,0x0000}},
{0x00,0x1F4A,{0x1F42,0x0000,0x0000,0x0000}},
{0x00,0x1F4B,{0x1F43,0x0000,0x0000,0x0000}},
{0x00,0x1F4C,{0x1F44,0x0000,0x0000,0x0000}},
{0x00,0x1F4D,{0x1F45,0x0000,0x0000,0x0000}},
{0x00,0x1F50,{0x03C5,0x0313,0x0000,0x0000}},
{0x00,0x1F52,{0x03C5,0x0313,0x0300,0x0000}},
{0x00,0x1F54,{0x03C5,0x0313,0x0301,0x0000}},
{0x00,0x1F56,{0x03C5,0x0313,0x0342,0x0000}},
{0x00,0x1F59,{0x1F51,0x0000,0x0000,0x0000}},
{0x00,0x1F5B,{0x1F53,0x0000,0x0000,0x0000}},
{0x00,0x1F5D,{0x1F55,0x0000,0x0000,0x0000}},
{0x00,0x1F5F,{0x1F57,0x0000,0x0000,0x0000}},
{0x00,0x1F68,{0x1F60,0x0000,0x0000,0x0000}},
{0x00,0x1F69,{0x1F61,0x0000,0x0000,0x0000}},
{0x00,0x1F6A,{0x1F62,0x0000,0x0000,0x0000}},
{0x00,0x1F6B,{0x1F63,0x0000,0x0000,0x0000}},
{0x00,0x1F6C,{0x1F64,0x0000,0x0000,0x0000}},
{0x00,0x1F6D,{0x1F65,0x0000,0x0000,0x0000}},
{0x00,0x1F6E,{0x1F66,0x0000,0x0000,0x0000}},
{0x00,0x1F6F,{0x1F67,0x0000,0x0000,0x0000}},
{0x00,0x1F80,{0x1F00,0x03B9,0x0000,0x0000}},
{0x00,0x1F81,{0x1F01,0x03B9,0x0000,0x0000}},
{0x00,0x1F82,{0x1F02,0x03B9,0x0000,0x0000}},
{0x00,0x1F83,{0x1F03,0x03B9,0x0000,0x0000}},
{0x00,0x1F84,{0x1F04,0x03B9,0x0000,0x0000}},
{0x00,0x1F85,{0x1F05,0x03B9,0x0000,0x0000}},
{0x00,0x1F86,{0x1F06,0x03B9,0x0000,0x0000}},
{0x00,0x1F87,{0x1F07,0x03B9,0x0000,0x0000}},
{0x00,0x1F88,{0x1F00,0x03B9,0x0000,0x0000}},
{0x00,0x1F89,{0x1F01,0x03B9,0x0000,0x0000}},
{0x00,0x1F8A,{0x1F02,0x03B9,0x0000,0x0000}},
{0x00,0x1F8B,{0x1F03,0x03B9,0x0000,0x0000}},
{0x00,0x1F8C,{0x1F04,0x03B9,0x0000,0x0000}},
{0x00,0x1F8D,{0x1F05,0x03B9,0x0000,0x0000}},
{0x00,0x1F8E,{0x1F06,0x03B9,0x0000,0x0000}},
{0x00,0x1F8F,{0x1F07,0x03B9,0x0000,0x0000}},
{0x00,0x1F90,{0x1F20,0x03B9,0x0000,0x0000}},
{0x00,0x1F91,{0x1F21,0x03B9,0x0000,0x0000}},
{0x00,0x1F92,{0x1F22,0x03B9,0x0000,0x0000}},
{0x00,0x1F93,{0x1F23,0x03B9,0x0000,0x0000}},
{0x00,0x1F94,{0x1F24,0x03B9,0x0000,0x0000}},
{0x00,0x1F95,{0x1F25,0x03B9,0x0000,0x0000}},
{0x00,0x1F96,{0x1F26,0x03B9,0x0000,0x0000}},
{0x00,0x1F97,{0x1F27,0x03B9,0x0000,0x0000}},
{0x00,0x1F98,{0x1F20,0x03B9,0x0000,0x0000}},
{0x00,0x1F99,{0x1F21,0x03B9,0x0000,0x0000}},
{0x00,0x1F9A,{0x1F22,0x03B9,0x0000,0x0000}},
{0x00,0x1F9B,{0x1F23,0x03B9,0x0000,0x0000}},
{0x00,0x1F9C,{0x1F24,0x03B9,0x0000,0x0000}},
{0x00,0x1F9D,{0x1F25,0x03B9,0x0000,0x0000}},
{0x00,0x1F9E,{0x1F26,0x03B9,0x0000,0x0000}},
{0x00,0x1F9F,{0x1F27,0x03B9,0x0000,0x0000}},
{0x00,0x1FA0,{0x1F60,0x03B9,0x0000,0x0000}},
{0x00,0x1FA1,{0x1F61,0x03B9,0x0000,0x0000}},
{0x00,0x1FA2,{0x1F62,0x03B9,0x0000,0x0000}},
{0x00,0x1FA3,{0x1F63,0x03B9,0x0000,0x0000}},
{0x00,0x1FA4,{0x1F64,0x03B9,0x0000,0x0000}},
{0x00,0x1FA5,{0x1F65,0x03B9,0x0000,0x0000}},
{0x00,0x1FA6,{0x1F66,0x03B9,0x0000,0x0000}},
{0x00,0x1FA7,{0x1F67,0x03B9,0x0000,0x0000}},
{0x00,0x1FA8,{0x1F60,0x03B9,0x0000,0x0000}},
{0x00,0x1FA9,{0x1F61,0x03B9,0x0000,0x0000}},
{0x00,0x1FAA,{0x1F62,0x03B9,0x0000,0x0000}},
{0x00,0x1FAB,{0x1F63,0x03B9,0x0000,0x0000}},
{0x00,0x1FAC,{0x1F64,0x03B9,0x0000,0x0000}},
{0x00,0x1FAD,{0x1F65,0x03B9,0x0000,0x0000}},
{0x00,0x1FAE,{0x1F66,0x03B9,0x0000,0x0000}},
{0x00,0x1FAF,{0x1F67,0x03B9,0x0000,0x0000}},
{0x00,0x1FB2,{0x1F70,0x03B9,0x0000,0x0000}},
{0x00,0x1FB3,{0x03B1,0x03B9,0x0000,0x0000}},
{0x00,0x1FB4,{0x03AC,0x03B9,0x0000,0x0000}},
{0x00,0x1FB6,{0x03B1,0x0342,0x0000,0x0000}},
{0x00,0x1FB7,{0x03B1,0x0342,0x03B9,0x0000}},
{0x00,0x1FB8,{0x1FB0,0x0000,0x0000,0x0000}},
{0x00,0x1FB9,{0x1FB1,0x0000,0x0000,0x0000}},
{0x00,0x1FBA,{0x1F70,0x0000,0x0000,0x0000}},
{0x00,0x1FBB,{0x1F71,0x0000,0x0000,0x0000}},
{0x00,0x1FBC,{0x03B1,0x03B9,0x0000,0x0000}},
{0x00,0x1FBE,{0x03B9,0x0000,0x0000,0x0000}},
{0x00,0x1FC2,{0x1F74,0x03B9,0x0000,0x0000}},
{0x00,0x1FC3,{0x03B7,0x03B9,0x0000,0x0000}},
{0x00,0x1FC4,{0x03AE,0x03B9,0x0000,0x0000}},
{0x00,0x1FC6,{0x03B7,0x0342,0x0000,0x0000}},
{0x00,0x1FC7,{0x03B7,0x0342,0x03B9,0x0000}},
{0x00,0x1FC8,{0x1F72,0x0000,0x0000,0x0000}},
{0x00,0x1FC9,{0x1F73,0x0000,0x0000,0x0000}},
{0x00,0x1FCA,{0x1F74,0x0000,0x0000,0x0000}},
{0x00,0x1FCB,{0x1F75,0x0000,0x0000,0x0000}},
{0x00,0x1FCC,{0x03B7,0x03B9,0x0000,0x0000}},
{0x00,0x1FD2,{0x03B9,0x0308,0x0300,0x0000}},
{0x00,0x1FD3,{0x03B9,0x0308,0x0301,0x0000}},
{0x00,0x1FD6,{0x03B9,0x0342,0x0000,0x0000}},
{0x00,0x1FD7,{0x03B9,0x0308,0x0342,0x0000}},
{0x00,0x1FD8,{0x1FD0,0x0000,0x0000,0x0000}},
{0x00,0x1FD9,{0x1FD1,0x0000,0x0000,0x0000}},
{0x00,0x1FDA,{0x1F76,0x0000,0x0000,0x0000}},
{0x00,0x1FDB,{0x1F77,0x0000,0x0000,0x0000}},
{0x00,0x1FE2,{0x03C5,0x0308,0x0300,0x0000}},
{0x00,0x1FE3,{0x03C5,0x0308,0x0301,0x0000}},
{0x00,0x1FE4,{0x03C1,0x0313,0x0000,0x0000}},
{0x00,0x1FE6,{0x03C5,0x0342,0x0000,0x0000}},
{0x00,0x1FE7,{0x03C5,0x0308,0x0342,0x0000}},
{0x00,0x1FE8,{0x1FE0,0x0000,0x0000,0x0000}},
{0x00,0x1FE9,{0x1FE1,0x0000,0x0000,0x0000}},
{0x00,0x1FEA,{0x1F7A,0x0000,0x0000,0x0000}},
{0x00,0x1FEB,{0x1F7B,0x0000,0x0000,0x0000}},
{0x00,0x1FEC,{0x1FE5,0x0000,0x0000,0x0000}},
{0x00,0x1FF2,{0x1F7C,0x03B9,0x0000,0x0000}},
{0x00,0x1FF3,{0x03C9,0x03B9,0x0000,0x0000}},
{0x00,0x1FF4,{0x03CE,0x03B9,0x0000,0x0000}},
{0x00,0x1FF6,{0x03C9,0x0342,0x0000,0x0000}},
{0x00,0x1FF7,{0x03C9,0x0342,0x03B9,0x0000}},
{0x00,0x1FF8,{0x1F78,0x0000,0x0000,0x0000}},
{0x00,0x1FF9,{0x1F79,0x0000,0x0000,0x0000}},
{0x00,0x1FFA,{0x1F7C,0x0000,0x0000,0x0000}},
{0x00,0x1FFB,{0x1F7D,0x0000,0x0000,0x0000}},
{0x00,0x1FFC,{0x03C9,0x03B9,0x0000,0x0000}},
{0x00,0x20A8,{0x0072,0x0073,0x0000,0x0000}},
{0x00,0x2102,{0x0063,0x0000,0x0000,0x0000}},
{0x00,0x2103,{0x00B0,0x0063,0x0000,0x0000}},
{0x00,0x2107,{0x025B,0x0000,0x0000,0x0000}},
{0x00,0x2109,{0x00B0,0x0066,0x0000,0x0000}},
{0x00,0x210B,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0x210C,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0x210D,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0x2110,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x2111,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x2112,{0x006C,0x0000,0x0000,0x0000}},
{0x00,0x2115,{0x006E,0x0000,0x0000,0x0000}},
{0x00,0x2116,{0x006E,0x006F,0x0000,0x0000}},
{0x00,0x2119,{0x0070,0x0000,0x0000,0x0000}},
{0x00,0x211A,{0x0071,0x0000,0x0000,0x0000}},
{0x00,0x211B,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0x211C,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0x211D,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0x2120,{0x0073,0x006D,0x0000,0x0000}},
{0x00,0x2121,{0x0074,0x0065,0x006C,0x0000}},
{0x00,0x2122,{0x0074,0x006D,0x0000,0x0000}},
{0x00,0x2124,{0x007A,0x0000,0x0000,0x0000}},
{0x00,0x2126,{0x03C9,0x0000,0x0000,0x0000}},
{0x00,0x2128,{0x007A,0x0000,0x0000,0x0000}},
{0x00,0x212A,{0x006B,0x0000,0x0000,0x0000}},
{0x00,0x212B,{0x00E5,0x0000,0x0000,0x0000}},
{0x00,0x212C,{0x0062,0x0000,0x0000,0x0000}},
{0x00,0x212D,{0x0063,0x0000,0x0000,0x0000}},
{0x00,0x2130,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0x2131,{0x0066,0x0000,0x0000,0x0000}},
{0x00,0x2133,{0x006D,0x0000,0x0000,0x0000}},
{0x00,0x213E,{0x03B3,0x0000,0x0000,0x0000}},
{0x00,0x213F,{0x03C0,0x0000,0x0000,0x0000}},
{0x00,0x2145,{0x0064,0x0000,0x0000,0x0000}},
{0x00,0x2160,{0x2170,0x0000,0x0000,0x0000}},
{0x00,0x2161,{0x2171,0x0000,0x0000,0x0000}},
{0x00,0x2162,{0x2172,0x0000,0x0000,0x0000}},
{0x00,0x2163,{0x2173,0x0000,0x0000,0x0000}},
{0x00,0x2164,{0x2174,0x0000,0x0000,0x0000}},
{0x00,0x2165,{0x2175,0x0000,0x0000,0x0000}},
{0x00,0x2166,{0x2176,0x0000,0x0000,0x0000}},
{0x00,0x2167,{0x2177,0x0000,0x0000,0x0000}},
{0x00,0x2168,{0x2178,0x0000,0x0000,0x0000}},
{0x00,0x2169,{0x2179,0x0000,0x0000,0x0000}},
{0x00,0x216A,{0x217A,0x0000,0x0000,0x0000}},
{0x00,0x216B,{0x217B,0x0000,0x0000,0x0000}},
{0x00,0x216C,{0x217C,0x0000,0x0000,0x0000}},
{0x00,0x216D,{0x217D,0x0000,0x0000,0x0000}},
{0x00,0x216E,{0x217E,0x0000,0x0000,0x0000}},
{0x00,0x216F,{0x217F,0x0000,0x0000,0x0000}},
{0x00,0x24B6,{0x24D0,0x0000,0x0000,0x0000}},
{0x00,0x24B7,{0x24D1,0x0000,0x0000,0x0000}},
{0x00,0x24B8,{0x24D2,0x0000,0x0000,0x0000}},
{0x00,0x24B9,{0x24D3,0x0000,0x0000,0x0000}},
{0x00,0x24BA,{0x24D4,0x0000,0x0000,0x0000}},
{0x00,0x24BB,{0x24D5,0x0000,0x0000,0x0000}},
{0x00,0x24BC,{0x24D6,0x0000,0x0000,0x0000}},
{0x00,0x24BD,{0x24D7,0x0000,0x0000,0x0000}},
{0x00,0x24BE,{0x24D8,0x0000,0x0000,0x0000}},
{0x00,0x24BF,{0x24D9,0x0000,0x0000,0x0000}},
{0x00,0x24C0,{0x24DA,0x0000,0x0000,0x0000}},
{0x00,0x24C1,{0x24DB,0x0000,0x0000,0x0000}},
{0x00,0x24C2,{0x24DC,0x0000,0x0000,0x0000}},
{0x00,0x24C3,{0x24DD,0x0000,0x0000,0x0000}},
{0x00,0x24C4,{0x24DE,0x0000,0x0000,0x0000}},
{0x00,0x24C5,{0x24DF,0x0000,0x0000,0x0000}},
{0x00,0x24C6,{0x24E0,0x0000,0x0000,0x0000}},
{0x00,0x24C7,{0x24E1,0x0000,0x0000,0x0000}},
{0x00,0x24C8,{0x24E2,0x0000,0x0000,0x0000}},
{0x00,0x24C9,{0x24E3,0x0000,0x0000,0x0000}},
{0x00,0x24CA,{0x24E4,0x0000,0x0000,0x0000}},
{0x00,0x24CB,{0x24E5,0x0000,0x0000,0x0000}},
{0x00,0x24CC,{0x24E6,0x0000,0x0000,0x0000}},
{0x00,0x24CD,{0x24E7,0x0000,0x0000,0x0000}},
{0x00,0x24CE,{0x24E8,0x0000,0x0000,0x0000}},
{0x00,0x24CF,{0x24E9,0x0000,0x0000,0x0000}},
{0x00,0x3371,{0x0068,0x0070,0x0061,0x0000}},
{0x00,0x3373,{0x0061,0x0075,0x0000,0x0000}},
{0x00,0x3375,{0x006F,0x0076,0x0000,0x0000}},
{0x00,0x3380,{0x0070,0x0061,0x0000,0x0000}},
{0x00,0x3381,{0x006E,0x0061,0x0000,0x0000}},
{0x00,0x3382,{0x03BC,0x0061,0x0000,0x0000}},
{0x00,0x3383,{0x006D,0x0061,0x0000,0x0000}},
{0x00,0x3384,{0x006B,0x0061,0x0000,0x0000}},
{0x00,0x3385,{0x006B,0x0062,0x0000,0x0000}},
{0x00,0x3386,{0x006D,0x0062,0x0000,0x0000}},
{0x00,0x3387,{0x0067,0x0062,0x0000,0x0000}},
{0x00,0x338A,{0x0070,0x0066,0x0000,0x0000}},
{0x00,0x338B,{0x006E,0x0066,0x0000,0x0000}},
{0x00,0x338C,{0x03BC,0x0066,0x0000,0x0000}},
{0x00,0x3390,{0x0068,0x007A,0x0000,0x0000}},
{0x00,0x3391,{0x006B,0x0068,0x007A,0x0000}},
{0x00,0x3392,{0x006D,0x0068,0x007A,0x0000}},
{0x00,0x3393,{0x0067,0x0068,0x007A,0x0000}},
{0x00,0x3394,{0x0074,0x0068,0x007A,0x0000}},
{0x00,0x33A9,{0x0070,0x0061,0x0000,0x0000}},
{0x00,0x33AA,{0x006B,0x0070,0x0061,0x0000}},
{0x00,0x33AB,{0x006D,0x0070,0x0061,0x0000}},
{0x00,0x33AC,{0x0067,0x0070,0x0061,0x0000}},
{0x00,0x33B4,{0x0070,0x0076,0x0000,0x0000}},
{0x00,0x33B5,{0x006E,0x0076,0x0000,0x0000}},
{0x00,0x33B6,{0x03BC,0x0076,0x0000,0x0000}},
{0x00,0x33B7,{0x006D,0x0076,0x0000,0x0000}},
{0x00,0x33B8,{0x006B,0x0076,0x0000,0x0000}},
{0x00,0x33B9,{0x006D,0x0076,0x0000,0x0000}},
{0x00,0x33BA,{0x0070,0x0077,0x0000,0x0000}},
{0x00,0x33BB,{0x006E,0x0077,0x0000,0x0000}},
{0x00,0x33BC,{0x03BC,0x0077,0x0000,0x0000}},
{0x00,0x33BD,{0x006D,0x0077,0x0000,0x0000}},
{0x00,0x33BE,{0x006B,0x0077,0x0000,0x0000}},
{0x00,0x33BF,{0x006D,0x0077,0x0000,0x0000}},
{0x00,0x33C0,{0x006B,0x03C9,0x0000,0x0000}},
{0x00,0x33C1,{0x006D,0x03C9,0x0000,0x0000}},
{0x00,0x33C3,{0x0062,0x0071,0x0000,0x0000}},
{0x00,0x33C6,{0x0063,0x2215,0x006B,0x0067}},
{0x00,0x33C7,{0x0063,0x006F,0x002E,0x0000}},
{0x00,0x33C8,{0x0064,0x0062,0x0000,0x0000}},
{0x00,0x33C9,{0x0067,0x0079,0x0000,0x0000}},
{0x00,0x33CB,{0x0068,0x0070,0x0000,0x0000}},
{0x00,0x33CD,{0x006B,0x006B,0x0000,0x0000}},
{0x00,0x33CE,{0x006B,0x006D,0x0000,0x0000}},
{0x00,0x33D7,{0x0070,0x0068,0x0000,0x0000}},
{0x00,0x33D9,{0x0070,0x0070,0x006D,0x0000}},
{0x00,0x33DA,{0x0070,0x0072,0x0000,0x0000}},
{0x00,0x33DC,{0x0073,0x0076,0x0000,0x0000}},
{0x00,0x33DD,{0x0077,0x0062,0x0000,0x0000}},
{0x00,0xFB00,{0x0066,0x0066,0x0000,0x0000}},
{0x00,0xFB01,{0x0066,0x0069,0x0000,0x0000}},
{0x00,0xFB02,{0x0066,0x006C,0x0000,0x0000}},
{0x00,0xFB03,{0x0066,0x0066,0x0069,0x0000}},
{0x00,0xFB04,{0x0066,0x0066,0x006C,0x0000}},
{0x00,0xFB05,{0x0073,0x0074,0x0000,0x0000}},
{0x00,0xFB06,{0x0073,0x0074,0x0000,0x0000}},
{0x00,0xFB13,{0x0574,0x0576,0x0000,0x0000}},
{0x00,0xFB14,{0x0574,0x0565,0x0000,0x0000}},
{0x00,0xFB15,{0x0574,0x056B,0x0000,0x0000}},
{0x00,0xFB16,{0x057E,0x0576,0x0000,0x0000}},
{0x00,0xFB17,{0x0574,0x056D,0x0000,0x0000}},
{0x00,0xFF21,{0xFF41,0x0000,0x0000,0x0000}},
{0x00,0xFF22,{0xFF42,0x0000,0x0000,0x0000}},
{0x00,0xFF23,{0xFF43,0x0000,0x0000,0x0000}},
{0x00,0xFF24,{0xFF44,0x0000,0x0000,0x0000}},
{0x00,0xFF25,{0xFF45,0x0000,0x0000,0x0000}},
{0x00,0xFF26,{0xFF46,0x0000,0x0000,0x0000}},
{0x00,0xFF27,{0xFF47,0x0000,0x0000,0x0000}},
{0x00,0xFF28,{0xFF48,0x0000,0x0000,0x0000}},
{0x00,0xFF29,{0xFF49,0x0000,0x0000,0x0000}},
{0x00,0xFF2A,{0xFF4A,0x0000,0x0000,0x0000}},
{0x00,0xFF2B,{0xFF4B,0x0000,0x0000,0x0000}},
{0x00,0xFF2C,{0xFF4C,0x0000,0x0000,0x0000}},
{0x00,0xFF2D,{0xFF4D,0x0000,0x0000,0x0000}},
{0x00,0xFF2E,{0xFF4E,0x0000,0x0000,0x0000}},
{0x00,0xFF2F,{0xFF4F,0x0000,0x0000,0x0000}},
{0x00,0xFF30,{0xFF50,0x0000,0x0000,0x0000}},
{0x00,0xFF31,{0xFF51,0x0000,0x0000,0x0000}},
{0x00,0xFF32,{0xFF52,0x0000,0x0000,0x0000}},
{0x00,0xFF33,{0xFF53,0x0000,0x0000,0x0000}},
{0x00,0xFF34,{0xFF54,0x0000,0x0000,0x0000}},
{0x00,0xFF35,{0xFF55,0x0000,0x0000,0x0000}},
{0x00,0xFF36,{0xFF56,0x0000,0x0000,0x0000}},
{0x00,0xFF37,{0xFF57,0x0000,0x0000,0x0000}},
{0x00,0xFF38,{0xFF58,0x0000,0x0000,0x0000}},
{0x00,0xFF39,{0xFF59,0x0000,0x0000,0x0000}},
{0x00,0xFF3A,{0xFF5A,0x0000,0x0000,0x0000}},
{0x11,0x0400,{0x0428,0x0000,0x0000,0x0000}},
{0x11,0x0401,{0x0429,0x0000,0x0000,0x0000}},
{0x11,0x0402,{0x042A,0x0000,0x0000,0x0000}},
{0x11,0x0403,{0x042B,0x0000,0x0000,0x0000}},
{0x11,0x0404,{0x042C,0x0000,0x0000,0x0000}},
{0x11,0x0405,{0x042D,0x0000,0x0000,0x0000}},
{0x11,0x0406,{0x042E,0x0000,0x0000,0x0000}},
{0x11,0x0407,{0x042F,0x0000,0x0000,0x0000}},
{0x11,0x0408,{0x0430,0x0000,0x0000,0x0000}},
{0x11,0x0409,{0x0431,0x0000,0x0000,0x0000}},
{0x11,0x040A,{0x0432,0x0000,0x0000,0x0000}},
{0x11,0x040B,{0x0433,0x0000,0x0000,0x0000}},
{0x11,0x040C,{0x0434,0x0000,0x0000,0x0000}},
{0x11,0x040D,{0x0435,0x0000,0x0000,0x0000}},
{0x11,0x040E,{0x0436,0x0000,0x0000,0x0000}},
{0x11,0x040F,{0x0437,0x0000,0x0000,0x0000}},
{0x11,0x0410,{0x0438,0x0000,0x0000,0x0000}},
{0x11,0x0411,{0x0439,0x0000,0x0000,0x0000}},
{0x11,0x0412,{0x043A,0x0000,0x0000,0x0000}},
{0x11,0x0413,{0x043B,0x0000,0x0000,0x0000}},
{0x11,0x0414,{0x043C,0x0000,0x0000,0x0000}},
{0x11,0x0415,{0x043D,0x0000,0x0000,0x0000}},
{0x11,0x0416,{0x043E,0x0000,0x0000,0x0000}},
{0x11,0x0417,{0x043F,0x0000,0x0000,0x0000}},
{0x11,0x0418,{0x0440,0x0000,0x0000,0x0000}},
{0x11,0x0419,{0x0441,0x0000,0x0000,0x0000}},
{0x11,0x041A,{0x0442,0x0000,0x0000,0x0000}},
{0x11,0x041B,{0x0443,0x0000,0x0000,0x0000}},
{0x11,0x041C,{0x0444,0x0000,0x0000,0x0000}},
{0x11,0x041D,{0x0445,0x0000,0x0000,0x0000}},
{0x11,0x041E,{0x0446,0x0000,0x0000,0x0000}},
{0x11,0x041F,{0x0447,0x0000,0x0000,0x0000}},
{0x11,0x0420,{0x0448,0x0000,0x0000,0x0000}},
{0x11,0x0421,{0x0449,0x0000,0x0000,0x0000}},
{0x11,0x0422,{0x044A,0x0000,0x0000,0x0000}},
{0x11,0x0423,{0x044B,0x0000,0x0000,0x0000}},
{0x11,0x0424,{0x044C,0x0000,0x0000,0x0000}},
{0x11,0x0425,{0x044D,0x0000,0x0000,0x0000}},
{0x01,0xD400,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD401,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD402,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD403,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD404,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD405,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD406,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD407,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD408,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD409,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD40A,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD40B,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD40C,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD40D,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD40E,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD40F,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD410,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD411,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD412,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD413,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD414,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD415,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD416,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD417,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD418,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD419,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD434,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD435,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD436,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD437,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD438,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD439,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD43A,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD43B,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD43C,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD43D,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD43E,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD43F,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD440,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD441,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD442,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD443,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD444,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD445,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD446,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD447,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD448,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD449,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD44A,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD44B,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD44C,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD44D,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD468,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD469,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD46A,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD46B,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD46C,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD46D,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD46E,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD46F,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD470,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD471,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD472,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD473,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD474,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD475,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD476,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD477,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD478,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD479,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD47A,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD47B,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD47C,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD47D,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD47E,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD47F,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD480,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD481,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD49C,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD49E,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD49F,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD4A2,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD4A5,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD4A6,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD4A9,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD4AA,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD4AB,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD4AC,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD4AE,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD4AF,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD4B0,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD4B1,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD4B2,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD4B3,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD4B4,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD4B5,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD4D0,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD4D1,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD4D2,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD4D3,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD4D4,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD4D5,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD4D6,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD4D7,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD4D8,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD4D9,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD4DA,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD4DB,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD4DC,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD4DD,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD4DE,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD4DF,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD4E0,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD4E1,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD4E2,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD4E3,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD4E4,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD4E5,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD4E6,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD4E7,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD4E8,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD4E9,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD504,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD505,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD507,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD508,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD509,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD50A,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD50D,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD50E,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD50F,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD510,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD511,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD512,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD513,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD514,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD516,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD517,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD518,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD519,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD51A,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD51B,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD51C,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD538,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD539,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD53B,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD53C,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD53D,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD53E,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD540,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD541,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD542,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD543,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD544,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD546,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD54A,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD54B,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD54C,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD54D,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD54E,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD54F,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD550,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD56C,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD56D,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD56E,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD56F,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD570,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD571,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD572,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD573,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD574,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD575,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD576,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD577,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD578,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD579,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD57A,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD57B,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD57C,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD57D,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD57E,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD57F,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD580,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD581,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD582,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD583,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD584,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD585,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD5A0,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD5A1,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD5A2,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD5A3,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD5A4,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD5A5,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD5A6,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD5A7,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD5A8,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD5A9,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD5AA,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD5AB,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD5AC,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD5AD,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD5AE,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD5AF,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD5B0,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD5B1,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD5B2,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD5B3,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD5B4,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD5B5,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD5B6,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD5B7,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD5B8,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD5B9,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD5D4,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD5D5,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD5D6,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD5D7,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD5D8,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD5D9,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD5DA,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD5DB,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD5DC,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD5DD,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD5DE,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD5DF,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD5E0,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD5E1,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD5E2,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD5E3,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD5E4,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD5E5,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD5E6,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD5E7,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD5E8,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD5E9,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD5EA,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD5EB,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD5EC,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD5ED,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD608,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD609,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD60A,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD60B,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD60C,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD60D,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD60E,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD60F,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD610,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD611,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD612,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD613,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD614,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD615,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD616,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD617,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD618,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD619,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD61A,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD61B,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD61C,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD61D,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD61E,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD61F,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD620,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD621,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD63C,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD63D,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD63E,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD63F,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD640,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD641,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD642,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD643,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD644,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD645,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD646,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD647,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD648,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD649,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD64A,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD64B,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD64C,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD64D,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD64E,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD64F,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD650,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD651,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD652,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD653,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD654,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD655,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD670,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD671,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD672,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD673,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD674,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD675,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD676,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD677,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD678,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD679,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD67A,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD67B,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD67C,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD67D,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD67E,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD67F,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD680,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD681,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD682,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD683,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD684,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD685,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD686,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD687,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD688,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD689,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD6A8,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD6A9,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD6AA,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD6AB,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD6AC,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD6AD,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD6AE,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD6AF,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD6B0,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD6B1,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD6B2,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD6B3,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD6B4,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD6B5,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD6B6,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD6B7,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD6B8,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD6B9,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD6BA,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD6BB,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD6BC,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD6BD,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD6BE,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD6BF,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD6C0,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD6D3,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD6E2,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD6E3,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD6E4,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD6E5,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD6E6,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD6E7,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD6E8,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD6E9,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD6EA,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD6EB,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD6EC,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD6ED,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD6EE,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD6EF,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD6F0,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD6F1,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD6F2,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD6F3,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD6F4,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD6F5,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD6F6,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD6F7,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD6F8,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD6F9,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD6FA,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD70D,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD71C,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD71D,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD71E,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD71F,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD720,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD721,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD722,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD723,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD724,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD725,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD726,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD727,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD728,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD729,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD72A,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD72B,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD72C,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD72D,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD72E,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD72F,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD730,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD731,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD732,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD733,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD734,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD747,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD756,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD757,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD758,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD759,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD75A,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD75B,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD75C,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD75D,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD75E,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD75F,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD760,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD761,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD762,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD763,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD764,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD765,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD766,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD767,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD768,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD769,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD76A,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD76B,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD76C,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD76D,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD76E,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD781,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD790,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD791,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD792,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD793,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD794,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD795,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD796,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD797,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD798,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD799,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD79A,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD79B,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD79C,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD79D,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD79E,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD79F,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD7A0,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD7A1,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD7A2,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD7A3,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD7A4,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD7A5,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD7A6,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD7A7,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD7A8,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD7BB,{0x03C3,0x0000,0x0000,0x0000}}
};

const struct UnicodeInterval UCProhibited[NUCPROHIBITED]={
{0x00,0x0000,0x001F}, // [CONTROL CHARACTERS]
{0x00,0x0020,0x0020}, // SPACE
{0x00,0x007F,0x007F}, // DELETE
{0x00,0x0080,0x009F}, // [CONTROL CHARACTERS]
{0x00,0x00A0,0x00A0}, // NO-BREAK SPACE
{0x00,0x0340,0x0340}, // COMBINING GRAVE TONE MARK
{0x00,0x0341,0x0341}, // COMBINING ACUTE TONE MARK
{0x00,0x06DD,0x06DD}, // ARABIC END OF AYAH
{0x00,0x070F,0x070F}, // SYRIAC ABBREVIATION MARK
{0x00,0x1680,0x1680}, // OGHAM SPACE MARK
{0x00,0x180E,0x180E}, // MONGOLIAN VOWEL SEPARATOR
{0x00,0x2000,0x200F}, // EN QUAD,EM QUAD,EN SPACE,EM SPACE,THREE-PER-EM SPACE,FOUR-PER-EM SPACE,SIX-PER-EM SPACE,FIGURE SPACE,PUNCTUATION SPACE,THIN SPACE,HAIR SPACE,ZERO WIDTH SPACE,ZERO WIDTH NON-JOINER,ZERO WIDTH JOINER,LEFT-TO-RIGHT MARK,RIGHT-TO-LEFT MARK
{0x00,0x2028,0x202F}, // LINE SEPARATOR,PARAGRAPH SEPARATOR,LEFT-TO-RIGHT EMBEDDING,RIGHT-TO-LEFT EMBEDDING,POP DIRECTIONAL FORMATTING,LEFT-TO-RIGHT OVERRIDE,RIGHT-TO-LEFT OVERRIDEN,ARROW NO-BREAK SPACE
{0x00,0x205F,0x2063}, // MEDIUM MATHEMATICAL SPACE,WORD JOINER,FUNCTION APPLICATION,INVISIBLE TIMES,INVISIBLE SEPARATOR
{0x00,0x206A,0x206F}, // [CONTROL CHARACTERS]
{0x00,0x2FF0,0x2FFB}, // [IDEOGRAPHIC DESCRIPTION CHARACTERS]
{0x00,0x3000,0x3000}, // IDEOGRAPHIC SPACE
{0x00,0xD800,0xDFFF}, // [SURROGATE CODES]
{0x00,0xE000,0xF8FF}, // [PRIVATE USE, PLANE 0]
{0x00,0xFDD0,0xFDEF}, // [NONCHARACTER CODE POINTS]
{0x00,0xFEFF,0xFEFF}, // ZERO WIDTH NO-BREAK SPACE
{0x00,0xFFF9,0xFFFF}, // [CONTROL CHARACTERS],[NONCHARACTER CODE POINTS]
{0x01,0xD173,0xD17A}, // [MUSICAL CONTROL CHARACTERS]
{0x01,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x02,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x03,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x04,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x05,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x06,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x07,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x08,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x09,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x0A,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x0B,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x0C,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x0D,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x0E,0x0001,0x0001}, // LANGUAGE TAG
{0x0E,0x0020,0x007F}, // [TAGGING CHARACTERS]
{0x0E,0xFFFE,0xFFFF}, // [NONCHARACTER CODE POINTS]
{0x0F,0x0000,0xFFFF}, // [PRIVATE USE, PLANE 15],[NONCHARACTER CODE POINTS]
{0x10,0x0000,0xFFFF}, // [PRIVATE USE, PLANE 16],[NONCHARACTER CODE POINTS]
};

const struct UnicodeInterval UCBidiCatL[NUCBIDICATL]={
{0x00,0x0041,0x005A},
{0x00,0x0061,0x007A},
{0x00,0x00AA,0x00AA},
{0x00,0x00B5,0x00B5},
{0x00,0x00BA,0x00BA},
{0x00,0x00C0,0x00D6},
{0x00,0x00D8,0x00F6},
{0x00,0x00F8,0x0220},
{0x00,0x0222,0x0233},
{0x00,0x0250,0x02AD},
{0x00,0x02B0,0x02B8},
{0x00,0x02BB,0x02C1},
{0x00,0x02D0,0x02D1},
{0x00,0x02E0,0x02E4},
{0x00,0x02EE,0x02EE},
{0x00,0x037A,0x037A},
{0x00,0x0386,0x0386},
{0x00,0x0388,0x038A},
{0x00,0x038C,0x038C},
{0x00,0x038E,0x03A1},
{0x00,0x03A3,0x03CE},
{0x00,0x03D0,0x03F5},
{0x00,0x0400,0x0482},
{0x00,0x048A,0x04CE},
{0x00,0x04D0,0x04F5},
{0x00,0x04F8,0x04F9},
{0x00,0x0500,0x050F},
{0x00,0x0531,0x0556},
{0x00,0x0559,0x055F},
{0x00,0x0561,0x0587},
{0x00,0x0589,0x0589},
{0x00,0x0903,0x0903},
{0x00,0x0905,0x0939},
{0x00,0x093D,0x0940},
{0x00,0x0949,0x094C},
{0x00,0x0950,0x0950},
{0x00,0x0958,0x0961},
{0x00,0x0964,0x0970},
{0x00,0x0982,0x0983},
{0x00,0x0985,0x098C},
{0x00,0x098F,0x0990},
{0x00,0x0993,0x09A8},
{0x00,0x09AA,0x09B0},
{0x00,0x09B2,0x09B2},
{0x00,0x09B6,0x09B9},
{0x00,0x09BE,0x09C0},
{0x00,0x09C7,0x09C8},
{0x00,0x09CB,0x09CC},
{0x00,0x09D7,0x09D7},
{0x00,0x09DC,0x09DD},
{0x00,0x09DF,0x09E1},
{0x00,0x09E6,0x09F1},
{0x00,0x09F4,0x09FA},
{0x00,0x0A05,0x0A0A},
{0x00,0x0A0F,0x0A10},
{0x00,0x0A13,0x0A28},
{0x00,0x0A2A,0x0A30},
{0x00,0x0A32,0x0A33},
{0x00,0x0A35,0x0A36},
{0x00,0x0A38,0x0A39},
{0x00,0x0A3E,0x0A40},
{0x00,0x0A59,0x0A5C},
{0x00,0x0A5E,0x0A5E},
{0x00,0x0A66,0x0A6F},
{0x00,0x0A72,0x0A74},
{0x00,0x0A83,0x0A83},
{0x00,0x0A85,0x0A8B},
{0x00,0x0A8D,0x0A8D},
{0x00,0x0A8F,0x0A91},
{0x00,0x0A93,0x0AA8},
{0x00,0x0AAA,0x0AB0},
{0x00,0x0AB2,0x0AB3},
{0x00,0x0AB5,0x0AB9},
{0x00,0x0ABD,0x0AC0},
{0x00,0x0AC9,0x0AC9},
{0x00,0x0ACB,0x0ACC},
{0x00,0x0AD0,0x0AD0},
{0x00,0x0AE0,0x0AE0},
{0x00,0x0AE6,0x0AEF},
{0x00,0x0B02,0x0B03},
{0x00,0x0B05,0x0B0C},
{0x00,0x0B0F,0x0B10},
{0x00,0x0B13,0x0B28},
{0x00,0x0B2A,0x0B30},
{0x00,0x0B32,0x0B33},
{0x00,0x0B36,0x0B39},
{0x00,0x0B3D,0x0B3E},
{0x00,0x0B40,0x0B40},
{0x00,0x0B47,0x0B48},
{0x00,0x0B4B,0x0B4C},
{0x00,0x0B57,0x0B57},
{0x00,0x0B5C,0x0B5D},
{0x00,0x0B5F,0x0B61},
{0x00,0x0B66,0x0B70},
{0x00,0x0B83,0x0B83},
{0x00,0x0B85,0x0B8A},
{0x00,0x0B8E,0x0B90},
{0x00,0x0B92,0x0B95},
{0x00,0x0B99,0x0B9A},
{0x00,0x0B9C,0x0B9C},
{0x00,0x0B9E,0x0B9F},
{0x00,0x0BA3,0x0BA4},
{0x00,0x0BA8,0x0BAA},
{0x00,0x0BAE,0x0BB5},
{0x00,0x0BB7,0x0BB9},
{0x00,0x0BBE,0x0BBF},
{0x00,0x0BC1,0x0BC2},
{0x00,0x0BC6,0x0BC8},
{0x00,0x0BCA,0x0BCC},
{0x00,0x0BD7,0x0BD7},
{0x00,0x0BE7,0x0BF2},
{0x00,0x0C01,0x0C03},
{0x00,0x0C05,0x0C0C},
{0x00,0x0C0E,0x0C10},
{0x00,0x0C12,0x0C28},
{0x00,0x0C2A,0x0C33},
{0x00,0x0C35,0x0C39},
{0x00,0x0C41,0x0C44},
{0x00,0x0C60,0x0C61},
{0x00,0x0C66,0x0C6F},
{0x00,0x0C82,0x0C83},
{0x00,0x0C85,0x0C8C},
{0x00,0x0C8E,0x0C90},
{0x00,0x0C92,0x0CA8},
{0x00,0x0CAA,0x0CB3},
{0x00,0x0CB5,0x0CB9},
{0x00,0x0CBE,0x0CBE},
{0x00,0x0CC0,0x0CC4},
{0x00,0x0CC7,0x0CC8},
{0x00,0x0CCA,0x0CCB},
{0x00,0x0CD5,0x0CD6},
{0x00,0x0CDE,0x0CDE},
{0x00,0x0CE0,0x0CE1},
{0x00,0x0CE6,0x0CEF},
{0x00,0x0D02,0x0D03},
{0x00,0x0D05,0x0D0C},
{0x00,0x0D0E,0x0D10},
{0x00,0x0D12,0x0D28},
{0x00,0x0D2A,0x0D39},
{0x00,0x0D3E,0x0D40},
{0x00,0x0D46,0x0D48},
{0x00,0x0D4A,0x0D4C},
{0x00,0x0D57,0x0D57},
{0x00,0x0D60,0x0D61},
{0x00,0x0D66,0x0D6F},
{0x00,0x0D82,0x0D83},
{0x00,0x0D85,0x0D96},
{0x00,0x0D9A,0x0DB1},
{0x00,0x0DB3,0x0DBB},
{0x00,0x0DBD,0x0DBD},
{0x00,0x0DC0,0x0DC6},
{0x00,0x0DCF,0x0DD1},
{0x00,0x0DD8,0x0DDF},
{0x00,0x0DF2,0x0DF4},
{0x00,0x0E01,0x0E30},
{0x00,0x0E32,0x0E33},
{0x00,0x0E40,0x0E46},
{0x00,0x0E4F,0x0E5B},
{0x00,0x0E81,0x0E82},
{0x00,0x0E84,0x0E84},
{0x00,0x0E87,0x0E88},
{0x00,0x0E8A,0x0E8A},
{0x00,0x0E8D,0x0E8D},
{0x00,0x0E94,0x0E97},
{0x00,0x0E99,0x0E9F},
{0x00,0x0EA1,0x0EA3},
{0x00,0x0EA5,0x0EA5},
{0x00,0x0EA7,0x0EA7},
{0x00,0x0EAA,0x0EAB},
{0x00,0x0EAD,0x0EB0},
{0x00,0x0EB2,0x0EB3},
{0x00,0x0EBD,0x0EBD},
{0x00,0x0EC0,0x0EC4},
{0x00,0x0EC6,0x0EC6},
{0x00,0x0ED0,0x0ED9},
{0x00,0x0EDC,0x0EDD},
{0x00,0x0F00,0x0F17},
{0x00,0x0F1A,0x0F34},
{0x00,0x0F36,0x0F36},
{0x00,0x0F38,0x0F38},
{0x00,0x0F3E,0x0F47},
{0x00,0x0F49,0x0F6A},
{0x00,0x0F7F,0x0F7F},
{0x00,0x0F85,0x0F85},
{0x00,0x0F88,0x0F8B},
{0x00,0x0FBE,0x0FC5},
{0x00,0x0FC7,0x0FCC},
{0x00,0x0FCF,0x0FCF},
{0x00,0x1000,0x1021},
{0x00,0x1023,0x1027},
{0x00,0x1029,0x102A},
{0x00,0x102C,0x102C},
{0x00,0x1031,0x1031},
{0x00,0x1038,0x1038},
{0x00,0x1040,0x1057},
{0x00,0x10A0,0x10C5},
{0x00,0x10D0,0x10F8},
{0x00,0x10FB,0x10FB},
{0x00,0x1100,0x1159},
{0x00,0x115F,0x11A2},
{0x00,0x11A8,0x11F9},
{0x00,0x1200,0x1206},
{0x00,0x1208,0x1246},
{0x00,0x1248,0x1248},
{0x00,0x124A,0x124D},
{0x00,0x1250,0x1256},
{0x00,0x1258,0x1258},
{0x00,0x125A,0x125D},
{0x00,0x1260,0x1286},
{0x00,0x1288,0x1288},
{0x00,0x128A,0x128D},
{0x00,0x1290,0x12AE},
{0x00,0x12B0,0x12B0},
{0x00,0x12B2,0x12B5},
{0x00,0x12B8,0x12BE},
{0x00,0x12C0,0x12C0},
{0x00,0x12C2,0x12C5},
{0x00,0x12C8,0x12CE},
{0x00,0x12D0,0x12D6},
{0x00,0x12D8,0x12EE},
{0x00,0x12F0,0x130E},
{0x00,0x1310,0x1310},
{0x00,0x1312,0x1315},
{0x00,0x1318,0x131E},
{0x00,0x1320,0x1346},
{0x00,0x1348,0x135A},
{0x00,0x1361,0x137C},
{0x00,0x13A0,0x13F4},
{0x00,0x1401,0x1676},
{0x00,0x1681,0x169A},
{0x00,0x16A0,0x16F0},
{0x00,0x1700,0x170C},
{0x00,0x170E,0x1711},
{0x00,0x1720,0x1731},
{0x00,0x1735,0x1736},
{0x00,0x1740,0x1751},
{0x00,0x1760,0x176C},
{0x00,0x176E,0x1770},
{0x00,0x1780,0x17B6},
{0x00,0x17BE,0x17C5},
{0x00,0x17C7,0x17C8},
{0x00,0x17D4,0x17DA},
{0x00,0x17DC,0x17DC},
{0x00,0x17E0,0x17E9},
{0x00,0x1810,0x1819},
{0x00,0x1820,0x1877},
{0x00,0x1880,0x18A8},
{0x00,0x1E00,0x1E9B},
{0x00,0x1EA0,0x1EF9},
{0x00,0x1F00,0x1F15},
{0x00,0x1F18,0x1F1D},
{0x00,0x1F20,0x1F45},
{0x00,0x1F48,0x1F4D},
{0x00,0x1F50,0x1F57},
{0x00,0x1F59,0x1F59},
{0x00,0x1F5B,0x1F5B},
{0x00,0x1F5D,0x1F5D},
{0x00,0x1F5F,0x1F7D},
{0x00,0x1F80,0x1FB4},
{0x00,0x1FB6,0x1FBC},
{0x00,0x1FBE,0x1FBE},
{0x00,0x1FC2,0x1FC4},
{0x00,0x1FC6,0x1FCC},
{0x00,0x1FD0,0x1FD3},
{0x00,0x1FD6,0x1FDB},
{0x00,0x1FE0,0x1FEC},
{0x00,0x1FF2,0x1FF4},
{0x00,0x1FF6,0x1FFC},
{0x00,0x200E,0x200E},
{0x00,0x2071,0x2071},
{0x00,0x207F,0x207F},
{0x00,0x2102,0x2102},
{0x00,0x2107,0x2107},
{0x00,0x210A,0x2113},
{0x00,0x2115,0x2115},
{0x00,0x2119,0x211D},
{0x00,0x2124,0x2124},
{0x00,0x2126,0x2126},
{0x00,0x2128,0x2128},
{0x00,0x212A,0x212D},
{0x00,0x212F,0x2131},
{0x00,0x2133,0x2139},
{0x00,0x213D,0x213F},
{0x00,0x2145,0x2149},
{0x00,0x2160,0x2183},
{0x00,0x2336,0x237A},
{0x00,0x2395,0x2395},
{0x00,0x249C,0x24E9},
{0x00,0x3005,0x3007},
{0x00,0x3021,0x3029},
{0x00,0x3031,0x3035},
{0x00,0x3038,0x303C},
{0x00,0x3041,0x3096},
{0x00,0x309D,0x309F},
{0x00,0x30A1,0x30FA},
{0x00,0x30FC,0x30FF},
{0x00,0x3105,0x312C},
{0x00,0x3131,0x318E},
{0x00,0x3190,0x31B7},
{0x00,0x31F0,0x321C},
{0x00,0x3220,0x3243},
{0x00,0x3260,0x327B},
{0x00,0x327F,0x32B0},
{0x00,0x32C0,0x32CB},
{0x00,0x32D0,0x32FE},
{0x00,0x3300,0x3376},
{0x00,0x337B,0x33DD},
{0x00,0x33E0,0x33FE},
{0x00,0x3400,0x4DB5},
{0x00,0x4E00,0x9FA5},
{0x00,0xA000,0xA48C},
{0x00,0xAC00,0xD7A3},
{0x00,0xD800,0xFA2D},
{0x00,0xFA30,0xFA6A},
{0x00,0xFB00,0xFB06},
{0x00,0xFB13,0xFB17},
{0x00,0xFF21,0xFF3A},
{0x00,0xFF41,0xFF5A},
{0x00,0xFF66,0xFFBE},
{0x00,0xFFC2,0xFFC7},
{0x00,0xFFCA,0xFFCF},
{0x00,0xFFD2,0xFFD7},
{0x00,0xFFDA,0xFFDC},
{0x01,0x0300,0x031E},
{0x01,0x0320,0x0323},
{0x01,0x0330,0x034A},
{0x01,0x0400,0x0425},
{0x01,0x0428,0x044D},
{0x01,0xD000,0xD0F5},
{0x01,0xD100,0xD126},
{0x01,0xD12A,0xD166},
{0x01,0xD16A,0xD172},
{0x01,0xD183,0xD184},
{0x01,0xD18C,0xD1A9},
{0x01,0xD1AE,0xD1DD},
{0x01,0xD400,0xD454},
{0x01,0xD456,0xD49C},
{0x01,0xD49E,0xD49F},
{0x01,0xD4A2,0xD4A2},
{0x01,0xD4A5,0xD4A6},
{0x01,0xD4A9,0xD4AC},
{0x01,0xD4AE,0xD4B9},
{0x01,0xD4BB,0xD4BB},
{0x01,0xD4BD,0xD4C0},
{0x01,0xD4C2,0xD4C3},
{0x01,0xD4C5,0xD505},
{0x01,0xD507,0xD50A},
{0x01,0xD50D,0xD514},
{0x01,0xD516,0xD51C},
{0x01,0xD51E,0xD539},
{0x01,0xD53B,0xD53E},
{0x01,0xD540,0xD544},
{0x01,0xD546,0xD546},
{0x01,0xD54A,0xD550},
{0x01,0xD552,0xD6A3},
{0x01,0xD6A8,0xD7C9},
{0x02,0x0000,0xA6D6},
{0x02,0xF800,0xFA1D},
{0x0F,0x0000,0xFFFD},
{0x10,0x0000,0xFFFD}
};

const struct UnicodeInterval UCBidiCatR_AL[NUCBIDICATR_AL]={
{0x00,0x05BE,0x05BE},
{0x00,0x05C0,0x05C0},
{0x00,0x05C3,0x05C3},
{0x00,0x05D0,0x05EA},
{0x00,0x05F0,0x05F4},
{0x00,0x061B,0x061B},
{0x00,0x061F,0x061F},
{0x00,0x0621,0x063A},
{0x00,0x0640,0x064A},
{0x00,0x066D,0x066F},
{0x00,0x0671,0x06D5},
{0x00,0x06DD,0x06DD},
{0x00,0x06E5,0x06E6},
{0x00,0x06FA,0x06FE},
{0x00,0x0700,0x070D},
{0x00,0x0710,0x0710},
{0x00,0x0712,0x072C},
{0x00,0x0780,0x07A5},
{0x00,0x07B1,0x07B1},
{0x00,0x200F,0x200F},
{0x00,0xFB1D,0xFB1D},
{0x00,0xFB1F,0xFB28},
{0x00,0xFB2A,0xFB36},
{0x00,0xFB38,0xFB3C},
{0x00,0xFB3E,0xFB3E},
{0x00,0xFB40,0xFB41},
{0x00,0xFB43,0xFB44},
{0x00,0xFB46,0xFBB1},
{0x00,0xFBD3,0xFD3D},
{0x00,0xFD50,0xFD8F},
{0x00,0xFD92,0xFDC7},
{0x00,0xFDF0,0xFDFC},
{0x00,0xFE70,0xFE74},
{0x00,0xFE76,0xFEFC}
};

const struct UnicodeInterval UCClassIntervals[NUCCLASSES]={
{0x00,0x0300,0x0314},
{0x00,0x0315,0x0315},
{0x00,0x0316,0x0319},
{0x00,0x031A,0x031A},
{0x00,0x031B,0x031B},
{0x00,0x031C,0x0320},
{0x00,0x0321,0x0322},
{0x00,0x0323,0x0326},
{0x00,0x0327,0x0328},
{0x00,0x0329,0x0333},
{0x00,0x0334,0x0338},
{0x00,0x0339,0x033C},
{0x00,0x033D,0x0344},
{0x00,0x0345,0x0345},
{0x00,0x0346,0x0346},
{0x00,0x0347,0x0349},
{0x00,0x034A,0x034C},
{0x00,0x034D,0x034E},
{0x00,0x0350,0x0352},
{0x00,0x0353,0x0356},
{0x00,0x0357,0x0357},
{0x00,0x0358,0x0358},
{0x00,0x0359,0x035A},
{0x00,0x035B,0x035B},
{0x00,0x035C,0x035C},
{0x00,0x035D,0x035E},
{0x00,0x035F,0x035F},
{0x00,0x0360,0x0361},
{0x00,0x0362,0x0362},
{0x00,0x0363,0x036F},
{0x00,0x0483,0x0486},
{0x00,0x0591,0x0595},
{0x00,0x0596,0x0596},
{0x00,0x0597,0x0599},
{0x00,0x059A,0x059A},
{0x00,0x059B,0x059B},
{0x00,0x059C,0x05A1},
{0x00,0x05A2,0x05A7},
{0x00,0x05A8,0x05A9},
{0x00,0x05AA,0x05AA},
{0x00,0x05AB,0x05AC},
{0x00,0x05AD,0x05AD},
{0x00,0x05AE,0x05AE},
{0x00,0x05AF,0x05AF},
{0x00,0x05B0,0x05B0},
{0x00,0x05B1,0x05B1},
{0x00,0x05B2,0x05B2},
{0x00,0x05B3,0x05B3},
{0x00,0x05B4,0x05B4},
{0x00,0x05B5,0x05B5},
{0x00,0x05B6,0x05B6},
{0x00,0x05B7,0x05B7},
{0x00,0x05B8,0x05B8},
{0x00,0x05B9,0x05B9},
{0x00,0x05BB,0x05BB},
{0x00,0x05BC,0x05BC},
{0x00,0x05BD,0x05BD},
{0x00,0x05BF,0x05BF},
{0x00,0x05C1,0x05C1},
{0x00,0x05C2,0x05C2},
{0x00,0x05C4,0x05C4},
{0x00,0x05C5,0x05C5},
{0x00,0x05C7,0x05C7},
{0x00,0x0610,0x0615},
{0x00,0x064B,0x064B},
{0x00,0x064C,0x064C},
{0x00,0x064D,0x064D},
{0x00,0x064E,0x064E},
{0x00,0x064F,0x064F},
{0x00,0x0650,0x0650},
{0x00,0x0651,0x0651},
{0x00,0x0652,0x0652},
{0x00,0x0653,0x065B},
{0x00,0x065C,0x065C},
{0x00,0x065D,0x065D},
{0x00,0x065E,0x065E},
{0x00,0x0670,0x0670},
{0x00,0x06D6,0x06DC},
{0x00,0x06DF,0x06E4},
{0x00,0x06E7,0x06E8},
{0x00,0x06EA,0x06ED},
{0x00,0x0711,0x0711},
{0x00,0x0730,0x0730},
{0x00,0x0731,0x0731},
{0x00,0x0732,0x0733},
{0x00,0x0734,0x0734},
{0x00,0x0735,0x0736},
{0x00,0x0737,0x0739},
{0x00,0x073A,0x073A},
{0x00,0x073B,0x073C},
{0x00,0x073D,0x073D},
{0x00,0x073E,0x073E},
{0x00,0x073F,0x0741},
{0x00,0x0742,0x0742},
{0x00,0x0743,0x0743},
{0x00,0x0744,0x0744},
{0x00,0x0745,0x0745},
{0x00,0x0746,0x0746},
{0x00,0x0747,0x0747},
{0x00,0x0748,0x0748},
{0x00,0x0749,0x074A},
{0x00,0x093C,0x093C},
{0x00,0x094D,0x094D},
{0x00,0x0951,0x0951},
{0x00,0x0952,0x0952},
{0x00,0x0953,0x0954},
{0x00,0x09BC,0x09BC},
{0x00,0x09CD,0x09CD},
{0x00,0x0A3C,0x0A3C},
{0x00,0x0A4D,0x0A4D},
{0x00,0x0ABC,0x0ABC},
{0x00,0x0ACD,0x0ACD},
{0x00,0x0B3C,0x0B3C},
{0x00,0x0B4D,0x0B4D},
{0x00,0x0BCD,0x0BCD},
{0x00,0x0C4D,0x0C4D},
{0x00,0x0C55,0x0C55},
{0x00,0x0C56,0x0C56},
{0x00,0x0CBC,0x0CBC},
{0x00,0x0CCD,0x0CCD},
{0x00,0x0D4D,0x0D4D},
{0x00,0x0DCA,0x0DCA},
{0x00,0x0E38,0x0E39},
{0x00,0x0E3A,0x0E3A},
{0x00,0x0E48,0x0E4B},
{0x00,0x0EB8,0x0EB9},
{0x00,0x0EC8,0x0ECB},
{0x00,0x0F18,0x0F19},
{0x00,0x0F35,0x0F35},
{0x00,0x0F37,0x0F37},
{0x00,0x0F39,0x0F39},
{0x00,0x0F71,0x0F71},
{0x00,0x0F72,0x0F72},
{0x00,0x0F74,0x0F74},
{0x00,0x0F7A,0x0F7D},
{0x00,0x0F80,0x0F80},
{0x00,0x0F82,0x0F83},
{0x00,0x0F84,0x0F84},
{0x00,0x0F86,0x0F87},
{0x00,0x0FC6,0x0FC6},
{0x00,0x1037,0x1037},
{0x00,0x1039,0x1039},
{0x00,0x135F,0x135F},
{0x00,0x1714,0x1714},
{0x00,0x1734,0x1734},
{0x00,0x17D2,0x17D2},
{0x00,0x17DD,0x17DD},
{0x00,0x18A9,0x18A9},
{0x00,0x1939,0x1939},
{0x00,0x193A,0x193A},
{0x00,0x193B,0x193B},
{0x00,0x1A17,0x1A17},
{0x00,0x1A18,0x1A18},
{0x00,0x1DC0,0x1DC1},
{0x00,0x1DC2,0x1DC2},
{0x00,0x1DC3,0x1DC3},
{0x00,0x20D0,0x20D1},
{0x00,0x20D2,0x20D3},
{0x00,0x20D4,0x20D7},
{0x00,0x20D8,0x20DA},
{0x00,0x20DB,0x20DC},
{0x00,0x20E1,0x20E1},
{0x00,0x20E5,0x20E6},
{0x00,0x20E7,0x20E7},
{0x00,0x20E8,0x20E8},
{0x00,0x20E9,0x20E9},
{0x00,0x20EA,0x20EB},
{0x00,0x302A,0x302A},
{0x00,0x302B,0x302B},
{0x00,0x302C,0x302C},
{0x00,0x302D,0x302D},
{0x00,0x302E,0x302F},
{0x00,0x3099,0x309A},
{0x00,0xA806,0xA806},
{0x00,0xFB1E,0xFB1E},
{0x00,0xFE20,0xFE23},
{0x01,0x0A0D,0x0A0D},
{0x01,0x0A0F,0x0A0F},
{0x01,0x0A38,0x0A38},
{0x01,0x0A39,0x0A39},
{0x01,0x0A3A,0x0A3A},
{0x01,0x0A3F,0x0A3F},
{0x01,0xD165,0xD166},
{0x01,0xD167,0xD169},
{0x01,0xD16D,0xD16D},
{0x01,0xD16E,0xD172},
{0x01,0xD17B,0xD182},
{0x01,0xD185,0xD189},
{0x01,0xD18A,0xD18B},
{0x01,0xD1AA,0xD1AD},
{0x01,0xD242,0xD244}
};

const BYTE UCClassLookup[NUCCLASSES]={
230,
232,
220,
232,
216,
220,
202,
220,
202,
220,
1,
220,
230,
240,
230,
220,
230,
220,
230,
220,
230,
232,
220,
230,
233,
234,
233,
234,
233,
230,
230,
220,
220,
230,
222,
220,
230,
220,
230,
220,
230,
222,
228,
230,
10,
11,
12,
13,
14,
15,
16,
17,
18,
19,
20,
21,
22,
23,
24,
25,
230,
220,
18,
230,
27,
28,
29,
30,
31,
32,
33,
34,
230,
220,
230,
230,
35,
230,
230,
230,
220,
36,
230,
220,
230,
220,
230,
220,
230,
220,
230,
220,
230,
220,
230,
220,
230,
220,
230,
220,
230,
7,
9,
230,
220,
230,
7,
9,
7,
9,
7,
9,
7,
9,
9,
9,
84,
91,
7,
9,
9,
9,
103,
9,
107,
118,
122,
220,
220,
220,
216,
129,
130,
132,
130,
130,
230,
9,
230,
220,
7,
9,
230,
9,
9,
9,
230,
228,
222,
230,
220,
230,
220,
230,
220,
230,
230,
1,
230,
1,
230,
230,
1,
230,
220,
230,
1,
218,
228,
232,
222,
224,
8,
9,
26,
230,
220,
230,
230,
1,
220,
9,
216,
1,
226,
216,
220,
230,
220,
230,
230
};

const struct UnicodeMappingDecomp UCDecomp[NUCDECOMP]={
{0x00,0x00E0,{0x0061,0x0300}},
{0x00,0x00E1,{0x0061,0x0301}},
{0x00,0x00E2,{0x0061,0x0302}},
{0x00,0x00E3,{0x0061,0x0303}},
{0x00,0x00E4,{0x0061,0x0308}},
{0x00,0x00E5,{0x0061,0x030A}},
{0x00,0x00E7,{0x0063,0x0327}},
{0x00,0x00E8,{0x0065,0x0300}},
{0x00,0x00E9,{0x0065,0x0301}},
{0x00,0x00EA,{0x0065,0x0302}},
{0x00,0x00EB,{0x0065,0x0308}},
{0x00,0x00EC,{0x0069,0x0300}},
{0x00,0x00ED,{0x0069,0x0301}},
{0x00,0x00EE,{0x0069,0x0302}},
{0x00,0x00EF,{0x0069,0x0308}},
{0x00,0x00F1,{0x006E,0x0303}},
{0x00,0x00F2,{0x006F,0x0300}},
{0x00,0x00F3,{0x006F,0x0301}},
{0x00,0x00F4,{0x006F,0x0302}},
{0x00,0x00F5,{0x006F,0x0303}},
{0x00,0x00F6,{0x006F,0x0308}},
{0x00,0x00F9,{0x0075,0x0300}},
{0x00,0x00FA,{0x0075,0x0301}},
{0x00,0x00FB,{0x0075,0x0302}},
{0x00,0x00FC,{0x0075,0x0308}},
{0x00,0x00FD,{0x0079,0x0301}},
{0x00,0x00FF,{0x0079,0x0308}},
{0x00,0x0101,{0x0061,0x0304}},
{0x00,0x0103,{0x0061,0x0306}},
{0x00,0x0105,{0x0061,0x0328}},
{0x00,0x0107,{0x0063,0x0301}},
{0x00,0x0109,{0x0063,0x0302}},
{0x00,0x010B,{0x0063,0x0307}},
{0x00,0x010D,{0x0063,0x030C}},
{0x00,0x010F,{0x0064,0x030C}},
{0x00,0x0113,{0x0065,0x0304}},
{0x00,0x0115,{0x0065,0x0306}},
{0x00,0x0117,{0x0065,0x0307}},
{0x00,0x0119,{0x0065,0x0328}},
{0x00,0x011B,{0x0065,0x030C}},
{0x00,0x011D,{0x0067,0x0302}},
{0x00,0x011F,{0x0067,0x0306}},
{0x00,0x0121,{0x0067,0x0307}},
{0x00,0x0123,{0x0067,0x0327}},
{0x00,0x0125,{0x0068,0x0302}},
{0x00,0x0129,{0x0069,0x0303}},
{0x00,0x012B,{0x0069,0x0304}},
{0x00,0x012D,{0x0069,0x0306}},
{0x00,0x012F,{0x0069,0x0328}},
{0x00,0x0135,{0x006A,0x0302}},
{0x00,0x0137,{0x006B,0x0327}},
{0x00,0x013A,{0x006C,0x0301}},
{0x00,0x013C,{0x006C,0x0327}},
{0x00,0x013E,{0x006C,0x030C}},
{0x00,0x0144,{0x006E,0x0301}},
{0x00,0x0146,{0x006E,0x0327}},
{0x00,0x0148,{0x006E,0x030C}},
{0x00,0x014D,{0x006F,0x0304}},
{0x00,0x014F,{0x006F,0x0306}},
{0x00,0x0151,{0x006F,0x030B}},
{0x00,0x0155,{0x0072,0x0301}},
{0x00,0x0157,{0x0072,0x0327}},
{0x00,0x0159,{0x0072,0x030C}},
{0x00,0x015B,{0x0073,0x0301}},
{0x00,0x015D,{0x0073,0x0302}},
{0x00,0x015F,{0x0073,0x0327}},
{0x00,0x0161,{0x0073,0x030C}},
{0x00,0x0163,{0x0074,0x0327}},
{0x00,0x0165,{0x0074,0x030C}},
{0x00,0x0169,{0x0075,0x0303}},
{0x00,0x016B,{0x0075,0x0304}},
{0x00,0x016D,{0x0075,0x0306}},
{0x00,0x016F,{0x0075,0x030A}},
{0x00,0x0171,{0x0075,0x030B}},
{0x00,0x0173,{0x0075,0x0328}},
{0x00,0x0175,{0x0077,0x0302}},
{0x00,0x0177,{0x0079,0x0302}},
{0x00,0x017A,{0x007A,0x0301}},
{0x00,0x017C,{0x007A,0x0307}},
{0x00,0x017E,{0x007A,0x030C}},
{0x00,0x01A1,{0x006F,0x031B}},
{0x00,0x01B0,{0x0075,0x031B}},
{0x00,0x01CE,{0x0061,0x030C}},
{0x00,0x01D0,{0x0069,0x030C}},
{0x00,0x01D2,{0x006F,0x030C}},
{0x00,0x01D4,{0x0075,0x030C}},
{0x00,0x01D6,{0x00FC,0x0304}},
{0x00,0x01D8,{0x00FC,0x0301}},
{0x00,0x01DA,{0x00FC,0x030C}},
{0x00,0x01DC,{0x00FC,0x0300}},
{0x00,0x01DF,{0x00E4,0x0304}},
{0x00,0x01E1,{0x0227,0x0304}},
{0x00,0x01E3,{0x00E6,0x0304}},
{0x00,0x01E7,{0x0067,0x030C}},
{0x00,0x01E9,{0x006B,0x030C}},
{0x00,0x01EB,{0x006F,0x0328}},
{0x00,0x01ED,{0x01EB,0x0304}},
{0x00,0x01EF,{0x0292,0x030C}},
{0x00,0x01F5,{0x0067,0x0301}},
{0x00,0x01F9,{0x006E,0x0300}},
{0x00,0x01FB,{0x00E5,0x0301}},
{0x00,0x01FD,{0x00E6,0x0301}},
{0x00,0x01FF,{0x00F8,0x0301}},
{0x00,0x0201,{0x0061,0x030F}},
{0x00,0x0203,{0x0061,0x0311}},
{0x00,0x0205,{0x0065,0x030F}},
{0x00,0x0207,{0x0065,0x0311}},
{0x00,0x0209,{0x0069,0x030F}},
{0x00,0x020B,{0x0069,0x0311}},
{0x00,0x020D,{0x006F,0x030F}},
{0x00,0x020F,{0x006F,0x0311}},
{0x00,0x0211,{0x0072,0x030F}},
{0x00,0x0213,{0x0072,0x0311}},
{0x00,0x0215,{0x0075,0x030F}},
{0x00,0x0217,{0x0075,0x0311}},
{0x00,0x0219,{0x0073,0x0326}},
{0x00,0x021B,{0x0074,0x0326}},
{0x00,0x021F,{0x0068,0x030C}},
{0x00,0x0227,{0x0061,0x0307}},
{0x00,0x0229,{0x0065,0x0327}},
{0x00,0x022B,{0x00F6,0x0304}},
{0x00,0x022D,{0x00F5,0x0304}},
{0x00,0x022F,{0x006F,0x0307}},
{0x00,0x0231,{0x022F,0x0304}},
{0x00,0x0233,{0x0079,0x0304}},
{0x00,0x0344,{0x0308,0x0301}},
{0x00,0x0385,{0x00A8,0x0301}},
{0x00,0x03AC,{0x03B1,0x0301}},
{0x00,0x03AD,{0x03B5,0x0301}},
{0x00,0x03AE,{0x03B7,0x0301}},
{0x00,0x03AF,{0x03B9,0x0301}},
{0x00,0x03CA,{0x03B9,0x0308}},
{0x00,0x03CB,{0x03C5,0x0308}},
{0x00,0x03CC,{0x03BF,0x0301}},
{0x00,0x03CD,{0x03C5,0x0301}},
{0x00,0x03CE,{0x03C9,0x0301}},
{0x00,0x0439,{0x0438,0x0306}},
{0x00,0x0450,{0x0435,0x0300}},
{0x00,0x0451,{0x0435,0x0308}},
{0x00,0x0453,{0x0433,0x0301}},
{0x00,0x0457,{0x0456,0x0308}},
{0x00,0x045C,{0x043A,0x0301}},
{0x00,0x045D,{0x0438,0x0300}},
{0x00,0x045E,{0x0443,0x0306}},
{0x00,0x0477,{0x0475,0x030F}},
{0x00,0x04C2,{0x0436,0x0306}},
{0x00,0x04D1,{0x0430,0x0306}},
{0x00,0x04D3,{0x0430,0x0308}},
{0x00,0x04D7,{0x0435,0x0306}},
{0x00,0x04DB,{0x04D9,0x0308}},
{0x00,0x04DD,{0x0436,0x0308}},
{0x00,0x04DF,{0x0437,0x0308}},
{0x00,0x04E3,{0x0438,0x0304}},
{0x00,0x04E5,{0x0438,0x0308}},
{0x00,0x04E7,{0x043E,0x0308}},
{0x00,0x04EB,{0x04E9,0x0308}},
{0x00,0x04ED,{0x044D,0x0308}},
{0x00,0x04EF,{0x0443,0x0304}},
{0x00,0x04F1,{0x0443,0x0308}},
{0x00,0x04F3,{0x0443,0x030B}},
{0x00,0x04F5,{0x0447,0x0308}},
{0x00,0x04F9,{0x044B,0x0308}},
{0x00,0x0622,{0x0627,0x0653}},
{0x00,0x0623,{0x0627,0x0654}},
{0x00,0x0624,{0x0648,0x0654}},
{0x00,0x0625,{0x0627,0x0655}},
{0x00,0x0626,{0x064A,0x0654}},
{0x00,0x06C0,{0x06D5,0x0654}},
{0x00,0x06C2,{0x06C1,0x0654}},
{0x00,0x06D3,{0x06D2,0x0654}},
{0x00,0x0929,{0x0928,0x093C}},
{0x00,0x0931,{0x0930,0x093C}},
{0x00,0x0934,{0x0933,0x093C}},
{0x00,0x0958,{0x0915,0x093C}},
{0x00,0x0959,{0x0916,0x093C}},
{0x00,0x095A,{0x0917,0x093C}},
{0x00,0x095B,{0x091C,0x093C}},
{0x00,0x095C,{0x0921,0x093C}},
{0x00,0x095D,{0x0922,0x093C}},
{0x00,0x095E,{0x092B,0x093C}},
{0x00,0x095F,{0x092F,0x093C}},
{0x00,0x09CB,{0x09C7,0x09BE}},
{0x00,0x09CC,{0x09C7,0x09D7}},
{0x00,0x09DC,{0x09A1,0x09BC}},
{0x00,0x09DD,{0x09A2,0x09BC}},
{0x00,0x09DF,{0x09AF,0x09BC}},
{0x00,0x0A33,{0x0A32,0x0A3C}},
{0x00,0x0A36,{0x0A38,0x0A3C}},
{0x00,0x0A59,{0x0A16,0x0A3C}},
{0x00,0x0A5A,{0x0A17,0x0A3C}},
{0x00,0x0A5B,{0x0A1C,0x0A3C}},
{0x00,0x0A5E,{0x0A2B,0x0A3C}},
{0x00,0x0B48,{0x0B47,0x0B56}},
{0x00,0x0B4B,{0x0B47,0x0B3E}},
{0x00,0x0B4C,{0x0B47,0x0B57}},
{0x00,0x0B5C,{0x0B21,0x0B3C}},
{0x00,0x0B5D,{0x0B22,0x0B3C}},
{0x00,0x0B94,{0x0B92,0x0BD7}},
{0x00,0x0BCA,{0x0BC6,0x0BBE}},
{0x00,0x0BCB,{0x0BC7,0x0BBE}},
{0x00,0x0BCC,{0x0BC6,0x0BD7}},
{0x00,0x0C48,{0x0C46,0x0C56}},
{0x00,0x0CC0,{0x0CBF,0x0CD5}},
{0x00,0x0CC7,{0x0CC6,0x0CD5}},
{0x00,0x0CC8,{0x0CC6,0x0CD6}},
{0x00,0x0CCA,{0x0CC6,0x0CC2}},
{0x00,0x0CCB,{0x0CCA,0x0CD5}},
{0x00,0x0D4A,{0x0D46,0x0D3E}},
{0x00,0x0D4B,{0x0D47,0x0D3E}},
{0x00,0x0D4C,{0x0D46,0x0D57}},
{0x00,0x0DDA,{0x0DD9,0x0DCA}},
{0x00,0x0DDC,{0x0DD9,0x0DCF}},
{0x00,0x0DDD,{0x0DDC,0x0DCA}},
{0x00,0x0DDE,{0x0DD9,0x0DDF}},
{0x00,0x0F43,{0x0F42,0x0FB7}},
{0x00,0x0F4D,{0x0F4C,0x0FB7}},
{0x00,0x0F52,{0x0F51,0x0FB7}},
{0x00,0x0F57,{0x0F56,0x0FB7}},
{0x00,0x0F5C,{0x0F5B,0x0FB7}},
{0x00,0x0F69,{0x0F40,0x0FB5}},
{0x00,0x0F73,{0x0F71,0x0F72}},
{0x00,0x0F75,{0x0F71,0x0F74}},
{0x00,0x0F76,{0x0FB2,0x0F80}},
{0x00,0x0F78,{0x0FB3,0x0F80}},
{0x00,0x0F81,{0x0F71,0x0F80}},
{0x00,0x0F93,{0x0F92,0x0FB7}},
{0x00,0x0F9D,{0x0F9C,0x0FB7}},
{0x00,0x0FA2,{0x0FA1,0x0FB7}},
{0x00,0x0FA7,{0x0FA6,0x0FB7}},
{0x00,0x0FAC,{0x0FAB,0x0FB7}},
{0x00,0x0FB9,{0x0F90,0x0FB5}},
{0x00,0x1026,{0x1025,0x102E}},
{0x00,0x1E01,{0x0061,0x0325}},
{0x00,0x1E03,{0x0062,0x0307}},
{0x00,0x1E05,{0x0062,0x0323}},
{0x00,0x1E07,{0x0062,0x0331}},
{0x00,0x1E09,{0x00E7,0x0301}},
{0x00,0x1E0B,{0x0064,0x0307}},
{0x00,0x1E0D,{0x0064,0x0323}},
{0x00,0x1E0F,{0x0064,0x0331}},
{0x00,0x1E11,{0x0064,0x0327}},
{0x00,0x1E13,{0x0064,0x032D}},
{0x00,0x1E15,{0x0113,0x0300}},
{0x00,0x1E17,{0x0113,0x0301}},
{0x00,0x1E19,{0x0065,0x032D}},
{0x00,0x1E1B,{0x0065,0x0330}},
{0x00,0x1E1D,{0x0229,0x0306}},
{0x00,0x1E1F,{0x0066,0x0307}},
{0x00,0x1E21,{0x0067,0x0304}},
{0x00,0x1E23,{0x0068,0x0307}},
{0x00,0x1E25,{0x0068,0x0323}},
{0x00,0x1E27,{0x0068,0x0308}},
{0x00,0x1E29,{0x0068,0x0327}},
{0x00,0x1E2B,{0x0068,0x032E}},
{0x00,0x1E2D,{0x0069,0x0330}},
{0x00,0x1E2F,{0x00EF,0x0301}},
{0x00,0x1E31,{0x006B,0x0301}},
{0x00,0x1E33,{0x006B,0x0323}},
{0x00,0x1E35,{0x006B,0x0331}},
{0x00,0x1E37,{0x006C,0x0323}},
{0x00,0x1E39,{0x1E37,0x0304}},
{0x00,0x1E3B,{0x006C,0x0331}},
{0x00,0x1E3D,{0x006C,0x032D}},
{0x00,0x1E3F,{0x006D,0x0301}},
{0x00,0x1E41,{0x006D,0x0307}},
{0x00,0x1E43,{0x006D,0x0323}},
{0x00,0x1E45,{0x006E,0x0307}},
{0x00,0x1E47,{0x006E,0x0323}},
{0x00,0x1E49,{0x006E,0x0331}},
{0x00,0x1E4B,{0x006E,0x032D}},
{0x00,0x1E4D,{0x00F5,0x0301}},
{0x00,0x1E4F,{0x00F5,0x0308}},
{0x00,0x1E51,{0x014D,0x0300}},
{0x00,0x1E53,{0x014D,0x0301}},
{0x00,0x1E55,{0x0070,0x0301}},
{0x00,0x1E57,{0x0070,0x0307}},
{0x00,0x1E59,{0x0072,0x0307}},
{0x00,0x1E5B,{0x0072,0x0323}},
{0x00,0x1E5D,{0x1E5B,0x0304}},
{0x00,0x1E5F,{0x0072,0x0331}},
{0x00,0x1E61,{0x0073,0x0307}},
{0x00,0x1E63,{0x0073,0x0323}},
{0x00,0x1E65,{0x015B,0x0307}},
{0x00,0x1E67,{0x0161,0x0307}},
{0x00,0x1E69,{0x1E63,0x0307}},
{0x00,0x1E6B,{0x0074,0x0307}},
{0x00,0x1E6D,{0x0074,0x0323}},
{0x00,0x1E6F,{0x0074,0x0331}},
{0x00,0x1E71,{0x0074,0x032D}},
{0x00,0x1E73,{0x0075,0x0324}},
{0x00,0x1E75,{0x0075,0x0330}},
{0x00,0x1E77,{0x0075,0x032D}},
{0x00,0x1E79,{0x0169,0x0301}},
{0x00,0x1E7B,{0x016B,0x0308}},
{0x00,0x1E7D,{0x0076,0x0303}},
{0x00,0x1E7F,{0x0076,0x0323}},
{0x00,0x1E81,{0x0077,0x0300}},
{0x00,0x1E83,{0x0077,0x0301}},
{0x00,0x1E85,{0x0077,0x0308}},
{0x00,0x1E87,{0x0077,0x0307}},
{0x00,0x1E89,{0x0077,0x0323}},
{0x00,0x1E8B,{0x0078,0x0307}},
{0x00,0x1E8D,{0x0078,0x0308}},
{0x00,0x1E8F,{0x0079,0x0307}},
{0x00,0x1E91,{0x007A,0x0302}},
{0x00,0x1E93,{0x007A,0x0323}},
{0x00,0x1E95,{0x007A,0x0331}},
{0x00,0x1EA1,{0x0061,0x0323}},
{0x00,0x1EA3,{0x0061,0x0309}},
{0x00,0x1EA5,{0x00E2,0x0301}},
{0x00,0x1EA7,{0x00E2,0x0300}},
{0x00,0x1EA9,{0x00E2,0x0309}},
{0x00,0x1EAB,{0x00E2,0x0303}},
{0x00,0x1EAD,{0x1EA1,0x0302}},
{0x00,0x1EAF,{0x0103,0x0301}},
{0x00,0x1EB1,{0x0103,0x0300}},
{0x00,0x1EB3,{0x0103,0x0309}},
{0x00,0x1EB5,{0x0103,0x0303}},
{0x00,0x1EB7,{0x1EA1,0x0306}},
{0x00,0x1EB9,{0x0065,0x0323}},
{0x00,0x1EBB,{0x0065,0x0309}},
{0x00,0x1EBD,{0x0065,0x0303}},
{0x00,0x1EBF,{0x00EA,0x0301}},
{0x00,0x1EC1,{0x00EA,0x0300}},
{0x00,0x1EC3,{0x00EA,0x0309}},
{0x00,0x1EC5,{0x00EA,0x0303}},
{0x00,0x1EC7,{0x1EB9,0x0302}},
{0x00,0x1EC9,{0x0069,0x0309}},
{0x00,0x1ECB,{0x0069,0x0323}},
{0x00,0x1ECD,{0x006F,0x0323}},
{0x00,0x1ECF,{0x006F,0x0309}},
{0x00,0x1ED1,{0x00F4,0x0301}},
{0x00,0x1ED3,{0x00F4,0x0300}},
{0x00,0x1ED5,{0x00F4,0x0309}},
{0x00,0x1ED7,{0x00F4,0x0303}},
{0x00,0x1ED9,{0x1ECD,0x0302}},
{0x00,0x1EDB,{0x01A1,0x0301}},
{0x00,0x1EDD,{0x01A1,0x0300}},
{0x00,0x1EDF,{0x01A1,0x0309}},
{0x00,0x1EE1,{0x01A1,0x0303}},
{0x00,0x1EE3,{0x01A1,0x0323}},
{0x00,0x1EE5,{0x0075,0x0323}},
{0x00,0x1EE7,{0x0075,0x0309}},
{0x00,0x1EE9,{0x01B0,0x0301}},
{0x00,0x1EEB,{0x01B0,0x0300}},
{0x00,0x1EED,{0x01B0,0x0309}},
{0x00,0x1EEF,{0x01B0,0x0303}},
{0x00,0x1EF1,{0x01B0,0x0323}},
{0x00,0x1EF3,{0x0079,0x0300}},
{0x00,0x1EF5,{0x0079,0x0323}},
{0x00,0x1EF7,{0x0079,0x0309}},
{0x00,0x1EF9,{0x0079,0x0303}},
{0x00,0x1F00,{0x03B1,0x0313}},
{0x00,0x1F01,{0x03B1,0x0314}},
{0x00,0x1F02,{0x1F00,0x0300}},
{0x00,0x1F03,{0x1F01,0x0300}},
{0x00,0x1F04,{0x1F00,0x0301}},
{0x00,0x1F05,{0x1F01,0x0301}},
{0x00,0x1F06,{0x1F00,0x0342}},
{0x00,0x1F07,{0x1F01,0x0342}},
{0x00,0x1F10,{0x03B5,0x0313}},
{0x00,0x1F11,{0x03B5,0x0314}},
{0x00,0x1F12,{0x1F10,0x0300}},
{0x00,0x1F13,{0x1F11,0x0300}},
{0x00,0x1F14,{0x1F10,0x0301}},
{0x00,0x1F15,{0x1F11,0x0301}},
{0x00,0x1F20,{0x03B7,0x0313}},
{0x00,0x1F21,{0x03B7,0x0314}},
{0x00,0x1F22,{0x1F20,0x0300}},
{0x00,0x1F23,{0x1F21,0x0300}},
{0x00,0x1F24,{0x1F20,0x0301}},
{0x00,0x1F25,{0x1F21,0x0301}},
{0x00,0x1F26,{0x1F20,0x0342}},
{0x00,0x1F27,{0x1F21,0x0342}},
{0x00,0x1F30,{0x03B9,0x0313}},
{0x00,0x1F31,{0x03B9,0x0314}},
{0x00,0x1F32,{0x1F30,0x0300}},
{0x00,0x1F33,{0x1F31,0x0300}},
{0x00,0x1F34,{0x1F30,0x0301}},
{0x00,0x1F35,{0x1F31,0x0301}},
{0x00,0x1F36,{0x1F30,0x0342}},
{0x00,0x1F37,{0x1F31,0x0342}},
{0x00,0x1F40,{0x03BF,0x0313}},
{0x00,0x1F41,{0x03BF,0x0314}},
{0x00,0x1F42,{0x1F40,0x0300}},
{0x00,0x1F43,{0x1F41,0x0300}},
{0x00,0x1F44,{0x1F40,0x0301}},
{0x00,0x1F45,{0x1F41,0x0301}},
{0x00,0x1F51,{0x03C5,0x0314}},
{0x00,0x1F53,{0x1F51,0x0300}},
{0x00,0x1F55,{0x1F51,0x0301}},
{0x00,0x1F57,{0x1F51,0x0342}},
{0x00,0x1F60,{0x03C9,0x0313}},
{0x00,0x1F61,{0x03C9,0x0314}},
{0x00,0x1F62,{0x1F60,0x0300}},
{0x00,0x1F63,{0x1F61,0x0300}},
{0x00,0x1F64,{0x1F60,0x0301}},
{0x00,0x1F65,{0x1F61,0x0301}},
{0x00,0x1F66,{0x1F60,0x0342}},
{0x00,0x1F67,{0x1F61,0x0342}},
{0x00,0x1F70,{0x03B1,0x0300}},
{0x00,0x1F72,{0x03B5,0x0300}},
{0x00,0x1F74,{0x03B7,0x0300}},
{0x00,0x1F76,{0x03B9,0x0300}},
{0x00,0x1F78,{0x03BF,0x0300}},
{0x00,0x1F7A,{0x03C5,0x0300}},
{0x00,0x1F7C,{0x03C9,0x0300}},
{0x00,0x1FB0,{0x03B1,0x0306}},
{0x00,0x1FB1,{0x03B1,0x0304}},
{0x00,0x1FC1,{0x00A8,0x0342}},
{0x00,0x1FCD,{0x1FBF,0x0300}},
{0x00,0x1FCE,{0x1FBF,0x0301}},
{0x00,0x1FCF,{0x1FBF,0x0342}},
{0x00,0x1FD0,{0x03B9,0x0306}},
{0x00,0x1FD1,{0x03B9,0x0304}},
{0x00,0x1FDD,{0x1FFE,0x0300}},
{0x00,0x1FDE,{0x1FFE,0x0301}},
{0x00,0x1FDF,{0x1FFE,0x0342}},
{0x00,0x1FE0,{0x03C5,0x0306}},
{0x00,0x1FE1,{0x03C5,0x0304}},
{0x00,0x1FE5,{0x03C1,0x0314}},
{0x00,0x1FED,{0x00A8,0x0300}},
{0x00,0x219A,{0x2190,0x0338}},
{0x00,0x219B,{0x2192,0x0338}},
{0x00,0x21AE,{0x2194,0x0338}},
{0x00,0x21CD,{0x21D0,0x0338}},
{0x00,0x21CE,{0x21D4,0x0338}},
{0x00,0x21CF,{0x21D2,0x0338}},
{0x00,0x2204,{0x2203,0x0338}},
{0x00,0x2209,{0x2208,0x0338}},
{0x00,0x220C,{0x220B,0x0338}},
{0x00,0x2224,{0x2223,0x0338}},
{0x00,0x2226,{0x2225,0x0338}},
{0x00,0x2241,{0x223C,0x0338}},
{0x00,0x2244,{0x2243,0x0338}},
{0x00,0x2247,{0x2245,0x0338}},
{0x00,0x2249,{0x2248,0x0338}},
{0x00,0x2260,{0x003D,0x0338}},
{0x00,0x2262,{0x2261,0x0338}},
{0x00,0x226D,{0x224D,0x0338}},
{0x00,0x226E,{0x003C,0x0338}},
{0x00,0x226F,{0x003E,0x0338}},
{0x00,0x2270,{0x2264,0x0338}},
{0x00,0x2271,{0x2265,0x0338}},
{0x00,0x2274,{0x2272,0x0338}},
{0x00,0x2275,{0x2273,0x0338}},
{0x00,0x2278,{0x2276,0x0338}},
{0x00,0x2279,{0x2277,0x0338}},
{0x00,0x2280,{0x227A,0x0338}},
{0x00,0x2281,{0x227B,0x0338}},
{0x00,0x2284,{0x2282,0x0338}},
{0x00,0x2285,{0x2283,0x0338}},
{0x00,0x2288,{0x2286,0x0338}},
{0x00,0x2289,{0x2287,0x0338}},
{0x00,0x22AC,{0x22A2,0x0338}},
{0x00,0x22AD,{0x22A8,0x0338}},
{0x00,0x22AE,{0x22A9,0x0338}},
{0x00,0x22AF,{0x22AB,0x0338}},
{0x00,0x22E0,{0x227C,0x0338}},
{0x00,0x22E1,{0x227D,0x0338}},
{0x00,0x22E2,{0x2291,0x0338}},
{0x00,0x22E3,{0x2292,0x0338}},
{0x00,0x22EA,{0x22B2,0x0338}},
{0x00,0x22EB,{0x22B3,0x0338}},
{0x00,0x22EC,{0x22B4,0x0338}},
{0x00,0x22ED,{0x22B5,0x0338}},
{0x00,0x2ADC,{0x2ADD,0x0338}},
{0x00,0x304C,{0x304B,0x3099}},
{0x00,0x304E,{0x304D,0x3099}},
{0x00,0x3050,{0x304F,0x3099}},
{0x00,0x3052,{0x3051,0x3099}},
{0x00,0x3054,{0x3053,0x3099}},
{0x00,0x3056,{0x3055,0x3099}},
{0x00,0x3058,{0x3057,0x3099}},
{0x00,0x305A,{0x3059,0x3099}},
{0x00,0x305C,{0x305B,0x3099}},
{0x00,0x305E,{0x305D,0x3099}},
{0x00,0x3060,{0x305F,0x3099}},
{0x00,0x3062,{0x3061,0x3099}},
{0x00,0x3065,{0x3064,0x3099}},
{0x00,0x3067,{0x3066,0x3099}},
{0x00,0x3069,{0x3068,0x3099}},
{0x00,0x3070,{0x306F,0x3099}},
{0x00,0x3071,{0x306F,0x309A}},
{0x00,0x3073,{0x3072,0x3099}},
{0x00,0x3074,{0x3072,0x309A}},
{0x00,0x3076,{0x3075,0x3099}},
{0x00,0x3077,{0x3075,0x309A}},
{0x00,0x3079,{0x3078,0x3099}},
{0x00,0x307A,{0x3078,0x309A}},
{0x00,0x307C,{0x307B,0x3099}},
{0x00,0x307D,{0x307B,0x309A}},
{0x00,0x3094,{0x3046,0x3099}},
{0x00,0x309E,{0x309D,0x3099}},
{0x00,0x30AC,{0x30AB,0x3099}},
{0x00,0x30AE,{0x30AD,0x3099}},
{0x00,0x30B0,{0x30AF,0x3099}},
{0x00,0x30B2,{0x30B1,0x3099}},
{0x00,0x30B4,{0x30B3,0x3099}},
{0x00,0x30B6,{0x30B5,0x3099}},
{0x00,0x30B8,{0x30B7,0x3099}},
{0x00,0x30BA,{0x30B9,0x3099}},
{0x00,0x30BC,{0x30BB,0x3099}},
{0x00,0x30BE,{0x30BD,0x3099}},
{0x00,0x30C0,{0x30BF,0x3099}},
{0x00,0x30C2,{0x30C1,0x3099}},
{0x00,0x30C5,{0x30C4,0x3099}},
{0x00,0x30C7,{0x30C6,0x3099}},
{0x00,0x30C9,{0x30C8,0x3099}},
{0x00,0x30D0,{0x30CF,0x3099}},
{0x00,0x30D1,{0x30CF,0x309A}},
{0x00,0x30D3,{0x30D2,0x3099}},
{0x00,0x30D4,{0x30D2,0x309A}},
{0x00,0x30D6,{0x30D5,0x3099}},
{0x00,0x30D7,{0x30D5,0x309A}},
{0x00,0x30D9,{0x30D8,0x3099}},
{0x00,0x30DA,{0x30D8,0x309A}},
{0x00,0x30DC,{0x30DB,0x3099}},
{0x00,0x30DD,{0x30DB,0x309A}},
{0x00,0x30F4,{0x30A6,0x3099}},
{0x00,0x30F7,{0x30EF,0x3099}},
{0x00,0x30F8,{0x30F0,0x3099}},
{0x00,0x30F9,{0x30F1,0x3099}},
{0x00,0x30FA,{0x30F2,0x3099}},
{0x00,0x30FE,{0x30FD,0x3099}},
{0x00,0xFB1D,{0x05D9,0x05B4}},
{0x00,0xFB1F,{0x05F2,0x05B7}},
{0x00,0xFB2A,{0x05E9,0x05C1}},
{0x00,0xFB2B,{0x05E9,0x05C2}},
{0x00,0xFB2C,{0xFB49,0x05C1}},
{0x00,0xFB2D,{0xFB49,0x05C2}},
{0x00,0xFB2E,{0x05D0,0x05B7}},
{0x00,0xFB2F,{0x05D0,0x05B8}},
{0x00,0xFB30,{0x05D0,0x05BC}},
{0x00,0xFB31,{0x05D1,0x05BC}},
{0x00,0xFB32,{0x05D2,0x05BC}},
{0x00,0xFB33,{0x05D3,0x05BC}},
{0x00,0xFB34,{0x05D4,0x05BC}},
{0x00,0xFB35,{0x05D5,0x05BC}},
{0x00,0xFB36,{0x05D6,0x05BC}},
{0x00,0xFB38,{0x05D8,0x05BC}},
{0x00,0xFB39,{0x05D9,0x05BC}},
{0x00,0xFB3A,{0x05DA,0x05BC}},
{0x00,0xFB3B,{0x05DB,0x05BC}},
{0x00,0xFB3C,{0x05DC,0x05BC}},
{0x00,0xFB3E,{0x05DE,0x05BC}},
{0x00,0xFB40,{0x05E0,0x05BC}},
{0x00,0xFB41,{0x05E1,0x05BC}},
{0x00,0xFB43,{0x05E3,0x05BC}},
{0x00,0xFB44,{0x05E4,0x05BC}},
{0x00,0xFB46,{0x05E6,0x05BC}},
{0x00,0xFB47,{0x05E7,0x05BC}},
{0x00,0xFB48,{0x05E8,0x05BC}},
{0x00,0xFB49,{0x05E9,0x05BC}},
{0x00,0xFB4A,{0x05EA,0x05BC}},
{0x00,0xFB4B,{0x05D5,0x05B9}},
{0x00,0xFB4C,{0x05D1,0x05BF}},
{0x00,0xFB4D,{0x05DB,0x05BF}},
{0x00,0xFB4E,{0x05E4,0x05BF}},
{0x11,0xD15E,{0xD157,0xD165}},
{0x11,0xD15F,{0xD158,0xD165}},
{0x11,0xD160,{0xD15F,0xD16E}},
{0x11,0xD161,{0xD15F,0xD16F}},
{0x11,0xD162,{0xD15F,0xD170}},
{0x11,0xD163,{0xD15F,0xD171}},
{0x11,0xD164,{0xD15F,0xD172}},
{0x11,0xD1BB,{0xD1B9,0xD165}},
{0x11,0xD1BC,{0xD1BA,0xD165}},
{0x11,0xD1BD,{0xD1BB,0xD16E}},
{0x11,0xD1BE,{0xD1BC,0xD16E}},
{0x11,0xD1BF,{0xD1BB,0xD16F}},
{0x11,0xD1C0,{0xD1BC,0xD16F}}
};

const struct UnicodeMappingDecomp UCComposites[NUCCOMPOSITES]={
{0x00,0x226E,{0x003C,0x0338}},
{0x00,0x2260,{0x003D,0x0338}},
{0x00,0x226F,{0x003E,0x0338}},
{0x00,0x00E0,{0x0061,0x0300}},
{0x00,0x00E1,{0x0061,0x0301}},
{0x00,0x00E2,{0x0061,0x0302}},
{0x00,0x00E3,{0x0061,0x0303}},
{0x00,0x0101,{0x0061,0x0304}},
{0x00,0x0103,{0x0061,0x0306}},
{0x00,0x0227,{0x0061,0x0307}},
{0x00,0x00E4,{0x0061,0x0308}},
{0x00,0x1EA3,{0x0061,0x0309}},
{0x00,0x00E5,{0x0061,0x030A}},
{0x00,0x01CE,{0x0061,0x030C}},
{0x00,0x0201,{0x0061,0x030F}},
{0x00,0x0203,{0x0061,0x0311}},
{0x00,0x1EA1,{0x0061,0x0323}},
{0x00,0x1E01,{0x0061,0x0325}},
{0x00,0x0105,{0x0061,0x0328}},
{0x00,0x1E03,{0x0062,0x0307}},
{0x00,0x1E05,{0x0062,0x0323}},
{0x00,0x1E07,{0x0062,0x0331}},
{0x00,0x0107,{0x0063,0x0301}},
{0x00,0x0109,{0x0063,0x0302}},
{0x00,0x010B,{0x0063,0x0307}},
{0x00,0x010D,{0x0063,0x030C}},
{0x00,0x00E7,{0x0063,0x0327}},
{0x00,0x1E0B,{0x0064,0x0307}},
{0x00,0x010F,{0x0064,0x030C}},
{0x00,0x1E0D,{0x0064,0x0323}},
{0x00,0x1E11,{0x0064,0x0327}},
{0x00,0x1E13,{0x0064,0x032D}},
{0x00,0x1E0F,{0x0064,0x0331}},
{0x00,0x00E8,{0x0065,0x0300}},
{0x00,0x00E9,{0x0065,0x0301}},
{0x00,0x00EA,{0x0065,0x0302}},
{0x00,0x1EBD,{0x0065,0x0303}},
{0x00,0x0113,{0x0065,0x0304}},
{0x00,0x0115,{0x0065,0x0306}},
{0x00,0x0117,{0x0065,0x0307}},
{0x00,0x00EB,{0x0065,0x0308}},
{0x00,0x1EBB,{0x0065,0x0309}},
{0x00,0x011B,{0x0065,0x030C}},
{0x00,0x0205,{0x0065,0x030F}},
{0x00,0x0207,{0x0065,0x0311}},
{0x00,0x1EB9,{0x0065,0x0323}},
{0x00,0x0229,{0x0065,0x0327}},
{0x00,0x0119,{0x0065,0x0328}},
{0x00,0x1E19,{0x0065,0x032D}},
{0x00,0x1E1B,{0x0065,0x0330}},
{0x00,0x1E1F,{0x0066,0x0307}},
{0x00,0x01F5,{0x0067,0x0301}},
{0x00,0x011D,{0x0067,0x0302}},
{0x00,0x1E21,{0x0067,0x0304}},
{0x00,0x011F,{0x0067,0x0306}},
{0x00,0x0121,{0x0067,0x0307}},
{0x00,0x01E7,{0x0067,0x030C}},
{0x00,0x0123,{0x0067,0x0327}},
{0x00,0x0125,{0x0068,0x0302}},
{0x00,0x1E23,{0x0068,0x0307}},
{0x00,0x1E27,{0x0068,0x0308}},
{0x00,0x021F,{0x0068,0x030C}},
{0x00,0x1E25,{0x0068,0x0323}},
{0x00,0x1E29,{0x0068,0x0327}},
{0x00,0x1E2B,{0x0068,0x032E}},
{0x00,0x00EC,{0x0069,0x0300}},
{0x00,0x00ED,{0x0069,0x0301}},
{0x00,0x00EE,{0x0069,0x0302}},
{0x00,0x0129,{0x0069,0x0303}},
{0x00,0x012B,{0x0069,0x0304}},
{0x00,0x012D,{0x0069,0x0306}},
{0x00,0x00EF,{0x0069,0x0308}},
{0x00,0x1EC9,{0x0069,0x0309}},
{0x00,0x01D0,{0x0069,0x030C}},
{0x00,0x0209,{0x0069,0x030F}},
{0x00,0x020B,{0x0069,0x0311}},
{0x00,0x1ECB,{0x0069,0x0323}},
{0x00,0x012F,{0x0069,0x0328}},
{0x00,0x1E2D,{0x0069,0x0330}},
{0x00,0x0135,{0x006A,0x0302}},
{0x00,0x1E31,{0x006B,0x0301}},
{0x00,0x01E9,{0x006B,0x030C}},
{0x00,0x1E33,{0x006B,0x0323}},
{0x00,0x0137,{0x006B,0x0327}},
{0x00,0x1E35,{0x006B,0x0331}},
{0x00,0x013A,{0x006C,0x0301}},
{0x00,0x013E,{0x006C,0x030C}},
{0x00,0x1E37,{0x006C,0x0323}},
{0x00,0x013C,{0x006C,0x0327}},
{0x00,0x1E3D,{0x006C,0x032D}},
{0x00,0x1E3B,{0x006C,0x0331}},
{0x00,0x1E3F,{0x006D,0x0301}},
{0x00,0x1E41,{0x006D,0x0307}},
{0x00,0x1E43,{0x006D,0x0323}},
{0x00,0x01F9,{0x006E,0x0300}},
{0x00,0x0144,{0x006E,0x0301}},
{0x00,0x00F1,{0x006E,0x0303}},
{0x00,0x1E45,{0x006E,0x0307}},
{0x00,0x0148,{0x006E,0x030C}},
{0x00,0x1E47,{0x006E,0x0323}},
{0x00,0x0146,{0x006E,0x0327}},
{0x00,0x1E4B,{0x006E,0x032D}},
{0x00,0x1E49,{0x006E,0x0331}},
{0x00,0x00F2,{0x006F,0x0300}},
{0x00,0x00F3,{0x006F,0x0301}},
{0x00,0x00F4,{0x006F,0x0302}},
{0x00,0x00F5,{0x006F,0x0303}},
{0x00,0x014D,{0x006F,0x0304}},
{0x00,0x014F,{0x006F,0x0306}},
{0x00,0x022F,{0x006F,0x0307}},
{0x00,0x00F6,{0x006F,0x0308}},
{0x00,0x1ECF,{0x006F,0x0309}},
{0x00,0x0151,{0x006F,0x030B}},
{0x00,0x01D2,{0x006F,0x030C}},
{0x00,0x020D,{0x006F,0x030F}},
{0x00,0x020F,{0x006F,0x0311}},
{0x00,0x01A1,{0x006F,0x031B}},
{0x00,0x1ECD,{0x006F,0x0323}},
{0x00,0x01EB,{0x006F,0x0328}},
{0x00,0x1E55,{0x0070,0x0301}},
{0x00,0x1E57,{0x0070,0x0307}},
{0x00,0x0155,{0x0072,0x0301}},
{0x00,0x1E59,{0x0072,0x0307}},
{0x00,0x0159,{0x0072,0x030C}},
{0x00,0x0211,{0x0072,0x030F}},
{0x00,0x0213,{0x0072,0x0311}},
{0x00,0x1E5B,{0x0072,0x0323}},
{0x00,0x0157,{0x0072,0x0327}},
{0x00,0x1E5F,{0x0072,0x0331}},
{0x00,0x015B,{0x0073,0x0301}},
{0x00,0x015D,{0x0073,0x0302}},
{0x00,0x1E61,{0x0073,0x0307}},
{0x00,0x0161,{0x0073,0x030C}},
{0x00,0x1E63,{0x0073,0x0323}},
{0x00,0x0219,{0x0073,0x0326}},
{0x00,0x015F,{0x0073,0x0327}},
{0x00,0x1E6B,{0x0074,0x0307}},
{0x00,0x0165,{0x0074,0x030C}},
{0x00,0x1E6D,{0x0074,0x0323}},
{0x00,0x021B,{0x0074,0x0326}},
{0x00,0x0163,{0x0074,0x0327}},
{0x00,0x1E71,{0x0074,0x032D}},
{0x00,0x1E6F,{0x0074,0x0331}},
{0x00,0x00F9,{0x0075,0x0300}},
{0x00,0x00FA,{0x0075,0x0301}},
{0x00,0x00FB,{0x0075,0x0302}},
{0x00,0x0169,{0x0075,0x0303}},
{0x00,0x016B,{0x0075,0x0304}},
{0x00,0x016D,{0x0075,0x0306}},
{0x00,0x00FC,{0x0075,0x0308}},
{0x00,0x1EE7,{0x0075,0x0309}},
{0x00,0x016F,{0x0075,0x030A}},
{0x00,0x0171,{0x0075,0x030B}},
{0x00,0x01D4,{0x0075,0x030C}},
{0x00,0x0215,{0x0075,0x030F}},
{0x00,0x0217,{0x0075,0x0311}},
{0x00,0x01B0,{0x0075,0x031B}},
{0x00,0x1EE5,{0x0075,0x0323}},
{0x00,0x1E73,{0x0075,0x0324}},
{0x00,0x0173,{0x0075,0x0328}},
{0x00,0x1E77,{0x0075,0x032D}},
{0x00,0x1E75,{0x0075,0x0330}},
{0x00,0x1E7D,{0x0076,0x0303}},
{0x00,0x1E7F,{0x0076,0x0323}},
{0x00,0x1E81,{0x0077,0x0300}},
{0x00,0x1E83,{0x0077,0x0301}},
{0x00,0x0175,{0x0077,0x0302}},
{0x00,0x1E87,{0x0077,0x0307}},
{0x00,0x1E85,{0x0077,0x0308}},
{0x00,0x1E89,{0x0077,0x0323}},
{0x00,0x1E8B,{0x0078,0x0307}},
{0x00,0x1E8D,{0x0078,0x0308}},
{0x00,0x1EF3,{0x0079,0x0300}},
{0x00,0x00FD,{0x0079,0x0301}},
{0x00,0x0177,{0x0079,0x0302}},
{0x00,0x1EF9,{0x0079,0x0303}},
{0x00,0x0233,{0x0079,0x0304}},
{0x00,0x1E8F,{0x0079,0x0307}},
{0x00,0x00FF,{0x0079,0x0308}},
{0x00,0x1EF7,{0x0079,0x0309}},
{0x00,0x1EF5,{0x0079,0x0323}},
{0x00,0x017A,{0x007A,0x0301}},
{0x00,0x1E91,{0x007A,0x0302}},
{0x00,0x017C,{0x007A,0x0307}},
{0x00,0x017E,{0x007A,0x030C}},
{0x00,0x1E93,{0x007A,0x0323}},
{0x00,0x1E95,{0x007A,0x0331}},
{0x00,0x1FED,{0x00A8,0x0300}},
{0x00,0x0385,{0x00A8,0x0301}},
{0x00,0x1FC1,{0x00A8,0x0342}},
{0x00,0x1EA7,{0x00E2,0x0300}},
{0x00,0x1EA5,{0x00E2,0x0301}},
{0x00,0x1EAB,{0x00E2,0x0303}},
{0x00,0x1EA9,{0x00E2,0x0309}},
{0x00,0x01DF,{0x00E4,0x0304}},
{0x00,0x01FB,{0x00E5,0x0301}},
{0x00,0x01FD,{0x00E6,0x0301}},
{0x00,0x01E3,{0x00E6,0x0304}},
{0x00,0x1E09,{0x00E7,0x0301}},
{0x00,0x1EC1,{0x00EA,0x0300}},
{0x00,0x1EBF,{0x00EA,0x0301}},
{0x00,0x1EC5,{0x00EA,0x0303}},
{0x00,0x1EC3,{0x00EA,0x0309}},
{0x00,0x1E2F,{0x00EF,0x0301}},
{0x00,0x1ED3,{0x00F4,0x0300}},
{0x00,0x1ED1,{0x00F4,0x0301}},
{0x00,0x1ED7,{0x00F4,0x0303}},
{0x00,0x1ED5,{0x00F4,0x0309}},
{0x00,0x1E4D,{0x00F5,0x0301}},
{0x00,0x022D,{0x00F5,0x0304}},
{0x00,0x1E4F,{0x00F5,0x0308}},
{0x00,0x022B,{0x00F6,0x0304}},
{0x00,0x01FF,{0x00F8,0x0301}},
{0x00,0x01DC,{0x00FC,0x0300}},
{0x00,0x01D8,{0x00FC,0x0301}},
{0x00,0x01D6,{0x00FC,0x0304}},
{0x00,0x01DA,{0x00FC,0x030C}},
{0x00,0x1EB1,{0x0103,0x0300}},
{0x00,0x1EAF,{0x0103,0x0301}},
{0x00,0x1EB5,{0x0103,0x0303}},
{0x00,0x1EB3,{0x0103,0x0309}},
{0x00,0x1E15,{0x0113,0x0300}},
{0x00,0x1E17,{0x0113,0x0301}},
{0x00,0x1E51,{0x014D,0x0300}},
{0x00,0x1E53,{0x014D,0x0301}},
{0x00,0x1E65,{0x015B,0x0307}},
{0x00,0x1E67,{0x0161,0x0307}},
{0x00,0x1E79,{0x0169,0x0301}},
{0x00,0x1E7B,{0x016B,0x0308}},
{0x00,0x1EDD,{0x01A1,0x0300}},
{0x00,0x1EDB,{0x01A1,0x0301}},
{0x00,0x1EE1,{0x01A1,0x0303}},
{0x00,0x1EDF,{0x01A1,0x0309}},
{0x00,0x1EE3,{0x01A1,0x0323}},
{0x00,0x1EEB,{0x01B0,0x0300}},
{0x00,0x1EE9,{0x01B0,0x0301}},
{0x00,0x1EEF,{0x01B0,0x0303}},
{0x00,0x1EED,{0x01B0,0x0309}},
{0x00,0x1EF1,{0x01B0,0x0323}},
{0x00,0x01ED,{0x01EB,0x0304}},
{0x00,0x01E1,{0x0227,0x0304}},
{0x00,0x1E1D,{0x0229,0x0306}},
{0x00,0x0231,{0x022F,0x0304}},
{0x00,0x01EF,{0x0292,0x030C}},
{0x00,0x0344,{0x0308,0x0301}},
{0x00,0x1F70,{0x03B1,0x0300}},
{0x00,0x03AC,{0x03B1,0x0301}},
{0x00,0x1FB1,{0x03B1,0x0304}},
{0x00,0x1FB0,{0x03B1,0x0306}},
{0x00,0x1F00,{0x03B1,0x0313}},
{0x00,0x1F01,{0x03B1,0x0314}},
{0x00,0x1F72,{0x03B5,0x0300}},
{0x00,0x03AD,{0x03B5,0x0301}},
{0x00,0x1F10,{0x03B5,0x0313}},
{0x00,0x1F11,{0x03B5,0x0314}},
{0x00,0x1F74,{0x03B7,0x0300}},
{0x00,0x03AE,{0x03B7,0x0301}},
{0x00,0x1F20,{0x03B7,0x0313}},
{0x00,0x1F21,{0x03B7,0x0314}},
{0x00,0x1F76,{0x03B9,0x0300}},
{0x00,0x03AF,{0x03B9,0x0301}},
{0x00,0x1FD1,{0x03B9,0x0304}},
{0x00,0x1FD0,{0x03B9,0x0306}},
{0x00,0x03CA,{0x03B9,0x0308}},
{0x00,0x1F30,{0x03B9,0x0313}},
{0x00,0x1F31,{0x03B9,0x0314}},
{0x00,0x1F78,{0x03BF,0x0300}},
{0x00,0x03CC,{0x03BF,0x0301}},
{0x00,0x1F40,{0x03BF,0x0313}},
{0x00,0x1F41,{0x03BF,0x0314}},
{0x00,0x1FE5,{0x03C1,0x0314}},
{0x00,0x1F7A,{0x03C5,0x0300}},
{0x00,0x03CD,{0x03C5,0x0301}},
{0x00,0x1FE1,{0x03C5,0x0304}},
{0x00,0x1FE0,{0x03C5,0x0306}},
{0x00,0x03CB,{0x03C5,0x0308}},
{0x00,0x1F51,{0x03C5,0x0314}},
{0x00,0x1F7C,{0x03C9,0x0300}},
{0x00,0x03CE,{0x03C9,0x0301}},
{0x00,0x1F60,{0x03C9,0x0313}},
{0x00,0x1F61,{0x03C9,0x0314}},
{0x00,0x04D1,{0x0430,0x0306}},
{0x00,0x04D3,{0x0430,0x0308}},
{0x00,0x0453,{0x0433,0x0301}},
{0x00,0x0450,{0x0435,0x0300}},
{0x00,0x04D7,{0x0435,0x0306}},
{0x00,0x0451,{0x0435,0x0308}},
{0x00,0x04C2,{0x0436,0x0306}},
{0x00,0x04DD,{0x0436,0x0308}},
{0x00,0x04DF,{0x0437,0x0308}},
{0x00,0x045D,{0x0438,0x0300}},
{0x00,0x04E3,{0x0438,0x0304}},
{0x00,0x0439,{0x0438,0x0306}},
{0x00,0x04E5,{0x0438,0x0308}},
{0x00,0x045C,{0x043A,0x0301}},
{0x00,0x04E7,{0x043E,0x0308}},
{0x00,0x04EF,{0x0443,0x0304}},
{0x00,0x045E,{0x0443,0x0306}},
{0x00,0x04F1,{0x0443,0x0308}},
{0x00,0x04F3,{0x0443,0x030B}},
{0x00,0x04F5,{0x0447,0x0308}},
{0x00,0x04F9,{0x044B,0x0308}},
{0x00,0x04ED,{0x044D,0x0308}},
{0x00,0x0457,{0x0456,0x0308}},
{0x00,0x0477,{0x0475,0x030F}},
{0x00,0x04DB,{0x04D9,0x0308}},
{0x00,0x04EB,{0x04E9,0x0308}},
{0x00,0x0622,{0x0627,0x0653}},
{0x00,0x0623,{0x0627,0x0654}},
{0x00,0x0625,{0x0627,0x0655}},
{0x00,0x0624,{0x0648,0x0654}},
{0x00,0x0626,{0x064A,0x0654}},
{0x00,0x06C2,{0x06C1,0x0654}},
{0x00,0x06D3,{0x06D2,0x0654}},
{0x00,0x06C0,{0x06D5,0x0654}},
{0x00,0x0929,{0x0928,0x093C}},
{0x00,0x0931,{0x0930,0x093C}},
{0x00,0x0934,{0x0933,0x093C}},
{0x00,0x09CB,{0x09C7,0x09BE}},
{0x00,0x09CC,{0x09C7,0x09D7}},
{0x00,0x0B4B,{0x0B47,0x0B3E}},
{0x00,0x0B48,{0x0B47,0x0B56}},
{0x00,0x0B4C,{0x0B47,0x0B57}},
{0x00,0x0B94,{0x0B92,0x0BD7}},
{0x00,0x0BCA,{0x0BC6,0x0BBE}},
{0x00,0x0BCC,{0x0BC6,0x0BD7}},
{0x00,0x0BCB,{0x0BC7,0x0BBE}},
{0x00,0x0C48,{0x0C46,0x0C56}},
{0x00,0x0CC0,{0x0CBF,0x0CD5}},
{0x00,0x0CCA,{0x0CC6,0x0CC2}},
{0x00,0x0CC7,{0x0CC6,0x0CD5}},
{0x00,0x0CC8,{0x0CC6,0x0CD6}},
{0x00,0x0CCB,{0x0CCA,0x0CD5}},
{0x00,0x0D4A,{0x0D46,0x0D3E}},
{0x00,0x0D4C,{0x0D46,0x0D57}},
{0x00,0x0D4B,{0x0D47,0x0D3E}},
{0x00,0x0DDA,{0x0DD9,0x0DCA}},
{0x00,0x0DDC,{0x0DD9,0x0DCF}},
{0x00,0x0DDE,{0x0DD9,0x0DDF}},
{0x00,0x0DDD,{0x0DDC,0x0DCA}},
{0x00,0x1026,{0x1025,0x102E}},
{0x00,0x1E39,{0x1E37,0x0304}},
{0x00,0x1E5D,{0x1E5B,0x0304}},
{0x00,0x1E69,{0x1E63,0x0307}},
{0x00,0x1EAD,{0x1EA1,0x0302}},
{0x00,0x1EB7,{0x1EA1,0x0306}},
{0x00,0x1EC7,{0x1EB9,0x0302}},
{0x00,0x1ED9,{0x1ECD,0x0302}},
{0x00,0x1F02,{0x1F00,0x0300}},
{0x00,0x1F04,{0x1F00,0x0301}},
{0x00,0x1F06,{0x1F00,0x0342}},
{0x00,0x1F03,{0x1F01,0x0300}},
{0x00,0x1F05,{0x1F01,0x0301}},
{0x00,0x1F07,{0x1F01,0x0342}},
{0x00,0x1F12,{0x1F10,0x0300}},
{0x00,0x1F14,{0x1F10,0x0301}},
{0x00,0x1F13,{0x1F11,0x0300}},
{0x00,0x1F15,{0x1F11,0x0301}},
{0x00,0x1F22,{0x1F20,0x0300}},
{0x00,0x1F24,{0x1F20,0x0301}},
{0x00,0x1F26,{0x1F20,0x0342}},
{0x00,0x1F23,{0x1F21,0x0300}},
{0x00,0x1F25,{0x1F21,0x0301}},
{0x00,0x1F27,{0x1F21,0x0342}},
{0x00,0x1F32,{0x1F30,0x0300}},
{0x00,0x1F34,{0x1F30,0x0301}},
{0x00,0x1F36,{0x1F30,0x0342}},
{0x00,0x1F33,{0x1F31,0x0300}},
{0x00,0x1F35,{0x1F31,0x0301}},
{0x00,0x1F37,{0x1F31,0x0342}},
{0x00,0x1F42,{0x1F40,0x0300}},
{0x00,0x1F44,{0x1F40,0x0301}},
{0x00,0x1F43,{0x1F41,0x0300}},
{0x00,0x1F45,{0x1F41,0x0301}},
{0x00,0x1F53,{0x1F51,0x0300}},
{0x00,0x1F55,{0x1F51,0x0301}},
{0x00,0x1F57,{0x1F51,0x0342}},
{0x00,0x1F62,{0x1F60,0x0300}},
{0x00,0x1F64,{0x1F60,0x0301}},
{0x00,0x1F66,{0x1F60,0x0342}},
{0x00,0x1F63,{0x1F61,0x0300}},
{0x00,0x1F65,{0x1F61,0x0301}},
{0x00,0x1F67,{0x1F61,0x0342}},
{0x00,0x1FCD,{0x1FBF,0x0300}},
{0x00,0x1FCE,{0x1FBF,0x0301}},
{0x00,0x1FCF,{0x1FBF,0x0342}},
{0x00,0x1FDD,{0x1FFE,0x0300}},
{0x00,0x1FDE,{0x1FFE,0x0301}},
{0x00,0x1FDF,{0x1FFE,0x0342}},
{0x00,0x219A,{0x2190,0x0338}},
{0x00,0x219B,{0x2192,0x0338}},
{0x00,0x21AE,{0x2194,0x0338}},
{0x00,0x21CD,{0x21D0,0x0338}},
{0x00,0x21CF,{0x21D2,0x0338}},
{0x00,0x21CE,{0x21D4,0x0338}},
{0x00,0x2204,{0x2203,0x0338}},
{0x00,0x2209,{0x2208,0x0338}},
{0x00,0x220C,{0x220B,0x0338}},
{0x00,0x2224,{0x2223,0x0338}},
{0x00,0x2226,{0x2225,0x0338}},
{0x00,0x2241,{0x223C,0x0338}},
{0x00,0x2244,{0x2243,0x0338}},
{0x00,0x2247,{0x2245,0x0338}},
{0x00,0x2249,{0x2248,0x0338}},
{0x00,0x226D,{0x224D,0x0338}},
{0x00,0x2262,{0x2261,0x0338}},
{0x00,0x2270,{0x2264,0x0338}},
{0x00,0x2271,{0x2265,0x0338}},
{0x00,0x2274,{0x2272,0x0338}},
{0x00,0x2275,{0x2273,0x0338}},
{0x00,0x2278,{0x2276,0x0338}},
{0x00,0x2279,{0x2277,0x0338}},
{0x00,0x2280,{0x227A,0x0338}},
{0x00,0x2281,{0x227B,0x0338}},
{0x00,0x22E0,{0x227C,0x0338}},
{0x00,0x22E1,{0x227D,0x0338}},
{0x00,0x2284,{0x2282,0x0338}},
{0x00,0x2285,{0x2283,0x0338}},
{0x00,0x2288,{0x2286,0x0338}},
{0x00,0x2289,{0x2287,0x0338}},
{0x00,0x22E2,{0x2291,0x0338}},
{0x00,0x22E3,{0x2292,0x0338}},
{0x00,0x22AC,{0x22A2,0x0338}},
{0x00,0x22AD,{0x22A8,0x0338}},
{0x00,0x22AE,{0x22A9,0x0338}},
{0x00,0x22AF,{0x22AB,0x0338}},
{0x00,0x22EA,{0x22B2,0x0338}},
{0x00,0x22EB,{0x22B3,0x0338}},
{0x00,0x22EC,{0x22B4,0x0338}},
{0x00,0x22ED,{0x22B5,0x0338}},
{0x00,0x2ADC,{0x2ADD,0x0338}},
{0x00,0x3094,{0x3046,0x3099}},
{0x00,0x304C,{0x304B,0x3099}},
{0x00,0x304E,{0x304D,0x3099}},
{0x00,0x3050,{0x304F,0x3099}},
{0x00,0x3052,{0x3051,0x3099}},
{0x00,0x3054,{0x3053,0x3099}},
{0x00,0x3056,{0x3055,0x3099}},
{0x00,0x3058,{0x3057,0x3099}},
{0x00,0x305A,{0x3059,0x3099}},
{0x00,0x305C,{0x305B,0x3099}},
{0x00,0x305E,{0x305D,0x3099}},
{0x00,0x3060,{0x305F,0x3099}},
{0x00,0x3062,{0x3061,0x3099}},
{0x00,0x3065,{0x3064,0x3099}},
{0x00,0x3067,{0x3066,0x3099}},
{0x00,0x3069,{0x3068,0x3099}},
{0x00,0x3070,{0x306F,0x3099}},
{0x00,0x3071,{0x306F,0x309A}},
{0x00,0x3073,{0x3072,0x3099}},
{0x00,0x3074,{0x3072,0x309A}},
{0x00,0x3076,{0x3075,0x3099}},
{0x00,0x3077,{0x3075,0x309A}},
{0x00,0x3079,{0x3078,0x3099}},
{0x00,0x307A,{0x3078,0x309A}},
{0x00,0x307C,{0x307B,0x3099}},
{0x00,0x307D,{0x307B,0x309A}},
{0x00,0x309E,{0x309D,0x3099}},
{0x00,0x30F4,{0x30A6,0x3099}},
{0x00,0x30AC,{0x30AB,0x3099}},
{0x00,0x30AE,{0x30AD,0x3099}},
{0x00,0x30B0,{0x30AF,0x3099}},
{0x00,0x30B2,{0x30B1,0x3099}},
{0x00,0x30B4,{0x30B3,0x3099}},
{0x00,0x30B6,{0x30B5,0x3099}},
{0x00,0x30B8,{0x30B7,0x3099}},
{0x00,0x30BA,{0x30B9,0x3099}},
{0x00,0x30BC,{0x30BB,0x3099}},
{0x00,0x30BE,{0x30BD,0x3099}},
{0x00,0x30C0,{0x30BF,0x3099}},
{0x00,0x30C2,{0x30C1,0x3099}},
{0x00,0x30C5,{0x30C4,0x3099}},
{0x00,0x30C7,{0x30C6,0x3099}},
{0x00,0x30C9,{0x30C8,0x3099}},
{0x00,0x30D0,{0x30CF,0x3099}},
{0x00,0x30D1,{0x30CF,0x309A}},
{0x00,0x30D3,{0x30D2,0x3099}},
{0x00,0x30D4,{0x30D2,0x309A}},
{0x00,0x30D6,{0x30D5,0x3099}},
{0x00,0x30D7,{0x30D5,0x309A}},
{0x00,0x30D9,{0x30D8,0x3099}},
{0x00,0x30DA,{0x30D8,0x309A}},
{0x00,0x30DC,{0x30DB,0x3099}},
{0x00,0x30DD,{0x30DB,0x309A}},
{0x00,0x30F7,{0x30EF,0x3099}},
{0x00,0x30F8,{0x30F0,0x3099}},
{0x00,0x30F9,{0x30F1,0x3099}},
{0x00,0x30FA,{0x30F2,0x3099}},
{0x00,0x30FE,{0x30FD,0x3099}}
};

const struct UnicodeMappingSingleton UCSingletons[NUCSINGLETONS]={
{0x00,0x0340,0x0300},
{0x00,0x0341,0x0301},
{0x00,0x0343,0x0313},
{0x00,0x0374,0x02B9},
{0x00,0x037E,0x003B},
{0x00,0x0387,0x00B7},
{0x00,0x1F71,0x03AC},
{0x00,0x1F73,0x03AD},
{0x00,0x1F75,0x03AE},
{0x00,0x1F77,0x03AF},
{0x00,0x1F79,0x03CC},
{0x00,0x1F7B,0x03CD},
{0x00,0x1F7D,0x03CE},
{0x00,0x1FEE,0x0385},
{0x00,0x1FEF,0x0060},
{0x00,0x1FFD,0x00B4},
{0x00,0x2000,0x2002},
{0x00,0x2001,0x2003},
{0x00,0x2329,0x3008},
{0x00,0x232A,0x3009},
{0x00,0xF900,0x8C48},
{0x00,0xF901,0x66F4},
{0x00,0xF902,0x8ECA},
{0x00,0xF903,0x8CC8},
{0x00,0xF904,0x6ED1},
{0x00,0xF905,0x4E32},
{0x00,0xF906,0x53E5},
{0x00,0xF907,0x9F9C},
{0x00,0xF908,0x9F9C},
{0x00,0xF909,0x5951},
{0x00,0xF90A,0x91D1},
{0x00,0xF90B,0x5587},
{0x00,0xF90C,0x5948},
{0x00,0xF90D,0x61F6},
{0x00,0xF90E,0x7669},
{0x00,0xF90F,0x7F85},
{0x00,0xF910,0x863F},
{0x00,0xF911,0x87BA},
{0x00,0xF912,0x88F8},
{0x00,0xF913,0x908F},
{0x00,0xF914,0x6A02},
{0x00,0xF915,0x6D1B},
{0x00,0xF916,0x70D9},
{0x00,0xF917,0x73DE},
{0x00,0xF918,0x843D},
{0x00,0xF919,0x916A},
{0x00,0xF91A,0x99F1},
{0x00,0xF91B,0x4E82},
{0x00,0xF91C,0x5375},
{0x00,0xF91D,0x6B04},
{0x00,0xF91E,0x721B},
{0x00,0xF91F,0x862D},
{0x00,0xF920,0x9E1E},
{0x00,0xF921,0x5D50},
{0x00,0xF922,0x6FEB},
{0x00,0xF923,0x85CD},
{0x00,0xF924,0x8964},
{0x00,0xF925,0x62C9},
{0x00,0xF926,0x81D8},
{0x00,0xF927,0x881F},
{0x00,0xF928,0x5ECA},
{0x00,0xF929,0x6717},
{0x00,0xF92A,0x6D6A},
{0x00,0xF92B,0x72FC},
{0x00,0xF92C,0x90CE},
{0x00,0xF92D,0x4F86},
{0x00,0xF92E,0x51B7},
{0x00,0xF92F,0x52DE},
{0x00,0xF930,0x64C4},
{0x00,0xF931,0x6AD3},
{0x00,0xF932,0x7210},
{0x00,0xF933,0x76E7},
{0x00,0xF934,0x8001},
{0x00,0xF935,0x8606},
{0x00,0xF936,0x865C},
{0x00,0xF937,0x8DEF},
{0x00,0xF938,0x9732},
{0x00,0xF939,0x9B6F},
{0x00,0xF93A,0x9DFA},
{0x00,0xF93B,0x788C},
{0x00,0xF93C,0x797F},
{0x00,0xF93D,0x7DA0},
{0x00,0xF93E,0x83C9},
{0x00,0xF93F,0x9304},
{0x00,0xF940,0x9E7F},
{0x00,0xF941,0x8AD6},
{0x00,0xF942,0x58DF},
{0x00,0xF943,0x5F04},
{0x00,0xF944,0x7C60},
{0x00,0xF945,0x807E},
{0x00,0xF946,0x7262},
{0x00,0xF947,0x78CA},
{0x00,0xF948,0x8CC2},
{0x00,0xF949,0x96F7},
{0x00,0xF94A,0x58D8},
{0x00,0xF94B,0x5C62},
{0x00,0xF94C,0x6A13},
{0x00,0xF94D,0x6DDA},
{0x00,0xF94E,0x6F0F},
{0x00,0xF94F,0x7D2F},
{0x00,0xF950,0x7E37},
{0x00,0xF951,0x964B},
{0x00,0xF952,0x52D2},
{0x00,0xF953,0x808B},
{0x00,0xF954,0x51DC},
{0x00,0xF955,0x51CC},
{0x00,0xF956,0x7A1C},
{0x00,0xF957,0x7DBE},
{0x00,0xF958,0x83F1},
{0x00,0xF959,0x9675},
{0x00,0xF95A,0x8B80},
{0x00,0xF95B,0x62CF},
{0x00,0xF95C,0x6A02},
{0x00,0xF95D,0x8AFE},
{0x00,0xF95E,0x4E39},
{0x00,0xF95F,0x5BE7},
{0x00,0xF960,0x6012},
{0x00,0xF961,0x7387},
{0x00,0xF962,0x7570},
{0x00,0xF963,0x5317},
{0x00,0xF964,0x78FB},
{0x00,0xF965,0x4FBF},
{0x00,0xF966,0x5FA9},
{0x00,0xF967,0x4E0D},
{0x00,0xF968,0x6CCC},
{0x00,0xF969,0x6578},
{0x00,0xF96A,0x7D22},
{0x00,0xF96B,0x53C3},
{0x00,0xF96C,0x585E},
{0x00,0xF96D,0x7701},
{0x00,0xF96E,0x8449},
{0x00,0xF96F,0x8AAA},
{0x00,0xF970,0x6BBA},
{0x00,0xF971,0x8FB0},
{0x00,0xF972,0x6C88},
{0x00,0xF973,0x62FE},
{0x00,0xF974,0x82E5},
{0x00,0xF975,0x63A0},
{0x00,0xF976,0x7565},
{0x00,0xF977,0x4EAE},
{0x00,0xF978,0x5169},
{0x00,0xF979,0x51C9},
{0x00,0xF97A,0x6881},
{0x00,0xF97B,0x7CE7},
{0x00,0xF97C,0x826F},
{0x00,0xF97D,0x8AD2},
{0x00,0xF97E,0x91CF},
{0x00,0xF97F,0x52F5},
{0x00,0xF980,0x5442},
{0x00,0xF981,0x5973},
{0x00,0xF982,0x5EEC},
{0x00,0xF983,0x65C5},
{0x00,0xF984,0x6FFE},
{0x00,0xF985,0x792A},
{0x00,0xF986,0x95AD},
{0x00,0xF987,0x9A6A},
{0x00,0xF988,0x9E97},
{0x00,0xF989,0x9ECE},
{0x00,0xF98A,0x529B},
{0x00,0xF98B,0x66C6},
{0x00,0xF98C,0x6B77},
{0x00,0xF98D,0x8F62},
{0x00,0xF98E,0x5E74},
{0x00,0xF98F,0x6190},
{0x00,0xF990,0x6200},
{0x00,0xF991,0x649A},
{0x00,0xF992,0x6F23},
{0x00,0xF993,0x7149},
{0x00,0xF994,0x7489},
{0x00,0xF995,0x79CA},
{0x00,0xF996,0x7DF4},
{0x00,0xF997,0x806F},
{0x00,0xF998,0x8F26},
{0x00,0xF999,0x84EE},
{0x00,0xF99A,0x9023},
{0x00,0xF99B,0x934A},
{0x00,0xF99C,0x5217},
{0x00,0xF99D,0x52A3},
{0x00,0xF99E,0x54BD},
{0x00,0xF99F,0x70C8},
{0x00,0xF9A0,0x88C2},
{0x00,0xF9A1,0x8AAA},
{0x00,0xF9A2,0x5EC9},
{0x00,0xF9A3,0x5FF5},
{0x00,0xF9A4,0x637B},
{0x00,0xF9A5,0x6BAE},
{0x00,0xF9A6,0x7C3E},
{0x00,0xF9A7,0x7375},
{0x00,0xF9A8,0x4EE4},
{0x00,0xF9A9,0x56F9},
{0x00,0xF9AA,0x5BE7},
{0x00,0xF9AB,0x5DBA},
{0x00,0xF9AC,0x601C},
{0x00,0xF9AD,0x73B2},
{0x00,0xF9AE,0x7469},
{0x00,0xF9AF,0x7F9A},
{0x00,0xF9B0,0x8046},
{0x00,0xF9B1,0x9234},
{0x00,0xF9B2,0x96F6},
{0x00,0xF9B3,0x9748},
{0x00,0xF9B4,0x9818},
{0x00,0xF9B5,0x4F8B},
{0x00,0xF9B6,0x79AE},
{0x00,0xF9B7,0x91B4},
{0x00,0xF9B8,0x96B8},
{0x00,0xF9B9,0x60E1},
{0x00,0xF9BA,0x4E86},
{0x00,0xF9BB,0x50DA},
{0x00,0xF9BC,0x5BEE},
{0x00,0xF9BD,0x5C3F},
{0x00,0xF9BE,0x6599},
{0x00,0xF9BF,0x6A02},
{0x00,0xF9C0,0x71CE},
{0x00,0xF9C1,0x7642},
{0x00,0xF9C2,0x84FC},
{0x00,0xF9C3,0x907C},
{0x00,0xF9C4,0x9F8D},
{0x00,0xF9C5,0x6688},
{0x00,0xF9C6,0x962E},
{0x00,0xF9C7,0x5289},
{0x00,0xF9C8,0x677B},
{0x00,0xF9C9,0x67F3},
{0x00,0xF9CA,0x6D41},
{0x00,0xF9CB,0x6E9C},
{0x00,0xF9CC,0x7409},
{0x00,0xF9CD,0x7559},
{0x00,0xF9CE,0x786B},
{0x00,0xF9CF,0x7D10},
{0x00,0xF9D0,0x985E},
{0x00,0xF9D1,0x516D},
{0x00,0xF9D2,0x622E},
{0x00,0xF9D3,0x9678},
{0x00,0xF9D4,0x502B},
{0x00,0xF9D5,0x5D19},
{0x00,0xF9D6,0x6DEA},
{0x00,0xF9D7,0x8F2A},
{0x00,0xF9D8,0x5F8B},
{0x00,0xF9D9,0x6144},
{0x00,0xF9DA,0x6817},
{0x00,0xF9DB,0x7387},
{0x00,0xF9DC,0x9686},
{0x00,0xF9DD,0x5229},
{0x00,0xF9DE,0x540F},
{0x00,0xF9DF,0x5C65},
{0x00,0xF9E0,0x6613},
{0x00,0xF9E1,0x674E},
{0x00,0xF9E2,0x68A8},
{0x00,0xF9E3,0x6CE5},
{0x00,0xF9E4,0x7406},
{0x00,0xF9E5,0x75E2},
{0x00,0xF9E6,0x7F79},
{0x00,0xF9E7,0x88CF},
{0x00,0xF9E8,0x88E1},
{0x00,0xF9E9,0x91CC},
{0x00,0xF9EA,0x96E2},
{0x00,0xF9EB,0x533F},
{0x00,0xF9EC,0x6EBA},
{0x00,0xF9ED,0x541D},
{0x00,0xF9EE,0x71D0},
{0x00,0xF9EF,0x7498},
{0x00,0xF9F0,0x85FA},
{0x00,0xF9F1,0x96A3},
{0x00,0xF9F2,0x9C57},
{0x00,0xF9F3,0x9E9F},
{0x00,0xF9F4,0x6797},
{0x00,0xF9F5,0x6DCB},
{0x00,0xF9F6,0x81E8},
{0x00,0xF9F7,0x7ACB},
{0x00,0xF9F8,0x7B20},
{0x00,0xF9F9,0x7C92},
{0x00,0xF9FA,0x72C0},
{0x00,0xF9FB,0x7099},
{0x00,0xF9FC,0x8B58},
{0x00,0xF9FD,0x4EC0},
{0x00,0xF9FE,0x8336},
{0x00,0xF9FF,0x523A},
{0x00,0xFA00,0x5207},
{0x00,0xFA01,0x5EA6},
{0x00,0xFA02,0x62D3},
{0x00,0xFA03,0x7CD6},
{0x00,0xFA04,0x5B85},
{0x00,0xFA05,0x6D1E},
{0x00,0xFA06,0x66B4},
{0x00,0xFA07,0x8F3B},
{0x00,0xFA08,0x884C},
{0x00,0xFA09,0x964D},
{0x00,0xFA0A,0x898B},
{0x00,0xFA0B,0x5ED3},
{0x00,0xFA0C,0x5140},
{0x00,0xFA0D,0x55C0},
{0x00,0xFA10,0x585A},
{0x00,0xFA12,0x6674},
{0x00,0xFA15,0x51DE},
{0x00,0xFA16,0x732A},
{0x00,0xFA17,0x76CA},
{0x00,0xFA18,0x793C},
{0x00,0xFA19,0x795E},
{0x00,0xFA1A,0x7965},
{0x00,0xFA1B,0x798F},
{0x00,0xFA1C,0x9756},
{0x00,0xFA1D,0x7CBE},
{0x00,0xFA1E,0x7FBD},
{0x00,0xFA20,0x8612},
{0x00,0xFA22,0x8AF8},
{0x00,0xFA25,0x9038},
{0x00,0xFA26,0x90FD},
{0x00,0xFA2A,0x98EF},
{0x00,0xFA2B,0x98FC},
{0x00,0xFA2C,0x9928},
{0x00,0xFA2D,0x9DB4},
{0x00,0xFA30,0x4FAE},
{0x00,0xFA31,0x50E7},
{0x00,0xFA32,0x514D},
{0x00,0xFA33,0x52C9},
{0x00,0xFA34,0x52E4},
{0x00,0xFA35,0x5351},
{0x00,0xFA36,0x559D},
{0x00,0xFA37,0x5606},
{0x00,0xFA38,0x5668},
{0x00,0xFA39,0x5840},
{0x00,0xFA3A,0x58A8},
{0x00,0xFA3B,0x5C64},
{0x00,0xFA3C,0x5C6E},
{0x00,0xFA3D,0x6094},
{0x00,0xFA3E,0x6168},
{0x00,0xFA3F,0x618E},
{0x00,0xFA40,0x61F2},
{0x00,0xFA41,0x654F},
{0x00,0xFA42,0x65E2},
{0x00,0xFA43,0x6691},
{0x00,0xFA44,0x6885},
{0x00,0xFA45,0x6D77},
{0x00,0xFA46,0x6E1A},
{0x00,0xFA47,0x6F22},
{0x00,0xFA48,0x716E},
{0x00,0xFA49,0x722B},
{0x00,0xFA4A,0x7422},
{0x00,0xFA4B,0x7891},
{0x00,0xFA4C,0x793E},
{0x00,0xFA4D,0x7949},
{0x00,0xFA4E,0x7948},
{0x00,0xFA4F,0x7950},
{0x00,0xFA50,0x7956},
{0x00,0xFA51,0x795D},
{0x00,0xFA52,0x798D},
{0x00,0xFA53,0x798E},
{0x00,0xFA54,0x7A40},
{0x00,0xFA55,0x7A81},
{0x00,0xFA56,0x7BC0},
{0x00,0xFA57,0x7DF4},
{0x00,0xFA58,0x7E09},
{0x00,0xFA59,0x7E41},
{0x00,0xFA5A,0x7F72},
{0x00,0xFA5B,0x8005},
{0x00,0xFA5C,0x81ED},
{0x00,0xFA5D,0x8279},
{0x00,0xFA5E,0x8279},
{0x00,0xFA5F,0x8457},
{0x00,0xFA60,0x8910},
{0x00,0xFA61,0x8996},
{0x00,0xFA62,0x8B01},
{0x00,0xFA63,0x8B39},
{0x00,0xFA64,0x8CD3},
{0x00,0xFA65,0x8D08},
{0x00,0xFA66,0x8FB6},
{0x00,0xFA67,0x9038},
{0x00,0xFA68,0x96E3},
{0x00,0xFA69,0x97FF},
{0x00,0xFA6A,0x983B},
{0x00,0xFA70,0x4E26},
{0x00,0xFA71,0x51B5},
{0x00,0xFA72,0x5168},
{0x00,0xFA73,0x4F80},
{0x00,0xFA74,0x5145},
{0x00,0xFA75,0x5180},
{0x00,0xFA76,0x52C7},
{0x00,0xFA77,0x52FA},
{0x00,0xFA78,0x559D},
{0x00,0xFA79,0x5555},
{0x00,0xFA7A,0x5599},
{0x00,0xFA7B,0x55E2},
{0x00,0xFA7C,0x585A},
{0x00,0xFA7D,0x58B3},
{0x00,0xFA7E,0x5944},
{0x00,0xFA7F,0x5954},
{0x00,0xFA80,0x5A62},
{0x00,0xFA81,0x5B28},
{0x00,0xFA82,0x5ED2},
{0x00,0xFA83,0x5ED9},
{0x00,0xFA84,0x5F69},
{0x00,0xFA85,0x5FAD},
{0x00,0xFA86,0x60D8},
{0x00,0xFA87,0x614E},
{0x00,0xFA88,0x6108},
{0x00,0xFA89,0x618E},
{0x00,0xFA8A,0x6160},
{0x00,0xFA8B,0x61F2},
{0x00,0xFA8C,0x6234},
{0x00,0xFA8D,0x63C4},
{0x00,0xFA8E,0x641C},
{0x00,0xFA8F,0x6452},
{0x00,0xFA90,0x6556},
{0x00,0xFA91,0x6674},
{0x00,0xFA92,0x6717},
{0x00,0xFA93,0x671B},
{0x00,0xFA94,0x6756},
{0x00,0xFA95,0x6B79},
{0x00,0xFA96,0x6BBA},
{0x00,0xFA97,0x6D41},
{0x00,0xFA98,0x6EDB},
{0x00,0xFA99,0x6ECB},
{0x00,0xFA9A,0x6F22},
{0x00,0xFA9B,0x701E},
{0x00,0xFA9C,0x716E},
{0x00,0xFA9D,0x77A7},
{0x00,0xFA9E,0x7235},
{0x00,0xFA9F,0x72AF},
{0x00,0xFAA0,0x732A},
{0x00,0xFAA1,0x7471},
{0x00,0xFAA2,0x7506},
{0x00,0xFAA3,0x753B},
{0x00,0xFAA4,0x761D},
{0x00,0xFAA5,0x761F},
{0x00,0xFAA6,0x76CA},
{0x00,0xFAA7,0x76DB},
{0x00,0xFAA8,0x76F4},
{0x00,0xFAA9,0x774A},
{0x00,0xFAAA,0x7740},
{0x00,0xFAAB,0x78CC},
{0x00,0xFAAC,0x7AB1},
{0x00,0xFAAD,0x7BC0},
{0x00,0xFAAE,0x7C7B},
{0x00,0xFAAF,0x7D5B},
{0x00,0xFAB0,0x7DF4},
{0x00,0xFAB1,0x7F3E},
{0x00,0xFAB2,0x8005},
{0x00,0xFAB3,0x8352},
{0x00,0xFAB4,0x83EF},
{0x00,0xFAB5,0x8779},
{0x00,0xFAB6,0x8941},
{0x00,0xFAB7,0x8986},
{0x00,0xFAB8,0x8996},
{0x00,0xFAB9,0x8ABF},
{0x00,0xFABA,0x8AF8},
{0x00,0xFABB,0x8ACB},
{0x00,0xFABC,0x8B01},
{0x00,0xFABD,0x8AFE},
{0x00,0xFABE,0x8AED},
{0x00,0xFABF,0x8B39},
{0x00,0xFAC0,0x8B8A},
{0x00,0xFAC1,0x8D08},
{0x00,0xFAC2,0x8F38},
{0x00,0xFAC3,0x9072},
{0x00,0xFAC4,0x9199},
{0x00,0xFAC5,0x9276},
{0x00,0xFAC6,0x967C},
{0x00,0xFAC7,0x96E3},
{0x00,0xFAC8,0x9756},
{0x00,0xFAC9,0x97DB},
{0x00,0xFACA,0x97FF},
{0x00,0xFACB,0x980B},
{0x00,0xFACC,0x983B},
{0x00,0xFACD,0x9B12},
{0x00,0xFACE,0x9F9C},
{0x00,0xFACF,0x284A},
{0x00,0xFAD0,0x2844},
{0x00,0xFAD1,0x33D5},
{0x00,0xFAD2,0x3B9D},
{0x00,0xFAD3,0x4018},
{0x00,0xFAD4,0x4039},
{0x00,0xFAD5,0x5249},
{0x00,0xFAD6,0x5CD0},
{0x00,0xFAD7,0x7ED3},
{0x00,0xFAD8,0x9F43},
{0x00,0xFAD9,0x9F8E},
{0x02,0xF800,0x4E3D},
{0x02,0xF801,0x4E38},
{0x02,0xF802,0x4E41},
{0x22,0xF803,0x0122},
{0x02,0xF804,0x4F60},
{0x02,0xF805,0x4FAE},
{0x02,0xF806,0x4FBB},
{0x02,0xF807,0x5002},
{0x02,0xF808,0x507A},
{0x02,0xF809,0x5099},
{0x02,0xF80A,0x50E7},
{0x02,0xF80B,0x50CF},
{0x02,0xF80C,0x349E},
{0x22,0xF80D,0x063A},
{0x02,0xF80E,0x514D},
{0x02,0xF80F,0x5154},
{0x02,0xF810,0x5164},
{0x02,0xF811,0x5177},
{0x22,0xF812,0x051C},
{0x02,0xF813,0x34B9},
{0x02,0xF814,0x5167},
{0x02,0xF815,0x518D},
{0x22,0xF816,0x054B},
{0x02,0xF817,0x5197},
{0x02,0xF818,0x51A4},
{0x02,0xF819,0x4ECC},
{0x02,0xF81A,0x51AC},
{0x02,0xF81B,0x51B5},
{0x22,0xF81C,0x91DF},
{0x02,0xF81D,0x51F5},
{0x02,0xF81E,0x5203},
{0x02,0xF81F,0x34DF},
{0x02,0xF820,0x523B},
{0x02,0xF821,0x5246},
{0x02,0xF822,0x5272},
{0x02,0xF823,0x5277},
{0x02,0xF824,0x3515},
{0x02,0xF825,0x52C7},
{0x02,0xF826,0x52C9},
{0x02,0xF827,0x52E4},
{0x02,0xF828,0x52FA},
{0x02,0xF829,0x5305},
{0x02,0xF82A,0x5306},
{0x02,0xF82B,0x5317},
{0x02,0xF82C,0x5349},
{0x02,0xF82D,0x5351},
{0x02,0xF82E,0x535A},
{0x02,0xF82F,0x5373},
{0x02,0xF830,0x537D},
{0x02,0xF831,0x537F},
{0x02,0xF832,0x537F},
{0x02,0xF833,0x537F},
{0x22,0xF834,0x0A2C},
{0x02,0xF835,0x7070},
{0x02,0xF836,0x53CA},
{0x02,0xF837,0x53DF},
{0x22,0xF838,0x0B63},
{0x02,0xF839,0x53EB},
{0x02,0xF83A,0x53F1},
{0x02,0xF83B,0x5406},
{0x02,0xF83C,0x549E},
{0x02,0xF83D,0x5438},
{0x02,0xF83E,0x5448},
{0x02,0xF83F,0x5468},
{0x02,0xF840,0x54A2},
{0x02,0xF841,0x54F6},
{0x02,0xF842,0x5510},
{0x02,0xF843,0x5553},
{0x02,0xF844,0x5563},
{0x02,0xF845,0x5584},
{0x02,0xF846,0x5584},
{0x02,0xF847,0x5599},
{0x02,0xF848,0x55AB},
{0x02,0xF849,0x55B3},
{0x02,0xF84A,0x55C2},
{0x02,0xF84B,0x5716},
{0x02,0xF84C,0x5606},
{0x02,0xF84D,0x5717},
{0x02,0xF84E,0x5651},
{0x02,0xF84F,0x5674},
{0x02,0xF850,0x5207},
{0x02,0xF851,0x58EE},
{0x02,0xF852,0x57CE},
{0x02,0xF853,0x57F4},
{0x02,0xF854,0x580D},
{0x02,0xF855,0x578B},
{0x02,0xF856,0x5832},
{0x02,0xF857,0x5831},
{0x02,0xF858,0x58AC},
{0x22,0xF859,0x14E4},
{0x02,0xF85A,0x58F2},
{0x02,0xF85B,0x58F7},
{0x02,0xF85C,0x5906},
{0x02,0xF85D,0x591A},
{0x02,0xF85E,0x5922},
{0x02,0xF85F,0x5962},
{0x22,0xF860,0x16A8},
{0x22,0xF861,0x16EA},
{0x02,0xF862,0x59EC},
{0x02,0xF863,0x5A1B},
{0x02,0xF864,0x5A27},
{0x02,0xF865,0x59D8},
{0x02,0xF866,0x5A66},
{0x02,0xF867,0x36EE},
{0x02,0xF868,0x36FC},
{0x02,0xF869,0x5B08},
{0x02,0xF86A,0x5B3E},
{0x02,0xF86B,0x5B3E},
{0x22,0xF86C,0x19C8},
{0x02,0xF86D,0x5BC3},
{0x02,0xF86E,0x5BD8},
{0x02,0xF86F,0x5BE7},
{0x02,0xF870,0x5BF3},
{0x22,0xF871,0x1B18},
{0x02,0xF872,0x5BFF},
{0x02,0xF873,0x5C06},
{0x02,0xF874,0x5F53},
{0x02,0xF875,0x5C22},
{0x02,0xF876,0x3781},
{0x02,0xF877,0x5C60},
{0x02,0xF878,0x5C6E},
{0x02,0xF879,0x5CC0},
{0x02,0xF87A,0x5C8D},
{0x22,0xF87B,0x1DE4},
{0x02,0xF87C,0x5D43},
{0x22,0xF87D,0x1DE6},
{0x02,0xF87E,0x5D6E},
{0x02,0xF87F,0x5D6B},
{0x02,0xF880,0x5D7C},
{0x02,0xF881,0x5DE1},
{0x02,0xF882,0x5DE2},
{0x02,0xF883,0x382F},
{0x02,0xF884,0x5DFD},
{0x02,0xF885,0x5E28},
{0x02,0xF886,0x5E3D},
{0x02,0xF887,0x5E69},
{0x02,0xF888,0x3862},
{0x22,0xF889,0x2183},
{0x02,0xF88A,0x387C},
{0x02,0xF88B,0x5EB0},
{0x02,0xF88C,0x5EB3},
{0x02,0xF88D,0x5EB6},
{0x02,0xF88E,0x5ECA},
{0x22,0xF88F,0xA392},
{0x02,0xF890,0x5EFE},
{0x22,0xF891,0x2331},
{0x22,0xF892,0x2331},
{0x02,0xF893,0x8201},
{0x02,0xF894,0x5F22},
{0x02,0xF895,0x5F22},
{0x02,0xF896,0x38C7},
{0x22,0xF897,0x32B8},
{0x22,0xF898,0x61DA},
{0x02,0xF899,0x5F62},
{0x02,0xF89A,0x5F6B},
{0x02,0xF89B,0x38E3},
{0x02,0xF89C,0x5F9A},
{0x02,0xF89D,0x5FCD},
{0x02,0xF89E,0x5FD7},
{0x02,0xF89F,0x5FF9},
{0x02,0xF8A0,0x6081},
{0x02,0xF8A1,0x393A},
{0x02,0xF8A2,0x391C},
{0x02,0xF8A3,0x6094},
{0x22,0xF8A4,0x26D4},
{0x02,0xF8A5,0x60C7},
{0x02,0xF8A6,0x6148},
{0x02,0xF8A7,0x614C},
{0x02,0xF8A8,0x614E},
{0x02,0xF8A9,0x614C},
{0x02,0xF8AA,0x617A},
{0x02,0xF8AB,0x618E},
{0x02,0xF8AC,0x61B2},
{0x02,0xF8AD,0x61A4},
{0x02,0xF8AE,0x61AF},
{0x02,0xF8AF,0x61DE},
{0x02,0xF8B0,0x61F2},
{0x02,0xF8B1,0x61F6},
{0x02,0xF8B2,0x6210},
{0x02,0xF8B3,0x621B},
{0x02,0xF8B4,0x625D},
{0x02,0xF8B5,0x62B1},
{0x02,0xF8B6,0x62D4},
{0x02,0xF8B7,0x6350},
{0x22,0xF8B8,0x2B0C},
{0x02,0xF8B9,0x633D},
{0x02,0xF8BA,0x62FC},
{0x02,0xF8BB,0x6368},
{0x02,0xF8BC,0x6383},
{0x02,0xF8BD,0x63E4},
{0x22,0xF8BE,0x2BF1},
{0x02,0xF8BF,0x6422},
{0x02,0xF8C0,0x63C5},
{0x02,0xF8C1,0x63A9},
{0x02,0xF8C2,0x3A2E},
{0x02,0xF8C3,0x6469},
{0x02,0xF8C4,0x647E},
{0x02,0xF8C5,0x649D},
{0x02,0xF8C6,0x6477},
{0x02,0xF8C7,0x3A6C},
{0x02,0xF8C8,0x654F},
{0x02,0xF8C9,0x656C},
{0x22,0xF8CA,0x300A},
{0x02,0xF8CB,0x65E3},
{0x02,0xF8CC,0x66F8},
{0x02,0xF8CD,0x6649},
{0x02,0xF8CE,0x3B19},
{0x02,0xF8CF,0x6691},
{0x02,0xF8D0,0x3B08},
{0x02,0xF8D1,0x3AE4},
{0x02,0xF8D2,0x5192},
{0x02,0xF8D3,0x5195},
{0x02,0xF8D4,0x6700},
{0x02,0xF8D5,0x669C},
{0x02,0xF8D6,0x80AD},
{0x02,0xF8D7,0x43D9},
{0x02,0xF8D8,0x6717},
{0x02,0xF8D9,0x671B},
{0x02,0xF8DA,0x6721},
{0x02,0xF8DB,0x675E},
{0x02,0xF8DC,0x6753},
{0x22,0xF8DD,0x33C3},
{0x02,0xF8DE,0x3B49},
{0x02,0xF8DF,0x67FA},
{0x02,0xF8E0,0x6785},
{0x02,0xF8E1,0x6852},
{0x02,0xF8E2,0x6885},
{0x22,0xF8E3,0x346D},
{0x02,0xF8E4,0x688E},
{0x02,0xF8E5,0x681F},
{0x02,0xF8E6,0x6914},
{0x02,0xF8E7,0x3B9D},
{0x02,0xF8E8,0x6942},
{0x02,0xF8E9,0x69A3},
{0x02,0xF8EA,0x69EA},
{0x02,0xF8EB,0x6AA8},
{0x22,0xF8EC,0x36A3},
{0x02,0xF8ED,0x6ADB},
{0x02,0xF8EE,0x3C18},
{0x02,0xF8EF,0x6B21},
{0x22,0xF8F0,0x38A7},
{0x02,0xF8F1,0x6B54},
{0x02,0xF8F2,0x3C4E},
{0x02,0xF8F3,0x6B72},
{0x02,0xF8F4,0x6B9F},
{0x02,0xF8F5,0x6BBA},
{0x02,0xF8F6,0x6BBB},
{0x22,0xF8F7,0x3A8D},
{0x22,0xF8F8,0x1D0B},
{0x22,0xF8F9,0x3AFA},
{0x02,0xF8FA,0x6C4E},
{0x22,0xF8FB,0x3CBC},
{0x02,0xF8FC,0x6CBF},
{0x02,0xF8FD,0x6CCD},
{0x02,0xF8FE,0x6C67},
{0x02,0xF8FF,0x6D16},
{0x02,0xF900,0x6D3E},
{0x02,0xF901,0x6D77},
{0x02,0xF902,0x6D41},
{0x02,0xF903,0x6D69},
{0x02,0xF904,0x6D78},
{0x02,0xF905,0x6D85},
{0x22,0xF906,0x3D1E},
{0x02,0xF907,0x6D34},
{0x02,0xF908,0x6E2F},
{0x02,0xF909,0x6E6E},
{0x02,0xF90A,0x3D33},
{0x02,0xF90B,0x6ECB},
{0x02,0xF90C,0x6EC7},
{0x22,0xF90D,0x3ED1},
{0x02,0xF90E,0x6DF9},
{0x02,0xF90F,0x6F6E},
{0x22,0xF910,0x3F5E},
{0x22,0xF911,0x3F8E},
{0x02,0xF912,0x6FC6},
{0x02,0xF913,0x7039},
{0x02,0xF914,0x701E},
{0x02,0xF915,0x701B},
{0x02,0xF916,0x3D96},
{0x02,0xF917,0x704A},
{0x02,0xF918,0x707D},
{0x02,0xF919,0x7077},
{0x02,0xF91A,0x70AD},
{0x22,0xF91B,0x0525},
{0x02,0xF91C,0x7145},
{0x22,0xF91D,0x4263},
{0x02,0xF91E,0x719C},
{0x22,0xF91F,0x43AB},
{0x02,0xF920,0x7228},
{0x02,0xF921,0x7235},
{0x02,0xF922,0x7250},
{0x22,0xF923,0x4608},
{0x02,0xF924,0x7280},
{0x02,0xF925,0x7295},
{0x22,0xF926,0x4735},
{0x22,0xF927,0x4814},
{0x02,0xF928,0x737A},
{0x02,0xF929,0x738B},
{0x02,0xF92A,0x3EAC},
{0x02,0xF92B,0x73A5},
{0x02,0xF92C,0x3EB8},
{0x02,0xF92D,0x3EB8},
{0x02,0xF92E,0x7447},
{0x02,0xF92F,0x745C},
{0x02,0xF930,0x7471},
{0x02,0xF931,0x7485},
{0x02,0xF932,0x74CA},
{0x02,0xF933,0x3F1B},
{0x02,0xF934,0x7524},
{0x22,0xF935,0x4C36},
{0x02,0xF936,0x753E},
{0x22,0xF937,0x4C92},
{0x02,0xF938,0x7570},
{0x22,0xF939,0x219F},
{0x02,0xF93A,0x7610},
{0x22,0xF93B,0x4FA1},
{0x22,0xF93C,0x4FB8},
{0x22,0xF93D,0x5044},
{0x02,0xF93E,0x3FFC},
{0x02,0xF93F,0x4008},
{0x02,0xF940,0x76F4},
{0x22,0xF941,0x50F3},
{0x22,0xF942,0x50F2},
{0x22,0xF943,0x5119},
{0x22,0xF944,0x5133},
{0x02,0xF945,0x771E},
{0x02,0xF946,0x771F},
{0x02,0xF947,0x771F},
{0x02,0xF948,0x774A},
{0x02,0xF949,0x4039},
{0x02,0xF94A,0x778B},
{0x02,0xF94B,0x4046},
{0x02,0xF94C,0x4096},
{0x22,0xF94D,0x541D},
{0x02,0xF94E,0x784E},
{0x02,0xF94F,0x788C},
{0x02,0xF950,0x78CC},
{0x02,0xF951,0x40E3},
{0x22,0xF952,0x5626},
{0x02,0xF953,0x7956},
{0x22,0xF954,0x569A},
{0x22,0xF955,0x56C5},
{0x02,0xF956,0x798F},
{0x02,0xF957,0x79EB},
{0x02,0xF958,0x412F},
{0x02,0xF959,0x7A40},
{0x02,0xF95A,0x7A4A},
{0x02,0xF95B,0x7A4F},
{0x22,0xF95C,0x597C},
{0x22,0xF95D,0x5AA7},
{0x22,0xF95E,0x5AA7},
{0x02,0xF95F,0x7AEE},
{0x02,0xF960,0x4202},
{0x22,0xF961,0x5BAB},
{0x02,0xF962,0x7BC6},
{0x02,0xF963,0x7BC9},
{0x02,0xF964,0x4227},
{0x22,0xF965,0x5C80},
{0x02,0xF966,0x7CD2},
{0x02,0xF967,0x42A0},
{0x02,0xF968,0x7CE8},
{0x02,0xF969,0x7CE3},
{0x02,0xF96A,0x7D00},
{0x22,0xF96B,0x5F86},
{0x02,0xF96C,0x7D63},
{0x02,0xF96D,0x4301},
{0x02,0xF96E,0x7DC7},
{0x02,0xF96F,0x7E02},
{0x02,0xF970,0x7E45},
{0x02,0xF971,0x4334},
{0x22,0xF972,0x6228},
{0x22,0xF973,0x6247},
{0x02,0xF974,0x4359},
{0x22,0xF975,0x62D9},
{0x02,0xF976,0x7F7A},
{0x22,0xF977,0x633E},
{0x02,0xF978,0x7F95},
{0x02,0xF979,0x7FFA},
{0x02,0xF97A,0x8005},
{0x22,0xF97B,0x64DA},
{0x22,0xF97C,0x6523},
{0x02,0xF97D,0x8060},
{0x22,0xF97E,0x65A8},
{0x02,0xF97F,0x8070},
{0x22,0xF980,0x335F},
{0x02,0xF981,0x43D5},
{0x02,0xF982,0x80B2},
{0x02,0xF983,0x8103},
{0x02,0xF984,0x440B},
{0x02,0xF985,0x813E},
{0x02,0xF986,0x5AB5},
{0x22,0xF987,0x67A7},
{0x22,0xF988,0x67B5},
{0x22,0xF989,0x3393},
{0x22,0xF98A,0x339C},
{0x02,0xF98B,0x8201},
{0x02,0xF98C,0x8204},
{0x02,0xF98D,0x8F9E},
{0x02,0xF98E,0x446B},
{0x02,0xF98F,0x8291},
{0x02,0xF990,0x828B},
{0x02,0xF991,0x829D},
{0x02,0xF992,0x52B3},
{0x02,0xF993,0x82B1},
{0x02,0xF994,0x82B3},
{0x02,0xF995,0x82BD},
{0x02,0xF996,0x82E6},
{0x22,0xF997,0x6B3C},
{0x02,0xF998,0x82E5},
{0x02,0xF999,0x831D},
{0x02,0xF99A,0x8363},
{0x02,0xF99B,0x83AD},
{0x02,0xF99C,0x8323},
{0x02,0xF99D,0x83BD},
{0x02,0xF99E,0x83E7},
{0x02,0xF99F,0x8457},
{0x02,0xF9A0,0x8353},
{0x02,0xF9A1,0x83CA},
{0x02,0xF9A2,0x83CC},
{0x02,0xF9A3,0x83DC},
{0x22,0xF9A4,0x6C36},
{0x22,0xF9A5,0x6D6B},
{0x22,0xF9A6,0x6CD5},
{0x02,0xF9A7,0x452B},
{0x02,0xF9A8,0x84F1},
{0x02,0xF9A9,0x84F3},
{0x02,0xF9AA,0x8516},
{0x22,0xF9AB,0x73CA},
{0x02,0xF9AC,0x8564},
{0x22,0xF9AD,0x6F2C},
{0x02,0xF9AE,0x455D},
{0x02,0xF9AF,0x4561},
{0x22,0xF9B0,0x6FB1},
{0x22,0xF9B1,0x70D2},
{0x02,0xF9B2,0x456B},
{0x02,0xF9B3,0x8650},
{0x02,0xF9B4,0x865C},
{0x02,0xF9B5,0x8667},
{0x02,0xF9B6,0x8669},
{0x02,0xF9B7,0x86A9},
{0x02,0xF9B8,0x8688},
{0x02,0xF9B9,0x870E},
{0x02,0xF9BA,0x86E2},
{0x02,0xF9BB,0x8779},
{0x02,0xF9BC,0x8728},
{0x02,0xF9BD,0x876B},
{0x02,0xF9BE,0x8786},
{0x02,0xF9BF,0x45D7},
{0x02,0xF9C0,0x87E1},
{0x02,0xF9C1,0x8801},
{0x02,0xF9C2,0x45F9},
{0x02,0xF9C3,0x8860},
{0x02,0xF9C4,0x8863},
{0x22,0xF9C5,0x7667},
{0x02,0xF9C6,0x88D7},
{0x02,0xF9C7,0x88DE},
{0x02,0xF9C8,0x4635},
{0x02,0xF9C9,0x88FA},
{0x02,0xF9CA,0x34BB},
{0x22,0xF9CB,0x78AE},
{0x22,0xF9CC,0x7966},
{0x02,0xF9CD,0x46BE},
{0x02,0xF9CE,0x46C7},
{0x02,0xF9CF,0x8AA0},
{0x02,0xF9D0,0x8AED},
{0x02,0xF9D1,0x8B8A},
{0x02,0xF9D2,0x8C55},
{0x22,0xF9D3,0x7CA8},
{0x02,0xF9D4,0x8CAB},
{0x02,0xF9D5,0x8CC1},
{0x02,0xF9D6,0x8D1B},
{0x02,0xF9D7,0x8D77},
{0x22,0xF9D8,0x7F2F},
{0x22,0xF9D9,0x0804},
{0x02,0xF9DA,0x8DCB},
{0x02,0xF9DB,0x8DBC},
{0x02,0xF9DC,0x8DF0},
{0x22,0xF9DD,0x08DE},
{0x02,0xF9DE,0x8ED4},
{0x02,0xF9DF,0x8F38},
{0x22,0xF9E0,0x85D2},
{0x22,0xF9E1,0x85ED},
{0x02,0xF9E2,0x9094},
{0x02,0xF9E3,0x90F1},
{0x02,0xF9E4,0x9111},
{0x22,0xF9E5,0x872E},
{0x02,0xF9E6,0x911B},
{0x02,0xF9E7,0x9238},
{0x02,0xF9E8,0x92D7},
{0x02,0xF9E9,0x92D8},
{0x02,0xF9EA,0x927C},
{0x02,0xF9EB,0x93F9},
{0x02,0xF9EC,0x9415},
{0x22,0xF9ED,0x8BFA},
{0x02,0xF9EE,0x958B},
{0x02,0xF9EF,0x4995},
{0x02,0xF9F0,0x95B7},
{0x22,0xF9F1,0x8D77},
{0x02,0xF9F2,0x49E6},
{0x02,0xF9F3,0x96C3},
{0x02,0xF9F4,0x5DB2},
{0x02,0xF9F5,0x9723},
{0x22,0xF9F6,0x9145},
{0x22,0xF9F7,0x921A},
{0x02,0xF9F8,0x4A6E},
{0x02,0xF9F9,0x4A76},
{0x02,0xF9FA,0x97E0},
{0x22,0xF9FB,0x940A},
{0x02,0xF9FC,0x4AB2},
{0x22,0xF9FD,0x9496},
{0x02,0xF9FE,0x980B},
{0x02,0xF9FF,0x980B},
{0x02,0xFA00,0x9829},
{0x22,0xFA01,0x95B6},
{0x02,0xFA02,0x98E2},
{0x02,0xFA03,0x4B33},
{0x02,0xFA04,0x9929},
{0x02,0xFA05,0x99A7},
{0x02,0xFA06,0x99C2},
{0x02,0xFA07,0x99FE},
{0x02,0xFA08,0x4BCE},
{0x22,0xFA09,0x9B30},
{0x02,0xFA0A,0x9B12},
{0x02,0xFA0B,0x9C40},
{0x02,0xFA0C,0x9CFD},
{0x02,0xFA0D,0x4CCE},
{0x02,0xFA0E,0x4CED},
{0x02,0xFA0F,0x9D67},
{0x22,0xFA10,0xA0CE},
{0x02,0xFA11,0x4CF8},
{0x22,0xFA12,0xA105},
{0x22,0xFA13,0xA20E},
{0x22,0xFA14,0xA291},
{0x02,0xFA15,0x9EBB},
{0x02,0xFA16,0x4D56},
{0x02,0xFA17,0x9EF9},
{0x02,0xFA18,0x9EFE},
{0x02,0xFA19,0x9F05},
{0x02,0xFA1A,0x9F0F},
{0x02,0xFA1B,0x9F16},
{0x02,0xFA1C,0x9F3B},
{0x22,0xFA1D,0xA600}
};

const struct UnicodeMapping UCCompat[NUCCOMPAT]={
{0x00,0x02B0,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0x02B1,{0x0266,0x0000,0x0000,0x0000}},
{0x00,0x02B2,{0x006A,0x0000,0x0000,0x0000}},
{0x00,0x02B3,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0x02B4,{0x0279,0x0000,0x0000,0x0000}},
{0x00,0x02B5,{0x027B,0x0000,0x0000,0x0000}},
{0x00,0x02B6,{0x0281,0x0000,0x0000,0x0000}},
{0x00,0x02B7,{0x0077,0x0000,0x0000,0x0000}},
{0x00,0x02B8,{0x0079,0x0000,0x0000,0x0000}},
{0x00,0x02D8,{0x0020,0x0306,0x0306,0x0000}},
{0x00,0x02D9,{0x0020,0x0307,0x0307,0x0000}},
{0x00,0x02DA,{0x0020,0x030A,0x030A,0x0000}},
{0x00,0x02DB,{0x0020,0x0328,0x0328,0x0000}},
{0x00,0x02DC,{0x0020,0x0303,0x0303,0x0000}},
{0x00,0x02DD,{0x0020,0x030B,0x030B,0x0000}},
{0x00,0x02E0,{0x0263,0x0000,0x0000,0x0000}},
{0x00,0x02E1,{0x006C,0x0000,0x0000,0x0000}},
{0x00,0x02E2,{0x0073,0x0000,0x0000,0x0000}},
{0x00,0x02E3,{0x0078,0x0000,0x0000,0x0000}},
{0x00,0x02E4,{0x0295,0x0000,0x0000,0x0000}},
{0x00,0x0384,{0x0020,0x0301,0x0301,0x0000}},
{0x00,0x03F9,{0x03A3,0x0000,0x0000,0x0000}},
{0x00,0x0675,{0x0627,0x0674,0x0674,0x0000}},
{0x00,0x0676,{0x0648,0x0674,0x0674,0x0000}},
{0x00,0x0677,{0x06C7,0x0674,0x0674,0x0000}},
{0x00,0x0678,{0x064A,0x0674,0x0674,0x0000}},
{0x00,0x0E33,{0x0E4D,0x0E32,0x0E32,0x0000}},
{0x00,0x0EB3,{0x0ECD,0x0EB2,0x0EB2,0x0000}},
{0x00,0x0EDC,{0x0EAB,0x0E99,0x0E99,0x0000}},
{0x00,0x0EDD,{0x0EAB,0x0EA1,0x0EA1,0x0000}},
{0x00,0x0F0C,{0x0F0B,0x0000,0x0000,0x0000}},
{0x00,0x0F77,{0x0FB2,0x0F81,0x0F81,0x0000}},
{0x00,0x0F79,{0x0FB3,0x0F81,0x0F81,0x0000}},
{0x00,0x10FC,{0x10DC,0x0000,0x0000,0x0000}},
{0x00,0x1D2C,{0x0041,0x0000,0x0000,0x0000}},
{0x00,0x1D2D,{0x00C6,0x0000,0x0000,0x0000}},
{0x00,0x1D2E,{0x0042,0x0000,0x0000,0x0000}},
{0x00,0x1D30,{0x0044,0x0000,0x0000,0x0000}},
{0x00,0x1D31,{0x0045,0x0000,0x0000,0x0000}},
{0x00,0x1D32,{0x018E,0x0000,0x0000,0x0000}},
{0x00,0x1D33,{0x0047,0x0000,0x0000,0x0000}},
{0x00,0x1D34,{0x0048,0x0000,0x0000,0x0000}},
{0x00,0x1D35,{0x0049,0x0000,0x0000,0x0000}},
{0x00,0x1D36,{0x004A,0x0000,0x0000,0x0000}},
{0x00,0x1D37,{0x004B,0x0000,0x0000,0x0000}},
{0x00,0x1D38,{0x004C,0x0000,0x0000,0x0000}},
{0x00,0x1D39,{0x004D,0x0000,0x0000,0x0000}},
{0x00,0x1D3A,{0x004E,0x0000,0x0000,0x0000}},
{0x00,0x1D3C,{0x004F,0x0000,0x0000,0x0000}},
{0x00,0x1D3D,{0x0222,0x0000,0x0000,0x0000}},
{0x00,0x1D3E,{0x0050,0x0000,0x0000,0x0000}},
{0x00,0x1D3F,{0x0052,0x0000,0x0000,0x0000}},
{0x00,0x1D40,{0x0054,0x0000,0x0000,0x0000}},
{0x00,0x1D41,{0x0055,0x0000,0x0000,0x0000}},
{0x00,0x1D42,{0x0057,0x0000,0x0000,0x0000}},
{0x00,0x1D43,{0x0061,0x0000,0x0000,0x0000}},
{0x00,0x1D44,{0x0250,0x0000,0x0000,0x0000}},
{0x00,0x1D45,{0x0251,0x0000,0x0000,0x0000}},
{0x00,0x1D46,{0x1D02,0x0000,0x0000,0x0000}},
{0x00,0x1D47,{0x0062,0x0000,0x0000,0x0000}},
{0x00,0x1D48,{0x0064,0x0000,0x0000,0x0000}},
{0x00,0x1D49,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0x1D4A,{0x0259,0x0000,0x0000,0x0000}},
{0x00,0x1D4B,{0x025B,0x0000,0x0000,0x0000}},
{0x00,0x1D4C,{0x025C,0x0000,0x0000,0x0000}},
{0x00,0x1D4D,{0x0067,0x0000,0x0000,0x0000}},
{0x00,0x1D4F,{0x006B,0x0000,0x0000,0x0000}},
{0x00,0x1D50,{0x006D,0x0000,0x0000,0x0000}},
{0x00,0x1D51,{0x014B,0x0000,0x0000,0x0000}},
{0x00,0x1D52,{0x006F,0x0000,0x0000,0x0000}},
{0x00,0x1D53,{0x0254,0x0000,0x0000,0x0000}},
{0x00,0x1D54,{0x1D16,0x0000,0x0000,0x0000}},
{0x00,0x1D55,{0x1D17,0x0000,0x0000,0x0000}},
{0x00,0x1D56,{0x0070,0x0000,0x0000,0x0000}},
{0x00,0x1D57,{0x0074,0x0000,0x0000,0x0000}},
{0x00,0x1D58,{0x0075,0x0000,0x0000,0x0000}},
{0x00,0x1D59,{0x1D1D,0x0000,0x0000,0x0000}},
{0x00,0x1D5A,{0x026F,0x0000,0x0000,0x0000}},
{0x00,0x1D5B,{0x0076,0x0000,0x0000,0x0000}},
{0x00,0x1D5C,{0x1D25,0x0000,0x0000,0x0000}},
{0x00,0x1D5D,{0x03B2,0x0000,0x0000,0x0000}},
{0x00,0x1D5E,{0x03B3,0x0000,0x0000,0x0000}},
{0x00,0x1D5F,{0x03B4,0x0000,0x0000,0x0000}},
{0x00,0x1D60,{0x03C6,0x0000,0x0000,0x0000}},
{0x00,0x1D61,{0x03C7,0x0000,0x0000,0x0000}},
{0x00,0x1D62,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x1D63,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0x1D64,{0x0075,0x0000,0x0000,0x0000}},
{0x00,0x1D65,{0x0076,0x0000,0x0000,0x0000}},
{0x00,0x1D66,{0x03B2,0x0000,0x0000,0x0000}},
{0x00,0x1D67,{0x03B3,0x0000,0x0000,0x0000}},
{0x00,0x1D68,{0x03C1,0x0000,0x0000,0x0000}},
{0x00,0x1D69,{0x03C6,0x0000,0x0000,0x0000}},
{0x00,0x1D6A,{0x03C7,0x0000,0x0000,0x0000}},
{0x00,0x1D78,{0x043D,0x0000,0x0000,0x0000}},
{0x00,0x1D9B,{0x0252,0x0000,0x0000,0x0000}},
{0x00,0x1D9C,{0x0063,0x0000,0x0000,0x0000}},
{0x00,0x1D9D,{0x0255,0x0000,0x0000,0x0000}},
{0x00,0x1D9E,{0x00F0,0x0000,0x0000,0x0000}},
{0x00,0x1D9F,{0x025C,0x0000,0x0000,0x0000}},
{0x00,0x1DA0,{0x0066,0x0000,0x0000,0x0000}},
{0x00,0x1DA1,{0x025F,0x0000,0x0000,0x0000}},
{0x00,0x1DA2,{0x0261,0x0000,0x0000,0x0000}},
{0x00,0x1DA3,{0x0265,0x0000,0x0000,0x0000}},
{0x00,0x1DA4,{0x0268,0x0000,0x0000,0x0000}},
{0x00,0x1DA5,{0x0269,0x0000,0x0000,0x0000}},
{0x00,0x1DA6,{0x026A,0x0000,0x0000,0x0000}},
{0x00,0x1DA7,{0x1D7B,0x0000,0x0000,0x0000}},
{0x00,0x1DA8,{0x029D,0x0000,0x0000,0x0000}},
{0x00,0x1DA9,{0x026D,0x0000,0x0000,0x0000}},
{0x00,0x1DAA,{0x1D85,0x0000,0x0000,0x0000}},
{0x00,0x1DAB,{0x029F,0x0000,0x0000,0x0000}},
{0x00,0x1DAC,{0x0271,0x0000,0x0000,0x0000}},
{0x00,0x1DAD,{0x0270,0x0000,0x0000,0x0000}},
{0x00,0x1DAE,{0x0272,0x0000,0x0000,0x0000}},
{0x00,0x1DAF,{0x0273,0x0000,0x0000,0x0000}},
{0x00,0x1DB0,{0x0274,0x0000,0x0000,0x0000}},
{0x00,0x1DB1,{0x0275,0x0000,0x0000,0x0000}},
{0x00,0x1DB2,{0x0278,0x0000,0x0000,0x0000}},
{0x00,0x1DB3,{0x0282,0x0000,0x0000,0x0000}},
{0x00,0x1DB4,{0x0283,0x0000,0x0000,0x0000}},
{0x00,0x1DB5,{0x01AB,0x0000,0x0000,0x0000}},
{0x00,0x1DB6,{0x0289,0x0000,0x0000,0x0000}},
{0x00,0x1DB7,{0x028A,0x0000,0x0000,0x0000}},
{0x00,0x1DB8,{0x1D1C,0x0000,0x0000,0x0000}},
{0x00,0x1DB9,{0x028B,0x0000,0x0000,0x0000}},
{0x00,0x1DBA,{0x028C,0x0000,0x0000,0x0000}},
{0x00,0x1DBB,{0x007A,0x0000,0x0000,0x0000}},
{0x00,0x1DBC,{0x0290,0x0000,0x0000,0x0000}},
{0x00,0x1DBD,{0x0291,0x0000,0x0000,0x0000}},
{0x00,0x1DBE,{0x0292,0x0000,0x0000,0x0000}},
{0x00,0x1DBF,{0x03B8,0x0000,0x0000,0x0000}},
{0x00,0x1FBD,{0x0020,0x0313,0x0313,0x0000}},
{0x00,0x1FBF,{0x0020,0x0313,0x0313,0x0000}},
{0x00,0x1FC0,{0x0020,0x0342,0x0342,0x0000}},
{0x00,0x1FFE,{0x0020,0x0314,0x0314,0x0000}},
{0x00,0x2002,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2003,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2004,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2005,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2006,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2007,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2008,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2009,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x200A,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2011,{0x2010,0x0000,0x0000,0x0000}},
{0x00,0x2017,{0x0020,0x0333,0x0333,0x0000}},
{0x00,0x2024,{0x002E,0x0000,0x0000,0x0000}},
{0x00,0x2025,{0x002E,0x002E,0x002E,0x0000}},
{0x00,0x2026,{0x002E,0x002E,0x002E,0x002E}},
{0x00,0x202F,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2033,{0x2032,0x2032,0x2032,0x0000}},
{0x00,0x2034,{0x2032,0x2032,0x2032,0x2032}},
{0x00,0x2036,{0x2035,0x2035,0x2035,0x0000}},
{0x00,0x2037,{0x2035,0x2035,0x2035,0x2035}},
{0x00,0x203C,{0x0021,0x0021,0x0021,0x0000}},
{0x00,0x203E,{0x0020,0x0305,0x0305,0x0000}},
{0x00,0x2047,{0x003F,0x003F,0x003F,0x0000}},
{0x00,0x2048,{0x003F,0x0021,0x0021,0x0000}},
{0x00,0x2049,{0x0021,0x003F,0x003F,0x0000}},
{0x00,0x205F,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x2070,{0x0030,0x0000,0x0000,0x0000}},
{0x00,0x2071,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x2074,{0x0034,0x0000,0x0000,0x0000}},
{0x00,0x2075,{0x0035,0x0000,0x0000,0x0000}},
{0x00,0x2076,{0x0036,0x0000,0x0000,0x0000}},
{0x00,0x2077,{0x0037,0x0000,0x0000,0x0000}},
{0x00,0x2078,{0x0038,0x0000,0x0000,0x0000}},
{0x00,0x2079,{0x0039,0x0000,0x0000,0x0000}},
{0x00,0x207A,{0x002B,0x0000,0x0000,0x0000}},
{0x00,0x207B,{0x2212,0x0000,0x0000,0x0000}},
{0x00,0x207C,{0x003D,0x0000,0x0000,0x0000}},
{0x00,0x207D,{0x0028,0x0000,0x0000,0x0000}},
{0x00,0x207E,{0x0029,0x0000,0x0000,0x0000}},
{0x00,0x207F,{0x006E,0x0000,0x0000,0x0000}},
{0x00,0x2080,{0x0030,0x0000,0x0000,0x0000}},
{0x00,0x2081,{0x0031,0x0000,0x0000,0x0000}},
{0x00,0x2082,{0x0032,0x0000,0x0000,0x0000}},
{0x00,0x2083,{0x0033,0x0000,0x0000,0x0000}},
{0x00,0x2084,{0x0034,0x0000,0x0000,0x0000}},
{0x00,0x2085,{0x0035,0x0000,0x0000,0x0000}},
{0x00,0x2086,{0x0036,0x0000,0x0000,0x0000}},
{0x00,0x2087,{0x0037,0x0000,0x0000,0x0000}},
{0x00,0x2088,{0x0038,0x0000,0x0000,0x0000}},
{0x00,0x2089,{0x0039,0x0000,0x0000,0x0000}},
{0x00,0x208A,{0x002B,0x0000,0x0000,0x0000}},
{0x00,0x208B,{0x2212,0x0000,0x0000,0x0000}},
{0x00,0x208C,{0x003D,0x0000,0x0000,0x0000}},
{0x00,0x208D,{0x0028,0x0000,0x0000,0x0000}},
{0x00,0x208E,{0x0029,0x0000,0x0000,0x0000}},
{0x00,0x2090,{0x0061,0x0000,0x0000,0x0000}},
{0x00,0x2091,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0x2092,{0x006F,0x0000,0x0000,0x0000}},
{0x00,0x2093,{0x0078,0x0000,0x0000,0x0000}},
{0x00,0x2094,{0x0259,0x0000,0x0000,0x0000}},
{0x00,0x2100,{0x0061,0x002F,0x002F,0x0063}},
{0x00,0x2101,{0x0061,0x002F,0x002F,0x0073}},
{0x00,0x2105,{0x0063,0x002F,0x002F,0x006F}},
{0x00,0x2106,{0x0063,0x002F,0x002F,0x0075}},
{0x00,0x210A,{0x0067,0x0000,0x0000,0x0000}},
{0x00,0x210E,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0x210F,{0x0127,0x0000,0x0000,0x0000}},
{0x00,0x2113,{0x006C,0x0000,0x0000,0x0000}},
{0x00,0x212F,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0x2134,{0x006F,0x0000,0x0000,0x0000}},
{0x00,0x2135,{0x05D0,0x0000,0x0000,0x0000}},
{0x00,0x2136,{0x05D1,0x0000,0x0000,0x0000}},
{0x00,0x2137,{0x05D2,0x0000,0x0000,0x0000}},
{0x00,0x2138,{0x05D3,0x0000,0x0000,0x0000}},
{0x00,0x2139,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x213B,{0x0046,0x0041,0x0041,0x0058}},
{0x00,0x213C,{0x03C0,0x0000,0x0000,0x0000}},
{0x00,0x213D,{0x03B3,0x0000,0x0000,0x0000}},
{0x00,0x2140,{0x2211,0x0000,0x0000,0x0000}},
{0x00,0x2146,{0x0064,0x0000,0x0000,0x0000}},
{0x00,0x2147,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0x2148,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x2149,{0x006A,0x0000,0x0000,0x0000}},
{0x00,0x2153,{0x0031,0x2044,0x2044,0x0033}},
{0x00,0x2154,{0x0032,0x2044,0x2044,0x0033}},
{0x00,0x2155,{0x0031,0x2044,0x2044,0x0035}},
{0x00,0x2156,{0x0032,0x2044,0x2044,0x0035}},
{0x00,0x2157,{0x0033,0x2044,0x2044,0x0035}},
{0x00,0x2158,{0x0034,0x2044,0x2044,0x0035}},
{0x00,0x2159,{0x0031,0x2044,0x2044,0x0036}},
{0x00,0x215A,{0x0035,0x2044,0x2044,0x0036}},
{0x00,0x215B,{0x0031,0x2044,0x2044,0x0038}},
{0x00,0x215C,{0x0033,0x2044,0x2044,0x0038}},
{0x00,0x215D,{0x0035,0x2044,0x2044,0x0038}},
{0x00,0x215E,{0x0037,0x2044,0x2044,0x0038}},
{0x00,0x215F,{0x0031,0x2044,0x2044,0x0000}},
{0x00,0x2170,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x2171,{0x0069,0x0069,0x0069,0x0000}},
{0x00,0x2172,{0x0069,0x0069,0x0069,0x0069}},
{0x00,0x2173,{0x0069,0x0076,0x0076,0x0000}},
{0x00,0x2174,{0x0076,0x0000,0x0000,0x0000}},
{0x00,0x2175,{0x0076,0x0069,0x0069,0x0000}},
{0x00,0x2176,{0x0076,0x0069,0x0069,0x0069}},
{0x00,0x2178,{0x0069,0x0078,0x0078,0x0000}},
{0x00,0x2179,{0x0078,0x0000,0x0000,0x0000}},
{0x00,0x217A,{0x0078,0x0069,0x0069,0x0000}},
{0x00,0x217B,{0x0078,0x0069,0x0069,0x0069}},
{0x00,0x217C,{0x006C,0x0000,0x0000,0x0000}},
{0x00,0x217D,{0x0063,0x0000,0x0000,0x0000}},
{0x00,0x217E,{0x0064,0x0000,0x0000,0x0000}},
{0x00,0x217F,{0x006D,0x0000,0x0000,0x0000}},
{0x00,0x222C,{0x222B,0x222B,0x222B,0x0000}},
{0x00,0x222D,{0x222B,0x222B,0x222B,0x222B}},
{0x00,0x222F,{0x222E,0x222E,0x222E,0x0000}},
{0x00,0x2230,{0x222E,0x222E,0x222E,0x222E}},
{0x00,0x2460,{0x0031,0x0000,0x0000,0x0000}},
{0x00,0x2461,{0x0032,0x0000,0x0000,0x0000}},
{0x00,0x2462,{0x0033,0x0000,0x0000,0x0000}},
{0x00,0x2463,{0x0034,0x0000,0x0000,0x0000}},
{0x00,0x2464,{0x0035,0x0000,0x0000,0x0000}},
{0x00,0x2465,{0x0036,0x0000,0x0000,0x0000}},
{0x00,0x2466,{0x0037,0x0000,0x0000,0x0000}},
{0x00,0x2467,{0x0038,0x0000,0x0000,0x0000}},
{0x00,0x2468,{0x0039,0x0000,0x0000,0x0000}},
{0x00,0x2469,{0x0031,0x0030,0x0030,0x0000}},
{0x00,0x246A,{0x0031,0x0031,0x0031,0x0000}},
{0x00,0x246B,{0x0031,0x0032,0x0032,0x0000}},
{0x00,0x246C,{0x0031,0x0033,0x0033,0x0000}},
{0x00,0x246D,{0x0031,0x0034,0x0034,0x0000}},
{0x00,0x246E,{0x0031,0x0035,0x0035,0x0000}},
{0x00,0x246F,{0x0031,0x0036,0x0036,0x0000}},
{0x00,0x2470,{0x0031,0x0037,0x0037,0x0000}},
{0x00,0x2471,{0x0031,0x0038,0x0038,0x0000}},
{0x00,0x2472,{0x0031,0x0039,0x0039,0x0000}},
{0x00,0x2473,{0x0032,0x0030,0x0030,0x0000}},
{0x00,0x2474,{0x0028,0x0031,0x0031,0x0029}},
{0x00,0x2475,{0x0028,0x0032,0x0032,0x0029}},
{0x00,0x2476,{0x0028,0x0033,0x0033,0x0029}},
{0x00,0x2477,{0x0028,0x0034,0x0034,0x0029}},
{0x00,0x2478,{0x0028,0x0035,0x0035,0x0029}},
{0x00,0x2479,{0x0028,0x0036,0x0036,0x0029}},
{0x00,0x247A,{0x0028,0x0037,0x0037,0x0029}},
{0x00,0x247B,{0x0028,0x0038,0x0038,0x0029}},
{0x00,0x247C,{0x0028,0x0039,0x0039,0x0029}},
{0x00,0x2488,{0x0031,0x002E,0x002E,0x0000}},
{0x00,0x2489,{0x0032,0x002E,0x002E,0x0000}},
{0x00,0x248A,{0x0033,0x002E,0x002E,0x0000}},
{0x00,0x248B,{0x0034,0x002E,0x002E,0x0000}},
{0x00,0x248C,{0x0035,0x002E,0x002E,0x0000}},
{0x00,0x248D,{0x0036,0x002E,0x002E,0x0000}},
{0x00,0x248E,{0x0037,0x002E,0x002E,0x0000}},
{0x00,0x248F,{0x0038,0x002E,0x002E,0x0000}},
{0x00,0x2490,{0x0039,0x002E,0x002E,0x0000}},
{0x00,0x2491,{0x0031,0x0030,0x0030,0x002E}},
{0x00,0x2492,{0x0031,0x0031,0x0031,0x002E}},
{0x00,0x2493,{0x0031,0x0032,0x0032,0x002E}},
{0x00,0x2494,{0x0031,0x0033,0x0033,0x002E}},
{0x00,0x2495,{0x0031,0x0034,0x0034,0x002E}},
{0x00,0x2496,{0x0031,0x0035,0x0035,0x002E}},
{0x00,0x2497,{0x0031,0x0036,0x0036,0x002E}},
{0x00,0x2498,{0x0031,0x0037,0x0037,0x002E}},
{0x00,0x2499,{0x0031,0x0038,0x0038,0x002E}},
{0x00,0x249A,{0x0031,0x0039,0x0039,0x002E}},
{0x00,0x249B,{0x0032,0x0030,0x0030,0x002E}},
{0x00,0x249C,{0x0028,0x0061,0x0061,0x0029}},
{0x00,0x249D,{0x0028,0x0062,0x0062,0x0029}},
{0x00,0x249E,{0x0028,0x0063,0x0063,0x0029}},
{0x00,0x249F,{0x0028,0x0064,0x0064,0x0029}},
{0x00,0x24A0,{0x0028,0x0065,0x0065,0x0029}},
{0x00,0x24A1,{0x0028,0x0066,0x0066,0x0029}},
{0x00,0x24A2,{0x0028,0x0067,0x0067,0x0029}},
{0x00,0x24A3,{0x0028,0x0068,0x0068,0x0029}},
{0x00,0x24A4,{0x0028,0x0069,0x0069,0x0029}},
{0x00,0x24A5,{0x0028,0x006A,0x006A,0x0029}},
{0x00,0x24A6,{0x0028,0x006B,0x006B,0x0029}},
{0x00,0x24A7,{0x0028,0x006C,0x006C,0x0029}},
{0x00,0x24A8,{0x0028,0x006D,0x006D,0x0029}},
{0x00,0x24A9,{0x0028,0x006E,0x006E,0x0029}},
{0x00,0x24AA,{0x0028,0x006F,0x006F,0x0029}},
{0x00,0x24AB,{0x0028,0x0070,0x0070,0x0029}},
{0x00,0x24AC,{0x0028,0x0071,0x0071,0x0029}},
{0x00,0x24AD,{0x0028,0x0072,0x0072,0x0029}},
{0x00,0x24AE,{0x0028,0x0073,0x0073,0x0029}},
{0x00,0x24AF,{0x0028,0x0074,0x0074,0x0029}},
{0x00,0x24B0,{0x0028,0x0075,0x0075,0x0029}},
{0x00,0x24B1,{0x0028,0x0076,0x0076,0x0029}},
{0x00,0x24B2,{0x0028,0x0077,0x0077,0x0029}},
{0x00,0x24B3,{0x0028,0x0078,0x0078,0x0029}},
{0x00,0x24B4,{0x0028,0x0079,0x0079,0x0029}},
{0x00,0x24B5,{0x0028,0x007A,0x007A,0x0029}},
{0x00,0x24D0,{0x0061,0x0000,0x0000,0x0000}},
{0x00,0x24D1,{0x0062,0x0000,0x0000,0x0000}},
{0x00,0x24D2,{0x0063,0x0000,0x0000,0x0000}},
{0x00,0x24D3,{0x0064,0x0000,0x0000,0x0000}},
{0x00,0x24D4,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0x24D5,{0x0066,0x0000,0x0000,0x0000}},
{0x00,0x24D6,{0x0067,0x0000,0x0000,0x0000}},
{0x00,0x24D7,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0x24D8,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0x24D9,{0x006A,0x0000,0x0000,0x0000}},
{0x00,0x24DA,{0x006B,0x0000,0x0000,0x0000}},
{0x00,0x24DB,{0x006C,0x0000,0x0000,0x0000}},
{0x00,0x24DC,{0x006D,0x0000,0x0000,0x0000}},
{0x00,0x24DD,{0x006E,0x0000,0x0000,0x0000}},
{0x00,0x24DE,{0x006F,0x0000,0x0000,0x0000}},
{0x00,0x24DF,{0x0070,0x0000,0x0000,0x0000}},
{0x00,0x24E0,{0x0071,0x0000,0x0000,0x0000}},
{0x00,0x24E1,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0x24E2,{0x0073,0x0000,0x0000,0x0000}},
{0x00,0x24E3,{0x0074,0x0000,0x0000,0x0000}},
{0x00,0x24E4,{0x0075,0x0000,0x0000,0x0000}},
{0x00,0x24E5,{0x0076,0x0000,0x0000,0x0000}},
{0x00,0x24E6,{0x0077,0x0000,0x0000,0x0000}},
{0x00,0x24E7,{0x0078,0x0000,0x0000,0x0000}},
{0x00,0x24E8,{0x0079,0x0000,0x0000,0x0000}},
{0x00,0x24E9,{0x007A,0x0000,0x0000,0x0000}},
{0x00,0x24EA,{0x0030,0x0000,0x0000,0x0000}},
{0x00,0x2A74,{0x003A,0x003A,0x003A,0x003D}},
{0x00,0x2A75,{0x003D,0x003D,0x003D,0x0000}},
{0x00,0x2A76,{0x003D,0x003D,0x003D,0x003D}},
{0x00,0x2D6F,{0x2D61,0x0000,0x0000,0x0000}},
{0x00,0x2E9F,{0x6BCD,0x0000,0x0000,0x0000}},
{0x00,0x2EF3,{0x9F9F,0x0000,0x0000,0x0000}},
{0x00,0x2F00,{0x4E00,0x0000,0x0000,0x0000}},
{0x00,0x2F01,{0x4E28,0x0000,0x0000,0x0000}},
{0x00,0x2F02,{0x4E36,0x0000,0x0000,0x0000}},
{0x00,0x2F03,{0x4E3F,0x0000,0x0000,0x0000}},
{0x00,0x2F04,{0x4E59,0x0000,0x0000,0x0000}},
{0x00,0x2F05,{0x4E85,0x0000,0x0000,0x0000}},
{0x00,0x2F06,{0x4E8C,0x0000,0x0000,0x0000}},
{0x00,0x2F07,{0x4EA0,0x0000,0x0000,0x0000}},
{0x00,0x2F08,{0x4EBA,0x0000,0x0000,0x0000}},
{0x00,0x2F09,{0x513F,0x0000,0x0000,0x0000}},
{0x00,0x2F0A,{0x5165,0x0000,0x0000,0x0000}},
{0x00,0x2F0B,{0x516B,0x0000,0x0000,0x0000}},
{0x00,0x2F0C,{0x5182,0x0000,0x0000,0x0000}},
{0x00,0x2F0D,{0x5196,0x0000,0x0000,0x0000}},
{0x00,0x2F0E,{0x51AB,0x0000,0x0000,0x0000}},
{0x00,0x2F0F,{0x51E0,0x0000,0x0000,0x0000}},
{0x00,0x2F10,{0x51F5,0x0000,0x0000,0x0000}},
{0x00,0x2F11,{0x5200,0x0000,0x0000,0x0000}},
{0x00,0x2F12,{0x529B,0x0000,0x0000,0x0000}},
{0x00,0x2F13,{0x52F9,0x0000,0x0000,0x0000}},
{0x00,0x2F14,{0x5315,0x0000,0x0000,0x0000}},
{0x00,0x2F15,{0x531A,0x0000,0x0000,0x0000}},
{0x00,0x2F16,{0x5338,0x0000,0x0000,0x0000}},
{0x00,0x2F17,{0x5341,0x0000,0x0000,0x0000}},
{0x00,0x2F18,{0x535C,0x0000,0x0000,0x0000}},
{0x00,0x2F19,{0x5369,0x0000,0x0000,0x0000}},
{0x00,0x2F1A,{0x5382,0x0000,0x0000,0x0000}},
{0x00,0x2F1B,{0x53B6,0x0000,0x0000,0x0000}},
{0x00,0x2F1C,{0x53C8,0x0000,0x0000,0x0000}},
{0x00,0x2F1D,{0x53E3,0x0000,0x0000,0x0000}},
{0x00,0x2F1E,{0x56D7,0x0000,0x0000,0x0000}},
{0x00,0x2F1F,{0x571F,0x0000,0x0000,0x0000}},
{0x00,0x2F20,{0x58EB,0x0000,0x0000,0x0000}},
{0x00,0x2F21,{0x5902,0x0000,0x0000,0x0000}},
{0x00,0x2F22,{0x590A,0x0000,0x0000,0x0000}},
{0x00,0x2F23,{0x5915,0x0000,0x0000,0x0000}},
{0x00,0x2F24,{0x5927,0x0000,0x0000,0x0000}},
{0x00,0x2F25,{0x5973,0x0000,0x0000,0x0000}},
{0x00,0x2F26,{0x5B50,0x0000,0x0000,0x0000}},
{0x00,0x2F27,{0x5B80,0x0000,0x0000,0x0000}},
{0x00,0x2F28,{0x5BF8,0x0000,0x0000,0x0000}},
{0x00,0x2F29,{0x5C0F,0x0000,0x0000,0x0000}},
{0x00,0x2F2A,{0x5C22,0x0000,0x0000,0x0000}},
{0x00,0x2F2B,{0x5C38,0x0000,0x0000,0x0000}},
{0x00,0x2F2C,{0x5C6E,0x0000,0x0000,0x0000}},
{0x00,0x2F2D,{0x5C71,0x0000,0x0000,0x0000}},
{0x00,0x2F2E,{0x5DDB,0x0000,0x0000,0x0000}},
{0x00,0x2F2F,{0x5DE5,0x0000,0x0000,0x0000}},
{0x00,0x2F30,{0x5DF1,0x0000,0x0000,0x0000}},
{0x00,0x2F31,{0x5DFE,0x0000,0x0000,0x0000}},
{0x00,0x2F32,{0x5E72,0x0000,0x0000,0x0000}},
{0x00,0x2F33,{0x5E7A,0x0000,0x0000,0x0000}},
{0x00,0x2F34,{0x5E7F,0x0000,0x0000,0x0000}},
{0x00,0x2F35,{0x5EF4,0x0000,0x0000,0x0000}},
{0x00,0x2F36,{0x5EFE,0x0000,0x0000,0x0000}},
{0x00,0x2F37,{0x5F0B,0x0000,0x0000,0x0000}},
{0x00,0x2F38,{0x5F13,0x0000,0x0000,0x0000}},
{0x00,0x2F39,{0x5F50,0x0000,0x0000,0x0000}},
{0x00,0x2F3A,{0x5F61,0x0000,0x0000,0x0000}},
{0x00,0x2F3B,{0x5F73,0x0000,0x0000,0x0000}},
{0x00,0x2F3C,{0x5FC3,0x0000,0x0000,0x0000}},
{0x00,0x2F3D,{0x6208,0x0000,0x0000,0x0000}},
{0x00,0x2F3E,{0x6236,0x0000,0x0000,0x0000}},
{0x00,0x2F3F,{0x624B,0x0000,0x0000,0x0000}},
{0x00,0x2F40,{0x652F,0x0000,0x0000,0x0000}},
{0x00,0x2F41,{0x6534,0x0000,0x0000,0x0000}},
{0x00,0x2F42,{0x6587,0x0000,0x0000,0x0000}},
{0x00,0x2F43,{0x6597,0x0000,0x0000,0x0000}},
{0x00,0x2F44,{0x65A4,0x0000,0x0000,0x0000}},
{0x00,0x2F45,{0x65B9,0x0000,0x0000,0x0000}},
{0x00,0x2F46,{0x65E0,0x0000,0x0000,0x0000}},
{0x00,0x2F47,{0x65E5,0x0000,0x0000,0x0000}},
{0x00,0x2F48,{0x66F0,0x0000,0x0000,0x0000}},
{0x00,0x2F49,{0x6708,0x0000,0x0000,0x0000}},
{0x00,0x2F4A,{0x6728,0x0000,0x0000,0x0000}},
{0x00,0x2F4B,{0x6B20,0x0000,0x0000,0x0000}},
{0x00,0x2F4C,{0x6B62,0x0000,0x0000,0x0000}},
{0x00,0x2F4D,{0x6B79,0x0000,0x0000,0x0000}},
{0x00,0x2F4E,{0x6BB3,0x0000,0x0000,0x0000}},
{0x00,0x2F4F,{0x6BCB,0x0000,0x0000,0x0000}},
{0x00,0x2F50,{0x6BD4,0x0000,0x0000,0x0000}},
{0x00,0x2F51,{0x6BDB,0x0000,0x0000,0x0000}},
{0x00,0x2F52,{0x6C0F,0x0000,0x0000,0x0000}},
{0x00,0x2F53,{0x6C14,0x0000,0x0000,0x0000}},
{0x00,0x2F54,{0x6C34,0x0000,0x0000,0x0000}},
{0x00,0x2F55,{0x706B,0x0000,0x0000,0x0000}},
{0x00,0x2F56,{0x722A,0x0000,0x0000,0x0000}},
{0x00,0x2F57,{0x7236,0x0000,0x0000,0x0000}},
{0x00,0x2F58,{0x723B,0x0000,0x0000,0x0000}},
{0x00,0x2F59,{0x723F,0x0000,0x0000,0x0000}},
{0x00,0x2F5A,{0x7247,0x0000,0x0000,0x0000}},
{0x00,0x2F5B,{0x7259,0x0000,0x0000,0x0000}},
{0x00,0x2F5C,{0x725B,0x0000,0x0000,0x0000}},
{0x00,0x2F5D,{0x72AC,0x0000,0x0000,0x0000}},
{0x00,0x2F5E,{0x7384,0x0000,0x0000,0x0000}},
{0x00,0x2F5F,{0x7389,0x0000,0x0000,0x0000}},
{0x00,0x2F60,{0x74DC,0x0000,0x0000,0x0000}},
{0x00,0x2F61,{0x74E6,0x0000,0x0000,0x0000}},
{0x00,0x2F62,{0x7518,0x0000,0x0000,0x0000}},
{0x00,0x2F63,{0x751F,0x0000,0x0000,0x0000}},
{0x00,0x2F64,{0x7528,0x0000,0x0000,0x0000}},
{0x00,0x2F65,{0x7530,0x0000,0x0000,0x0000}},
{0x00,0x2F66,{0x758B,0x0000,0x0000,0x0000}},
{0x00,0x2F67,{0x7592,0x0000,0x0000,0x0000}},
{0x00,0x2F68,{0x7676,0x0000,0x0000,0x0000}},
{0x00,0x2F69,{0x767D,0x0000,0x0000,0x0000}},
{0x00,0x2F6A,{0x76AE,0x0000,0x0000,0x0000}},
{0x00,0x2F6B,{0x76BF,0x0000,0x0000,0x0000}},
{0x00,0x2F6C,{0x76EE,0x0000,0x0000,0x0000}},
{0x00,0x2F6D,{0x77DB,0x0000,0x0000,0x0000}},
{0x00,0x2F6E,{0x77E2,0x0000,0x0000,0x0000}},
{0x00,0x2F6F,{0x77F3,0x0000,0x0000,0x0000}},
{0x00,0x2F70,{0x793A,0x0000,0x0000,0x0000}},
{0x00,0x2F71,{0x79B8,0x0000,0x0000,0x0000}},
{0x00,0x2F72,{0x79BE,0x0000,0x0000,0x0000}},
{0x00,0x2F73,{0x7A74,0x0000,0x0000,0x0000}},
{0x00,0x2F74,{0x7ACB,0x0000,0x0000,0x0000}},
{0x00,0x2F75,{0x7AF9,0x0000,0x0000,0x0000}},
{0x00,0x2F76,{0x7C73,0x0000,0x0000,0x0000}},
{0x00,0x2F77,{0x7CF8,0x0000,0x0000,0x0000}},
{0x00,0x2F78,{0x7F36,0x0000,0x0000,0x0000}},
{0x00,0x2F79,{0x7F51,0x0000,0x0000,0x0000}},
{0x00,0x2F7A,{0x7F8A,0x0000,0x0000,0x0000}},
{0x00,0x2F7B,{0x7FBD,0x0000,0x0000,0x0000}},
{0x00,0x2F7C,{0x8001,0x0000,0x0000,0x0000}},
{0x00,0x2F7D,{0x800C,0x0000,0x0000,0x0000}},
{0x00,0x2F7E,{0x8012,0x0000,0x0000,0x0000}},
{0x00,0x2F7F,{0x8033,0x0000,0x0000,0x0000}},
{0x00,0x2F80,{0x807F,0x0000,0x0000,0x0000}},
{0x00,0x2F81,{0x8089,0x0000,0x0000,0x0000}},
{0x00,0x2F82,{0x81E3,0x0000,0x0000,0x0000}},
{0x00,0x2F83,{0x81EA,0x0000,0x0000,0x0000}},
{0x00,0x2F84,{0x81F3,0x0000,0x0000,0x0000}},
{0x00,0x2F85,{0x81FC,0x0000,0x0000,0x0000}},
{0x00,0x2F86,{0x820C,0x0000,0x0000,0x0000}},
{0x00,0x2F87,{0x821B,0x0000,0x0000,0x0000}},
{0x00,0x2F88,{0x821F,0x0000,0x0000,0x0000}},
{0x00,0x2F89,{0x826E,0x0000,0x0000,0x0000}},
{0x00,0x2F8A,{0x8272,0x0000,0x0000,0x0000}},
{0x00,0x2F8B,{0x8278,0x0000,0x0000,0x0000}},
{0x00,0x2F8C,{0x864D,0x0000,0x0000,0x0000}},
{0x00,0x2F8D,{0x866B,0x0000,0x0000,0x0000}},
{0x00,0x2F8E,{0x8840,0x0000,0x0000,0x0000}},
{0x00,0x2F8F,{0x884C,0x0000,0x0000,0x0000}},
{0x00,0x2F90,{0x8863,0x0000,0x0000,0x0000}},
{0x00,0x2F91,{0x897E,0x0000,0x0000,0x0000}},
{0x00,0x2F92,{0x898B,0x0000,0x0000,0x0000}},
{0x00,0x2F93,{0x89D2,0x0000,0x0000,0x0000}},
{0x00,0x2F94,{0x8A00,0x0000,0x0000,0x0000}},
{0x00,0x2F95,{0x8C37,0x0000,0x0000,0x0000}},
{0x00,0x2F96,{0x8C46,0x0000,0x0000,0x0000}},
{0x00,0x2F97,{0x8C55,0x0000,0x0000,0x0000}},
{0x00,0x2F98,{0x8C78,0x0000,0x0000,0x0000}},
{0x00,0x2F99,{0x8C9D,0x0000,0x0000,0x0000}},
{0x00,0x2F9A,{0x8D64,0x0000,0x0000,0x0000}},
{0x00,0x2F9B,{0x8D70,0x0000,0x0000,0x0000}},
{0x00,0x2F9C,{0x8DB3,0x0000,0x0000,0x0000}},
{0x00,0x2F9D,{0x8EAB,0x0000,0x0000,0x0000}},
{0x00,0x2F9E,{0x8ECA,0x0000,0x0000,0x0000}},
{0x00,0x2F9F,{0x8F9B,0x0000,0x0000,0x0000}},
{0x00,0x2FA0,{0x8FB0,0x0000,0x0000,0x0000}},
{0x00,0x2FA1,{0x8FB5,0x0000,0x0000,0x0000}},
{0x00,0x2FA2,{0x9091,0x0000,0x0000,0x0000}},
{0x00,0x2FA3,{0x9149,0x0000,0x0000,0x0000}},
{0x00,0x2FA4,{0x91C6,0x0000,0x0000,0x0000}},
{0x00,0x2FA5,{0x91CC,0x0000,0x0000,0x0000}},
{0x00,0x2FA6,{0x91D1,0x0000,0x0000,0x0000}},
{0x00,0x2FA7,{0x9577,0x0000,0x0000,0x0000}},
{0x00,0x2FA8,{0x9580,0x0000,0x0000,0x0000}},
{0x00,0x2FA9,{0x961C,0x0000,0x0000,0x0000}},
{0x00,0x2FAA,{0x96B6,0x0000,0x0000,0x0000}},
{0x00,0x2FAB,{0x96B9,0x0000,0x0000,0x0000}},
{0x00,0x2FAC,{0x96E8,0x0000,0x0000,0x0000}},
{0x00,0x2FAD,{0x9751,0x0000,0x0000,0x0000}},
{0x00,0x2FAE,{0x975E,0x0000,0x0000,0x0000}},
{0x00,0x2FAF,{0x9762,0x0000,0x0000,0x0000}},
{0x00,0x2FB0,{0x9769,0x0000,0x0000,0x0000}},
{0x00,0x2FB1,{0x97CB,0x0000,0x0000,0x0000}},
{0x00,0x2FB2,{0x97ED,0x0000,0x0000,0x0000}},
{0x00,0x2FB3,{0x97F3,0x0000,0x0000,0x0000}},
{0x00,0x2FB4,{0x9801,0x0000,0x0000,0x0000}},
{0x00,0x2FB5,{0x98A8,0x0000,0x0000,0x0000}},
{0x00,0x2FB6,{0x98DB,0x0000,0x0000,0x0000}},
{0x00,0x2FB7,{0x98DF,0x0000,0x0000,0x0000}},
{0x00,0x2FB8,{0x9996,0x0000,0x0000,0x0000}},
{0x00,0x2FB9,{0x9999,0x0000,0x0000,0x0000}},
{0x00,0x2FBA,{0x99AC,0x0000,0x0000,0x0000}},
{0x00,0x2FBB,{0x9AA8,0x0000,0x0000,0x0000}},
{0x00,0x2FBC,{0x9AD8,0x0000,0x0000,0x0000}},
{0x00,0x2FBD,{0x9ADF,0x0000,0x0000,0x0000}},
{0x00,0x2FBE,{0x9B25,0x0000,0x0000,0x0000}},
{0x00,0x2FBF,{0x9B2F,0x0000,0x0000,0x0000}},
{0x00,0x2FC0,{0x9B32,0x0000,0x0000,0x0000}},
{0x00,0x2FC1,{0x9B3C,0x0000,0x0000,0x0000}},
{0x00,0x2FC2,{0x9B5A,0x0000,0x0000,0x0000}},
{0x00,0x2FC3,{0x9CE5,0x0000,0x0000,0x0000}},
{0x00,0x2FC4,{0x9E75,0x0000,0x0000,0x0000}},
{0x00,0x2FC5,{0x9E7F,0x0000,0x0000,0x0000}},
{0x00,0x2FC6,{0x9EA5,0x0000,0x0000,0x0000}},
{0x00,0x2FC7,{0x9EBB,0x0000,0x0000,0x0000}},
{0x00,0x2FC8,{0x9EC3,0x0000,0x0000,0x0000}},
{0x00,0x2FC9,{0x9ECD,0x0000,0x0000,0x0000}},
{0x00,0x2FCA,{0x9ED1,0x0000,0x0000,0x0000}},
{0x00,0x2FCB,{0x9EF9,0x0000,0x0000,0x0000}},
{0x00,0x2FCC,{0x9EFD,0x0000,0x0000,0x0000}},
{0x00,0x2FCD,{0x9F0E,0x0000,0x0000,0x0000}},
{0x00,0x2FCE,{0x9F13,0x0000,0x0000,0x0000}},
{0x00,0x2FCF,{0x9F20,0x0000,0x0000,0x0000}},
{0x00,0x2FD0,{0x9F3B,0x0000,0x0000,0x0000}},
{0x00,0x2FD1,{0x9F4A,0x0000,0x0000,0x0000}},
{0x00,0x2FD2,{0x9F52,0x0000,0x0000,0x0000}},
{0x00,0x2FD3,{0x9F8D,0x0000,0x0000,0x0000}},
{0x00,0x2FD4,{0x9F9C,0x0000,0x0000,0x0000}},
{0x00,0x2FD5,{0x9FA0,0x0000,0x0000,0x0000}},
{0x00,0x3000,{0x0020,0x0000,0x0000,0x0000}},
{0x00,0x3036,{0x3012,0x0000,0x0000,0x0000}},
{0x00,0x3038,{0x5341,0x0000,0x0000,0x0000}},
{0x00,0x3039,{0x5344,0x0000,0x0000,0x0000}},
{0x00,0x303A,{0x5345,0x0000,0x0000,0x0000}},
{0x00,0x309B,{0x0020,0x3099,0x3099,0x0000}},
{0x00,0x309C,{0x0020,0x309A,0x309A,0x0000}},
{0x00,0x309F,{0x3088,0x308A,0x308A,0x0000}},
{0x00,0x30FF,{0x30B3,0x30C8,0x30C8,0x0000}},
{0x00,0x3131,{0x1100,0x0000,0x0000,0x0000}},
{0x00,0x3132,{0x1101,0x0000,0x0000,0x0000}},
{0x00,0x3133,{0x11AA,0x0000,0x0000,0x0000}},
{0x00,0x3134,{0x1102,0x0000,0x0000,0x0000}},
{0x00,0x3135,{0x11AC,0x0000,0x0000,0x0000}},
{0x00,0x3136,{0x11AD,0x0000,0x0000,0x0000}},
{0x00,0x3137,{0x1103,0x0000,0x0000,0x0000}},
{0x00,0x3138,{0x1104,0x0000,0x0000,0x0000}},
{0x00,0x3139,{0x1105,0x0000,0x0000,0x0000}},
{0x00,0x313A,{0x11B0,0x0000,0x0000,0x0000}},
{0x00,0x313B,{0x11B1,0x0000,0x0000,0x0000}},
{0x00,0x313C,{0x11B2,0x0000,0x0000,0x0000}},
{0x00,0x313D,{0x11B3,0x0000,0x0000,0x0000}},
{0x00,0x313E,{0x11B4,0x0000,0x0000,0x0000}},
{0x00,0x313F,{0x11B5,0x0000,0x0000,0x0000}},
{0x00,0x3140,{0x111A,0x0000,0x0000,0x0000}},
{0x00,0x3141,{0x1106,0x0000,0x0000,0x0000}},
{0x00,0x3142,{0x1107,0x0000,0x0000,0x0000}},
{0x00,0x3143,{0x1108,0x0000,0x0000,0x0000}},
{0x00,0x3144,{0x1121,0x0000,0x0000,0x0000}},
{0x00,0x3145,{0x1109,0x0000,0x0000,0x0000}},
{0x00,0x3146,{0x110A,0x0000,0x0000,0x0000}},
{0x00,0x3147,{0x110B,0x0000,0x0000,0x0000}},
{0x00,0x3148,{0x110C,0x0000,0x0000,0x0000}},
{0x00,0x3149,{0x110D,0x0000,0x0000,0x0000}},
{0x00,0x314A,{0x110E,0x0000,0x0000,0x0000}},
{0x00,0x314B,{0x110F,0x0000,0x0000,0x0000}},
{0x00,0x314C,{0x1110,0x0000,0x0000,0x0000}},
{0x00,0x314D,{0x1111,0x0000,0x0000,0x0000}},
{0x00,0x314E,{0x1112,0x0000,0x0000,0x0000}},
{0x00,0x314F,{0x1161,0x0000,0x0000,0x0000}},
{0x00,0x3150,{0x1162,0x0000,0x0000,0x0000}},
{0x00,0x3151,{0x1163,0x0000,0x0000,0x0000}},
{0x00,0x3152,{0x1164,0x0000,0x0000,0x0000}},
{0x00,0x3153,{0x1165,0x0000,0x0000,0x0000}},
{0x00,0x3154,{0x1166,0x0000,0x0000,0x0000}},
{0x00,0x3155,{0x1167,0x0000,0x0000,0x0000}},
{0x00,0x3156,{0x1168,0x0000,0x0000,0x0000}},
{0x00,0x3157,{0x1169,0x0000,0x0000,0x0000}},
{0x00,0x3158,{0x116A,0x0000,0x0000,0x0000}},
{0x00,0x3159,{0x116B,0x0000,0x0000,0x0000}},
{0x00,0x315A,{0x116C,0x0000,0x0000,0x0000}},
{0x00,0x315B,{0x116D,0x0000,0x0000,0x0000}},
{0x00,0x315C,{0x116E,0x0000,0x0000,0x0000}},
{0x00,0x315D,{0x116F,0x0000,0x0000,0x0000}},
{0x00,0x315E,{0x1170,0x0000,0x0000,0x0000}},
{0x00,0x315F,{0x1171,0x0000,0x0000,0x0000}},
{0x00,0x3160,{0x1172,0x0000,0x0000,0x0000}},
{0x00,0x3161,{0x1173,0x0000,0x0000,0x0000}},
{0x00,0x3162,{0x1174,0x0000,0x0000,0x0000}},
{0x00,0x3163,{0x1175,0x0000,0x0000,0x0000}},
{0x00,0x3164,{0x1160,0x0000,0x0000,0x0000}},
{0x00,0x3165,{0x1114,0x0000,0x0000,0x0000}},
{0x00,0x3166,{0x1115,0x0000,0x0000,0x0000}},
{0x00,0x3167,{0x11C7,0x0000,0x0000,0x0000}},
{0x00,0x3168,{0x11C8,0x0000,0x0000,0x0000}},
{0x00,0x3169,{0x11CC,0x0000,0x0000,0x0000}},
{0x00,0x316A,{0x11CE,0x0000,0x0000,0x0000}},
{0x00,0x316B,{0x11D3,0x0000,0x0000,0x0000}},
{0x00,0x316C,{0x11D7,0x0000,0x0000,0x0000}},
{0x00,0x316D,{0x11D9,0x0000,0x0000,0x0000}},
{0x00,0x316E,{0x111C,0x0000,0x0000,0x0000}},
{0x00,0x316F,{0x11DD,0x0000,0x0000,0x0000}},
{0x00,0x3170,{0x11DF,0x0000,0x0000,0x0000}},
{0x00,0x3171,{0x111D,0x0000,0x0000,0x0000}},
{0x00,0x3172,{0x111E,0x0000,0x0000,0x0000}},
{0x00,0x3173,{0x1120,0x0000,0x0000,0x0000}},
{0x00,0x3174,{0x1122,0x0000,0x0000,0x0000}},
{0x00,0x3175,{0x1123,0x0000,0x0000,0x0000}},
{0x00,0x3176,{0x1127,0x0000,0x0000,0x0000}},
{0x00,0x3177,{0x1129,0x0000,0x0000,0x0000}},
{0x00,0x3178,{0x112B,0x0000,0x0000,0x0000}},
{0x00,0x3179,{0x112C,0x0000,0x0000,0x0000}},
{0x00,0x317A,{0x112D,0x0000,0x0000,0x0000}},
{0x00,0x317B,{0x112E,0x0000,0x0000,0x0000}},
{0x00,0x317C,{0x112F,0x0000,0x0000,0x0000}},
{0x00,0x317D,{0x1132,0x0000,0x0000,0x0000}},
{0x00,0x317E,{0x1136,0x0000,0x0000,0x0000}},
{0x00,0x317F,{0x1140,0x0000,0x0000,0x0000}},
{0x00,0x3180,{0x1147,0x0000,0x0000,0x0000}},
{0x00,0x3181,{0x114C,0x0000,0x0000,0x0000}},
{0x00,0x3182,{0x11F1,0x0000,0x0000,0x0000}},
{0x00,0x3183,{0x11F2,0x0000,0x0000,0x0000}},
{0x00,0x3184,{0x1157,0x0000,0x0000,0x0000}},
{0x00,0x3185,{0x1158,0x0000,0x0000,0x0000}},
{0x00,0x3186,{0x1159,0x0000,0x0000,0x0000}},
{0x00,0x3187,{0x1184,0x0000,0x0000,0x0000}},
{0x00,0x3188,{0x1185,0x0000,0x0000,0x0000}},
{0x00,0x3189,{0x1188,0x0000,0x0000,0x0000}},
{0x00,0x318A,{0x1191,0x0000,0x0000,0x0000}},
{0x00,0x318B,{0x1192,0x0000,0x0000,0x0000}},
{0x00,0x318C,{0x1194,0x0000,0x0000,0x0000}},
{0x00,0x318D,{0x119E,0x0000,0x0000,0x0000}},
{0x00,0x318E,{0x11A1,0x0000,0x0000,0x0000}},
{0x00,0x3192,{0x4E00,0x0000,0x0000,0x0000}},
{0x00,0x3193,{0x4E8C,0x0000,0x0000,0x0000}},
{0x00,0x3194,{0x4E09,0x0000,0x0000,0x0000}},
{0x00,0x3195,{0x56DB,0x0000,0x0000,0x0000}},
{0x00,0x3196,{0x4E0A,0x0000,0x0000,0x0000}},
{0x00,0x3197,{0x4E2D,0x0000,0x0000,0x0000}},
{0x00,0x3198,{0x4E0B,0x0000,0x0000,0x0000}},
{0x00,0x3199,{0x7532,0x0000,0x0000,0x0000}},
{0x00,0x319A,{0x4E59,0x0000,0x0000,0x0000}},
{0x00,0x319B,{0x4E19,0x0000,0x0000,0x0000}},
{0x00,0x319C,{0x4E01,0x0000,0x0000,0x0000}},
{0x00,0x319D,{0x5929,0x0000,0x0000,0x0000}},
{0x00,0x319E,{0x5730,0x0000,0x0000,0x0000}},
{0x00,0x319F,{0x4EBA,0x0000,0x0000,0x0000}},
{0x00,0x3200,{0x0028,0x1100,0x1100,0x0029}},
{0x00,0x3201,{0x0028,0x1102,0x1102,0x0029}},
{0x00,0x3202,{0x0028,0x1103,0x1103,0x0029}},
{0x00,0x3203,{0x0028,0x1105,0x1105,0x0029}},
{0x00,0x3204,{0x0028,0x1106,0x1106,0x0029}},
{0x00,0x3205,{0x0028,0x1107,0x1107,0x0029}},
{0x00,0x3206,{0x0028,0x1109,0x1109,0x0029}},
{0x00,0x3207,{0x0028,0x110B,0x110B,0x0029}},
{0x00,0x3208,{0x0028,0x110C,0x110C,0x0029}},
{0x00,0x3209,{0x0028,0x110E,0x110E,0x0029}},
{0x00,0x320A,{0x0028,0x110F,0x110F,0x0029}},
{0x00,0x320B,{0x0028,0x1110,0x1110,0x0029}},
{0x00,0x320C,{0x0028,0x1111,0x1111,0x0029}},
{0x00,0x320D,{0x0028,0x1112,0x1112,0x0029}},
{0x00,0x3220,{0x0028,0x4E00,0x4E00,0x0029}},
{0x00,0x3221,{0x0028,0x4E8C,0x4E8C,0x0029}},
{0x00,0x3222,{0x0028,0x4E09,0x4E09,0x0029}},
{0x00,0x3223,{0x0028,0x56DB,0x56DB,0x0029}},
{0x00,0x3224,{0x0028,0x4E94,0x4E94,0x0029}},
{0x00,0x3225,{0x0028,0x516D,0x516D,0x0029}},
{0x00,0x3226,{0x0028,0x4E03,0x4E03,0x0029}},
{0x00,0x3227,{0x0028,0x516B,0x516B,0x0029}},
{0x00,0x3228,{0x0028,0x4E5D,0x4E5D,0x0029}},
{0x00,0x3229,{0x0028,0x5341,0x5341,0x0029}},
{0x00,0x322A,{0x0028,0x6708,0x6708,0x0029}},
{0x00,0x322B,{0x0028,0x706B,0x706B,0x0029}},
{0x00,0x322C,{0x0028,0x6C34,0x6C34,0x0029}},
{0x00,0x322D,{0x0028,0x6728,0x6728,0x0029}},
{0x00,0x322E,{0x0028,0x91D1,0x91D1,0x0029}},
{0x00,0x322F,{0x0028,0x571F,0x571F,0x0029}},
{0x00,0x3230,{0x0028,0x65E5,0x65E5,0x0029}},
{0x00,0x3231,{0x0028,0x682A,0x682A,0x0029}},
{0x00,0x3232,{0x0028,0x6709,0x6709,0x0029}},
{0x00,0x3233,{0x0028,0x793E,0x793E,0x0029}},
{0x00,0x3234,{0x0028,0x540D,0x540D,0x0029}},
{0x00,0x3235,{0x0028,0x7279,0x7279,0x0029}},
{0x00,0x3236,{0x0028,0x8CA1,0x8CA1,0x0029}},
{0x00,0x3237,{0x0028,0x795D,0x795D,0x0029}},
{0x00,0x3238,{0x0028,0x52B4,0x52B4,0x0029}},
{0x00,0x3239,{0x0028,0x4EE3,0x4EE3,0x0029}},
{0x00,0x323A,{0x0028,0x547C,0x547C,0x0029}},
{0x00,0x323B,{0x0028,0x5B66,0x5B66,0x0029}},
{0x00,0x323C,{0x0028,0x76E3,0x76E3,0x0029}},
{0x00,0x323D,{0x0028,0x4F01,0x4F01,0x0029}},
{0x00,0x323E,{0x0028,0x8CC7,0x8CC7,0x0029}},
{0x00,0x323F,{0x0028,0x5354,0x5354,0x0029}},
{0x00,0x3240,{0x0028,0x796D,0x796D,0x0029}},
{0x00,0x3241,{0x0028,0x4F11,0x4F11,0x0029}},
{0x00,0x3242,{0x0028,0x81EA,0x81EA,0x0029}},
{0x00,0x3243,{0x0028,0x81F3,0x81F3,0x0029}},
{0x00,0x3250,{0x0050,0x0054,0x0054,0x0045}},
{0x00,0x3251,{0x0032,0x0031,0x0031,0x0000}},
{0x00,0x3252,{0x0032,0x0032,0x0032,0x0000}},
{0x00,0x3253,{0x0032,0x0033,0x0033,0x0000}},
{0x00,0x3254,{0x0032,0x0034,0x0034,0x0000}},
{0x00,0x3255,{0x0032,0x0035,0x0035,0x0000}},
{0x00,0x3256,{0x0032,0x0036,0x0036,0x0000}},
{0x00,0x3257,{0x0032,0x0037,0x0037,0x0000}},
{0x00,0x3258,{0x0032,0x0038,0x0038,0x0000}},
{0x00,0x3259,{0x0032,0x0039,0x0039,0x0000}},
{0x00,0x325A,{0x0033,0x0030,0x0030,0x0000}},
{0x00,0x325B,{0x0033,0x0031,0x0031,0x0000}},
{0x00,0x325C,{0x0033,0x0032,0x0032,0x0000}},
{0x00,0x325D,{0x0033,0x0033,0x0033,0x0000}},
{0x00,0x325E,{0x0033,0x0034,0x0034,0x0000}},
{0x00,0x325F,{0x0033,0x0035,0x0035,0x0000}},
{0x00,0x3260,{0x1100,0x0000,0x0000,0x0000}},
{0x00,0x3261,{0x1102,0x0000,0x0000,0x0000}},
{0x00,0x3262,{0x1103,0x0000,0x0000,0x0000}},
{0x00,0x3263,{0x1105,0x0000,0x0000,0x0000}},
{0x00,0x3264,{0x1106,0x0000,0x0000,0x0000}},
{0x00,0x3265,{0x1107,0x0000,0x0000,0x0000}},
{0x00,0x3266,{0x1109,0x0000,0x0000,0x0000}},
{0x00,0x3267,{0x110B,0x0000,0x0000,0x0000}},
{0x00,0x3268,{0x110C,0x0000,0x0000,0x0000}},
{0x00,0x3269,{0x110E,0x0000,0x0000,0x0000}},
{0x00,0x326A,{0x110F,0x0000,0x0000,0x0000}},
{0x00,0x326B,{0x1110,0x0000,0x0000,0x0000}},
{0x00,0x326C,{0x1111,0x0000,0x0000,0x0000}},
{0x00,0x326D,{0x1112,0x0000,0x0000,0x0000}},
{0x00,0x326E,{0x1100,0x1161,0x1161,0x0000}},
{0x00,0x326F,{0x1102,0x1161,0x1161,0x0000}},
{0x00,0x3270,{0x1103,0x1161,0x1161,0x0000}},
{0x00,0x3271,{0x1105,0x1161,0x1161,0x0000}},
{0x00,0x3272,{0x1106,0x1161,0x1161,0x0000}},
{0x00,0x3273,{0x1107,0x1161,0x1161,0x0000}},
{0x00,0x3274,{0x1109,0x1161,0x1161,0x0000}},
{0x00,0x3275,{0x110B,0x1161,0x1161,0x0000}},
{0x00,0x3276,{0x110C,0x1161,0x1161,0x0000}},
{0x00,0x3277,{0x110E,0x1161,0x1161,0x0000}},
{0x00,0x3278,{0x110F,0x1161,0x1161,0x0000}},
{0x00,0x3279,{0x1110,0x1161,0x1161,0x0000}},
{0x00,0x327A,{0x1111,0x1161,0x1161,0x0000}},
{0x00,0x327B,{0x1112,0x1161,0x1161,0x0000}},
{0x00,0x327E,{0x110B,0x116E,0x116E,0x0000}},
{0x00,0x3280,{0x4E00,0x0000,0x0000,0x0000}},
{0x00,0x3281,{0x4E8C,0x0000,0x0000,0x0000}},
{0x00,0x3282,{0x4E09,0x0000,0x0000,0x0000}},
{0x00,0x3283,{0x56DB,0x0000,0x0000,0x0000}},
{0x00,0x3284,{0x4E94,0x0000,0x0000,0x0000}},
{0x00,0x3285,{0x516D,0x0000,0x0000,0x0000}},
{0x00,0x3286,{0x4E03,0x0000,0x0000,0x0000}},
{0x00,0x3287,{0x516B,0x0000,0x0000,0x0000}},
{0x00,0x3288,{0x4E5D,0x0000,0x0000,0x0000}},
{0x00,0x3289,{0x5341,0x0000,0x0000,0x0000}},
{0x00,0x328A,{0x6708,0x0000,0x0000,0x0000}},
{0x00,0x328B,{0x706B,0x0000,0x0000,0x0000}},
{0x00,0x328C,{0x6C34,0x0000,0x0000,0x0000}},
{0x00,0x328D,{0x6728,0x0000,0x0000,0x0000}},
{0x00,0x328E,{0x91D1,0x0000,0x0000,0x0000}},
{0x00,0x328F,{0x571F,0x0000,0x0000,0x0000}},
{0x00,0x3290,{0x65E5,0x0000,0x0000,0x0000}},
{0x00,0x3291,{0x682A,0x0000,0x0000,0x0000}},
{0x00,0x3292,{0x6709,0x0000,0x0000,0x0000}},
{0x00,0x3293,{0x793E,0x0000,0x0000,0x0000}},
{0x00,0x3294,{0x540D,0x0000,0x0000,0x0000}},
{0x00,0x3295,{0x7279,0x0000,0x0000,0x0000}},
{0x00,0x3296,{0x8CA1,0x0000,0x0000,0x0000}},
{0x00,0x3297,{0x795D,0x0000,0x0000,0x0000}},
{0x00,0x3298,{0x52B4,0x0000,0x0000,0x0000}},
{0x00,0x3299,{0x79D8,0x0000,0x0000,0x0000}},
{0x00,0x329A,{0x7537,0x0000,0x0000,0x0000}},
{0x00,0x329B,{0x5973,0x0000,0x0000,0x0000}},
{0x00,0x329C,{0x9069,0x0000,0x0000,0x0000}},
{0x00,0x329D,{0x512A,0x0000,0x0000,0x0000}},
{0x00,0x329E,{0x5370,0x0000,0x0000,0x0000}},
{0x00,0x329F,{0x6CE8,0x0000,0x0000,0x0000}},
{0x00,0x32A0,{0x9805,0x0000,0x0000,0x0000}},
{0x00,0x32A1,{0x4F11,0x0000,0x0000,0x0000}},
{0x00,0x32A2,{0x5199,0x0000,0x0000,0x0000}},
{0x00,0x32A3,{0x6B63,0x0000,0x0000,0x0000}},
{0x00,0x32A4,{0x4E0A,0x0000,0x0000,0x0000}},
{0x00,0x32A5,{0x4E2D,0x0000,0x0000,0x0000}},
{0x00,0x32A6,{0x4E0B,0x0000,0x0000,0x0000}},
{0x00,0x32A7,{0x5DE6,0x0000,0x0000,0x0000}},
{0x00,0x32A8,{0x53F3,0x0000,0x0000,0x0000}},
{0x00,0x32A9,{0x533B,0x0000,0x0000,0x0000}},
{0x00,0x32AA,{0x5B97,0x0000,0x0000,0x0000}},
{0x00,0x32AB,{0x5B66,0x0000,0x0000,0x0000}},
{0x00,0x32AC,{0x76E3,0x0000,0x0000,0x0000}},
{0x00,0x32AD,{0x4F01,0x0000,0x0000,0x0000}},
{0x00,0x32AE,{0x8CC7,0x0000,0x0000,0x0000}},
{0x00,0x32AF,{0x5354,0x0000,0x0000,0x0000}},
{0x00,0x32B0,{0x591C,0x0000,0x0000,0x0000}},
{0x00,0x32B1,{0x0033,0x0036,0x0036,0x0000}},
{0x00,0x32B2,{0x0033,0x0037,0x0037,0x0000}},
{0x00,0x32B3,{0x0033,0x0038,0x0038,0x0000}},
{0x00,0x32B4,{0x0033,0x0039,0x0039,0x0000}},
{0x00,0x32B5,{0x0034,0x0030,0x0030,0x0000}},
{0x00,0x32B6,{0x0034,0x0031,0x0031,0x0000}},
{0x00,0x32B7,{0x0034,0x0032,0x0032,0x0000}},
{0x00,0x32B8,{0x0034,0x0033,0x0033,0x0000}},
{0x00,0x32B9,{0x0034,0x0034,0x0034,0x0000}},
{0x00,0x32BA,{0x0034,0x0035,0x0035,0x0000}},
{0x00,0x32BB,{0x0034,0x0036,0x0036,0x0000}},
{0x00,0x32BC,{0x0034,0x0037,0x0037,0x0000}},
{0x00,0x32BD,{0x0034,0x0038,0x0038,0x0000}},
{0x00,0x32BE,{0x0034,0x0039,0x0039,0x0000}},
{0x00,0x32BF,{0x0035,0x0030,0x0030,0x0000}},
{0x00,0x32C0,{0x0031,0x6708,0x6708,0x0000}},
{0x00,0x32C1,{0x0032,0x6708,0x6708,0x0000}},
{0x00,0x32C2,{0x0033,0x6708,0x6708,0x0000}},
{0x00,0x32C3,{0x0034,0x6708,0x6708,0x0000}},
{0x00,0x32C4,{0x0035,0x6708,0x6708,0x0000}},
{0x00,0x32C5,{0x0036,0x6708,0x6708,0x0000}},
{0x00,0x32C6,{0x0037,0x6708,0x6708,0x0000}},
{0x00,0x32C7,{0x0038,0x6708,0x6708,0x0000}},
{0x00,0x32C8,{0x0039,0x6708,0x6708,0x0000}},
{0x00,0x32C9,{0x0031,0x0030,0x0030,0x6708}},
{0x00,0x32CA,{0x0031,0x0031,0x0031,0x6708}},
{0x00,0x32CB,{0x0031,0x0032,0x0032,0x6708}},
{0x00,0x32CC,{0x0048,0x0067,0x0067,0x0000}},
{0x00,0x32CD,{0x0065,0x0072,0x0072,0x0067}},
{0x00,0x32CE,{0x0065,0x0056,0x0056,0x0000}},
{0x00,0x32CF,{0x004C,0x0054,0x0054,0x0044}},
{0x00,0x32D0,{0x30A2,0x0000,0x0000,0x0000}},
{0x00,0x32D1,{0x30A4,0x0000,0x0000,0x0000}},
{0x00,0x32D2,{0x30A6,0x0000,0x0000,0x0000}},
{0x00,0x32D3,{0x30A8,0x0000,0x0000,0x0000}},
{0x00,0x32D4,{0x30AA,0x0000,0x0000,0x0000}},
{0x00,0x32D5,{0x30AB,0x0000,0x0000,0x0000}},
{0x00,0x32D6,{0x30AD,0x0000,0x0000,0x0000}},
{0x00,0x32D7,{0x30AF,0x0000,0x0000,0x0000}},
{0x00,0x32D8,{0x30B1,0x0000,0x0000,0x0000}},
{0x00,0x32D9,{0x30B3,0x0000,0x0000,0x0000}},
{0x00,0x32DA,{0x30B5,0x0000,0x0000,0x0000}},
{0x00,0x32DB,{0x30B7,0x0000,0x0000,0x0000}},
{0x00,0x32DC,{0x30B9,0x0000,0x0000,0x0000}},
{0x00,0x32DD,{0x30BB,0x0000,0x0000,0x0000}},
{0x00,0x32DE,{0x30BD,0x0000,0x0000,0x0000}},
{0x00,0x32DF,{0x30BF,0x0000,0x0000,0x0000}},
{0x00,0x32E0,{0x30C1,0x0000,0x0000,0x0000}},
{0x00,0x32E1,{0x30C4,0x0000,0x0000,0x0000}},
{0x00,0x32E2,{0x30C6,0x0000,0x0000,0x0000}},
{0x00,0x32E3,{0x30C8,0x0000,0x0000,0x0000}},
{0x00,0x32E4,{0x30CA,0x0000,0x0000,0x0000}},
{0x00,0x32E5,{0x30CB,0x0000,0x0000,0x0000}},
{0x00,0x32E6,{0x30CC,0x0000,0x0000,0x0000}},
{0x00,0x32E7,{0x30CD,0x0000,0x0000,0x0000}},
{0x00,0x32E8,{0x30CE,0x0000,0x0000,0x0000}},
{0x00,0x32E9,{0x30CF,0x0000,0x0000,0x0000}},
{0x00,0x32EA,{0x30D2,0x0000,0x0000,0x0000}},
{0x00,0x32EB,{0x30D5,0x0000,0x0000,0x0000}},
{0x00,0x32EC,{0x30D8,0x0000,0x0000,0x0000}},
{0x00,0x32ED,{0x30DB,0x0000,0x0000,0x0000}},
{0x00,0x32EE,{0x30DE,0x0000,0x0000,0x0000}},
{0x00,0x32EF,{0x30DF,0x0000,0x0000,0x0000}},
{0x00,0x32F0,{0x30E0,0x0000,0x0000,0x0000}},
{0x00,0x32F1,{0x30E1,0x0000,0x0000,0x0000}},
{0x00,0x32F2,{0x30E2,0x0000,0x0000,0x0000}},
{0x00,0x32F3,{0x30E4,0x0000,0x0000,0x0000}},
{0x00,0x32F4,{0x30E6,0x0000,0x0000,0x0000}},
{0x00,0x32F5,{0x30E8,0x0000,0x0000,0x0000}},
{0x00,0x32F6,{0x30E9,0x0000,0x0000,0x0000}},
{0x00,0x32F7,{0x30EA,0x0000,0x0000,0x0000}},
{0x00,0x32F8,{0x30EB,0x0000,0x0000,0x0000}},
{0x00,0x32F9,{0x30EC,0x0000,0x0000,0x0000}},
{0x00,0x32FA,{0x30ED,0x0000,0x0000,0x0000}},
{0x00,0x32FB,{0x30EF,0x0000,0x0000,0x0000}},
{0x00,0x32FC,{0x30F0,0x0000,0x0000,0x0000}},
{0x00,0x32FD,{0x30F1,0x0000,0x0000,0x0000}},
{0x00,0x32FE,{0x30F2,0x0000,0x0000,0x0000}},
{0x00,0x3303,{0x30A2,0x30FC,0x30FC,0x30EB}},
{0x00,0x3305,{0x30A4,0x30F3,0x30F3,0x30C1}},
{0x00,0x3306,{0x30A6,0x30A9,0x30A9,0x30F3}},
{0x00,0x3309,{0x30AA,0x30F3,0x30F3,0x30B9}},
{0x00,0x330A,{0x30AA,0x30FC,0x30FC,0x30E0}},
{0x00,0x330B,{0x30AB,0x30A4,0x30A4,0x30EA}},
{0x00,0x330E,{0x30AC,0x30ED,0x30ED,0x30F3}},
{0x00,0x330F,{0x30AC,0x30F3,0x30F3,0x30DE}},
{0x00,0x3310,{0x30AE,0x30AC,0x30AC,0x0000}},
{0x00,0x3311,{0x30AE,0x30CB,0x30CB,0x30FC}},
{0x00,0x3314,{0x30AD,0x30ED,0x30ED,0x0000}},
{0x00,0x3318,{0x30B0,0x30E9,0x30E9,0x30E0}},
{0x00,0x331C,{0x30B1,0x30FC,0x30FC,0x30B9}},
{0x00,0x331D,{0x30B3,0x30EB,0x30EB,0x30CA}},
{0x00,0x331E,{0x30B3,0x30FC,0x30FC,0x30DD}},
{0x00,0x3322,{0x30BB,0x30F3,0x30F3,0x30C1}},
{0x00,0x3323,{0x30BB,0x30F3,0x30F3,0x30C8}},
{0x00,0x3324,{0x30C0,0x30FC,0x30FC,0x30B9}},
{0x00,0x3325,{0x30C7,0x30B7,0x30B7,0x0000}},
{0x00,0x3326,{0x30C9,0x30EB,0x30EB,0x0000}},
{0x00,0x3327,{0x30C8,0x30F3,0x30F3,0x0000}},
{0x00,0x3328,{0x30CA,0x30CE,0x30CE,0x0000}},
{0x00,0x3329,{0x30CE,0x30C3,0x30C3,0x30C8}},
{0x00,0x332A,{0x30CF,0x30A4,0x30A4,0x30C4}},
{0x00,0x332C,{0x30D1,0x30FC,0x30FC,0x30C4}},
{0x00,0x332F,{0x30D4,0x30AF,0x30AF,0x30EB}},
{0x00,0x3330,{0x30D4,0x30B3,0x30B3,0x0000}},
{0x00,0x3331,{0x30D3,0x30EB,0x30EB,0x0000}},
{0x00,0x3335,{0x30D5,0x30E9,0x30E9,0x30F3}},
{0x00,0x3337,{0x30DA,0x30BD,0x30BD,0x0000}},
{0x00,0x3338,{0x30DA,0x30CB,0x30CB,0x30D2}},
{0x00,0x3339,{0x30D8,0x30EB,0x30EB,0x30C4}},
{0x00,0x333A,{0x30DA,0x30F3,0x30F3,0x30B9}},
{0x00,0x333B,{0x30DA,0x30FC,0x30FC,0x30B8}},
{0x00,0x333C,{0x30D9,0x30FC,0x30FC,0x30BF}},
{0x00,0x333E,{0x30DC,0x30EB,0x30EB,0x30C8}},
{0x00,0x333F,{0x30DB,0x30F3,0x30F3,0x0000}},
{0x00,0x3340,{0x30DD,0x30F3,0x30F3,0x30C9}},
{0x00,0x3341,{0x30DB,0x30FC,0x30FC,0x30EB}},
{0x00,0x3342,{0x30DB,0x30FC,0x30FC,0x30F3}},
{0x00,0x3344,{0x30DE,0x30A4,0x30A4,0x30EB}},
{0x00,0x3345,{0x30DE,0x30C3,0x30C3,0x30CF}},
{0x00,0x3346,{0x30DE,0x30EB,0x30EB,0x30AF}},
{0x00,0x3349,{0x30DF,0x30EA,0x30EA,0x0000}},
{0x00,0x334B,{0x30E1,0x30AC,0x30AC,0x0000}},
{0x00,0x334E,{0x30E4,0x30FC,0x30FC,0x30C9}},
{0x00,0x334F,{0x30E4,0x30FC,0x30FC,0x30EB}},
{0x00,0x3350,{0x30E6,0x30A2,0x30A2,0x30F3}},
{0x00,0x3352,{0x30EA,0x30E9,0x30E9,0x0000}},
{0x00,0x3353,{0x30EB,0x30D4,0x30D4,0x30FC}},
{0x00,0x3355,{0x30EC,0x30E0,0x30E0,0x0000}},
{0x00,0x3357,{0x30EF,0x30C3,0x30C3,0x30C8}},
{0x00,0x3358,{0x0030,0x70B9,0x70B9,0x0000}},
{0x00,0x3359,{0x0031,0x70B9,0x70B9,0x0000}},
{0x00,0x335A,{0x0032,0x70B9,0x70B9,0x0000}},
{0x00,0x335B,{0x0033,0x70B9,0x70B9,0x0000}},
{0x00,0x335C,{0x0034,0x70B9,0x70B9,0x0000}},
{0x00,0x335D,{0x0035,0x70B9,0x70B9,0x0000}},
{0x00,0x335E,{0x0036,0x70B9,0x70B9,0x0000}},
{0x00,0x335F,{0x0037,0x70B9,0x70B9,0x0000}},
{0x00,0x3360,{0x0038,0x70B9,0x70B9,0x0000}},
{0x00,0x3361,{0x0039,0x70B9,0x70B9,0x0000}},
{0x00,0x3362,{0x0031,0x0030,0x0030,0x70B9}},
{0x00,0x3363,{0x0031,0x0031,0x0031,0x70B9}},
{0x00,0x3364,{0x0031,0x0032,0x0032,0x70B9}},
{0x00,0x3365,{0x0031,0x0033,0x0033,0x70B9}},
{0x00,0x3366,{0x0031,0x0034,0x0034,0x70B9}},
{0x00,0x3367,{0x0031,0x0035,0x0035,0x70B9}},
{0x00,0x3368,{0x0031,0x0036,0x0036,0x70B9}},
{0x00,0x3369,{0x0031,0x0037,0x0037,0x70B9}},
{0x00,0x336A,{0x0031,0x0038,0x0038,0x70B9}},
{0x00,0x336B,{0x0031,0x0039,0x0039,0x70B9}},
{0x00,0x336C,{0x0032,0x0030,0x0030,0x70B9}},
{0x00,0x336D,{0x0032,0x0031,0x0031,0x70B9}},
{0x00,0x336E,{0x0032,0x0032,0x0032,0x70B9}},
{0x00,0x336F,{0x0032,0x0033,0x0033,0x70B9}},
{0x00,0x3370,{0x0032,0x0034,0x0034,0x70B9}},
{0x00,0x3372,{0x0064,0x0061,0x0061,0x0000}},
{0x00,0x3374,{0x0062,0x0061,0x0061,0x0072}},
{0x00,0x3376,{0x0070,0x0063,0x0063,0x0000}},
{0x00,0x3377,{0x0064,0x006D,0x006D,0x0000}},
{0x00,0x3378,{0x0064,0x006D,0x006D,0x00B2}},
{0x00,0x3379,{0x0064,0x006D,0x006D,0x00B3}},
{0x00,0x337A,{0x0049,0x0055,0x0055,0x0000}},
{0x00,0x337B,{0x5E73,0x6210,0x6210,0x0000}},
{0x00,0x337C,{0x662D,0x548C,0x548C,0x0000}},
{0x00,0x337D,{0x5927,0x6B63,0x6B63,0x0000}},
{0x00,0x337E,{0x660E,0x6CBB,0x6CBB,0x0000}},
{0x00,0x3388,{0x0063,0x0061,0x0061,0x006C}},
{0x00,0x338D,{0x03BC,0x0067,0x0067,0x0000}},
{0x00,0x338E,{0x006D,0x0067,0x0067,0x0000}},
{0x00,0x338F,{0x006B,0x0067,0x0067,0x0000}},
{0x00,0x3395,{0x03BC,0x2113,0x2113,0x0000}},
{0x00,0x3396,{0x006D,0x2113,0x2113,0x0000}},
{0x00,0x3397,{0x0064,0x2113,0x2113,0x0000}},
{0x00,0x3398,{0x006B,0x2113,0x2113,0x0000}},
{0x00,0x3399,{0x0066,0x006D,0x006D,0x0000}},
{0x00,0x339A,{0x006E,0x006D,0x006D,0x0000}},
{0x00,0x339B,{0x03BC,0x006D,0x006D,0x0000}},
{0x00,0x339C,{0x006D,0x006D,0x006D,0x0000}},
{0x00,0x339D,{0x0063,0x006D,0x006D,0x0000}},
{0x00,0x339E,{0x006B,0x006D,0x006D,0x0000}},
{0x00,0x339F,{0x006D,0x006D,0x006D,0x00B2}},
{0x00,0x33A0,{0x0063,0x006D,0x006D,0x00B2}},
{0x00,0x33A1,{0x006D,0x00B2,0x00B2,0x0000}},
{0x00,0x33A2,{0x006B,0x006D,0x006D,0x00B2}},
{0x00,0x33A3,{0x006D,0x006D,0x006D,0x00B3}},
{0x00,0x33A4,{0x0063,0x006D,0x006D,0x00B3}},
{0x00,0x33A5,{0x006D,0x00B3,0x00B3,0x0000}},
{0x00,0x33A6,{0x006B,0x006D,0x006D,0x00B3}},
{0x00,0x33A7,{0x006D,0x2215,0x2215,0x0073}},
{0x00,0x33AD,{0x0072,0x0061,0x0061,0x0064}},
{0x00,0x33B0,{0x0070,0x0073,0x0073,0x0000}},
{0x00,0x33B1,{0x006E,0x0073,0x0073,0x0000}},
{0x00,0x33B2,{0x03BC,0x0073,0x0073,0x0000}},
{0x00,0x33B3,{0x006D,0x0073,0x0073,0x0000}},
{0x00,0x33C4,{0x0063,0x0063,0x0063,0x0000}},
{0x00,0x33C5,{0x0063,0x0064,0x0064,0x0000}},
{0x00,0x33CA,{0x0068,0x0061,0x0061,0x0000}},
{0x00,0x33CC,{0x0069,0x006E,0x006E,0x0000}},
{0x00,0x33CF,{0x006B,0x0074,0x0074,0x0000}},
{0x00,0x33D0,{0x006C,0x006D,0x006D,0x0000}},
{0x00,0x33D1,{0x006C,0x006E,0x006E,0x0000}},
{0x00,0x33D2,{0x006C,0x006F,0x006F,0x0067}},
{0x00,0x33D3,{0x006C,0x0078,0x0078,0x0000}},
{0x00,0x33D4,{0x006D,0x0062,0x0062,0x0000}},
{0x00,0x33D5,{0x006D,0x0069,0x0069,0x006C}},
{0x00,0x33D6,{0x006D,0x006F,0x006F,0x006C}},
{0x00,0x33DB,{0x0073,0x0072,0x0072,0x0000}},
{0x00,0x33DE,{0x0056,0x2215,0x2215,0x006D}},
{0x00,0x33DF,{0x0041,0x2215,0x2215,0x006D}},
{0x00,0x33E0,{0x0031,0x65E5,0x65E5,0x0000}},
{0x00,0x33E1,{0x0032,0x65E5,0x65E5,0x0000}},
{0x00,0x33E2,{0x0033,0x65E5,0x65E5,0x0000}},
{0x00,0x33E3,{0x0034,0x65E5,0x65E5,0x0000}},
{0x00,0x33E4,{0x0035,0x65E5,0x65E5,0x0000}},
{0x00,0x33E5,{0x0036,0x65E5,0x65E5,0x0000}},
{0x00,0x33E6,{0x0037,0x65E5,0x65E5,0x0000}},
{0x00,0x33E7,{0x0038,0x65E5,0x65E5,0x0000}},
{0x00,0x33E8,{0x0039,0x65E5,0x65E5,0x0000}},
{0x00,0x33E9,{0x0031,0x0030,0x0030,0x65E5}},
{0x00,0x33EA,{0x0031,0x0031,0x0031,0x65E5}},
{0x00,0x33EB,{0x0031,0x0032,0x0032,0x65E5}},
{0x00,0x33EC,{0x0031,0x0033,0x0033,0x65E5}},
{0x00,0x33ED,{0x0031,0x0034,0x0034,0x65E5}},
{0x00,0x33EE,{0x0031,0x0035,0x0035,0x65E5}},
{0x00,0x33EF,{0x0031,0x0036,0x0036,0x65E5}},
{0x00,0x33F0,{0x0031,0x0037,0x0037,0x65E5}},
{0x00,0x33F1,{0x0031,0x0038,0x0038,0x65E5}},
{0x00,0x33F2,{0x0031,0x0039,0x0039,0x65E5}},
{0x00,0x33F3,{0x0032,0x0030,0x0030,0x65E5}},
{0x00,0x33F4,{0x0032,0x0031,0x0031,0x65E5}},
{0x00,0x33F5,{0x0032,0x0032,0x0032,0x65E5}},
{0x00,0x33F6,{0x0032,0x0033,0x0033,0x65E5}},
{0x00,0x33F7,{0x0032,0x0034,0x0034,0x65E5}},
{0x00,0x33F8,{0x0032,0x0035,0x0035,0x65E5}},
{0x00,0x33F9,{0x0032,0x0036,0x0036,0x65E5}},
{0x00,0x33FA,{0x0032,0x0037,0x0037,0x65E5}},
{0x00,0x33FB,{0x0032,0x0038,0x0038,0x65E5}},
{0x00,0x33FC,{0x0032,0x0039,0x0039,0x65E5}},
{0x00,0x33FD,{0x0033,0x0030,0x0030,0x65E5}},
{0x00,0x33FE,{0x0033,0x0031,0x0031,0x65E5}},
{0x00,0x33FF,{0x0067,0x0061,0x0061,0x006C}},
{0x00,0xFB20,{0x05E2,0x0000,0x0000,0x0000}},
{0x00,0xFB21,{0x05D0,0x0000,0x0000,0x0000}},
{0x00,0xFB22,{0x05D3,0x0000,0x0000,0x0000}},
{0x00,0xFB23,{0x05D4,0x0000,0x0000,0x0000}},
{0x00,0xFB24,{0x05DB,0x0000,0x0000,0x0000}},
{0x00,0xFB25,{0x05DC,0x0000,0x0000,0x0000}},
{0x00,0xFB26,{0x05DD,0x0000,0x0000,0x0000}},
{0x00,0xFB27,{0x05E8,0x0000,0x0000,0x0000}},
{0x00,0xFB28,{0x05EA,0x0000,0x0000,0x0000}},
{0x00,0xFB29,{0x002B,0x0000,0x0000,0x0000}},
{0x00,0xFB4F,{0x05D0,0x05DC,0x05DC,0x0000}},
{0x00,0xFB50,{0x0671,0x0000,0x0000,0x0000}},
{0x00,0xFB51,{0x0671,0x0000,0x0000,0x0000}},
{0x00,0xFB52,{0x067B,0x0000,0x0000,0x0000}},
{0x00,0xFB53,{0x067B,0x0000,0x0000,0x0000}},
{0x00,0xFB54,{0x067B,0x0000,0x0000,0x0000}},
{0x00,0xFB55,{0x067B,0x0000,0x0000,0x0000}},
{0x00,0xFB56,{0x067E,0x0000,0x0000,0x0000}},
{0x00,0xFB57,{0x067E,0x0000,0x0000,0x0000}},
{0x00,0xFB58,{0x067E,0x0000,0x0000,0x0000}},
{0x00,0xFB59,{0x067E,0x0000,0x0000,0x0000}},
{0x00,0xFB5A,{0x0680,0x0000,0x0000,0x0000}},
{0x00,0xFB5B,{0x0680,0x0000,0x0000,0x0000}},
{0x00,0xFB5C,{0x0680,0x0000,0x0000,0x0000}},
{0x00,0xFB5D,{0x0680,0x0000,0x0000,0x0000}},
{0x00,0xFB5E,{0x067A,0x0000,0x0000,0x0000}},
{0x00,0xFB5F,{0x067A,0x0000,0x0000,0x0000}},
{0x00,0xFB60,{0x067A,0x0000,0x0000,0x0000}},
{0x00,0xFB61,{0x067A,0x0000,0x0000,0x0000}},
{0x00,0xFB62,{0x067F,0x0000,0x0000,0x0000}},
{0x00,0xFB63,{0x067F,0x0000,0x0000,0x0000}},
{0x00,0xFB64,{0x067F,0x0000,0x0000,0x0000}},
{0x00,0xFB65,{0x067F,0x0000,0x0000,0x0000}},
{0x00,0xFB66,{0x0679,0x0000,0x0000,0x0000}},
{0x00,0xFB67,{0x0679,0x0000,0x0000,0x0000}},
{0x00,0xFB68,{0x0679,0x0000,0x0000,0x0000}},
{0x00,0xFB69,{0x0679,0x0000,0x0000,0x0000}},
{0x00,0xFB6A,{0x06A4,0x0000,0x0000,0x0000}},
{0x00,0xFB6B,{0x06A4,0x0000,0x0000,0x0000}},
{0x00,0xFB6C,{0x06A4,0x0000,0x0000,0x0000}},
{0x00,0xFB6D,{0x06A4,0x0000,0x0000,0x0000}},
{0x00,0xFB6E,{0x06A6,0x0000,0x0000,0x0000}},
{0x00,0xFB6F,{0x06A6,0x0000,0x0000,0x0000}},
{0x00,0xFB70,{0x06A6,0x0000,0x0000,0x0000}},
{0x00,0xFB71,{0x06A6,0x0000,0x0000,0x0000}},
{0x00,0xFB72,{0x0684,0x0000,0x0000,0x0000}},
{0x00,0xFB73,{0x0684,0x0000,0x0000,0x0000}},
{0x00,0xFB74,{0x0684,0x0000,0x0000,0x0000}},
{0x00,0xFB75,{0x0684,0x0000,0x0000,0x0000}},
{0x00,0xFB76,{0x0683,0x0000,0x0000,0x0000}},
{0x00,0xFB77,{0x0683,0x0000,0x0000,0x0000}},
{0x00,0xFB78,{0x0683,0x0000,0x0000,0x0000}},
{0x00,0xFB79,{0x0683,0x0000,0x0000,0x0000}},
{0x00,0xFB7A,{0x0686,0x0000,0x0000,0x0000}},
{0x00,0xFB7B,{0x0686,0x0000,0x0000,0x0000}},
{0x00,0xFB7C,{0x0686,0x0000,0x0000,0x0000}},
{0x00,0xFB7D,{0x0686,0x0000,0x0000,0x0000}},
{0x00,0xFB7E,{0x0687,0x0000,0x0000,0x0000}},
{0x00,0xFB7F,{0x0687,0x0000,0x0000,0x0000}},
{0x00,0xFB80,{0x0687,0x0000,0x0000,0x0000}},
{0x00,0xFB81,{0x0687,0x0000,0x0000,0x0000}},
{0x00,0xFB82,{0x068D,0x0000,0x0000,0x0000}},
{0x00,0xFB83,{0x068D,0x0000,0x0000,0x0000}},
{0x00,0xFB84,{0x068C,0x0000,0x0000,0x0000}},
{0x00,0xFB85,{0x068C,0x0000,0x0000,0x0000}},
{0x00,0xFB86,{0x068E,0x0000,0x0000,0x0000}},
{0x00,0xFB87,{0x068E,0x0000,0x0000,0x0000}},
{0x00,0xFB88,{0x0688,0x0000,0x0000,0x0000}},
{0x00,0xFB89,{0x0688,0x0000,0x0000,0x0000}},
{0x00,0xFB8A,{0x0698,0x0000,0x0000,0x0000}},
{0x00,0xFB8B,{0x0698,0x0000,0x0000,0x0000}},
{0x00,0xFB8C,{0x0691,0x0000,0x0000,0x0000}},
{0x00,0xFB8D,{0x0691,0x0000,0x0000,0x0000}},
{0x00,0xFB8E,{0x06A9,0x0000,0x0000,0x0000}},
{0x00,0xFB8F,{0x06A9,0x0000,0x0000,0x0000}},
{0x00,0xFB90,{0x06A9,0x0000,0x0000,0x0000}},
{0x00,0xFB91,{0x06A9,0x0000,0x0000,0x0000}},
{0x00,0xFB92,{0x06AF,0x0000,0x0000,0x0000}},
{0x00,0xFB93,{0x06AF,0x0000,0x0000,0x0000}},
{0x00,0xFB94,{0x06AF,0x0000,0x0000,0x0000}},
{0x00,0xFB95,{0x06AF,0x0000,0x0000,0x0000}},
{0x00,0xFB96,{0x06B3,0x0000,0x0000,0x0000}},
{0x00,0xFB97,{0x06B3,0x0000,0x0000,0x0000}},
{0x00,0xFB98,{0x06B3,0x0000,0x0000,0x0000}},
{0x00,0xFB99,{0x06B3,0x0000,0x0000,0x0000}},
{0x00,0xFB9A,{0x06B1,0x0000,0x0000,0x0000}},
{0x00,0xFB9B,{0x06B1,0x0000,0x0000,0x0000}},
{0x00,0xFB9C,{0x06B1,0x0000,0x0000,0x0000}},
{0x00,0xFB9D,{0x06B1,0x0000,0x0000,0x0000}},
{0x00,0xFB9E,{0x06BA,0x0000,0x0000,0x0000}},
{0x00,0xFB9F,{0x06BA,0x0000,0x0000,0x0000}},
{0x00,0xFBA0,{0x06BB,0x0000,0x0000,0x0000}},
{0x00,0xFBA1,{0x06BB,0x0000,0x0000,0x0000}},
{0x00,0xFBA2,{0x06BB,0x0000,0x0000,0x0000}},
{0x00,0xFBA3,{0x06BB,0x0000,0x0000,0x0000}},
{0x00,0xFBA4,{0x06C0,0x0000,0x0000,0x0000}},
{0x00,0xFBA5,{0x06C0,0x0000,0x0000,0x0000}},
{0x00,0xFBA6,{0x06C1,0x0000,0x0000,0x0000}},
{0x00,0xFBA7,{0x06C1,0x0000,0x0000,0x0000}},
{0x00,0xFBA8,{0x06C1,0x0000,0x0000,0x0000}},
{0x00,0xFBA9,{0x06C1,0x0000,0x0000,0x0000}},
{0x00,0xFBAA,{0x06BE,0x0000,0x0000,0x0000}},
{0x00,0xFBAB,{0x06BE,0x0000,0x0000,0x0000}},
{0x00,0xFBAC,{0x06BE,0x0000,0x0000,0x0000}},
{0x00,0xFBAD,{0x06BE,0x0000,0x0000,0x0000}},
{0x00,0xFBAE,{0x06D2,0x0000,0x0000,0x0000}},
{0x00,0xFBAF,{0x06D2,0x0000,0x0000,0x0000}},
{0x00,0xFBB0,{0x06D3,0x0000,0x0000,0x0000}},
{0x00,0xFBB1,{0x06D3,0x0000,0x0000,0x0000}},
{0x00,0xFBD3,{0x06AD,0x0000,0x0000,0x0000}},
{0x00,0xFBD4,{0x06AD,0x0000,0x0000,0x0000}},
{0x00,0xFBD5,{0x06AD,0x0000,0x0000,0x0000}},
{0x00,0xFBD6,{0x06AD,0x0000,0x0000,0x0000}},
{0x00,0xFBD7,{0x06C7,0x0000,0x0000,0x0000}},
{0x00,0xFBD8,{0x06C7,0x0000,0x0000,0x0000}},
{0x00,0xFBD9,{0x06C6,0x0000,0x0000,0x0000}},
{0x00,0xFBDA,{0x06C6,0x0000,0x0000,0x0000}},
{0x00,0xFBDB,{0x06C8,0x0000,0x0000,0x0000}},
{0x00,0xFBDC,{0x06C8,0x0000,0x0000,0x0000}},
{0x00,0xFBDD,{0x0677,0x0000,0x0000,0x0000}},
{0x00,0xFBDE,{0x06CB,0x0000,0x0000,0x0000}},
{0x00,0xFBDF,{0x06CB,0x0000,0x0000,0x0000}},
{0x00,0xFBE0,{0x06C5,0x0000,0x0000,0x0000}},
{0x00,0xFBE1,{0x06C5,0x0000,0x0000,0x0000}},
{0x00,0xFBE2,{0x06C9,0x0000,0x0000,0x0000}},
{0x00,0xFBE3,{0x06C9,0x0000,0x0000,0x0000}},
{0x00,0xFBE4,{0x06D0,0x0000,0x0000,0x0000}},
{0x00,0xFBE5,{0x06D0,0x0000,0x0000,0x0000}},
{0x00,0xFBE6,{0x06D0,0x0000,0x0000,0x0000}},
{0x00,0xFBE7,{0x06D0,0x0000,0x0000,0x0000}},
{0x00,0xFBE8,{0x0649,0x0000,0x0000,0x0000}},
{0x00,0xFBE9,{0x0649,0x0000,0x0000,0x0000}},
{0x00,0xFBEA,{0x0626,0x0627,0x0627,0x0000}},
{0x00,0xFBEB,{0x0626,0x0627,0x0627,0x0000}},
{0x00,0xFBEC,{0x0626,0x06D5,0x06D5,0x0000}},
{0x00,0xFBED,{0x0626,0x06D5,0x06D5,0x0000}},
{0x00,0xFBEE,{0x0626,0x0648,0x0648,0x0000}},
{0x00,0xFBEF,{0x0626,0x0648,0x0648,0x0000}},
{0x00,0xFBF0,{0x0626,0x06C7,0x06C7,0x0000}},
{0x00,0xFBF1,{0x0626,0x06C7,0x06C7,0x0000}},
{0x00,0xFBF2,{0x0626,0x06C6,0x06C6,0x0000}},
{0x00,0xFBF3,{0x0626,0x06C6,0x06C6,0x0000}},
{0x00,0xFBF4,{0x0626,0x06C8,0x06C8,0x0000}},
{0x00,0xFBF5,{0x0626,0x06C8,0x06C8,0x0000}},
{0x00,0xFBF6,{0x0626,0x06D0,0x06D0,0x0000}},
{0x00,0xFBF7,{0x0626,0x06D0,0x06D0,0x0000}},
{0x00,0xFBF8,{0x0626,0x06D0,0x06D0,0x0000}},
{0x00,0xFBF9,{0x0626,0x0649,0x0649,0x0000}},
{0x00,0xFBFA,{0x0626,0x0649,0x0649,0x0000}},
{0x00,0xFBFB,{0x0626,0x0649,0x0649,0x0000}},
{0x00,0xFBFC,{0x06CC,0x0000,0x0000,0x0000}},
{0x00,0xFBFD,{0x06CC,0x0000,0x0000,0x0000}},
{0x00,0xFBFE,{0x06CC,0x0000,0x0000,0x0000}},
{0x00,0xFBFF,{0x06CC,0x0000,0x0000,0x0000}},
{0x00,0xFC00,{0x0626,0x062C,0x062C,0x0000}},
{0x00,0xFC01,{0x0626,0x062D,0x062D,0x0000}},
{0x00,0xFC02,{0x0626,0x0645,0x0645,0x0000}},
{0x00,0xFC03,{0x0626,0x0649,0x0649,0x0000}},
{0x00,0xFC04,{0x0626,0x064A,0x064A,0x0000}},
{0x00,0xFC05,{0x0628,0x062C,0x062C,0x0000}},
{0x00,0xFC06,{0x0628,0x062D,0x062D,0x0000}},
{0x00,0xFC07,{0x0628,0x062E,0x062E,0x0000}},
{0x00,0xFC08,{0x0628,0x0645,0x0645,0x0000}},
{0x00,0xFC09,{0x0628,0x0649,0x0649,0x0000}},
{0x00,0xFC0A,{0x0628,0x064A,0x064A,0x0000}},
{0x00,0xFC0B,{0x062A,0x062C,0x062C,0x0000}},
{0x00,0xFC0C,{0x062A,0x062D,0x062D,0x0000}},
{0x00,0xFC0D,{0x062A,0x062E,0x062E,0x0000}},
{0x00,0xFC0E,{0x062A,0x0645,0x0645,0x0000}},
{0x00,0xFC0F,{0x062A,0x0649,0x0649,0x0000}},
{0x00,0xFC10,{0x062A,0x064A,0x064A,0x0000}},
{0x00,0xFC11,{0x062B,0x062C,0x062C,0x0000}},
{0x00,0xFC12,{0x062B,0x0645,0x0645,0x0000}},
{0x00,0xFC13,{0x062B,0x0649,0x0649,0x0000}},
{0x00,0xFC14,{0x062B,0x064A,0x064A,0x0000}},
{0x00,0xFC15,{0x062C,0x062D,0x062D,0x0000}},
{0x00,0xFC16,{0x062C,0x0645,0x0645,0x0000}},
{0x00,0xFC17,{0x062D,0x062C,0x062C,0x0000}},
{0x00,0xFC18,{0x062D,0x0645,0x0645,0x0000}},
{0x00,0xFC19,{0x062E,0x062C,0x062C,0x0000}},
{0x00,0xFC1A,{0x062E,0x062D,0x062D,0x0000}},
{0x00,0xFC1B,{0x062E,0x0645,0x0645,0x0000}},
{0x00,0xFC1C,{0x0633,0x062C,0x062C,0x0000}},
{0x00,0xFC1D,{0x0633,0x062D,0x062D,0x0000}},
{0x00,0xFC1E,{0x0633,0x062E,0x062E,0x0000}},
{0x00,0xFC1F,{0x0633,0x0645,0x0645,0x0000}},
{0x00,0xFC20,{0x0635,0x062D,0x062D,0x0000}},
{0x00,0xFC21,{0x0635,0x0645,0x0645,0x0000}},
{0x00,0xFC22,{0x0636,0x062C,0x062C,0x0000}},
{0x00,0xFC23,{0x0636,0x062D,0x062D,0x0000}},
{0x00,0xFC24,{0x0636,0x062E,0x062E,0x0000}},
{0x00,0xFC25,{0x0636,0x0645,0x0645,0x0000}},
{0x00,0xFC26,{0x0637,0x062D,0x062D,0x0000}},
{0x00,0xFC27,{0x0637,0x0645,0x0645,0x0000}},
{0x00,0xFC28,{0x0638,0x0645,0x0645,0x0000}},
{0x00,0xFC29,{0x0639,0x062C,0x062C,0x0000}},
{0x00,0xFC2A,{0x0639,0x0645,0x0645,0x0000}},
{0x00,0xFC2B,{0x063A,0x062C,0x062C,0x0000}},
{0x00,0xFC2C,{0x063A,0x0645,0x0645,0x0000}},
{0x00,0xFC2D,{0x0641,0x062C,0x062C,0x0000}},
{0x00,0xFC2E,{0x0641,0x062D,0x062D,0x0000}},
{0x00,0xFC2F,{0x0641,0x062E,0x062E,0x0000}},
{0x00,0xFC30,{0x0641,0x0645,0x0645,0x0000}},
{0x00,0xFC31,{0x0641,0x0649,0x0649,0x0000}},
{0x00,0xFC32,{0x0641,0x064A,0x064A,0x0000}},
{0x00,0xFC33,{0x0642,0x062D,0x062D,0x0000}},
{0x00,0xFC34,{0x0642,0x0645,0x0645,0x0000}},
{0x00,0xFC35,{0x0642,0x0649,0x0649,0x0000}},
{0x00,0xFC36,{0x0642,0x064A,0x064A,0x0000}},
{0x00,0xFC37,{0x0643,0x0627,0x0627,0x0000}},
{0x00,0xFC38,{0x0643,0x062C,0x062C,0x0000}},
{0x00,0xFC39,{0x0643,0x062D,0x062D,0x0000}},
{0x00,0xFC3A,{0x0643,0x062E,0x062E,0x0000}},
{0x00,0xFC3B,{0x0643,0x0644,0x0644,0x0000}},
{0x00,0xFC3C,{0x0643,0x0645,0x0645,0x0000}},
{0x00,0xFC3D,{0x0643,0x0649,0x0649,0x0000}},
{0x00,0xFC3E,{0x0643,0x064A,0x064A,0x0000}},
{0x00,0xFC3F,{0x0644,0x062C,0x062C,0x0000}},
{0x00,0xFC40,{0x0644,0x062D,0x062D,0x0000}},
{0x00,0xFC41,{0x0644,0x062E,0x062E,0x0000}},
{0x00,0xFC42,{0x0644,0x0645,0x0645,0x0000}},
{0x00,0xFC43,{0x0644,0x0649,0x0649,0x0000}},
{0x00,0xFC44,{0x0644,0x064A,0x064A,0x0000}},
{0x00,0xFC45,{0x0645,0x062C,0x062C,0x0000}},
{0x00,0xFC46,{0x0645,0x062D,0x062D,0x0000}},
{0x00,0xFC47,{0x0645,0x062E,0x062E,0x0000}},
{0x00,0xFC48,{0x0645,0x0645,0x0645,0x0000}},
{0x00,0xFC49,{0x0645,0x0649,0x0649,0x0000}},
{0x00,0xFC4A,{0x0645,0x064A,0x064A,0x0000}},
{0x00,0xFC4B,{0x0646,0x062C,0x062C,0x0000}},
{0x00,0xFC4C,{0x0646,0x062D,0x062D,0x0000}},
{0x00,0xFC4D,{0x0646,0x062E,0x062E,0x0000}},
{0x00,0xFC4E,{0x0646,0x0645,0x0645,0x0000}},
{0x00,0xFC4F,{0x0646,0x0649,0x0649,0x0000}},
{0x00,0xFC50,{0x0646,0x064A,0x064A,0x0000}},
{0x00,0xFC51,{0x0647,0x062C,0x062C,0x0000}},
{0x00,0xFC52,{0x0647,0x0645,0x0645,0x0000}},
{0x00,0xFC53,{0x0647,0x0649,0x0649,0x0000}},
{0x00,0xFC54,{0x0647,0x064A,0x064A,0x0000}},
{0x00,0xFC55,{0x064A,0x062C,0x062C,0x0000}},
{0x00,0xFC56,{0x064A,0x062D,0x062D,0x0000}},
{0x00,0xFC57,{0x064A,0x062E,0x062E,0x0000}},
{0x00,0xFC58,{0x064A,0x0645,0x0645,0x0000}},
{0x00,0xFC59,{0x064A,0x0649,0x0649,0x0000}},
{0x00,0xFC5A,{0x064A,0x064A,0x064A,0x0000}},
{0x00,0xFC5B,{0x0630,0x0670,0x0670,0x0000}},
{0x00,0xFC5C,{0x0631,0x0670,0x0670,0x0000}},
{0x00,0xFC5D,{0x0649,0x0670,0x0670,0x0000}},
{0x00,0xFC5E,{0x0020,0x064C,0x064C,0x0651}},
{0x00,0xFC5F,{0x0020,0x064D,0x064D,0x0651}},
{0x00,0xFC60,{0x0020,0x064E,0x064E,0x0651}},
{0x00,0xFC61,{0x0020,0x064F,0x064F,0x0651}},
{0x00,0xFC62,{0x0020,0x0650,0x0650,0x0651}},
{0x00,0xFC63,{0x0020,0x0651,0x0651,0x0670}},
{0x00,0xFC64,{0x0626,0x0631,0x0631,0x0000}},
{0x00,0xFC65,{0x0626,0x0632,0x0632,0x0000}},
{0x00,0xFC66,{0x0626,0x0645,0x0645,0x0000}},
{0x00,0xFC67,{0x0626,0x0646,0x0646,0x0000}},
{0x00,0xFC68,{0x0626,0x0649,0x0649,0x0000}},
{0x00,0xFC69,{0x0626,0x064A,0x064A,0x0000}},
{0x00,0xFC6A,{0x0628,0x0631,0x0631,0x0000}},
{0x00,0xFC6B,{0x0628,0x0632,0x0632,0x0000}},
{0x00,0xFC6C,{0x0628,0x0645,0x0645,0x0000}},
{0x00,0xFC6D,{0x0628,0x0646,0x0646,0x0000}},
{0x00,0xFC6E,{0x0628,0x0649,0x0649,0x0000}},
{0x00,0xFC6F,{0x0628,0x064A,0x064A,0x0000}},
{0x00,0xFC70,{0x062A,0x0631,0x0631,0x0000}},
{0x00,0xFC71,{0x062A,0x0632,0x0632,0x0000}},
{0x00,0xFC72,{0x062A,0x0645,0x0645,0x0000}},
{0x00,0xFC73,{0x062A,0x0646,0x0646,0x0000}},
{0x00,0xFC74,{0x062A,0x0649,0x0649,0x0000}},
{0x00,0xFC75,{0x062A,0x064A,0x064A,0x0000}},
{0x00,0xFC76,{0x062B,0x0631,0x0631,0x0000}},
{0x00,0xFC77,{0x062B,0x0632,0x0632,0x0000}},
{0x00,0xFC78,{0x062B,0x0645,0x0645,0x0000}},
{0x00,0xFC79,{0x062B,0x0646,0x0646,0x0000}},
{0x00,0xFC7A,{0x062B,0x0649,0x0649,0x0000}},
{0x00,0xFC7B,{0x062B,0x064A,0x064A,0x0000}},
{0x00,0xFC7C,{0x0641,0x0649,0x0649,0x0000}},
{0x00,0xFC7D,{0x0641,0x064A,0x064A,0x0000}},
{0x00,0xFC7E,{0x0642,0x0649,0x0649,0x0000}},
{0x00,0xFC7F,{0x0642,0x064A,0x064A,0x0000}},
{0x00,0xFC80,{0x0643,0x0627,0x0627,0x0000}},
{0x00,0xFC81,{0x0643,0x0644,0x0644,0x0000}},
{0x00,0xFC82,{0x0643,0x0645,0x0645,0x0000}},
{0x00,0xFC83,{0x0643,0x0649,0x0649,0x0000}},
{0x00,0xFC84,{0x0643,0x064A,0x064A,0x0000}},
{0x00,0xFC85,{0x0644,0x0645,0x0645,0x0000}},
{0x00,0xFC86,{0x0644,0x0649,0x0649,0x0000}},
{0x00,0xFC87,{0x0644,0x064A,0x064A,0x0000}},
{0x00,0xFC88,{0x0645,0x0627,0x0627,0x0000}},
{0x00,0xFC89,{0x0645,0x0645,0x0645,0x0000}},
{0x00,0xFC8A,{0x0646,0x0631,0x0631,0x0000}},
{0x00,0xFC8B,{0x0646,0x0632,0x0632,0x0000}},
{0x00,0xFC8C,{0x0646,0x0645,0x0645,0x0000}},
{0x00,0xFC8D,{0x0646,0x0646,0x0646,0x0000}},
{0x00,0xFC8E,{0x0646,0x0649,0x0649,0x0000}},
{0x00,0xFC8F,{0x0646,0x064A,0x064A,0x0000}},
{0x00,0xFC90,{0x0649,0x0670,0x0670,0x0000}},
{0x00,0xFC91,{0x064A,0x0631,0x0631,0x0000}},
{0x00,0xFC92,{0x064A,0x0632,0x0632,0x0000}},
{0x00,0xFC93,{0x064A,0x0645,0x0645,0x0000}},
{0x00,0xFC94,{0x064A,0x0646,0x0646,0x0000}},
{0x00,0xFC95,{0x064A,0x0649,0x0649,0x0000}},
{0x00,0xFC96,{0x064A,0x064A,0x064A,0x0000}},
{0x00,0xFC97,{0x0626,0x062C,0x062C,0x0000}},
{0x00,0xFC98,{0x0626,0x062D,0x062D,0x0000}},
{0x00,0xFC99,{0x0626,0x062E,0x062E,0x0000}},
{0x00,0xFC9A,{0x0626,0x0645,0x0645,0x0000}},
{0x00,0xFC9B,{0x0626,0x0647,0x0647,0x0000}},
{0x00,0xFC9C,{0x0628,0x062C,0x062C,0x0000}},
{0x00,0xFC9D,{0x0628,0x062D,0x062D,0x0000}},
{0x00,0xFC9E,{0x0628,0x062E,0x062E,0x0000}},
{0x00,0xFC9F,{0x0628,0x0645,0x0645,0x0000}},
{0x00,0xFCA0,{0x0628,0x0647,0x0647,0x0000}},
{0x00,0xFCA1,{0x062A,0x062C,0x062C,0x0000}},
{0x00,0xFCA2,{0x062A,0x062D,0x062D,0x0000}},
{0x00,0xFCA3,{0x062A,0x062E,0x062E,0x0000}},
{0x00,0xFCA4,{0x062A,0x0645,0x0645,0x0000}},
{0x00,0xFCA5,{0x062A,0x0647,0x0647,0x0000}},
{0x00,0xFCA6,{0x062B,0x0645,0x0645,0x0000}},
{0x00,0xFCA7,{0x062C,0x062D,0x062D,0x0000}},
{0x00,0xFCA8,{0x062C,0x0645,0x0645,0x0000}},
{0x00,0xFCA9,{0x062D,0x062C,0x062C,0x0000}},
{0x00,0xFCAA,{0x062D,0x0645,0x0645,0x0000}},
{0x00,0xFCAB,{0x062E,0x062C,0x062C,0x0000}},
{0x00,0xFCAC,{0x062E,0x0645,0x0645,0x0000}},
{0x00,0xFCAD,{0x0633,0x062C,0x062C,0x0000}},
{0x00,0xFCAE,{0x0633,0x062D,0x062D,0x0000}},
{0x00,0xFCAF,{0x0633,0x062E,0x062E,0x0000}},
{0x00,0xFCB0,{0x0633,0x0645,0x0645,0x0000}},
{0x00,0xFCB1,{0x0635,0x062D,0x062D,0x0000}},
{0x00,0xFCB2,{0x0635,0x062E,0x062E,0x0000}},
{0x00,0xFCB3,{0x0635,0x0645,0x0645,0x0000}},
{0x00,0xFCB4,{0x0636,0x062C,0x062C,0x0000}},
{0x00,0xFCB5,{0x0636,0x062D,0x062D,0x0000}},
{0x00,0xFCB6,{0x0636,0x062E,0x062E,0x0000}},
{0x00,0xFCB7,{0x0636,0x0645,0x0645,0x0000}},
{0x00,0xFCB8,{0x0637,0x062D,0x062D,0x0000}},
{0x00,0xFCB9,{0x0638,0x0645,0x0645,0x0000}},
{0x00,0xFCBA,{0x0639,0x062C,0x062C,0x0000}},
{0x00,0xFCBB,{0x0639,0x0645,0x0645,0x0000}},
{0x00,0xFCBC,{0x063A,0x062C,0x062C,0x0000}},
{0x00,0xFCBD,{0x063A,0x0645,0x0645,0x0000}},
{0x00,0xFCBE,{0x0641,0x062C,0x062C,0x0000}},
{0x00,0xFCBF,{0x0641,0x062D,0x062D,0x0000}},
{0x00,0xFCC0,{0x0641,0x062E,0x062E,0x0000}},
{0x00,0xFCC1,{0x0641,0x0645,0x0645,0x0000}},
{0x00,0xFCC2,{0x0642,0x062D,0x062D,0x0000}},
{0x00,0xFCC3,{0x0642,0x0645,0x0645,0x0000}},
{0x00,0xFCC4,{0x0643,0x062C,0x062C,0x0000}},
{0x00,0xFCC5,{0x0643,0x062D,0x062D,0x0000}},
{0x00,0xFCC6,{0x0643,0x062E,0x062E,0x0000}},
{0x00,0xFCC7,{0x0643,0x0644,0x0644,0x0000}},
{0x00,0xFCC8,{0x0643,0x0645,0x0645,0x0000}},
{0x00,0xFCC9,{0x0644,0x062C,0x062C,0x0000}},
{0x00,0xFCCA,{0x0644,0x062D,0x062D,0x0000}},
{0x00,0xFCCB,{0x0644,0x062E,0x062E,0x0000}},
{0x00,0xFCCC,{0x0644,0x0645,0x0645,0x0000}},
{0x00,0xFCCD,{0x0644,0x0647,0x0647,0x0000}},
{0x00,0xFCCE,{0x0645,0x062C,0x062C,0x0000}},
{0x00,0xFCCF,{0x0645,0x062D,0x062D,0x0000}},
{0x00,0xFCD0,{0x0645,0x062E,0x062E,0x0000}},
{0x00,0xFCD1,{0x0645,0x0645,0x0645,0x0000}},
{0x00,0xFCD2,{0x0646,0x062C,0x062C,0x0000}},
{0x00,0xFCD3,{0x0646,0x062D,0x062D,0x0000}},
{0x00,0xFCD4,{0x0646,0x062E,0x062E,0x0000}},
{0x00,0xFCD5,{0x0646,0x0645,0x0645,0x0000}},
{0x00,0xFCD6,{0x0646,0x0647,0x0647,0x0000}},
{0x00,0xFCD7,{0x0647,0x062C,0x062C,0x0000}},
{0x00,0xFCD8,{0x0647,0x0645,0x0645,0x0000}},
{0x00,0xFCD9,{0x0647,0x0670,0x0670,0x0000}},
{0x00,0xFCDA,{0x064A,0x062C,0x062C,0x0000}},
{0x00,0xFCDB,{0x064A,0x062D,0x062D,0x0000}},
{0x00,0xFCDC,{0x064A,0x062E,0x062E,0x0000}},
{0x00,0xFCDD,{0x064A,0x0645,0x0645,0x0000}},
{0x00,0xFCDE,{0x064A,0x0647,0x0647,0x0000}},
{0x00,0xFCDF,{0x0626,0x0645,0x0645,0x0000}},
{0x00,0xFCE0,{0x0626,0x0647,0x0647,0x0000}},
{0x00,0xFCE1,{0x0628,0x0645,0x0645,0x0000}},
{0x00,0xFCE2,{0x0628,0x0647,0x0647,0x0000}},
{0x00,0xFCE3,{0x062A,0x0645,0x0645,0x0000}},
{0x00,0xFCE4,{0x062A,0x0647,0x0647,0x0000}},
{0x00,0xFCE5,{0x062B,0x0645,0x0645,0x0000}},
{0x00,0xFCE6,{0x062B,0x0647,0x0647,0x0000}},
{0x00,0xFCE7,{0x0633,0x0645,0x0645,0x0000}},
{0x00,0xFCE8,{0x0633,0x0647,0x0647,0x0000}},
{0x00,0xFCE9,{0x0634,0x0645,0x0645,0x0000}},
{0x00,0xFCEA,{0x0634,0x0647,0x0647,0x0000}},
{0x00,0xFCEB,{0x0643,0x0644,0x0644,0x0000}},
{0x00,0xFCEC,{0x0643,0x0645,0x0645,0x0000}},
{0x00,0xFCED,{0x0644,0x0645,0x0645,0x0000}},
{0x00,0xFCEE,{0x0646,0x0645,0x0645,0x0000}},
{0x00,0xFCEF,{0x0646,0x0647,0x0647,0x0000}},
{0x00,0xFCF0,{0x064A,0x0645,0x0645,0x0000}},
{0x00,0xFCF1,{0x064A,0x0647,0x0647,0x0000}},
{0x00,0xFCF2,{0x0640,0x064E,0x064E,0x0651}},
{0x00,0xFCF3,{0x0640,0x064F,0x064F,0x0651}},
{0x00,0xFCF4,{0x0640,0x0650,0x0650,0x0651}},
{0x00,0xFCF5,{0x0637,0x0649,0x0649,0x0000}},
{0x00,0xFCF6,{0x0637,0x064A,0x064A,0x0000}},
{0x00,0xFCF7,{0x0639,0x0649,0x0649,0x0000}},
{0x00,0xFCF8,{0x0639,0x064A,0x064A,0x0000}},
{0x00,0xFCF9,{0x063A,0x0649,0x0649,0x0000}},
{0x00,0xFCFA,{0x063A,0x064A,0x064A,0x0000}},
{0x00,0xFCFB,{0x0633,0x0649,0x0649,0x0000}},
{0x00,0xFCFC,{0x0633,0x064A,0x064A,0x0000}},
{0x00,0xFCFD,{0x0634,0x0649,0x0649,0x0000}},
{0x00,0xFCFE,{0x0634,0x064A,0x064A,0x0000}},
{0x00,0xFCFF,{0x062D,0x0649,0x0649,0x0000}},
{0x00,0xFD00,{0x062D,0x064A,0x064A,0x0000}},
{0x00,0xFD01,{0x062C,0x0649,0x0649,0x0000}},
{0x00,0xFD02,{0x062C,0x064A,0x064A,0x0000}},
{0x00,0xFD03,{0x062E,0x0649,0x0649,0x0000}},
{0x00,0xFD04,{0x062E,0x064A,0x064A,0x0000}},
{0x00,0xFD05,{0x0635,0x0649,0x0649,0x0000}},
{0x00,0xFD06,{0x0635,0x064A,0x064A,0x0000}},
{0x00,0xFD07,{0x0636,0x0649,0x0649,0x0000}},
{0x00,0xFD08,{0x0636,0x064A,0x064A,0x0000}},
{0x00,0xFD09,{0x0634,0x062C,0x062C,0x0000}},
{0x00,0xFD0A,{0x0634,0x062D,0x062D,0x0000}},
{0x00,0xFD0B,{0x0634,0x062E,0x062E,0x0000}},
{0x00,0xFD0C,{0x0634,0x0645,0x0645,0x0000}},
{0x00,0xFD0D,{0x0634,0x0631,0x0631,0x0000}},
{0x00,0xFD0E,{0x0633,0x0631,0x0631,0x0000}},
{0x00,0xFD0F,{0x0635,0x0631,0x0631,0x0000}},
{0x00,0xFD10,{0x0636,0x0631,0x0631,0x0000}},
{0x00,0xFD11,{0x0637,0x0649,0x0649,0x0000}},
{0x00,0xFD12,{0x0637,0x064A,0x064A,0x0000}},
{0x00,0xFD13,{0x0639,0x0649,0x0649,0x0000}},
{0x00,0xFD14,{0x0639,0x064A,0x064A,0x0000}},
{0x00,0xFD15,{0x063A,0x0649,0x0649,0x0000}},
{0x00,0xFD16,{0x063A,0x064A,0x064A,0x0000}},
{0x00,0xFD17,{0x0633,0x0649,0x0649,0x0000}},
{0x00,0xFD18,{0x0633,0x064A,0x064A,0x0000}},
{0x00,0xFD19,{0x0634,0x0649,0x0649,0x0000}},
{0x00,0xFD1A,{0x0634,0x064A,0x064A,0x0000}},
{0x00,0xFD1B,{0x062D,0x0649,0x0649,0x0000}},
{0x00,0xFD1C,{0x062D,0x064A,0x064A,0x0000}},
{0x00,0xFD1D,{0x062C,0x0649,0x0649,0x0000}},
{0x00,0xFD1E,{0x062C,0x064A,0x064A,0x0000}},
{0x00,0xFD1F,{0x062E,0x0649,0x0649,0x0000}},
{0x00,0xFD20,{0x062E,0x064A,0x064A,0x0000}},
{0x00,0xFD21,{0x0635,0x0649,0x0649,0x0000}},
{0x00,0xFD22,{0x0635,0x064A,0x064A,0x0000}},
{0x00,0xFD23,{0x0636,0x0649,0x0649,0x0000}},
{0x00,0xFD24,{0x0636,0x064A,0x064A,0x0000}},
{0x00,0xFD25,{0x0634,0x062C,0x062C,0x0000}},
{0x00,0xFD26,{0x0634,0x062D,0x062D,0x0000}},
{0x00,0xFD27,{0x0634,0x062E,0x062E,0x0000}},
{0x00,0xFD28,{0x0634,0x0645,0x0645,0x0000}},
{0x00,0xFD29,{0x0634,0x0631,0x0631,0x0000}},
{0x00,0xFD2A,{0x0633,0x0631,0x0631,0x0000}},
{0x00,0xFD2B,{0x0635,0x0631,0x0631,0x0000}},
{0x00,0xFD2C,{0x0636,0x0631,0x0631,0x0000}},
{0x00,0xFD2D,{0x0634,0x062C,0x062C,0x0000}},
{0x00,0xFD2E,{0x0634,0x062D,0x062D,0x0000}},
{0x00,0xFD2F,{0x0634,0x062E,0x062E,0x0000}},
{0x00,0xFD30,{0x0634,0x0645,0x0645,0x0000}},
{0x00,0xFD31,{0x0633,0x0647,0x0647,0x0000}},
{0x00,0xFD32,{0x0634,0x0647,0x0647,0x0000}},
{0x00,0xFD33,{0x0637,0x0645,0x0645,0x0000}},
{0x00,0xFD34,{0x0633,0x062C,0x062C,0x0000}},
{0x00,0xFD35,{0x0633,0x062D,0x062D,0x0000}},
{0x00,0xFD36,{0x0633,0x062E,0x062E,0x0000}},
{0x00,0xFD37,{0x0634,0x062C,0x062C,0x0000}},
{0x00,0xFD38,{0x0634,0x062D,0x062D,0x0000}},
{0x00,0xFD39,{0x0634,0x062E,0x062E,0x0000}},
{0x00,0xFD3A,{0x0637,0x0645,0x0645,0x0000}},
{0x00,0xFD3B,{0x0638,0x0645,0x0645,0x0000}},
{0x00,0xFD3C,{0x0627,0x064B,0x064B,0x0000}},
{0x00,0xFD3D,{0x0627,0x064B,0x064B,0x0000}},
{0x00,0xFD50,{0x062A,0x062C,0x062C,0x0645}},
{0x00,0xFD51,{0x062A,0x062D,0x062D,0x062C}},
{0x00,0xFD52,{0x062A,0x062D,0x062D,0x062C}},
{0x00,0xFD53,{0x062A,0x062D,0x062D,0x0645}},
{0x00,0xFD54,{0x062A,0x062E,0x062E,0x0645}},
{0x00,0xFD55,{0x062A,0x0645,0x0645,0x062C}},
{0x00,0xFD56,{0x062A,0x0645,0x0645,0x062D}},
{0x00,0xFD57,{0x062A,0x0645,0x0645,0x062E}},
{0x00,0xFD58,{0x062C,0x0645,0x0645,0x062D}},
{0x00,0xFD59,{0x062C,0x0645,0x0645,0x062D}},
{0x00,0xFD5A,{0x062D,0x0645,0x0645,0x064A}},
{0x00,0xFD5B,{0x062D,0x0645,0x0645,0x0649}},
{0x00,0xFD5C,{0x0633,0x062D,0x062D,0x062C}},
{0x00,0xFD5D,{0x0633,0x062C,0x062C,0x062D}},
{0x00,0xFD5E,{0x0633,0x062C,0x062C,0x0649}},
{0x00,0xFD5F,{0x0633,0x0645,0x0645,0x062D}},
{0x00,0xFD60,{0x0633,0x0645,0x0645,0x062D}},
{0x00,0xFD61,{0x0633,0x0645,0x0645,0x062C}},
{0x00,0xFD62,{0x0633,0x0645,0x0645,0x0645}},
{0x00,0xFD63,{0x0633,0x0645,0x0645,0x0645}},
{0x00,0xFD64,{0x0635,0x062D,0x062D,0x062D}},
{0x00,0xFD65,{0x0635,0x062D,0x062D,0x062D}},
{0x00,0xFD66,{0x0635,0x0645,0x0645,0x0645}},
{0x00,0xFD67,{0x0634,0x062D,0x062D,0x0645}},
{0x00,0xFD68,{0x0634,0x062D,0x062D,0x0645}},
{0x00,0xFD69,{0x0634,0x062C,0x062C,0x064A}},
{0x00,0xFD6A,{0x0634,0x0645,0x0645,0x062E}},
{0x00,0xFD6B,{0x0634,0x0645,0x0645,0x062E}},
{0x00,0xFD6C,{0x0634,0x0645,0x0645,0x0645}},
{0x00,0xFD6D,{0x0634,0x0645,0x0645,0x0645}},
{0x00,0xFD6E,{0x0636,0x062D,0x062D,0x0649}},
{0x00,0xFD6F,{0x0636,0x062E,0x062E,0x0645}},
{0x00,0xFD70,{0x0636,0x062E,0x062E,0x0645}},
{0x00,0xFD71,{0x0637,0x0645,0x0645,0x062D}},
{0x00,0xFD72,{0x0637,0x0645,0x0645,0x062D}},
{0x00,0xFD73,{0x0637,0x0645,0x0645,0x0645}},
{0x00,0xFD74,{0x0637,0x0645,0x0645,0x064A}},
{0x00,0xFD75,{0x0639,0x062C,0x062C,0x0645}},
{0x00,0xFD76,{0x0639,0x0645,0x0645,0x0645}},
{0x00,0xFD77,{0x0639,0x0645,0x0645,0x0645}},
{0x00,0xFD78,{0x0639,0x0645,0x0645,0x0649}},
{0x00,0xFD79,{0x063A,0x0645,0x0645,0x0645}},
{0x00,0xFD7A,{0x063A,0x0645,0x0645,0x064A}},
{0x00,0xFD7B,{0x063A,0x0645,0x0645,0x0649}},
{0x00,0xFD7C,{0x0641,0x062E,0x062E,0x0645}},
{0x00,0xFD7D,{0x0641,0x062E,0x062E,0x0645}},
{0x00,0xFD7E,{0x0642,0x0645,0x0645,0x062D}},
{0x00,0xFD7F,{0x0642,0x0645,0x0645,0x0645}},
{0x00,0xFD80,{0x0644,0x062D,0x062D,0x0645}},
{0x00,0xFD81,{0x0644,0x062D,0x062D,0x064A}},
{0x00,0xFD82,{0x0644,0x062D,0x062D,0x0649}},
{0x00,0xFD83,{0x0644,0x062C,0x062C,0x062C}},
{0x00,0xFD84,{0x0644,0x062C,0x062C,0x062C}},
{0x00,0xFD85,{0x0644,0x062E,0x062E,0x0645}},
{0x00,0xFD86,{0x0644,0x062E,0x062E,0x0645}},
{0x00,0xFD87,{0x0644,0x0645,0x0645,0x062D}},
{0x00,0xFD88,{0x0644,0x0645,0x0645,0x062D}},
{0x00,0xFD89,{0x0645,0x062D,0x062D,0x062C}},
{0x00,0xFD8A,{0x0645,0x062D,0x062D,0x0645}},
{0x00,0xFD8B,{0x0645,0x062D,0x062D,0x064A}},
{0x00,0xFD8C,{0x0645,0x062C,0x062C,0x062D}},
{0x00,0xFD8D,{0x0645,0x062C,0x062C,0x0645}},
{0x00,0xFD8E,{0x0645,0x062E,0x062E,0x062C}},
{0x00,0xFD8F,{0x0645,0x062E,0x062E,0x0645}},
{0x00,0xFD92,{0x0645,0x062C,0x062C,0x062E}},
{0x00,0xFD93,{0x0647,0x0645,0x0645,0x062C}},
{0x00,0xFD94,{0x0647,0x0645,0x0645,0x0645}},
{0x00,0xFD95,{0x0646,0x062D,0x062D,0x0645}},
{0x00,0xFD96,{0x0646,0x062D,0x062D,0x0649}},
{0x00,0xFD97,{0x0646,0x062C,0x062C,0x0645}},
{0x00,0xFD98,{0x0646,0x062C,0x062C,0x0645}},
{0x00,0xFD99,{0x0646,0x062C,0x062C,0x0649}},
{0x00,0xFD9A,{0x0646,0x0645,0x0645,0x064A}},
{0x00,0xFD9B,{0x0646,0x0645,0x0645,0x0649}},
{0x00,0xFD9C,{0x064A,0x0645,0x0645,0x0645}},
{0x00,0xFD9D,{0x064A,0x0645,0x0645,0x0645}},
{0x00,0xFD9E,{0x0628,0x062E,0x062E,0x064A}},
{0x00,0xFD9F,{0x062A,0x062C,0x062C,0x064A}},
{0x00,0xFDA0,{0x062A,0x062C,0x062C,0x0649}},
{0x00,0xFDA1,{0x062A,0x062E,0x062E,0x064A}},
{0x00,0xFDA2,{0x062A,0x062E,0x062E,0x0649}},
{0x00,0xFDA3,{0x062A,0x0645,0x0645,0x064A}},
{0x00,0xFDA4,{0x062A,0x0645,0x0645,0x0649}},
{0x00,0xFDA5,{0x062C,0x0645,0x0645,0x064A}},
{0x00,0xFDA6,{0x062C,0x062D,0x062D,0x0649}},
{0x00,0xFDA7,{0x062C,0x0645,0x0645,0x0649}},
{0x00,0xFDA8,{0x0633,0x062E,0x062E,0x0649}},
{0x00,0xFDA9,{0x0635,0x062D,0x062D,0x064A}},
{0x00,0xFDAA,{0x0634,0x062D,0x062D,0x064A}},
{0x00,0xFDAB,{0x0636,0x062D,0x062D,0x064A}},
{0x00,0xFDAC,{0x0644,0x062C,0x062C,0x064A}},
{0x00,0xFDAD,{0x0644,0x0645,0x0645,0x064A}},
{0x00,0xFDAE,{0x064A,0x062D,0x062D,0x064A}},
{0x00,0xFDAF,{0x064A,0x062C,0x062C,0x064A}},
{0x00,0xFDB0,{0x064A,0x0645,0x0645,0x064A}},
{0x00,0xFDB1,{0x0645,0x0645,0x0645,0x064A}},
{0x00,0xFDB2,{0x0642,0x0645,0x0645,0x064A}},
{0x00,0xFDB3,{0x0646,0x062D,0x062D,0x064A}},
{0x00,0xFDB4,{0x0642,0x0645,0x0645,0x062D}},
{0x00,0xFDB5,{0x0644,0x062D,0x062D,0x0645}},
{0x00,0xFDB6,{0x0639,0x0645,0x0645,0x064A}},
{0x00,0xFDB7,{0x0643,0x0645,0x0645,0x064A}},
{0x00,0xFDB8,{0x0646,0x062C,0x062C,0x062D}},
{0x00,0xFDB9,{0x0645,0x062E,0x062E,0x064A}},
{0x00,0xFDBA,{0x0644,0x062C,0x062C,0x0645}},
{0x00,0xFDBB,{0x0643,0x0645,0x0645,0x0645}},
{0x00,0xFDBC,{0x0644,0x062C,0x062C,0x0645}},
{0x00,0xFDBD,{0x0646,0x062C,0x062C,0x062D}},
{0x00,0xFDBE,{0x062C,0x062D,0x062D,0x064A}},
{0x00,0xFDBF,{0x062D,0x062C,0x062C,0x064A}},
{0x00,0xFDC0,{0x0645,0x062C,0x062C,0x064A}},
{0x00,0xFDC1,{0x0641,0x0645,0x0645,0x064A}},
{0x00,0xFDC2,{0x0628,0x062D,0x062D,0x064A}},
{0x00,0xFDC3,{0x0643,0x0645,0x0645,0x0645}},
{0x00,0xFDC4,{0x0639,0x062C,0x062C,0x0645}},
{0x00,0xFDC5,{0x0635,0x0645,0x0645,0x0645}},
{0x00,0xFDC6,{0x0633,0x062E,0x062E,0x064A}},
{0x00,0xFDC7,{0x0646,0x062C,0x062C,0x064A}},
{0x00,0xFDF0,{0x0635,0x0644,0x0644,0x06D2}},
{0x00,0xFDF1,{0x0642,0x0644,0x0644,0x06D2}},
{0x00,0xFDF9,{0x0635,0x0644,0x0644,0x0649}},
{0x00,0xFE10,{0x002C,0x0000,0x0000,0x0000}},
{0x00,0xFE11,{0x3001,0x0000,0x0000,0x0000}},
{0x00,0xFE12,{0x3002,0x0000,0x0000,0x0000}},
{0x00,0xFE13,{0x003A,0x0000,0x0000,0x0000}},
{0x00,0xFE14,{0x003B,0x0000,0x0000,0x0000}},
{0x00,0xFE15,{0x0021,0x0000,0x0000,0x0000}},
{0x00,0xFE16,{0x003F,0x0000,0x0000,0x0000}},
{0x00,0xFE17,{0x3016,0x0000,0x0000,0x0000}},
{0x00,0xFE18,{0x3017,0x0000,0x0000,0x0000}},
{0x00,0xFE19,{0x2026,0x0000,0x0000,0x0000}},
{0x00,0xFE30,{0x2025,0x0000,0x0000,0x0000}},
{0x00,0xFE31,{0x2014,0x0000,0x0000,0x0000}},
{0x00,0xFE32,{0x2013,0x0000,0x0000,0x0000}},
{0x00,0xFE33,{0x005F,0x0000,0x0000,0x0000}},
{0x00,0xFE34,{0x005F,0x0000,0x0000,0x0000}},
{0x00,0xFE35,{0x0028,0x0000,0x0000,0x0000}},
{0x00,0xFE36,{0x0029,0x0000,0x0000,0x0000}},
{0x00,0xFE37,{0x007B,0x0000,0x0000,0x0000}},
{0x00,0xFE38,{0x007D,0x0000,0x0000,0x0000}},
{0x00,0xFE39,{0x3014,0x0000,0x0000,0x0000}},
{0x00,0xFE3A,{0x3015,0x0000,0x0000,0x0000}},
{0x00,0xFE3B,{0x3010,0x0000,0x0000,0x0000}},
{0x00,0xFE3C,{0x3011,0x0000,0x0000,0x0000}},
{0x00,0xFE3D,{0x300A,0x0000,0x0000,0x0000}},
{0x00,0xFE3E,{0x300B,0x0000,0x0000,0x0000}},
{0x00,0xFE3F,{0x3008,0x0000,0x0000,0x0000}},
{0x00,0xFE40,{0x3009,0x0000,0x0000,0x0000}},
{0x00,0xFE41,{0x300C,0x0000,0x0000,0x0000}},
{0x00,0xFE42,{0x300D,0x0000,0x0000,0x0000}},
{0x00,0xFE43,{0x300E,0x0000,0x0000,0x0000}},
{0x00,0xFE44,{0x300F,0x0000,0x0000,0x0000}},
{0x00,0xFE47,{0x005B,0x0000,0x0000,0x0000}},
{0x00,0xFE48,{0x005D,0x0000,0x0000,0x0000}},
{0x00,0xFE49,{0x203E,0x0000,0x0000,0x0000}},
{0x00,0xFE4A,{0x203E,0x0000,0x0000,0x0000}},
{0x00,0xFE4B,{0x203E,0x0000,0x0000,0x0000}},
{0x00,0xFE4C,{0x203E,0x0000,0x0000,0x0000}},
{0x00,0xFE4D,{0x005F,0x0000,0x0000,0x0000}},
{0x00,0xFE4E,{0x005F,0x0000,0x0000,0x0000}},
{0x00,0xFE4F,{0x005F,0x0000,0x0000,0x0000}},
{0x00,0xFE50,{0x002C,0x0000,0x0000,0x0000}},
{0x00,0xFE51,{0x3001,0x0000,0x0000,0x0000}},
{0x00,0xFE52,{0x002E,0x0000,0x0000,0x0000}},
{0x00,0xFE54,{0x003B,0x0000,0x0000,0x0000}},
{0x00,0xFE55,{0x003A,0x0000,0x0000,0x0000}},
{0x00,0xFE56,{0x003F,0x0000,0x0000,0x0000}},
{0x00,0xFE57,{0x0021,0x0000,0x0000,0x0000}},
{0x00,0xFE58,{0x2014,0x0000,0x0000,0x0000}},
{0x00,0xFE59,{0x0028,0x0000,0x0000,0x0000}},
{0x00,0xFE5A,{0x0029,0x0000,0x0000,0x0000}},
{0x00,0xFE5B,{0x007B,0x0000,0x0000,0x0000}},
{0x00,0xFE5C,{0x007D,0x0000,0x0000,0x0000}},
{0x00,0xFE5D,{0x3014,0x0000,0x0000,0x0000}},
{0x00,0xFE5E,{0x3015,0x0000,0x0000,0x0000}},
{0x00,0xFE5F,{0x0023,0x0000,0x0000,0x0000}},
{0x00,0xFE60,{0x0026,0x0000,0x0000,0x0000}},
{0x00,0xFE61,{0x002A,0x0000,0x0000,0x0000}},
{0x00,0xFE62,{0x002B,0x0000,0x0000,0x0000}},
{0x00,0xFE63,{0x002D,0x0000,0x0000,0x0000}},
{0x00,0xFE64,{0x003C,0x0000,0x0000,0x0000}},
{0x00,0xFE65,{0x003E,0x0000,0x0000,0x0000}},
{0x00,0xFE66,{0x003D,0x0000,0x0000,0x0000}},
{0x00,0xFE68,{0x005C,0x0000,0x0000,0x0000}},
{0x00,0xFE69,{0x0024,0x0000,0x0000,0x0000}},
{0x00,0xFE6A,{0x0025,0x0000,0x0000,0x0000}},
{0x00,0xFE6B,{0x0040,0x0000,0x0000,0x0000}},
{0x00,0xFE70,{0x0020,0x064B,0x064B,0x0000}},
{0x00,0xFE71,{0x0640,0x064B,0x064B,0x0000}},
{0x00,0xFE72,{0x0020,0x064C,0x064C,0x0000}},
{0x00,0xFE74,{0x0020,0x064D,0x064D,0x0000}},
{0x00,0xFE76,{0x0020,0x064E,0x064E,0x0000}},
{0x00,0xFE77,{0x0640,0x064E,0x064E,0x0000}},
{0x00,0xFE78,{0x0020,0x064F,0x064F,0x0000}},
{0x00,0xFE79,{0x0640,0x064F,0x064F,0x0000}},
{0x00,0xFE7A,{0x0020,0x0650,0x0650,0x0000}},
{0x00,0xFE7B,{0x0640,0x0650,0x0650,0x0000}},
{0x00,0xFE7C,{0x0020,0x0651,0x0651,0x0000}},
{0x00,0xFE7D,{0x0640,0x0651,0x0651,0x0000}},
{0x00,0xFE7E,{0x0020,0x0652,0x0652,0x0000}},
{0x00,0xFE7F,{0x0640,0x0652,0x0652,0x0000}},
{0x00,0xFE80,{0x0621,0x0000,0x0000,0x0000}},
{0x00,0xFE81,{0x0622,0x0000,0x0000,0x0000}},
{0x00,0xFE82,{0x0622,0x0000,0x0000,0x0000}},
{0x00,0xFE83,{0x0623,0x0000,0x0000,0x0000}},
{0x00,0xFE84,{0x0623,0x0000,0x0000,0x0000}},
{0x00,0xFE85,{0x0624,0x0000,0x0000,0x0000}},
{0x00,0xFE86,{0x0624,0x0000,0x0000,0x0000}},
{0x00,0xFE87,{0x0625,0x0000,0x0000,0x0000}},
{0x00,0xFE88,{0x0625,0x0000,0x0000,0x0000}},
{0x00,0xFE89,{0x0626,0x0000,0x0000,0x0000}},
{0x00,0xFE8A,{0x0626,0x0000,0x0000,0x0000}},
{0x00,0xFE8B,{0x0626,0x0000,0x0000,0x0000}},
{0x00,0xFE8C,{0x0626,0x0000,0x0000,0x0000}},
{0x00,0xFE8D,{0x0627,0x0000,0x0000,0x0000}},
{0x00,0xFE8E,{0x0627,0x0000,0x0000,0x0000}},
{0x00,0xFE8F,{0x0628,0x0000,0x0000,0x0000}},
{0x00,0xFE90,{0x0628,0x0000,0x0000,0x0000}},
{0x00,0xFE91,{0x0628,0x0000,0x0000,0x0000}},
{0x00,0xFE92,{0x0628,0x0000,0x0000,0x0000}},
{0x00,0xFE93,{0x0629,0x0000,0x0000,0x0000}},
{0x00,0xFE94,{0x0629,0x0000,0x0000,0x0000}},
{0x00,0xFE95,{0x062A,0x0000,0x0000,0x0000}},
{0x00,0xFE96,{0x062A,0x0000,0x0000,0x0000}},
{0x00,0xFE97,{0x062A,0x0000,0x0000,0x0000}},
{0x00,0xFE98,{0x062A,0x0000,0x0000,0x0000}},
{0x00,0xFE99,{0x062B,0x0000,0x0000,0x0000}},
{0x00,0xFE9A,{0x062B,0x0000,0x0000,0x0000}},
{0x00,0xFE9B,{0x062B,0x0000,0x0000,0x0000}},
{0x00,0xFE9C,{0x062B,0x0000,0x0000,0x0000}},
{0x00,0xFE9D,{0x062C,0x0000,0x0000,0x0000}},
{0x00,0xFE9E,{0x062C,0x0000,0x0000,0x0000}},
{0x00,0xFE9F,{0x062C,0x0000,0x0000,0x0000}},
{0x00,0xFEA0,{0x062C,0x0000,0x0000,0x0000}},
{0x00,0xFEA1,{0x062D,0x0000,0x0000,0x0000}},
{0x00,0xFEA2,{0x062D,0x0000,0x0000,0x0000}},
{0x00,0xFEA3,{0x062D,0x0000,0x0000,0x0000}},
{0x00,0xFEA4,{0x062D,0x0000,0x0000,0x0000}},
{0x00,0xFEA5,{0x062E,0x0000,0x0000,0x0000}},
{0x00,0xFEA6,{0x062E,0x0000,0x0000,0x0000}},
{0x00,0xFEA7,{0x062E,0x0000,0x0000,0x0000}},
{0x00,0xFEA8,{0x062E,0x0000,0x0000,0x0000}},
{0x00,0xFEA9,{0x062F,0x0000,0x0000,0x0000}},
{0x00,0xFEAA,{0x062F,0x0000,0x0000,0x0000}},
{0x00,0xFEAB,{0x0630,0x0000,0x0000,0x0000}},
{0x00,0xFEAC,{0x0630,0x0000,0x0000,0x0000}},
{0x00,0xFEAD,{0x0631,0x0000,0x0000,0x0000}},
{0x00,0xFEAE,{0x0631,0x0000,0x0000,0x0000}},
{0x00,0xFEAF,{0x0632,0x0000,0x0000,0x0000}},
{0x00,0xFEB0,{0x0632,0x0000,0x0000,0x0000}},
{0x00,0xFEB1,{0x0633,0x0000,0x0000,0x0000}},
{0x00,0xFEB2,{0x0633,0x0000,0x0000,0x0000}},
{0x00,0xFEB3,{0x0633,0x0000,0x0000,0x0000}},
{0x00,0xFEB4,{0x0633,0x0000,0x0000,0x0000}},
{0x00,0xFEB5,{0x0634,0x0000,0x0000,0x0000}},
{0x00,0xFEB6,{0x0634,0x0000,0x0000,0x0000}},
{0x00,0xFEB7,{0x0634,0x0000,0x0000,0x0000}},
{0x00,0xFEB8,{0x0634,0x0000,0x0000,0x0000}},
{0x00,0xFEB9,{0x0635,0x0000,0x0000,0x0000}},
{0x00,0xFEBA,{0x0635,0x0000,0x0000,0x0000}},
{0x00,0xFEBB,{0x0635,0x0000,0x0000,0x0000}},
{0x00,0xFEBC,{0x0635,0x0000,0x0000,0x0000}},
{0x00,0xFEBD,{0x0636,0x0000,0x0000,0x0000}},
{0x00,0xFEBE,{0x0636,0x0000,0x0000,0x0000}},
{0x00,0xFEBF,{0x0636,0x0000,0x0000,0x0000}},
{0x00,0xFEC0,{0x0636,0x0000,0x0000,0x0000}},
{0x00,0xFEC1,{0x0637,0x0000,0x0000,0x0000}},
{0x00,0xFEC2,{0x0637,0x0000,0x0000,0x0000}},
{0x00,0xFEC3,{0x0637,0x0000,0x0000,0x0000}},
{0x00,0xFEC4,{0x0637,0x0000,0x0000,0x0000}},
{0x00,0xFEC5,{0x0638,0x0000,0x0000,0x0000}},
{0x00,0xFEC6,{0x0638,0x0000,0x0000,0x0000}},
{0x00,0xFEC7,{0x0638,0x0000,0x0000,0x0000}},
{0x00,0xFEC8,{0x0638,0x0000,0x0000,0x0000}},
{0x00,0xFEC9,{0x0639,0x0000,0x0000,0x0000}},
{0x00,0xFECA,{0x0639,0x0000,0x0000,0x0000}},
{0x00,0xFECB,{0x0639,0x0000,0x0000,0x0000}},
{0x00,0xFECC,{0x0639,0x0000,0x0000,0x0000}},
{0x00,0xFECD,{0x063A,0x0000,0x0000,0x0000}},
{0x00,0xFECE,{0x063A,0x0000,0x0000,0x0000}},
{0x00,0xFECF,{0x063A,0x0000,0x0000,0x0000}},
{0x00,0xFED0,{0x063A,0x0000,0x0000,0x0000}},
{0x00,0xFED1,{0x0641,0x0000,0x0000,0x0000}},
{0x00,0xFED2,{0x0641,0x0000,0x0000,0x0000}},
{0x00,0xFED3,{0x0641,0x0000,0x0000,0x0000}},
{0x00,0xFED4,{0x0641,0x0000,0x0000,0x0000}},
{0x00,0xFED5,{0x0642,0x0000,0x0000,0x0000}},
{0x00,0xFED6,{0x0642,0x0000,0x0000,0x0000}},
{0x00,0xFED7,{0x0642,0x0000,0x0000,0x0000}},
{0x00,0xFED8,{0x0642,0x0000,0x0000,0x0000}},
{0x00,0xFED9,{0x0643,0x0000,0x0000,0x0000}},
{0x00,0xFEDA,{0x0643,0x0000,0x0000,0x0000}},
{0x00,0xFEDB,{0x0643,0x0000,0x0000,0x0000}},
{0x00,0xFEDC,{0x0643,0x0000,0x0000,0x0000}},
{0x00,0xFEDD,{0x0644,0x0000,0x0000,0x0000}},
{0x00,0xFEDE,{0x0644,0x0000,0x0000,0x0000}},
{0x00,0xFEDF,{0x0644,0x0000,0x0000,0x0000}},
{0x00,0xFEE0,{0x0644,0x0000,0x0000,0x0000}},
{0x00,0xFEE1,{0x0645,0x0000,0x0000,0x0000}},
{0x00,0xFEE2,{0x0645,0x0000,0x0000,0x0000}},
{0x00,0xFEE3,{0x0645,0x0000,0x0000,0x0000}},
{0x00,0xFEE4,{0x0645,0x0000,0x0000,0x0000}},
{0x00,0xFEE5,{0x0646,0x0000,0x0000,0x0000}},
{0x00,0xFEE6,{0x0646,0x0000,0x0000,0x0000}},
{0x00,0xFEE7,{0x0646,0x0000,0x0000,0x0000}},
{0x00,0xFEE8,{0x0646,0x0000,0x0000,0x0000}},
{0x00,0xFEE9,{0x0647,0x0000,0x0000,0x0000}},
{0x00,0xFEEA,{0x0647,0x0000,0x0000,0x0000}},
{0x00,0xFEEB,{0x0647,0x0000,0x0000,0x0000}},
{0x00,0xFEEC,{0x0647,0x0000,0x0000,0x0000}},
{0x00,0xFEED,{0x0648,0x0000,0x0000,0x0000}},
{0x00,0xFEEE,{0x0648,0x0000,0x0000,0x0000}},
{0x00,0xFEEF,{0x0649,0x0000,0x0000,0x0000}},
{0x00,0xFEF0,{0x0649,0x0000,0x0000,0x0000}},
{0x00,0xFEF1,{0x064A,0x0000,0x0000,0x0000}},
{0x00,0xFEF2,{0x064A,0x0000,0x0000,0x0000}},
{0x00,0xFEF3,{0x064A,0x0000,0x0000,0x0000}},
{0x00,0xFEF4,{0x064A,0x0000,0x0000,0x0000}},
{0x00,0xFEF5,{0x0644,0x0622,0x0622,0x0000}},
{0x00,0xFEF6,{0x0644,0x0622,0x0622,0x0000}},
{0x00,0xFEF7,{0x0644,0x0623,0x0623,0x0000}},
{0x00,0xFEF8,{0x0644,0x0623,0x0623,0x0000}},
{0x00,0xFEF9,{0x0644,0x0625,0x0625,0x0000}},
{0x00,0xFEFA,{0x0644,0x0625,0x0625,0x0000}},
{0x00,0xFEFB,{0x0644,0x0627,0x0627,0x0000}},
{0x00,0xFEFC,{0x0644,0x0627,0x0627,0x0000}},
{0x00,0xFF01,{0x0021,0x0000,0x0000,0x0000}},
{0x00,0xFF02,{0x0022,0x0000,0x0000,0x0000}},
{0x00,0xFF03,{0x0023,0x0000,0x0000,0x0000}},
{0x00,0xFF04,{0x0024,0x0000,0x0000,0x0000}},
{0x00,0xFF05,{0x0025,0x0000,0x0000,0x0000}},
{0x00,0xFF06,{0x0026,0x0000,0x0000,0x0000}},
{0x00,0xFF07,{0x0027,0x0000,0x0000,0x0000}},
{0x00,0xFF08,{0x0028,0x0000,0x0000,0x0000}},
{0x00,0xFF09,{0x0029,0x0000,0x0000,0x0000}},
{0x00,0xFF0A,{0x002A,0x0000,0x0000,0x0000}},
{0x00,0xFF0B,{0x002B,0x0000,0x0000,0x0000}},
{0x00,0xFF0C,{0x002C,0x0000,0x0000,0x0000}},
{0x00,0xFF0D,{0x002D,0x0000,0x0000,0x0000}},
{0x00,0xFF0E,{0x002E,0x0000,0x0000,0x0000}},
{0x00,0xFF0F,{0x002F,0x0000,0x0000,0x0000}},
{0x00,0xFF10,{0x0030,0x0000,0x0000,0x0000}},
{0x00,0xFF11,{0x0031,0x0000,0x0000,0x0000}},
{0x00,0xFF12,{0x0032,0x0000,0x0000,0x0000}},
{0x00,0xFF13,{0x0033,0x0000,0x0000,0x0000}},
{0x00,0xFF14,{0x0034,0x0000,0x0000,0x0000}},
{0x00,0xFF15,{0x0035,0x0000,0x0000,0x0000}},
{0x00,0xFF16,{0x0036,0x0000,0x0000,0x0000}},
{0x00,0xFF17,{0x0037,0x0000,0x0000,0x0000}},
{0x00,0xFF18,{0x0038,0x0000,0x0000,0x0000}},
{0x00,0xFF19,{0x0039,0x0000,0x0000,0x0000}},
{0x00,0xFF1A,{0x003A,0x0000,0x0000,0x0000}},
{0x00,0xFF1B,{0x003B,0x0000,0x0000,0x0000}},
{0x00,0xFF1C,{0x003C,0x0000,0x0000,0x0000}},
{0x00,0xFF1D,{0x003D,0x0000,0x0000,0x0000}},
{0x00,0xFF1E,{0x003E,0x0000,0x0000,0x0000}},
{0x00,0xFF1F,{0x003F,0x0000,0x0000,0x0000}},
{0x00,0xFF20,{0x0040,0x0000,0x0000,0x0000}},
{0x00,0xFF3B,{0x005B,0x0000,0x0000,0x0000}},
{0x00,0xFF3C,{0x005C,0x0000,0x0000,0x0000}},
{0x00,0xFF3D,{0x005D,0x0000,0x0000,0x0000}},
{0x00,0xFF3E,{0x005E,0x0000,0x0000,0x0000}},
{0x00,0xFF3F,{0x005F,0x0000,0x0000,0x0000}},
{0x00,0xFF40,{0x0060,0x0000,0x0000,0x0000}},
{0x00,0xFF41,{0x0061,0x0000,0x0000,0x0000}},
{0x00,0xFF42,{0x0062,0x0000,0x0000,0x0000}},
{0x00,0xFF43,{0x0063,0x0000,0x0000,0x0000}},
{0x00,0xFF44,{0x0064,0x0000,0x0000,0x0000}},
{0x00,0xFF45,{0x0065,0x0000,0x0000,0x0000}},
{0x00,0xFF46,{0x0066,0x0000,0x0000,0x0000}},
{0x00,0xFF47,{0x0067,0x0000,0x0000,0x0000}},
{0x00,0xFF48,{0x0068,0x0000,0x0000,0x0000}},
{0x00,0xFF49,{0x0069,0x0000,0x0000,0x0000}},
{0x00,0xFF4A,{0x006A,0x0000,0x0000,0x0000}},
{0x00,0xFF4B,{0x006B,0x0000,0x0000,0x0000}},
{0x00,0xFF4C,{0x006C,0x0000,0x0000,0x0000}},
{0x00,0xFF4D,{0x006D,0x0000,0x0000,0x0000}},
{0x00,0xFF4E,{0x006E,0x0000,0x0000,0x0000}},
{0x00,0xFF4F,{0x006F,0x0000,0x0000,0x0000}},
{0x00,0xFF50,{0x0070,0x0000,0x0000,0x0000}},
{0x00,0xFF51,{0x0071,0x0000,0x0000,0x0000}},
{0x00,0xFF52,{0x0072,0x0000,0x0000,0x0000}},
{0x00,0xFF53,{0x0073,0x0000,0x0000,0x0000}},
{0x00,0xFF54,{0x0074,0x0000,0x0000,0x0000}},
{0x00,0xFF55,{0x0075,0x0000,0x0000,0x0000}},
{0x00,0xFF56,{0x0076,0x0000,0x0000,0x0000}},
{0x00,0xFF57,{0x0077,0x0000,0x0000,0x0000}},
{0x00,0xFF58,{0x0078,0x0000,0x0000,0x0000}},
{0x00,0xFF59,{0x0079,0x0000,0x0000,0x0000}},
{0x00,0xFF5A,{0x007A,0x0000,0x0000,0x0000}},
{0x00,0xFF5B,{0x007B,0x0000,0x0000,0x0000}},
{0x00,0xFF5C,{0x007C,0x0000,0x0000,0x0000}},
{0x00,0xFF5D,{0x007D,0x0000,0x0000,0x0000}},
{0x00,0xFF5E,{0x007E,0x0000,0x0000,0x0000}},
{0x00,0xFF5F,{0x2985,0x0000,0x0000,0x0000}},
{0x00,0xFF60,{0x2986,0x0000,0x0000,0x0000}},
{0x00,0xFF61,{0x3002,0x0000,0x0000,0x0000}},
{0x00,0xFF62,{0x300C,0x0000,0x0000,0x0000}},
{0x00,0xFF63,{0x300D,0x0000,0x0000,0x0000}},
{0x00,0xFF64,{0x3001,0x0000,0x0000,0x0000}},
{0x00,0xFF65,{0x30FB,0x0000,0x0000,0x0000}},
{0x00,0xFF66,{0x30F2,0x0000,0x0000,0x0000}},
{0x00,0xFF67,{0x30A1,0x0000,0x0000,0x0000}},
{0x00,0xFF68,{0x30A3,0x0000,0x0000,0x0000}},
{0x00,0xFF69,{0x30A5,0x0000,0x0000,0x0000}},
{0x00,0xFF6A,{0x30A7,0x0000,0x0000,0x0000}},
{0x00,0xFF6B,{0x30A9,0x0000,0x0000,0x0000}},
{0x00,0xFF6C,{0x30E3,0x0000,0x0000,0x0000}},
{0x00,0xFF6D,{0x30E5,0x0000,0x0000,0x0000}},
{0x00,0xFF6E,{0x30E7,0x0000,0x0000,0x0000}},
{0x00,0xFF6F,{0x30C3,0x0000,0x0000,0x0000}},
{0x00,0xFF70,{0x30FC,0x0000,0x0000,0x0000}},
{0x00,0xFF71,{0x30A2,0x0000,0x0000,0x0000}},
{0x00,0xFF72,{0x30A4,0x0000,0x0000,0x0000}},
{0x00,0xFF73,{0x30A6,0x0000,0x0000,0x0000}},
{0x00,0xFF74,{0x30A8,0x0000,0x0000,0x0000}},
{0x00,0xFF75,{0x30AA,0x0000,0x0000,0x0000}},
{0x00,0xFF76,{0x30AB,0x0000,0x0000,0x0000}},
{0x00,0xFF77,{0x30AD,0x0000,0x0000,0x0000}},
{0x00,0xFF78,{0x30AF,0x0000,0x0000,0x0000}},
{0x00,0xFF79,{0x30B1,0x0000,0x0000,0x0000}},
{0x00,0xFF7A,{0x30B3,0x0000,0x0000,0x0000}},
{0x00,0xFF7B,{0x30B5,0x0000,0x0000,0x0000}},
{0x00,0xFF7C,{0x30B7,0x0000,0x0000,0x0000}},
{0x00,0xFF7D,{0x30B9,0x0000,0x0000,0x0000}},
{0x00,0xFF7E,{0x30BB,0x0000,0x0000,0x0000}},
{0x00,0xFF7F,{0x30BD,0x0000,0x0000,0x0000}},
{0x00,0xFF80,{0x30BF,0x0000,0x0000,0x0000}},
{0x00,0xFF81,{0x30C1,0x0000,0x0000,0x0000}},
{0x00,0xFF82,{0x30C4,0x0000,0x0000,0x0000}},
{0x00,0xFF83,{0x30C6,0x0000,0x0000,0x0000}},
{0x00,0xFF84,{0x30C8,0x0000,0x0000,0x0000}},
{0x00,0xFF85,{0x30CA,0x0000,0x0000,0x0000}},
{0x00,0xFF86,{0x30CB,0x0000,0x0000,0x0000}},
{0x00,0xFF87,{0x30CC,0x0000,0x0000,0x0000}},
{0x00,0xFF88,{0x30CD,0x0000,0x0000,0x0000}},
{0x00,0xFF89,{0x30CE,0x0000,0x0000,0x0000}},
{0x00,0xFF8A,{0x30CF,0x0000,0x0000,0x0000}},
{0x00,0xFF8B,{0x30D2,0x0000,0x0000,0x0000}},
{0x00,0xFF8C,{0x30D5,0x0000,0x0000,0x0000}},
{0x00,0xFF8D,{0x30D8,0x0000,0x0000,0x0000}},
{0x00,0xFF8E,{0x30DB,0x0000,0x0000,0x0000}},
{0x00,0xFF8F,{0x30DE,0x0000,0x0000,0x0000}},
{0x00,0xFF90,{0x30DF,0x0000,0x0000,0x0000}},
{0x00,0xFF91,{0x30E0,0x0000,0x0000,0x0000}},
{0x00,0xFF92,{0x30E1,0x0000,0x0000,0x0000}},
{0x00,0xFF93,{0x30E2,0x0000,0x0000,0x0000}},
{0x00,0xFF94,{0x30E4,0x0000,0x0000,0x0000}},
{0x00,0xFF95,{0x30E6,0x0000,0x0000,0x0000}},
{0x00,0xFF96,{0x30E8,0x0000,0x0000,0x0000}},
{0x00,0xFF97,{0x30E9,0x0000,0x0000,0x0000}},
{0x00,0xFF98,{0x30EA,0x0000,0x0000,0x0000}},
{0x00,0xFF99,{0x30EB,0x0000,0x0000,0x0000}},
{0x00,0xFF9A,{0x30EC,0x0000,0x0000,0x0000}},
{0x00,0xFF9B,{0x30ED,0x0000,0x0000,0x0000}},
{0x00,0xFF9C,{0x30EF,0x0000,0x0000,0x0000}},
{0x00,0xFF9D,{0x30F3,0x0000,0x0000,0x0000}},
{0x00,0xFF9E,{0x3099,0x0000,0x0000,0x0000}},
{0x00,0xFF9F,{0x309A,0x0000,0x0000,0x0000}},
{0x00,0xFFA0,{0x3164,0x0000,0x0000,0x0000}},
{0x00,0xFFA1,{0x3131,0x0000,0x0000,0x0000}},
{0x00,0xFFA2,{0x3132,0x0000,0x0000,0x0000}},
{0x00,0xFFA3,{0x3133,0x0000,0x0000,0x0000}},
{0x00,0xFFA4,{0x3134,0x0000,0x0000,0x0000}},
{0x00,0xFFA5,{0x3135,0x0000,0x0000,0x0000}},
{0x00,0xFFA6,{0x3136,0x0000,0x0000,0x0000}},
{0x00,0xFFA7,{0x3137,0x0000,0x0000,0x0000}},
{0x00,0xFFA8,{0x3138,0x0000,0x0000,0x0000}},
{0x00,0xFFA9,{0x3139,0x0000,0x0000,0x0000}},
{0x00,0xFFAA,{0x313A,0x0000,0x0000,0x0000}},
{0x00,0xFFAB,{0x313B,0x0000,0x0000,0x0000}},
{0x00,0xFFAC,{0x313C,0x0000,0x0000,0x0000}},
{0x00,0xFFAD,{0x313D,0x0000,0x0000,0x0000}},
{0x00,0xFFAE,{0x313E,0x0000,0x0000,0x0000}},
{0x00,0xFFAF,{0x313F,0x0000,0x0000,0x0000}},
{0x00,0xFFB0,{0x3140,0x0000,0x0000,0x0000}},
{0x00,0xFFB1,{0x3141,0x0000,0x0000,0x0000}},
{0x00,0xFFB2,{0x3142,0x0000,0x0000,0x0000}},
{0x00,0xFFB3,{0x3143,0x0000,0x0000,0x0000}},
{0x00,0xFFB4,{0x3144,0x0000,0x0000,0x0000}},
{0x00,0xFFB5,{0x3145,0x0000,0x0000,0x0000}},
{0x00,0xFFB6,{0x3146,0x0000,0x0000,0x0000}},
{0x00,0xFFB7,{0x3147,0x0000,0x0000,0x0000}},
{0x00,0xFFB8,{0x3148,0x0000,0x0000,0x0000}},
{0x00,0xFFB9,{0x3149,0x0000,0x0000,0x0000}},
{0x00,0xFFBA,{0x314A,0x0000,0x0000,0x0000}},
{0x00,0xFFBB,{0x314B,0x0000,0x0000,0x0000}},
{0x00,0xFFBC,{0x314C,0x0000,0x0000,0x0000}},
{0x00,0xFFBD,{0x314D,0x0000,0x0000,0x0000}},
{0x00,0xFFBE,{0x314E,0x0000,0x0000,0x0000}},
{0x00,0xFFC2,{0x314F,0x0000,0x0000,0x0000}},
{0x00,0xFFC3,{0x3150,0x0000,0x0000,0x0000}},
{0x00,0xFFC4,{0x3151,0x0000,0x0000,0x0000}},
{0x00,0xFFC5,{0x3152,0x0000,0x0000,0x0000}},
{0x00,0xFFC6,{0x3153,0x0000,0x0000,0x0000}},
{0x00,0xFFC7,{0x3154,0x0000,0x0000,0x0000}},
{0x00,0xFFCA,{0x3155,0x0000,0x0000,0x0000}},
{0x00,0xFFCB,{0x3156,0x0000,0x0000,0x0000}},
{0x00,0xFFCC,{0x3157,0x0000,0x0000,0x0000}},
{0x00,0xFFCD,{0x3158,0x0000,0x0000,0x0000}},
{0x00,0xFFCE,{0x3159,0x0000,0x0000,0x0000}},
{0x00,0xFFCF,{0x315A,0x0000,0x0000,0x0000}},
{0x00,0xFFD2,{0x315B,0x0000,0x0000,0x0000}},
{0x00,0xFFD3,{0x315C,0x0000,0x0000,0x0000}},
{0x00,0xFFD4,{0x315D,0x0000,0x0000,0x0000}},
{0x00,0xFFD5,{0x315E,0x0000,0x0000,0x0000}},
{0x00,0xFFD6,{0x315F,0x0000,0x0000,0x0000}},
{0x00,0xFFD7,{0x3160,0x0000,0x0000,0x0000}},
{0x00,0xFFDA,{0x3161,0x0000,0x0000,0x0000}},
{0x00,0xFFDB,{0x3162,0x0000,0x0000,0x0000}},
{0x00,0xFFDC,{0x3163,0x0000,0x0000,0x0000}},
{0x00,0xFFE0,{0x00A2,0x0000,0x0000,0x0000}},
{0x00,0xFFE1,{0x00A3,0x0000,0x0000,0x0000}},
{0x00,0xFFE2,{0x00AC,0x0000,0x0000,0x0000}},
{0x00,0xFFE3,{0x00AF,0x0000,0x0000,0x0000}},
{0x00,0xFFE4,{0x00A6,0x0000,0x0000,0x0000}},
{0x00,0xFFE5,{0x00A5,0x0000,0x0000,0x0000}},
{0x00,0xFFE6,{0x20A9,0x0000,0x0000,0x0000}},
{0x00,0xFFE8,{0x2502,0x0000,0x0000,0x0000}},
{0x00,0xFFE9,{0x2190,0x0000,0x0000,0x0000}},
{0x00,0xFFEA,{0x2191,0x0000,0x0000,0x0000}},
{0x00,0xFFEB,{0x2192,0x0000,0x0000,0x0000}},
{0x00,0xFFEC,{0x2193,0x0000,0x0000,0x0000}},
{0x00,0xFFED,{0x25A0,0x0000,0x0000,0x0000}},
{0x00,0xFFEE,{0x25CB,0x0000,0x0000,0x0000}},
{0x01,0xD41A,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD41B,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD41C,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD41D,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD41E,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD41F,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD420,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD421,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD422,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD423,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD424,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD425,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD426,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD427,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD428,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD429,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD42A,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD42B,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD42C,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD42D,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD42E,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD42F,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD430,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD431,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD432,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD433,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD44E,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD44F,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD450,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD451,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD452,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD453,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD454,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD456,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD457,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD458,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD459,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD45A,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD45B,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD45C,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD45D,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD45E,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD45F,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD460,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD461,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD462,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD463,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD464,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD465,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD466,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD467,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD482,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD483,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD484,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD485,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD486,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD487,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD488,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD489,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD48A,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD48B,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD48C,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD48D,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD48E,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD48F,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD490,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD491,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD492,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD493,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD494,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD495,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD496,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD497,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD498,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD499,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD49A,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD49B,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD4B6,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD4B7,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD4B8,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD4B9,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD4BB,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD4BD,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD4BE,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD4BF,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD4C0,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD4C1,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD4C2,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD4C3,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD4C5,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD4C6,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD4C7,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD4C8,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD4C9,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD4CA,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD4CB,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD4CC,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD4CD,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD4CE,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD4CF,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD4EA,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD4EB,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD4EC,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD4ED,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD4EE,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD4EF,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD4F0,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD4F1,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD4F2,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD4F3,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD4F4,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD4F5,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD4F6,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD4F7,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD4F8,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD4F9,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD4FA,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD4FB,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD4FC,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD4FD,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD4FE,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD4FF,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD500,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD501,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD502,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD503,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD51E,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD51F,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD520,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD521,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD522,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD523,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD524,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD525,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD526,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD527,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD528,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD529,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD52A,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD52B,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD52C,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD52D,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD52E,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD52F,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD530,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD531,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD532,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD533,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD534,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD535,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD536,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD537,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD552,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD553,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD554,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD555,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD556,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD557,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD558,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD559,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD55A,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD55B,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD55C,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD55D,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD55E,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD55F,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD560,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD561,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD562,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD563,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD564,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD565,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD566,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD567,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD568,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD569,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD56A,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD56B,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD586,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD587,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD588,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD589,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD58A,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD58B,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD58C,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD58D,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD58E,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD58F,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD590,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD591,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD592,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD593,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD594,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD595,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD596,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD597,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD598,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD599,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD59A,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD59B,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD59C,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD59D,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD59E,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD59F,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD5BA,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD5BB,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD5BC,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD5BD,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD5BE,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD5BF,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD5C0,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD5C1,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD5C2,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD5C3,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD5C4,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD5C5,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD5C6,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD5C7,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD5C8,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD5C9,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD5CA,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD5CB,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD5CC,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD5CD,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD5CE,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD5CF,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD5D0,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD5D1,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD5D2,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD5D3,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD5EE,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD5EF,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD5F0,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD5F1,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD5F2,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD5F3,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD5F4,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD5F5,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD5F6,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD5F7,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD5F8,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD5F9,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD5FA,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD5FB,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD5FC,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD5FD,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD5FE,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD5FF,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD600,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD601,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD602,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD603,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD604,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD605,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD606,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD607,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD622,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD623,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD624,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD625,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD626,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD627,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD628,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD629,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD62A,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD62B,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD62C,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD62D,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD62E,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD62F,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD630,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD631,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD632,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD633,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD634,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD635,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD636,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD637,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD638,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD639,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD63A,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD63B,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD656,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD657,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD658,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD659,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD65A,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD65B,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD65C,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD65D,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD65E,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD65F,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD660,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD661,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD662,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD663,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD664,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD665,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD666,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD667,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD668,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD669,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD66A,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD66B,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD66C,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD66D,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD66E,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD66F,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD68A,{0x0061,0x0000,0x0000,0x0000}},
{0x01,0xD68B,{0x0062,0x0000,0x0000,0x0000}},
{0x01,0xD68C,{0x0063,0x0000,0x0000,0x0000}},
{0x01,0xD68D,{0x0064,0x0000,0x0000,0x0000}},
{0x01,0xD68E,{0x0065,0x0000,0x0000,0x0000}},
{0x01,0xD68F,{0x0066,0x0000,0x0000,0x0000}},
{0x01,0xD690,{0x0067,0x0000,0x0000,0x0000}},
{0x01,0xD691,{0x0068,0x0000,0x0000,0x0000}},
{0x01,0xD692,{0x0069,0x0000,0x0000,0x0000}},
{0x01,0xD693,{0x006A,0x0000,0x0000,0x0000}},
{0x01,0xD694,{0x006B,0x0000,0x0000,0x0000}},
{0x01,0xD695,{0x006C,0x0000,0x0000,0x0000}},
{0x01,0xD696,{0x006D,0x0000,0x0000,0x0000}},
{0x01,0xD697,{0x006E,0x0000,0x0000,0x0000}},
{0x01,0xD698,{0x006F,0x0000,0x0000,0x0000}},
{0x01,0xD699,{0x0070,0x0000,0x0000,0x0000}},
{0x01,0xD69A,{0x0071,0x0000,0x0000,0x0000}},
{0x01,0xD69B,{0x0072,0x0000,0x0000,0x0000}},
{0x01,0xD69C,{0x0073,0x0000,0x0000,0x0000}},
{0x01,0xD69D,{0x0074,0x0000,0x0000,0x0000}},
{0x01,0xD69E,{0x0075,0x0000,0x0000,0x0000}},
{0x01,0xD69F,{0x0076,0x0000,0x0000,0x0000}},
{0x01,0xD6A0,{0x0077,0x0000,0x0000,0x0000}},
{0x01,0xD6A1,{0x0078,0x0000,0x0000,0x0000}},
{0x01,0xD6A2,{0x0079,0x0000,0x0000,0x0000}},
{0x01,0xD6A3,{0x007A,0x0000,0x0000,0x0000}},
{0x01,0xD6A4,{0x0131,0x0000,0x0000,0x0000}},
{0x01,0xD6A5,{0x0237,0x0000,0x0000,0x0000}},
{0x01,0xD6C1,{0x2207,0x0000,0x0000,0x0000}},
{0x01,0xD6C2,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD6C3,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD6C4,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD6C5,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD6C6,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD6C7,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD6C8,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD6C9,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD6CA,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD6CB,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD6CC,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD6CD,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD6CE,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD6CF,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD6D0,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD6D1,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD6D2,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD6D4,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD6D5,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD6D6,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD6D7,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD6D8,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD6D9,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD6DA,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD6DB,{0x2202,0x0000,0x0000,0x0000}},
{0x01,0xD6DC,{0x03F5,0x0000,0x0000,0x0000}},
{0x01,0xD6DD,{0x03D1,0x0000,0x0000,0x0000}},
{0x01,0xD6DE,{0x03F0,0x0000,0x0000,0x0000}},
{0x01,0xD6DF,{0x03D5,0x0000,0x0000,0x0000}},
{0x01,0xD6E0,{0x03F1,0x0000,0x0000,0x0000}},
{0x01,0xD6E1,{0x03D6,0x0000,0x0000,0x0000}},
{0x01,0xD6FB,{0x2207,0x0000,0x0000,0x0000}},
{0x01,0xD6FC,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD6FD,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD6FE,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD6FF,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD700,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD701,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD702,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD703,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD704,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD705,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD706,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD707,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD708,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD709,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD70A,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD70B,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD70C,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD70E,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD70F,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD710,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD711,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD712,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD713,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD714,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD715,{0x2202,0x0000,0x0000,0x0000}},
{0x01,0xD716,{0x03F5,0x0000,0x0000,0x0000}},
{0x01,0xD717,{0x03D1,0x0000,0x0000,0x0000}},
{0x01,0xD718,{0x03F0,0x0000,0x0000,0x0000}},
{0x01,0xD719,{0x03D5,0x0000,0x0000,0x0000}},
{0x01,0xD71A,{0x03F1,0x0000,0x0000,0x0000}},
{0x01,0xD71B,{0x03D6,0x0000,0x0000,0x0000}},
{0x01,0xD735,{0x2207,0x0000,0x0000,0x0000}},
{0x01,0xD736,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD737,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD738,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD739,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD73A,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD73B,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD73C,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD73D,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD73E,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD73F,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD740,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD741,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD742,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD743,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD744,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD745,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD746,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD748,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD749,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD74A,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD74B,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD74C,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD74D,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD74E,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD74F,{0x2202,0x0000,0x0000,0x0000}},
{0x01,0xD750,{0x03F5,0x0000,0x0000,0x0000}},
{0x01,0xD751,{0x03D1,0x0000,0x0000,0x0000}},
{0x01,0xD752,{0x03F0,0x0000,0x0000,0x0000}},
{0x01,0xD753,{0x03D5,0x0000,0x0000,0x0000}},
{0x01,0xD754,{0x03F1,0x0000,0x0000,0x0000}},
{0x01,0xD755,{0x03D6,0x0000,0x0000,0x0000}},
{0x01,0xD76F,{0x2207,0x0000,0x0000,0x0000}},
{0x01,0xD770,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD771,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD772,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD773,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD774,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD775,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD776,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD777,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD778,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD779,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD77A,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD77B,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD77C,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD77D,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD77E,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD77F,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD780,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD782,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD783,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD784,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD785,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD786,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD787,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD788,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD789,{0x2202,0x0000,0x0000,0x0000}},
{0x01,0xD78A,{0x03F5,0x0000,0x0000,0x0000}},
{0x01,0xD78B,{0x03D1,0x0000,0x0000,0x0000}},
{0x01,0xD78C,{0x03F0,0x0000,0x0000,0x0000}},
{0x01,0xD78D,{0x03D5,0x0000,0x0000,0x0000}},
{0x01,0xD78E,{0x03F1,0x0000,0x0000,0x0000}},
{0x01,0xD78F,{0x03D6,0x0000,0x0000,0x0000}},
{0x01,0xD7A9,{0x2207,0x0000,0x0000,0x0000}},
{0x01,0xD7AA,{0x03B1,0x0000,0x0000,0x0000}},
{0x01,0xD7AB,{0x03B2,0x0000,0x0000,0x0000}},
{0x01,0xD7AC,{0x03B3,0x0000,0x0000,0x0000}},
{0x01,0xD7AD,{0x03B4,0x0000,0x0000,0x0000}},
{0x01,0xD7AE,{0x03B5,0x0000,0x0000,0x0000}},
{0x01,0xD7AF,{0x03B6,0x0000,0x0000,0x0000}},
{0x01,0xD7B0,{0x03B7,0x0000,0x0000,0x0000}},
{0x01,0xD7B1,{0x03B8,0x0000,0x0000,0x0000}},
{0x01,0xD7B2,{0x03B9,0x0000,0x0000,0x0000}},
{0x01,0xD7B3,{0x03BA,0x0000,0x0000,0x0000}},
{0x01,0xD7B4,{0x03BB,0x0000,0x0000,0x0000}},
{0x01,0xD7B5,{0x03BC,0x0000,0x0000,0x0000}},
{0x01,0xD7B6,{0x03BD,0x0000,0x0000,0x0000}},
{0x01,0xD7B7,{0x03BE,0x0000,0x0000,0x0000}},
{0x01,0xD7B8,{0x03BF,0x0000,0x0000,0x0000}},
{0x01,0xD7B9,{0x03C0,0x0000,0x0000,0x0000}},
{0x01,0xD7BA,{0x03C1,0x0000,0x0000,0x0000}},
{0x01,0xD7BC,{0x03C3,0x0000,0x0000,0x0000}},
{0x01,0xD7BD,{0x03C4,0x0000,0x0000,0x0000}},
{0x01,0xD7BE,{0x03C5,0x0000,0x0000,0x0000}},
{0x01,0xD7BF,{0x03C6,0x0000,0x0000,0x0000}},
{0x01,0xD7C0,{0x03C7,0x0000,0x0000,0x0000}},
{0x01,0xD7C1,{0x03C8,0x0000,0x0000,0x0000}},
{0x01,0xD7C2,{0x03C9,0x0000,0x0000,0x0000}},
{0x01,0xD7C3,{0x2202,0x0000,0x0000,0x0000}},
{0x01,0xD7C4,{0x03F5,0x0000,0x0000,0x0000}},
{0x01,0xD7C5,{0x03D1,0x0000,0x0000,0x0000}},
{0x01,0xD7C6,{0x03F0,0x0000,0x0000,0x0000}},
{0x01,0xD7C7,{0x03D5,0x0000,0x0000,0x0000}},
{0x01,0xD7C8,{0x03F1,0x0000,0x0000,0x0000}},
{0x01,0xD7C9,{0x03D6,0x0000,0x0000,0x0000}},
{0x01,0xD7CE,{0x0030,0x0000,0x0000,0x0000}},
{0x01,0xD7CF,{0x0031,0x0000,0x0000,0x0000}},
{0x01,0xD7D0,{0x0032,0x0000,0x0000,0x0000}},
{0x01,0xD7D1,{0x0033,0x0000,0x0000,0x0000}},
{0x01,0xD7D2,{0x0034,0x0000,0x0000,0x0000}},
{0x01,0xD7D3,{0x0035,0x0000,0x0000,0x0000}},
{0x01,0xD7D4,{0x0036,0x0000,0x0000,0x0000}},
{0x01,0xD7D5,{0x0037,0x0000,0x0000,0x0000}},
{0x01,0xD7D6,{0x0038,0x0000,0x0000,0x0000}},
{0x01,0xD7D7,{0x0039,0x0000,0x0000,0x0000}},
{0x01,0xD7D8,{0x0030,0x0000,0x0000,0x0000}},
{0x01,0xD7D9,{0x0031,0x0000,0x0000,0x0000}},
{0x01,0xD7DA,{0x0032,0x0000,0x0000,0x0000}},
{0x01,0xD7DB,{0x0033,0x0000,0x0000,0x0000}},
{0x01,0xD7DC,{0x0034,0x0000,0x0000,0x0000}},
{0x01,0xD7DD,{0x0035,0x0000,0x0000,0x0000}},
{0x01,0xD7DE,{0x0036,0x0000,0x0000,0x0000}},
{0x01,0xD7DF,{0x0037,0x0000,0x0000,0x0000}},
{0x01,0xD7E0,{0x0038,0x0000,0x0000,0x0000}},
{0x01,0xD7E1,{0x0039,0x0000,0x0000,0x0000}},
{0x01,0xD7E2,{0x0030,0x0000,0x0000,0x0000}},
{0x01,0xD7E3,{0x0031,0x0000,0x0000,0x0000}},
{0x01,0xD7E4,{0x0032,0x0000,0x0000,0x0000}},
{0x01,0xD7E5,{0x0033,0x0000,0x0000,0x0000}},
{0x01,0xD7E6,{0x0034,0x0000,0x0000,0x0000}},
{0x01,0xD7E7,{0x0035,0x0000,0x0000,0x0000}},
{0x01,0xD7E8,{0x0036,0x0000,0x0000,0x0000}},
{0x01,0xD7E9,{0x0037,0x0000,0x0000,0x0000}},
{0x01,0xD7EA,{0x0038,0x0000,0x0000,0x0000}},
{0x01,0xD7EB,{0x0039,0x0000,0x0000,0x0000}},
{0x01,0xD7EC,{0x0030,0x0000,0x0000,0x0000}},
{0x01,0xD7ED,{0x0031,0x0000,0x0000,0x0000}},
{0x01,0xD7EE,{0x0032,0x0000,0x0000,0x0000}},
{0x01,0xD7EF,{0x0033,0x0000,0x0000,0x0000}},
{0x01,0xD7F0,{0x0034,0x0000,0x0000,0x0000}},
{0x01,0xD7F1,{0x0035,0x0000,0x0000,0x0000}},
{0x01,0xD7F2,{0x0036,0x0000,0x0000,0x0000}},
{0x01,0xD7F3,{0x0037,0x0000,0x0000,0x0000}},
{0x01,0xD7F4,{0x0038,0x0000,0x0000,0x0000}},
{0x01,0xD7F5,{0x0039,0x0000,0x0000,0x0000}},
{0x01,0xD7F6,{0x0030,0x0000,0x0000,0x0000}},
{0x01,0xD7F7,{0x0031,0x0000,0x0000,0x0000}},
{0x01,0xD7F8,{0x0032,0x0000,0x0000,0x0000}},
{0x01,0xD7F9,{0x0033,0x0000,0x0000,0x0000}},
{0x01,0xD7FA,{0x0034,0x0000,0x0000,0x0000}},
{0x01,0xD7FB,{0x0035,0x0000,0x0000,0x0000}},
{0x01,0xD7FC,{0x0036,0x0000,0x0000,0x0000}},
{0x01,0xD7FD,{0x0037,0x0000,0x0000,0x0000}},
{0x01,0xD7FE,{0x0038,0x0000,0x0000,0x0000}},
{0x01,0xD7FF,{0x0039,0x0000,0x0000,0x0000}}
};

const struct UnicodeMappingLarge UCCompatLarge[NUCCOMPATLARGE]={
{0x00,0x2057,{0x2032,0x2032,0x2032,0x2032,0x2032,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2177,{0x0076,0x0069,0x0069,0x0069,0x0069,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x247D,{0x0028,0x0031,0x0031,0x0030,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x247E,{0x0028,0x0031,0x0031,0x0031,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x247F,{0x0028,0x0031,0x0031,0x0032,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2480,{0x0028,0x0031,0x0031,0x0033,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2481,{0x0028,0x0031,0x0031,0x0034,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2482,{0x0028,0x0031,0x0031,0x0035,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2483,{0x0028,0x0031,0x0031,0x0036,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2484,{0x0028,0x0031,0x0031,0x0037,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2485,{0x0028,0x0031,0x0031,0x0038,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2486,{0x0028,0x0031,0x0031,0x0039,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2487,{0x0028,0x0032,0x0032,0x0030,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x2A0C,{0x222B,0x222B,0x222B,0x222B,0x222B,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x320E,{0x0028,0x1100,0x1100,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x320F,{0x0028,0x1102,0x1102,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3210,{0x0028,0x1103,0x1103,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3211,{0x0028,0x1105,0x1105,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3212,{0x0028,0x1106,0x1106,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3213,{0x0028,0x1107,0x1107,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3214,{0x0028,0x1109,0x1109,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3215,{0x0028,0x110B,0x110B,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3216,{0x0028,0x110C,0x110C,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3217,{0x0028,0x110E,0x110E,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3218,{0x0028,0x110F,0x110F,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3219,{0x0028,0x1110,0x1110,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x321A,{0x0028,0x1111,0x1111,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x321B,{0x0028,0x1112,0x1112,0x1161,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x321C,{0x0028,0x110C,0x110C,0x116E,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x321D,{0x0028,0x110B,0x110B,0x1169,0x110C,0x1165,0x11AB,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x321E,{0x0028,0x110B,0x110B,0x1169,0x1112,0x116E,0x0029,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x327C,{0x110E,0x1161,0x1161,0x11B7,0x1100,0x1169,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x327D,{0x110C,0x116E,0x116E,0x110B,0x1174,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3300,{0x30A2,0x30D1,0x30D1,0x30FC,0x30C8,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3301,{0x30A2,0x30EB,0x30EB,0x30D5,0x30A1,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3302,{0x30A2,0x30F3,0x30F3,0x30DA,0x30A2,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3304,{0x30A4,0x30CB,0x30CB,0x30F3,0x30B0,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3307,{0x30A8,0x30B9,0x30B9,0x30AF,0x30FC,0x30C9,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3308,{0x30A8,0x30FC,0x30FC,0x30AB,0x30FC,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x330C,{0x30AB,0x30E9,0x30E9,0x30C3,0x30C8,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x330D,{0x30AB,0x30ED,0x30ED,0x30EA,0x30FC,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3312,{0x30AD,0x30E5,0x30E5,0x30EA,0x30FC,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3313,{0x30AE,0x30EB,0x30EB,0x30C0,0x30FC,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3315,{0x30AD,0x30ED,0x30ED,0x30B0,0x30E9,0x30E0,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3316,{0x30AD,0x30ED,0x30ED,0x30E1,0x30FC,0x30C8,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3317,{0x30AD,0x30ED,0x30ED,0x30EF,0x30C3,0x30C8,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3319,{0x30B0,0x30E9,0x30E9,0x30E0,0x30C8,0x30F3,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x331A,{0x30AF,0x30EB,0x30EB,0x30BC,0x30A4,0x30ED,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x331B,{0x30AF,0x30ED,0x30ED,0x30FC,0x30CD,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x331F,{0x30B5,0x30A4,0x30A4,0x30AF,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3320,{0x30B5,0x30F3,0x30F3,0x30C1,0x30FC,0x30E0,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3321,{0x30B7,0x30EA,0x30EA,0x30F3,0x30B0,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x332B,{0x30D1,0x30FC,0x30FC,0x30BB,0x30F3,0x30C8,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x332D,{0x30D0,0x30FC,0x30FC,0x30EC,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x332E,{0x30D4,0x30A2,0x30A2,0x30B9,0x30C8,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3332,{0x30D5,0x30A1,0x30A1,0x30E9,0x30C3,0x30C9,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3333,{0x30D5,0x30A3,0x30A3,0x30FC,0x30C8,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3334,{0x30D6,0x30C3,0x30C3,0x30B7,0x30A7,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3336,{0x30D8,0x30AF,0x30AF,0x30BF,0x30FC,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x333D,{0x30DD,0x30A4,0x30A4,0x30F3,0x30C8,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3343,{0x30DE,0x30A4,0x30A4,0x30AF,0x30ED,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3347,{0x30DE,0x30F3,0x30F3,0x30B7,0x30E7,0x30F3,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3348,{0x30DF,0x30AF,0x30AF,0x30ED,0x30F3,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x334A,{0x30DF,0x30EA,0x30EA,0x30D0,0x30FC,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x334C,{0x30E1,0x30AC,0x30AC,0x30C8,0x30F3,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x334D,{0x30E1,0x30FC,0x30FC,0x30C8,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3351,{0x30EA,0x30C3,0x30C3,0x30C8,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3354,{0x30EB,0x30FC,0x30FC,0x30D6,0x30EB,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3356,{0x30EC,0x30F3,0x30F3,0x30C8,0x30B2,0x30F3,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x337F,{0x682A,0x5F0F,0x5F0F,0x4F1A,0x793E,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x3389,{0x006B,0x0063,0x0063,0x0061,0x006C,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x33A8,{0x006D,0x2215,0x2215,0x0073,0x00B2,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x33AE,{0x0072,0x0061,0x0061,0x0064,0x2215,0x0073,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x33AF,{0x0072,0x0061,0x0061,0x0064,0x2215,0x0073,0x00B2,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x33C2,{0x0061,0x002E,0x002E,0x006D,0x002E,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0x33D8,{0x0070,0x002E,0x002E,0x006D,0x002E,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDF2,{0x0627,0x0644,0x0644,0x0644,0x0647,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDF3,{0x0627,0x0643,0x0643,0x0628,0x0631,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDF4,{0x0645,0x062D,0x062D,0x0645,0x062F,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDF5,{0x0635,0x0644,0x0644,0x0639,0x0645,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDF6,{0x0631,0x0633,0x0633,0x0648,0x0644,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDF7,{0x0639,0x0644,0x0644,0x064A,0x0647,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDF8,{0x0648,0x0633,0x0633,0x0644,0x0645,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDFA,{0x0635,0x0644,0x0644,0x0649,0x0020,0x0627,0x0644,0x0644,0x0647,0x0020,0x0639,0x0644,0x064A,0x0647,0x0020,0x0648,0x0633,0x0644,0x0645}},
{0x00,0xFDFB,{0x062C,0x0644,0x0644,0x0020,0x062C,0x0644,0x0627,0x0644,0x0647,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}},
{0x00,0xFDFC,{0x0631,0x06CC,0x06CC,0x0627,0x0644,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000}}
};


CIDNA::CIDNA(void)
{
	nLabels=0;
	SchemeLen=0;
	UserPasswordLen=0;
}

CIDNA::~CIDNA(void)
{
}

TCHAR* CIDNA::EncodeHex(TCHAR *sIn,UINT CodePage)
{
	size_t nIn;

	StrCchLen(sIn,MAXURLLENGTH,nIn);
	return EncodeHex(sIn,nIn,CodePage);
}

TCHAR* CIDNA::EncodeHex(TCHAR *sIn,size_t nIn,UINT CodePage)
{
    TCHAR *sOut,*pOut;
    size_t nOut;
	UINT i;
	BYTE ch;
	BYTE MultiByteStr[MAXURLLENGTH];

	if(CodePage!=CODEPAGE_NO_ENCODING)
	{
		nOut=WideCharToMultiByte(CodePage,0,sIn,(int)nIn,(char *)MultiByteStr,sizeof MultiByteStr,"?",NULL);
		if(nOut)
		{
			pOut = sOut = new TCHAR[nOut*3+1];
			if(pOut)
			{
				i=0;
				while(i<nOut) // Encoding
				{
					ch=MultiByteStr[i];

					if(isalnum(ch) && ch < 128)
					{
						*pOut++ = ch;
					}
					else if(isspace(ch))
					{
						*pOut++ = L'+';
					}
					else
					{
						*pOut++ = L'%';
						*pOut++ = toHex(ch>>4);
						*pOut++ = toHex(ch%16);
					}
					i++;
				}
				*pOut = L'\0';
			}
		}
		else sOut=NULL;
	}
	else // No Encoding
	{
		sOut = new TCHAR[nIn+1];
		if(sOut) StringCchCopyN(sOut,nIn+1,sIn,nIn);
	}

	return sOut;
}

TCHAR* CIDNA::DecodeHex(TCHAR *sIn,UINT CodePage)
{	
	TCHAR *pSrc;
	TCHAR ch;
	size_t len;
	BYTE hi,lo;
	BYTE MultiByteStr[MAXURLLENGTH];

	if(CodePage!=CODEPAGE_NO_ENCODING)
	{
		pSrc=sIn;
		ch=*pSrc;

		len=0;
		while(ch && len<MAXURLLENGTH)
		{
			if(ch==L'+') ch=L' ';
			else if(ch==L'%')
			{
				if(pSrc[1] && pSrc[2])
				{
					hi=fromHex(pSrc[1]);
					lo=fromHex(pSrc[2]);

					if((hi&0xf0)==0 && (lo&0xf0)==0) // Decode
					{
						ch = (hi<<4) | lo;
						pSrc+=2;
					}
				}
			}
			else if(ch>0xFF) ch=L'?';

			MultiByteStr[len]=(BYTE)ch;
			len++;
			pSrc++;
			ch=*pSrc;
		}

		if(len)
		{
			len=MultiByteToWideChar(CodePage,0,(char*)MultiByteStr,(int)len,sIn,(int)len);
			sIn[len]=L'\0';
		}
	}

	return sIn;
}

TCHAR* CIDNA::EncodeUTF8(TCHAR *szSource)
{
	size_t len;
	
	StrCchLen(szSource,MAXURLLENGTH,len);
	return EncodeUTF8(szSource,len);
}

TCHAR* CIDNA::EncodeUTF8(TCHAR *szSource,size_t len)
{
	DWORD ch;
	BYTE bt1, bt2, bt3, bt4;
	TCHAR* sFinal,*sTemp;

	sFinal = sTemp = new TCHAR[len*9+1];

	if(sTemp)
	{
		while(len)
		{
			ch = DecodeUTF16(&szSource);

			if (ch< 128 && _istalnum((TCHAR)ch)) *sTemp++ = (TCHAR)ch;
			else if(_istspace((TCHAR)ch)) *sTemp++ = _T('+');
			else if (ch < 128)
			{
				*sTemp++ = _T('%');
				*sTemp++ = toHex((TCHAR)ch>>4);
				*sTemp++ = toHex((TCHAR)ch%16);
			}
			else if (ch <= 2047)
			{
				bt1 = (BYTE)(192 + (ch >> 6));
				bt2 = (BYTE)(128 + (ch & 0x3F));

				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt1>>4);
				*sTemp++ = toHex(bt1&0x0F);
				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt2>>4);
				*sTemp++ = toHex(bt2&0x0F);
			}
			else if (ch <= 65535)
			{
				bt1 = (BYTE)(224 + (ch >> 12));
				bt2 = (BYTE)(128 + ((ch >> 6) & 0x3F));
				bt3 = (BYTE)(128 + (ch & 0x3F));

				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt1>>4);
				*sTemp++ = toHex(bt1&0x0F);
				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt2>>4);
				*sTemp++ = toHex(bt2&0x0F);
				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt3>>4);
				*sTemp++ = toHex(bt3&0x0F);
			}
			else if (ch <= 0x1FFFFF)
			{
				bt1 = (BYTE)(240 + (ch >> 18));
				bt2 = (BYTE)(128 + ((ch >> 12) & 0x3F));
				bt3 = (BYTE)(128 + ((ch >> 6) & 0x3F));
				bt4 = (BYTE)(128 + (ch & 0x3F));

				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt1>>4);
				*sTemp++ = toHex(bt1&0x0F);
				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt2>>4);
				*sTemp++ = toHex(bt2&0x0F);
				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt3>>4);
				*sTemp++ = toHex(bt3&0x0F);
				*sTemp++ = _T('%');
				*sTemp++ = toHex(bt4>>4);
				*sTemp++ = toHex(bt4&0x0F);
			}
			szSource++;
			len--;
		}
		*sTemp=_T('\0');
	}
	return sFinal;
}

TCHAR* CIDNA::DecodeUTF8(TCHAR *szSource)
{
	TCHAR *pSrc,*pDest;
	TCHAR ch;
	DWORD cp;
	int skip;
	size_t len;

	BYTE z, y, x, w;
	BYTE lo,hi;

	pSrc=pDest=szSource;

	StrCchLen(pSrc,MAXURLLENGTH,len);
	ch=*pSrc;
	skip=1;
	while(ch)
	{
		if(ch==L'+') ch=L' ';

		*pDest=ch;

		if(ch==L'%' && len>=3)
		{
			hi=fromHex(pSrc[1]);	// Make byte
			lo=fromHex(pSrc[2]);

			if((hi&0xf0)==0 && (lo&0xf0)==0)
			{
				z = (hi<<4) | lo;
				if(z < 127)
				{
					*pDest=(TCHAR)z;
					skip=3;
				}
				else if((z&0xE0)==0xC0)
				{
					// Character is two bytes
					if(len>=6 && pSrc[3]==L'%')
					{
						hi=fromHex(pSrc[4]); // Make byte
						lo=fromHex(pSrc[5]);
						if((hi&0xf0)==0 && (lo&0xf0)==0)
						{
							y = (hi<<4) | lo;
							if((y&0xC0)==0x80) // Octet starts with bits: 10xxxxxx
							{
								cp=(z & 0x1F)<<6 | (y & 0x3F);
								if(cp>=0x80) *pDest = (TCHAR)cp; // Non-shortest form?
							}
							skip=6;
						}
					}
				}
				else if((z&0xF0)==0xE0)
				{
					// Character is three bytes
					if(len>=9 && pSrc[3]==L'%' && pSrc[6]==L'%')
					{
						hi=fromHex(pSrc[4]); // Make byte
						lo=fromHex(pSrc[5]);
						if((hi&0xf0)==0 && (lo&0xf0)==0)
						{
							y = (hi<<4) | lo;
							hi=fromHex(pSrc[7]); // Make byte
							lo=fromHex(pSrc[8]);
							if((hi&0xf0)==0 && (lo&0xf0)==0)
							{
								x = (hi<<4) | lo;
								if((y&0xC0)==0x80 && (x&0xC0)==0x80) // Octet starts with bits: 10xxxxxx
								{
									cp=(z & 0x0F)<<12 | (y & 0x3F)<<6 | (x & 0x3F);
									if(cp>=0x800) *pDest = (TCHAR)cp; // Non-shortest form?
								}
								skip=9;
							}
						}
					}
				}
				else if((z&0xF8)==0xF0)
				{
					// Character is four bytes
					if(len>=12 && pSrc[3]==L'%' && pSrc[6]==L'%' && pSrc[9]==L'%')
					{
						hi=fromHex(pSrc[4]); // Make byte
						lo=fromHex(pSrc[5]);
						if((hi&0xf0)==0 && (lo&0xf0)==0)
						{
							y = (hi<<4) | lo;
							hi=fromHex(pSrc[7]); // Make byte
							lo=fromHex(pSrc[8]);
							if((hi&0xf0)==0 && (lo&0xf0)==0)
							{
								x = (hi<<4) | lo;
								hi=fromHex(pSrc[10]); // Make byte
								lo=fromHex(pSrc[11]);
								if((hi&0xf0)==0 && (lo&0xf0)==0)
								{
									w = (hi<<4) | lo;
									if((y&0xC0)==0x80 && (x&0xC0)==0x80 && (w&0xC0)==0x80)
									{
										cp = (z & 0x07)<<18 | (y & 0x3F)<<12 | (x & 0x3F)<<6 | (w & 0x3F);
										if(cp>=0x10000)	EncodeUTF16(cp,&pDest); // Non-shortest form?
									}
									
									skip=12;
								}
							}
						}
					}
				}
				// else character is five+ bytes long (not handled yet)
			} // End z valid
		} // End ch==L'%' && len>=3

		pDest++;

		len-=skip;
		pSrc+=skip;
		if(skip>1) skip=1;
		ch=*pSrc;
	}

	*pDest=L'\0';

	return szSource;
}

DWORD CIDNA::DecodeUTF16(TCHAR **sIn)
{
	DWORD cp = **sIn;

	if(cp>=0xD800 && cp<=0xDBFF) // Surrogate Pair
	{
		WORD w2;

		w2=*(*sIn+1);
		if(w2>=0xDC00 && w2<=0xDFFF)
		{
			cp=(((cp&0x03FF)<<10) | (w2&0x03FF))+0x10000;
			(*sIn)++;
		}
	}
	return cp;
}

TCHAR* CIDNA::EncodeUTF16(DWORD cp,TCHAR **sOut)
{
	if(cp<0xD800) **sOut=(WORD)cp;
	else if(cp<=0xDFFF) **sOut=UNICODE_REPLACEMENT_CHARACTER;
	else if(cp<0x10000) **sOut=(WORD)cp;
	else // Surrogate Pair
	{
		if(cp<=0x10FFFF)
		{
			cp-=0x10000;
			**sOut=0xD800|(WORD)(cp>>10);
			(*sOut)++;
			**sOut=0xDC00|(WORD)(cp&0x03FF);
		}
		else **sOut=UNICODE_REPLACEMENT_CHARACTER;
	}

	return *sOut;
}

/*
 * decode_digit(cp) returns the numeric value of a basic code
 * point (for use in representing integers) in the range 0 to
 * base-1, or base if cp does not represent a value.
 */
DWORD CIDNA::decode_digit (TCHAR cp)
{
  return (cp - 48 < 10 ?
          cp - 22 : cp - 65 < 26 ?
          cp - 65 : cp - 97 < 26 ?
          cp - 97 : base);
}

/*
 * encode_digit(d,flag) returns the basic code point whose value
 * (when used for representing integers) is d, which needs to be in
 * the range 0 to base-1.  The lowercase form is used unless flag is
 * nonzero, in which case the uppercase form is used.  The behavior
 * is undefined if flag is nonzero and digit d has no uppercase form.
 */
TCHAR CIDNA::encode_digit (DWORD d, int flag)
{
	return (TCHAR)d + ((d<26)?(flag?L'A':L'a'):(TCHAR)22);
  //return ((TCHAR)d + 22 + 75 * (d < 26) - ((flag != 0) << 5));
  /*  0..25 map to ASCII a..z or A..Z */
  /* 26..35 map to ASCII 0..9         */
}

/*
 * encode_basic(bcp,flag) forces a basic code point to lowercase
 * if flag is zero, uppercase if flag is nonzero, and returns
 * the resulting code point.  The code point is unchanged if it
 * is caseless.  The behavior is undefined if bcp is not a basic
 * code point.
 */
TCHAR CIDNA::encode_basic (DWORD bcp, int flag)
{
	if(flag)
	{
		if(bcp>=L'a' && bcp<=L'z') bcp+=32;
	}
	else
	{
		if(bcp>=L'A' && bcp<=L'Z') bcp-=32;
	}
	
	return (TCHAR)bcp;
  //bcp -= (bcp - 97 < 26) << 5;
  //return TCHAR(bcp + ((!flag && (bcp - 65 < 26)) << 5));
}

DWORD CIDNA::adapt (DWORD delta, DWORD numpoints, int firsttime)
{
  DWORD k;

  delta = firsttime ? delta / damp : delta >> 1;
  /* delta >> 1 is a faster way of doing delta / 2
   */
  delta += delta / numpoints;

  for (k = 0; delta > ((base - tmin) * tmax) / 2; k += base)
      delta /= base - tmin;
  return k + (base - tmin + 1) * delta / (delta + skew);
}

/*
 *Main encode function
 */
punycode_status CIDNA::punycode_encode (size_t input_length,
                                        const DWORD *input,
                                        const BYTE *case_flags,
                                        size_t *output_length,
                                        TCHAR *output)
{
  DWORD n, delta, h, b, out, bias, j, m, q, k, t;
  size_t max_out;

  /* Initialize the state: */

  n = initial_n;
  delta = out = 0;
  max_out = *output_length;
  bias = initial_bias;

  /* Handle the basic code points: */
  for (j = 0; j < input_length; ++j)
  {
    if (basic(input[j]))
    {
	  if (max_out - out < 2) return (punycode_big_output);
      output[out++] = TCHAR(case_flags ? encode_basic (input[j], case_flags[j]) : input[j]);
    }
#if 0
    else if (input[j] < n) return (punycode_bad_input);
    /* (not needed for Punycode with unsigned code points) */
#endif
  }

  h = b = out;

  /* h is the number of code points that have been handled, b is the
   * number of basic code points, and out is the number of characters
   * that have been output.
   */
  if (b > 0) output[out++] = delimiter;

  /* Main encoding loop:
   */
  while (h < input_length)
  {
    /* All non-basic code points < n have been
     * handled already.  Find the next larger one:
     */
    for (m = MAXDWORD, j = 0; j < input_length; ++j)
    {
#if 0
      if (basic(input[j])) continue;
      /* (not needed for Punycode) */
#endif
      if (input[j] >= n && input[j] < m) m = input[j];
    }

    /* Increase delta enough to advance the decoder's
     * <n,i> state to <m,0>, but guard against overflow:
     */
    if (m - n > (MAXDWORD - delta) / (h + 1)) return (punycode_overflow);

    delta += (m - n) * (h + 1);
    n = m;

    for (j = 0; j < input_length; ++j)
    {
      if (input[j] < n)
      {
        if (++delta == 0) return (punycode_overflow);
      }

      if (input[j] == n)
      {
        /* Represent delta as a generalized variable-length integer:
         */
        for (q = delta, k = base;; k += base)
        {
          if (out >= max_out) return (punycode_big_output);

          t = k <= bias ? tmin :
              k >= bias + tmax ? tmax :
              k - bias;
          if (q < t)
             break;
          output[out++] = encode_digit ((t + (q - t) % (base - t)), 0);
          q = (q - t) / (base - t);
        }
        output[out++] = encode_digit (q, case_flags && case_flags[j]);
        bias = adapt (delta, h + 1, h == b);
        delta = 0;
        ++h;
      }
    }
    ++delta;
    ++n;
  }

  output[out]=0;
  *output_length = out;
  return (punycode_success);
}

/*** Main decode function ***/

punycode_status CIDNA::punycode_decode (size_t input_length,
                                        const TCHAR *input,
                                        size_t *output_length,
                                        DWORD *output,
                                        BYTE *case_flags)
{
  DWORD n, out, i, bias, b, j, in, oldi, w, k, digit, t;
  size_t max_out;

  /* Initialize the state: */

  n = initial_n;
  out = i = 0;
  max_out = *output_length;
  bias = initial_bias;

  /* Handle the basic code points:  Let b be the number of input code
   * points before the last delimiter, or 0 if there is none, then
   * copy the first b code points to the output.
   */
  for (b = j = 0; j < input_length; ++j)
      if (delim (input[j])) b = j;
  if (b > max_out) return (punycode_big_output);

  for (j = 0; j < b; ++j)
  {
    if (case_flags) case_flags[out] = flagged (input[j]);
    if (!basic (input[j])) return (punycode_bad_input);
    output[out++] = input[j];
  }

  /* Main decoding loop:  Start just after the last delimiter if any
   * basic code points were copied; start at the beginning otherwise.
   */
  for (in = b > 0 ? b + 1 : 0; in < input_length; ++out)
  {
    /* in is the index of the next character to be consumed, and
     * out is the number of code points in the output array.
     */

    /* Decode a generalized variable-length integer into delta,
     * which gets added to i.  The overflow checking is easier
     * if we increase i as we go, then subtract off its starting
     * value at the end to obtain delta.
     */
    for (oldi = i, w = 1, k = base;; k += base)
    {
      if (in >= input_length) return (punycode_bad_input);

      digit = decode_digit (input[in++]);
      if (digit >= base) return (punycode_bad_input);

      if (digit > (MAXDWORD - i) / w) return (punycode_overflow);

      i += digit * w;
      t = k <= bias ? tmin :
          k >= bias + tmax ? tmax :
          k - bias;
      if (digit < t) break;
      if (w > MAXDWORD / (base - t)) return (punycode_overflow);

      w *= (base - t);
    }

    bias = adapt (i - oldi, out + 1, oldi == 0);

    /* i was supposed to wrap around from out+1 to 0,
     * incrementing n each time, so we'll fix that now:
     */
    if (i / (out + 1) > MAXDWORD - n) return (punycode_overflow);

    n += i / (out + 1);
    i %= (out + 1);

    /* Insert n at position i of the output:
     */
#if 0
    /* not needed for Punycode: */
    if (decode_digit(n) <= base) return (punycode_invalid_input);
#endif
    if (out >= max_out) return (punycode_big_output);

    if (case_flags)
    {
      MoveMemory(case_flags + i + 1, case_flags + i, out - i);

      /* Case of last character determines uppercase flag:
       */
      case_flags[i] = flagged (input[in - 1]);
    }
    MoveMemory(output + i + 1, output + i, (out - i) * sizeof *output);
    output[i++] = n;
  }

  *output_length = out;
  return (punycode_success);
}

/*
 * Split a URL into labels
 */
bool CIDNA::split_labels (const TCHAR *url,bool bNoScheme)
{
	int i,len;
	const TCHAR *p,*sep;
	TCHAR ch;
	bool result=false;
  
	IsIPv4=false;
	IsIPv6=false;
	nLabels=0;
	p=url;

	// Extract scheme
	if(bNoScheme==false)
	{
		ch=*p;
		while(ch && ch!=L':')
		{
			p++;
			ch=*p;
		}
	}
	else ch=0;
	
	if(ch==L':')
	{
		p++;
		if(*p==L'/' && *(p+1)==L'/')
		{
			p+=2;
			i=3;
		}
		else i=1;

		len=(int)(p-url);
		if(len<=i || len>MAX_SCHEME_LEN+i) return false;
		StringCbCopyN(Scheme,sizeof Scheme,url,len*sizeof(TCHAR));
		SchemeLen=len;
	}
	else
	{
		Scheme[0]=L'\0';
		SchemeLen=0;
		p=url;
	}

	// Check if file URL has empty host name
	if(!StrCmpNI(Scheme,L"file",4) && *p==L'/')
	{
		UserPasswordLen=0;
		nLabels=0;
		StringCchCopy(URLPath,MAXURLLENGTH,p);
		return true;
	}

	// Test if user:password@ exists
	sep=p;
	while(*sep && *sep!=L'/' && *sep!=L'@') sep++;

	if(*sep==L'@')
	{
		sep++;
		len=(int)(sep-p);
		if(!len || len>MAX_USERPASSWORD_LEN) return false;
		StringCbCopyN(UserPassword,sizeof UserPassword,p,len*sizeof(TCHAR));
		p=sep;
	}
	else len=0;
	UserPassword[len]=L'\0';
	UserPasswordLen=len;

	// Is IPv6 address?
	if(*p==L'[')
	{
		len=IsIPv6reference(p);
		if(len)
		{
			StringCchCopyN(Labels[0],MAX_INT_LABEL_LEN,p,len);
			LabelLengths[0]=len;
			sep=p+len;
			nLabels=1;

			QDEBUG_PRINTF(L"IPv6",L"%s %d",sep,len);

			IsIPv6=true;
		}
		else return false;
	}
	else
	{
		// Split domain name labels
		i=0;
		sep=p;
		while(i<MAX_HOST_LABELS)
		{
			while(*sep && !IsUnicodeDot(*sep) && *sep!=L'/' && *sep!=L':' && *sep!=L'?') sep++;
        
			len=(int)(sep-p);

			if(len==0 || len>MAX_INT_LABEL_LEN) return false;
        
			StringCchCopyN(Labels[i],MAX_INT_LABEL_LEN,p,len);
			LabelLengths[i]=len;

			i++;

			if(IsUnicodeDot(*sep))
			{
				// Is Fully Qualified Domain Name?
				ch=sep[1];
				if(ch==0 || ch==L'/' || ch==L':' || ch==L'?')
				{
					sep++;
					IsFQDN=true;
					break;
				}
			}
			else
			{
				IsFQDN=false;
				break;
			}

			p=++sep;
		}

		nLabels=i;

		// Is IPv4 address
		if(nLabels==4)
		{
			i=0;
			while(i<4 && IsOctet(Labels[i])) i++;
			if(i==4) IsIPv4=true;
		}
	}

	if(nLabels && nLabels<=MAX_HOST_LABELS)
	{
		// Copy rest of URL
		StringCchCopy(URLPath,MAXURLLENGTH,sep);

		// Extract port number
		if(*sep==L':')
		{
			i=0;
			p=sep+1;
			ch=*p;
			PortNumber=0;
			while(ch>=L'0' && ch<=L'9' && i<5)
			{
				PortNumber=PortNumber*10+ch-L'0';
				i++;
				p++;
				ch=*p;
			}
			if(i==0) PortNumber=PORTNUMBER_DEFAULT;

			if(ch==L'\0' || ch==L'/' || ch==L'?')
			{
				if(PortNumber<0x10000) result=true;
			}
		}
		else
		{
			PortNumber=PORTNUMBER_OMITTED;
			result=true;
		}
	}

	return result;
}

int CIDNA::IsIPv6reference(const TCHAR *pIPv6reference)
{
	int len;
	int nwords;
	int noctets;
	bool bMixedIPv4;
	int QuadDotsPos;
	TCHAR ch,label[5];
	const TCHAR *sep;
	int i;

	len=0;

	if(*pIPv6reference==L'[')
	{
		sep=pIPv6reference+1;
		ch=*sep;
		i=0;
		nwords=noctets=0;
		bMixedIPv4=false;
		QuadDotsPos=-1;

		while( ch && ch!=L']' && (ch!=L'%' || sep[1]!=L'2' || sep[2]!=L'5') )
		{
			if(ch==L':')
			{
				if(bMixedIPv4==false)
				{
					if(QuadDotsPos==-1 && sep[1]==L':')
					{
						QuadDotsPos=nwords;
						sep++;
					}
					else
					{
						label[i]=0;
						if(IsHexWord(label)) nwords++;
						else break;
					}
				}
				else break;

				i=0;
			}
			else if(ch==L'.')
			{
				label[i]=0;
				if(IsOctet(label)) noctets++;
				else break;
				bMixedIPv4=true;
				i=0;
			}
			else if(i<4) label[i++]=ch;
			else break;

			sep++;
			ch=*sep;
		}

		if(ch==L'%') // Skip Scope Id
		{
			sep+=4; // Skip first character of Scopy Id
			ch=*sep;
			while(ch && ch!=L']')
			{
				sep++;
				ch=*sep;
			}
		}

		if(ch==L']')
		{
			sep++;
			ch=*sep;
			if(ch==L'\0' || ch==L'/' || ch==L':' || ch==L'?')
			{
				if(i)
				{
					label[i]=0;
					if(bMixedIPv4)
					{
						if(IsOctet(label)) noctets++;
						else nwords=-1;
					}
					else
					{
						if(IsHexWord(label)) nwords++;
						else nwords=-1;
					}
				}
				else if(QuadDotsPos!=nwords || bMixedIPv4) nwords=-1;

				if(nwords!=-1)
				{
					if(	(nwords==8 && bMixedIPv4==false && QuadDotsPos==-1) || 
						(nwords<8 && bMixedIPv4==false && QuadDotsPos>=0) ||
						(noctets==4 && ((nwords==6 && QuadDotsPos==-1) || (nwords<6 && QuadDotsPos>=0))) )
					{
						len=(int)(sep-pIPv6reference);
					}
				}
			}
		} // End L']'

	} // End L'['

	return len;
}

/* Retrieves the starting position of the relevant part of the address */
int CIDNA::GetDomainStartIndex(int *RegisteredDomainLabel)
{
	size_t DomainStartIndex;
	int i;
	TCHAR *p;

	DomainStartIndex=SchemeLen+UserPasswordLen;
	if(nLabels)
	{
		p=Labels[nLabels-1];
		while(*p)
		{
			if(!_istdigit(*p)) break;
			p++;
		}
		if(*p) // Is not numeric address
		{
			// For some ccTLD the third level domain is the relevant part of the hostname
			if(nLabels>1 && LabelLengths[nLabels-1]==2 && ( LabelLengths[nLabels-2]<=2 || !StrCmp(Labels[nLabels-1],L"uk") ||
				(!StrCmp(Labels[nLabels-2],L"com") || !StrCmp(Labels[nLabels-2],L"net") || !StrCmp(Labels[nLabels-2],L"org") )))
			{
				i=nLabels-3;
			}
			else // Second level domain is the relevant part of the hostname
			{
				i=nLabels-2;
			}

			if(RegisteredDomainLabel)
			{
				if(IsTLD(Labels[nLabels-1]) && i>=0) *RegisteredDomainLabel=i;
				else *RegisteredDomainLabel=-1;
			}

			while(i>0)
			{
				i--;
				DomainStartIndex+=LabelLengths[i]+1;
			}
		}
	}

	return (int)DomainStartIndex;
}

/* url  IN/OUT: native ASCII/ACE name */
/* size IN:     length of url buffer */
/*      OUT:    ACE encoded length */
int CIDNA::URLToAscii(TCHAR *url,size_t *size,int *HostStartIndex,int *HostEndIndex,int *DomainStartIndex)
{                         
	UINT i;
	size_t total_len,path_len;
    TCHAR *p;
	int domainstartidx;
	int RegisteredDomainLabel;
	int result=IDNA_UNMODIFIED;

	if(split_labels(url))
	{
		total_len=SchemeLen+UserPasswordLen+nLabels;
		if(nLabels) total_len--; // nLabels-1 dots
		if(IsFQDN) total_len++;
		if(HostStartIndex) *HostStartIndex=(int)(SchemeLen+UserPasswordLen);

		// Internationalize top level domains (map TLD to ascii)
		if(nLabels)
		{
			TCHAR *TLD;

			TLD=Labels[nLabels-1];
			result=IDNA_MODIFIED;
			if(!StrCmp(TLD,L"\x440\x443")) StringCchCopy(TLD,MAX_INT_LABEL_LEN,L"ru");
			else if(!StrCmp(TLD,L"\x43a\x43e\x43c")) StringCchCopy(TLD,MAX_INT_LABEL_LEN,L"com");
			else if(!StrCmp(TLD,L"\x43e\x440\x433")) StringCchCopy(TLD,MAX_INT_LABEL_LEN,L"org");
			else if(!StrCmp(TLD,L"\x43d\x435\x442")) StringCchCopy(TLD,MAX_INT_LABEL_LEN,L"net");
			else result=IDNA_UNMODIFIED;
		}

		domainstartidx=GetDomainStartIndex(&RegisteredDomainLabel);

		for(i=0;i<nLabels;i++)
		{
			int status;

			status=LabelToAscii(Labels[i],&LabelLengths[i]);		
			if(status&IDNA_ILLEGAL) return IDNA_ILLEGAL;

			if((int)i==RegisteredDomainLabel && (status&IDNA_ASCIIRULES_VIOLATED)) result|=IDNA_DOMAIN_VIOLATES_ASCIIRULES;

			result|=status;

			total_len+=LabelLengths[i];
		}
		if(total_len-domainstartidx > MAX_HOST_LEN) return IDNA_ILLEGAL;
		if(HostEndIndex) *HostEndIndex=(int)total_len;
		if(DomainStartIndex) *DomainStartIndex=domainstartidx;
		StrCchLen(URLPath,MAXURLLENGTH,path_len);
		total_len+=path_len;
		if(result&IDNA_MODIFIED)
		{
			if(total_len<*size)
			{
				if(SchemeLen)
				{
					StringCbCopy(url,sizeof Scheme,Scheme);
					p=url+SchemeLen;
				}
				else p=url;
				if(UserPasswordLen)
				{
					StringCbCopy(p,sizeof UserPassword,UserPassword);
					p+=UserPasswordLen;
				}
				for(i=0;i<nLabels;i++)
				{
					StringCchCopy(p,MAX_INT_LABEL_LEN,Labels[i]);
					p+=LabelLengths[i];
					if(i<nLabels-1) *(p++)=_T('.');
				}
				if(IsFQDN) *(p++)=_T('.');
				StringCchCopy(p,MAXURLLENGTH,URLPath); // Inaccurate destination buffer length (length checked above)
				*size=total_len;
			}
		}
	}
	else return IDNA_ILLEGAL; // split_labels failed

	*size=total_len;
	return result;
}

int CIDNA::LabelToAscii(TCHAR *label,size_t *label_len)
{
	UINT j;
	size_t len;
    TCHAR *p;
	DWORD codepoints[MAX_INT_LABEL_LEN];
	DWORD *pUCP;
	DWORD cp;
	punycode_status status;
	size_t out_len;
	int result=IDNA_UNMODIFIED;

	p=label;
	while(*p)
	{
		if(*p>0x7F)
		{
			// Nameprep
			if(!Nameprep(label,codepoints,&len)) return IDNA_ILLEGAL;

			// Check STD3ASCIIRules
			pUCP=codepoints;
			if(*pUCP==L'-' || *(pUCP+len-1)==L'-') result|=IDNA_ASCIIRULES_VIOLATED;
			j=0;
			while(j<len)
			{
				cp=*pUCP;
				if(cp<0x2C || cp==0x2E || cp==0x2F || (cp>=0x3A && cp<=0x40) || (cp>=0x5B && cp<=0x60) || (cp>=0x7B && cp<=0x7F))
				{
					if(cp>0x20 && cp<0x7F) result|=IDNA_ASCIIRULES_VIOLATED;
					else return IDNA_ILLEGAL; // Label contains ASCII control characters
				}
				pUCP++;
				j++;
			}

			p=label;
			pUCP=codepoints;
			j=0;
			while(j<len)
			{
				if(*pUCP>0x7F)
				{
					out_len=sizeof(buffer);
					
					// Verify that the sequence does not begin with the ACE prefix
					if(len>=ACE_PREFIX_LEN && codepoints[0]==L'x' && codepoints[1]==L'n' && codepoints[2]==L'-' && codepoints[3]==L'-') return IDNA_ILLEGAL;

					// Encode the sequence using the punycode algorithm
					status=punycode_encode(len,codepoints,NULL,&out_len,buffer);
					if (status != punycode_success || out_len>=MAX_INT_LABEL_LEN-ACE_PREFIX_LEN) return IDNA_ILLEGAL;

					// Prepend the ACE prefix
					StringCchCopy(label,MAX_INT_LABEL_LEN,ACE_PREFIX);
					StringCchCopy(label+ACE_PREFIX_LEN,MAX_INT_LABEL_LEN-ACE_PREFIX_LEN,buffer);
					*label_len=out_len+ACE_PREFIX_LEN;
					result|=IDNA_NONASCII;
					break;
				}
				else
				{
					*p=(TCHAR)*pUCP;
					p++;
				}
				pUCP++;
				j++;
			}

			if(j>=len)
			{
				*label_len=len;
				p--;
			}

			result|=IDNA_MODIFIED;
			break;
		}
		p++;
	}

	len=*label_len;

	if((result&IDNA_MODIFIED)==0)
	{
		// Check STD3ASCIIRules
		p=label;
		if(*p==L'-' || *(p+len-1)==L'-') result|=IDNA_ASCIIRULES_VIOLATED;
		j=0;
		while(j<len)
		{
			cp=*p;
			if(cp<0x2C || cp==0x2E || cp==0x2F || (cp>=0x3A && cp<=0x40) || (cp>=0x5B && cp<=0x60) || (cp>=0x7B && cp<=0x7F))
			{
				if(cp>0x20 && cp<0x7F) result|=IDNA_ASCIIRULES_VIOLATED;
				else return IDNA_ILLEGAL; // Label contains ASCII control characters
			}
			p++;
			j++;
		}
	}

	// Verify that the number of code points is in the range 1 to 63 inclusive
	if(len<1 || len>MAX_LABEL_LEN) return IDNA_ILLEGAL;

	return result;
}

/* url  IN/OUT: native ASCII/ACE name */
/* size IN:     length of url buffer */
/*      OUT:    ACE encoded length */
int CIDNA::URLToUnicode (TCHAR *url,size_t *size,int *HostStartIndex,int *HostEndIndex,int *DomainStartIndex)
{                         
	UINT i,j;
	size_t len;
	size_t total_len;
    TCHAR *p;
	TCHAR acelabel[MAX_INT_LABEL_LEN];
	TCHAR unicodelabel[MAX_INT_LABEL_LEN];
	DWORD codepoints[MAX_INT_LABEL_LEN];
	DWORD *pUCP;
	DWORD cp;
	punycode_status status;
	size_t out_len;
	int result=IDNA_UNMODIFIED;

	if(split_labels(url))
	{
		total_len=SchemeLen+UserPasswordLen+nLabels;
		if(nLabels) total_len--; // nLabels-1 dots
		if(IsFQDN) total_len++;
		if(HostStartIndex) *HostStartIndex=(int)(SchemeLen+UserPasswordLen);
		for(i=0;i<nLabels;i++)
		{
			p=Labels[i];

			// Check that all code points are in the range 0..7F
			while(*p)
			{
				if(*p>0x7F)
				{
					// Nameprep
					if(!Nameprep(Labels[i],codepoints,&len)) return IDNA_ILLEGAL;

					result|=IDNA_MODIFIED; // Not accurate but we assume that label was modified

					// Copy back result
					j=0;
					p=Labels[i];
					pUCP=codepoints;
					while(len && j<MAX_INT_LABEL_LEN-1)
					{
						cp=*pUCP;
						EncodeUTF16(cp,&p);
						if(IsUTF16BMP(cp)) j++;
						else j+=2;
						pUCP++;
						p++;
						len--;
					}
					*p=0;

					break;
				}
				p++;
			}

			p=Labels[i];

			// Verify that the sequence begins with the ACE prefix
			if(!StrCmpN(p,ACE_PREFIX,ACE_PREFIX_LEN) && LabelLengths[i]-ACE_PREFIX_LEN)
			{
				// Save a copy of the sequence
				StringCchCopy(acelabel,MAX_INT_LABEL_LEN,p);

				// Remove the ACE prefix and decode the sequence using the punycode algorithm
				out_len=sizeof(buffer);
				status=punycode_decode(LabelLengths[i]-ACE_PREFIX_LEN,p+ACE_PREFIX_LEN,&out_len,codepoints,NULL);
				if (status != punycode_success || out_len>=MAX_INT_LABEL_LEN) return IDNA_ILLEGAL;

				j=0;
				pUCP=codepoints;
				while(out_len && j<MAX_INT_LABEL_LEN-1)
				{
					cp=*pUCP;
					EncodeUTF16(cp,&p);
					if(IsUTF16BMP(cp)) j++;
					else j+=2;
					pUCP++;
					p++;
					out_len--;
				}
				*p=0;
				len=p-Labels[i];

				LabelLengths[i]=len;
				total_len+=len;

				// Check if result encodes to the original ACE string
				StringCchCopy(unicodelabel,MAX_INT_LABEL_LEN,Labels[i]);

				// Apply ToAscii
				result|=LabelToAscii(unicodelabel,&len);
				if(result&IDNA_ILLEGAL) return IDNA_ILLEGAL;

				// Verfiy that the result matches the original seqeuence
				if(StrCmpI(unicodelabel,acelabel)) return IDNA_ILLEGAL;

				// result|=IDNA_MODIFIED|IDNA_NONASCII; // Already set by LabelToAscii
			}
			else total_len+=LabelLengths[i];
		}
		if(HostEndIndex) *HostEndIndex=(int)total_len;
		if(DomainStartIndex) *DomainStartIndex=GetDomainStartIndex(NULL);
		StrCchLen(URLPath,MAXURLLENGTH,len);
		total_len+=len;
		if(result&IDNA_MODIFIED)
		{
			if(total_len<*size)
			{
				if(SchemeLen)
				{
					StringCbCopy(url,sizeof Scheme,Scheme);
					p=url+SchemeLen;
				}
				else p=url;
				if(UserPasswordLen)
				{
					StringCbCopy(p,sizeof UserPassword,UserPassword);
					p+=UserPasswordLen;
				}
				for(i=0;i<nLabels;i++)
				{
					StringCchCopy(p,MAX_INT_LABEL_LEN,Labels[i]);
					p+=LabelLengths[i];
					if(i<nLabels-1) *(p++)=_T('.');
				}
				if(IsFQDN) *(p++)=_T('.');
				StringCchCopy(p,MAXURLLENGTH,URLPath); // Inaccurate destination buffer length (length checked above)
				*size=total_len;
			}
		}
	}
	else return IDNA_ILLEGAL; // split_labels failed

	*size=total_len;
	return result;
}

#define N_URI_SCHEMES 7

UINT CIDNA::IsAddress(TCHAR *uri)
{
	const SchemeAddressType URISchemes[N_URI_SCHEMES]={
		{L"\\\\",2,ADDRESS_PATH}, // Windows network path
		{L"file://",7,ADDRESS_URL|ADDRESS_FILE_URL},
		{L"about:",6,ADDRESS_ABOUT},
		{L"javascript:",11,ADDRESS_JAVASCRIPT},
		{L"mailto:",7,ADDRESS_MAILTO},
		{L"res://",6,ADDRESS_RESOURCE},
		{L"::{",3,ADDRESS_PIDL}
		//if(!StrCmpNI(uri,_T("news:"),5) && (len<7 || uri[5]!=_T('/') || uri[6]!=_T('/'))) return ADDRESS_NEWSGROUP;
	};

	UINT result=ADDRESS_NONE;
	int i;
	size_t len;
	bool bNoScheme;

	if(SUCCEEDED_OK(StringCchLength(uri,MAXURLLENGTH,&len)) && uri[0]!=L'+' && uri[0]!=L'"' && uri[0]!=L'-')
	{
		if(len>=2)
		{
			for(i=0;i<N_URI_SCHEMES;i++)
			{
				if(!StrCmpNI(uri,URISchemes[i].Scheme,URISchemes[i].Length))
				{
					result=URISchemes[i].Type;
					break;
				}
			}

			 // DOS file path?
			if(uri[1]==_T(':') && ((uri[0]>=L'a' && uri[0]<=L'z') || (uri[0]>=L'A' && uri[0]<=L'Z')))
			{
				if(len==2) result=ADDRESS_DRIVE;
				else if(uri[2]==L'\\') result=ADDRESS_PATH;
			}
			
			// URL?
			if(result==ADDRESS_NONE)
			{
				i=0;
				bNoScheme=true;
				while(uri[i] && i<=MAX_SCHEME_LEN)
				{
					if(uri[i]==L':' && uri[i+1]==L'/' && uri[i+2]==L'/')
					{
						bNoScheme=false;
						break;
					}
					i++;
				}

				if(split_labels(uri,bNoScheme) && StrStr(uri,L" ")==NULL)
				{
					if(IsIPv4) result=ADDRESS_IPV4;
					if(IsIPv6) result=ADDRESS_IPV6;

					if(SchemeLen)
					{
						result|=ADDRESS_URL;
					}
					else
					{
						if( result!=ADDRESS_NONE || PortNumber==80 || PortNumber==8080 ||
							(PortNumber!=PORTNUMBER_OMITTED && nLabels==1 && !StrCmp(Labels[0],L"localhost")) ||
							(nLabels>1 && IsTLD(Labels[nLabels-1])) ) result|=ADDRESS_URL_WITHOUT_SCHEME;
					}

					if(result!=ADDRESS_NONE)
					{
						if(PortNumber!=PORTNUMBER_OMITTED) result|=ADDRESS_PORT_PRESENT;
						if(!StrCmpNI(Scheme,L"https:",6)) result|=ADDRESS_HTTPS_URL;
					}
				}
			}
		} // len>=2
	} // len succeeded, first char not ",+,-

	return result;
}

bool CIDNA::IsOctet(TCHAR *label)
{
	int octet;
	int i;

	octet=0;
	i=0;
	while(label[i])
	{
		if(label[i]>=L'0' && label[i]<=L'9') octet=octet*10+label[i]-L'0';
		else break;

		i++;
	}

	return (label[i]==L'\0' && octet<0x100 && i && i<=3);
}

bool CIDNA::IsHexWord(TCHAR *label)
{
	int word;
	int i;

	word=0;
	i=0;
	while(label[i])
	{
		word<<=4;
		if(label[i]>=L'0' && label[i]<=L'9') word+=label[i]-L'0';
		else if(label[i]>=L'a' && label[i]<=L'f') word+=10+label[i]-L'a';
		else if(label[i]>=L'A' && label[i]<=L'F') word+=10+label[i]-L'A';
		else break;

		i++;
	}

	return (label[i]==L'\0' && word<0x10000 && i && i<=4);
}

#define N_INTERNET_SCHEMES 9
#define INTERNET_SCHEMES_MAILTO 8

bool CIDNA::IsInternetURL(TCHAR *url)
{
	const SchemeAddressType InternetSchemes[N_INTERNET_SCHEMES]={
		{L"http:",5,0},
		{L"https:",6,0},
		{L"ftp:",4,0},
		{L"ftps:",5,0},
		{L"file://",7,0},
		{L"news://",7,0},
		{L"nntp://",7,0},
		{L"telnet:",7,0},
		{L"mailto:",7,0}
	};
	bool result;
	int i;

	i=0;
	while(i<N_INTERNET_SCHEMES && StrCmpNI(url,InternetSchemes[i].Scheme,InternetSchemes[i].Length)) i++;

	if(i<INTERNET_SCHEMES_MAILTO) result=true;
	else if(i==INTERNET_SCHEMES_MAILTO)
	{
		// Test if mailto address can be parsed
		TCHAR mailto_url[MAXURLLENGTH];
		size_t url_len;

		StringCchCopy(mailto_url,MAXURLLENGTH,url);
		url_len=MAXURLLENGTH;
		result=((URLToAscii(mailto_url,&url_len,NULL,NULL,NULL)&IDNA_ILLEGAL)==0);
	}
	else result=false;

	return result;
}

bool CIDNA::IsTLD(TCHAR * tld)
{
const static TCHAR tldlist[NTLD][7]={
			L"ac",L"ad",L"ae",L"aero",L"af",L"ag",L"ai",L"al",L"am",L"an",
			L"ao",L"aq",L"ar",L"arpa",L"as",L"asia",L"at",L"au",L"aw",L"ax",L"az",L"ba",
			L"bb",L"bd",L"be",L"bf",L"bg",L"bh",L"bi",L"biz",L"bj",L"bm",
			L"bn",L"bo",L"br",L"bs",L"bt",L"bv",L"bw",L"by",L"bz",L"ca",L"cat",
			L"cc",L"cd",L"cf",L"cg",L"ch",L"ci",L"ck",L"cl",L"cm",L"cn",
			L"co",L"com",L"coop",L"cr",L"cu",L"cv",L"cx",L"cy",L"cz",
			L"de",L"dj",L"dk",L"dm",L"do",L"dz",L"ec",L"edu",L"ee",L"eg",
			L"eh",L"er",L"es",L"et",L"eu",L"fi",L"fj",L"fk",L"fm",L"fo",L"fr",
			L"ga",L"gb",L"gd",L"ge",L"gf",L"gg",L"gh",L"gi",L"gl",L"gm",
			L"gn",L"gov",L"gp",L"gq",L"gr",L"gs",L"gt",L"gu",L"gw",L"gy",
			L"hk",L"hm",L"hn",L"hr",L"ht",L"hu",L"id",L"ie",L"il",L"im",
			L"in",L"info",L"int",L"io",L"iq",L"ir",L"is",L"it",L"je",L"jm",
			L"jo",L"jobs",L"jp",L"ke",L"kg",L"kh",L"ki",L"km",L"kn",L"kp",L"kr",
			L"kw",L"ky",L"kz",L"la",L"lb",L"lc",L"li",L"lk",L"lr",L"ls",
			L"lt",L"lu",L"lv",L"ly",L"ma",L"mc",L"md",L"me",L"mg",L"mh",L"mil",
			L"mk",L"ml",L"mm",L"mn",L"mo",L"mobi",L"mp",L"mq",L"mr",L"ms",L"mt",
			L"mu",L"museum",L"mv",L"mw",L"mx",L"my",L"mz",L"na",L"name",L"nc",
			L"ne",L"net",L"nf",L"ng",L"ni",L"nl",L"no",L"np",L"nr",L"nu",
			L"nz",L"om",L"org",L"pa",L"pe",L"pf",L"pg",L"ph",L"pk",L"pl",
			L"pm",L"pn",L"post",L"pr",L"pro",L"ps",L"pt",L"pw",L"py",L"qa",L"re",
			L"ro",L"rs",L"ru",L"rw",L"sa",L"sb",L"sc",L"sd",L"se",L"sg",L"sh",
			L"si",L"sj",L"sk",L"sl",L"sm",L"sn",L"so",L"sr",L"st",L"su",L"sv",
			L"sy",L"sz",L"tc",L"td",L"tel",L"tf",L"tg",L"th",L"tj",L"tk",L"tl",
			L"tm",L"tn",L"to",L"tp",L"tr",L"travel",L"tt",L"tv",L"tw",L"tz",L"ua",
			L"ug",L"uk",L"um",L"us",L"uy",L"uz",L"va",L"vc",L"ve",L"vg",
			L"vi",L"vn",L"vu",L"wf",L"ws",L"xxx",L"ye",L"yt",L"yu",L"za",L"zm",
			L"zw",
			L"\x43a\x43e\x43c",L"\x43a\x43e\x43c",L"\x43d\x435\x442",L"\x440\x443",L"\x440\x444"}; // com org net ru rf

	int start,end,mid;
	int cmp;

	start=0;
	end=NTLD-1;
	do
	{
		mid=start+(end-start)/2;

		cmp=StrCmpI(tld,tldlist[mid]);

		if(!cmp) return true;
		else if(cmp<0) end=mid-1;
		else start=mid+1;
	}
	while(start<=end);

	return false;
}

int CIDNA::IsElementOfUCTable(DWORD cp,const UnicodeInterval* UCTable,short n)
{
	short begin, end, mid;
	BYTE plane;
	WORD offset;
	const UnicodeInterval *CurrentInterval;
	bool continue_left;

	begin=0;
	end=n-1;
	plane=(BYTE)(cp>>16);
	offset=(WORD)(cp&0xFFFF);

	do
	{
		mid=begin+(end-begin)/2;
		CurrentInterval=UCTable+mid;
		if(CurrentInterval->Plane==plane)
		{
			if(CurrentInterval->Begin<=offset)
			{
				if(offset<=CurrentInterval->End) return mid;
				else continue_left=false;
			}
			else continue_left=true;
		}
		else continue_left=plane<CurrentInterval->Plane;
			
		if(continue_left) end=mid-1;
		else begin=mid+1;

	} while(begin<=end);

	return -1;
}

//template <typename M> int CIDNA::GetMapping(DWORD cp,const M* UCMapping,short n)
int CIDNA::GetMapping(DWORD cp,const UnicodeMapping* UCMapping,short n)
{
	short begin, end, mid;
	BYTE plane;
	WORD offset;
	const UnicodeMapping* CurrentMapping;
	bool continue_left;

	begin=0;
	end=n-1;
	plane=(BYTE)(cp>>16);
	offset=(WORD)(cp&0xFFFF);

	do
	{
		mid=begin+(end-begin)/2;
		CurrentMapping=UCMapping+mid;
		if((CurrentMapping->Plane&0x0F)==plane)
		{
			if(CurrentMapping->From==offset) return mid;
			else continue_left=offset<CurrentMapping->From;
		}
		else continue_left=plane<(CurrentMapping->Plane&0x0F);
			
		if(continue_left) end=mid-1;
		else begin=mid+1;

	} while(begin<=end);

	return -1;
}

int CIDNA::GetCompatMappingLarge(DWORD cp)
{
	short begin, end, mid;
	BYTE plane;
	WORD offset;
	const UnicodeMappingLarge *CurrentMapping;
	bool continue_left;

	begin=0;
	end=NUCCOMPATLARGE-1;
	plane=(BYTE)(cp>>16);
	offset=(WORD)(cp&0xFFFF);

	do
	{
		mid=begin+(end-begin)/2;
		CurrentMapping=UCCompatLarge+mid;
		if((CurrentMapping->Plane&0x0F)==plane)
		{
			if(CurrentMapping->From==offset) return mid;
			else continue_left=offset<CurrentMapping->From;
		}
		else continue_left=plane<(CurrentMapping->Plane&0x0F);
			
		if(continue_left) end=mid-1;
		else begin=mid+1;

	} while(begin<=end);

	return -1;
}

int CIDNA::GetCaseMapping(DWORD cp)
{
	return GetMapping(cp,UCMapping,NUCMAPPING);
}

int CIDNA::GetCompatMapping(DWORD cp)
{
	return GetMapping(cp,UCCompat,NUCCOMPAT);
}

DWORD CIDNA::GetSingletonMapping(DWORD cp)
{
	short begin, end, mid;
	BYTE plane;
	WORD offset;
	const UnicodeMappingSingleton *CurrentMapping;
	bool continue_left;

	begin=0;
	end=NUCSINGLETONS-1;
	plane=(BYTE)(cp>>16);
	offset=(WORD)(cp&0xFFFF);

	do
	{
		mid=begin+(end-begin)/2;
		CurrentMapping=UCSingletons+mid;
		if((CurrentMapping->Plane&0x0F)==plane)
		{
			if(CurrentMapping->From==offset) return CurrentMapping->To|((CurrentMapping->Plane&0xF0)<<12);
			else continue_left=offset<CurrentMapping->From;
		}
		else continue_left=plane<(CurrentMapping->Plane&0x0F);
			
		if(continue_left) end=mid-1;
		else begin=mid+1;

	} while(begin<=end);

	return 0;
}

int CIDNA::GetDecompMapping(DWORD cp)
{
	short begin, end, mid;
	BYTE plane;
	WORD offset;
	const UnicodeMappingDecomp* CurrentMapping;
	bool continue_left;

	begin=0;
	end=NUCDECOMP-1;
	plane=(BYTE)(cp>>16);
	offset=(WORD)(cp&0xFFFF);

	do
	{
		mid=begin+(end-begin)/2;
		CurrentMapping=UCDecomp+mid;
		if((CurrentMapping->Plane&0x0F)==plane)
		{
			if(CurrentMapping->From==offset) return mid;
			else continue_left=offset<CurrentMapping->From;
		}
		else continue_left=plane<(CurrentMapping->Plane&0x0F);
			
		if(continue_left) end=mid-1;
		else begin=mid+1;

	} while(begin<=end);

	return -1;
}

DWORD CIDNA::GetComposite(DWORD cp1,DWORD cp2)
{
	short begin, end, mid;
	BYTE plane;
	WORD offset1,offset2;
	const UnicodeMappingDecomp *CurrentComposite;
	bool continue_left;

	begin=0;
	end=NUCCOMPOSITES-1;
	plane=(BYTE)(cp1>>12)&0xF0;
	offset1=(WORD)(cp1&0xFFFF);
	offset2=(WORD)(cp2&0xFFFF);

	do
	{
		mid=begin+(end-begin)/2;
		CurrentComposite=UCComposites+mid;

		if((CurrentComposite->Plane&0xF0)==plane)
		{
			if(CurrentComposite->To[0]==offset1)
				if(CurrentComposite->To[1]==offset2) return CurrentComposite->From|((CurrentComposite->Plane&0x0F)<<16);
				else continue_left=offset2<CurrentComposite->To[1];
			else continue_left=offset1<CurrentComposite->To[0];
		}
		else continue_left=plane<(CurrentComposite->Plane&0xF0);
			
		if(continue_left) end=mid-1;
		else begin=mid+1;

	} while(begin<=end);

	return 0;
}

short CIDNA::GetCharSet(DWORD cp)
{
	int set;

	set=IsElementOfUCTable(cp,UCCharSets,NUCCHARSETS);
	if(set==-1) set=(cp>0x20 && cp<0x7F)?CHARSET_NONSTDASCII:CHARSET_UNKNOWN;
	
	return (short)set;
}

const TCHAR *CIDNA::GetCharSetName(short CharSet)
{
	return (CharSet<NUCCHARSETS_NAMES) ? UCCharSetNames[CharSet] : NULL;
}

bool CIDNA::IsIgnoredChar(DWORD cp)
{
	return IsElementOfUCTable(cp,UCIgnored,NUCIGNORED)!=-1;
}

bool CIDNA::IsProhibitedChar(DWORD cp)
{
	return IsElementOfUCTable(cp,UCProhibited,NUCPROHIBITED)!=-1;
}

bool CIDNA::IsBidiCatL(DWORD cp)
{
	return IsElementOfUCTable(cp,UCBidiCatL,NUCBIDICATL)!=-1;
}

bool CIDNA::IsBidiCatR_AL(DWORD cp)
{
	return IsElementOfUCTable(cp,UCBidiCatR_AL,NUCBIDICATR_AL)!=-1;
}

BYTE CIDNA::GetCanonicalClass(DWORD cp)
{
	int c;

	c=IsElementOfUCTable(cp,UCClassIntervals,NUCCLASSES);
	return (c!=-1) ? UCClassLookup[c] : 0;
}

bool CIDNA::Normalize(char form,DWORD *label,size_t *len)
{
	bool success;

	success=Decompose(form & NF_COMPATIBILITY_MASK,label,len);
	if(form & NF_COMPOSITION_MASK && success && *len>1) success=Compose(label,len);

	return success;
}

bool CIDNA::Decompose(bool compatability,DWORD *label,size_t *len)
{
	DWORD result[MAX_INT_LABEL_LEN];
	DWORD buffer[MAX_DECOMPOSITION_LEN];
	DWORD cp,decomp;
	DWORD *pLabel;
	size_t result_len;
	int i,k,n;
	BYTE decompClass,lastClass;

	result_len=0;
	pLabel=label;
	cp=*label;

	while(cp)
	{
		n=GetRecursiveDecomposition(compatability,cp,buffer);
		i=0;
		while(i<n && result_len<MAX_INT_LABEL_LEN-1)
		{
			decomp=buffer[i];
			decompClass=GetCanonicalClass(decomp);
			result[result_len]=decomp;
			if(decompClass!=0 && result_len)
			{
				// Bubble-sort combining marks as necessary
				k=(int)result_len-1;
				lastClass=GetCanonicalClass(result[k]);
				while(decompClass<lastClass)
				{
					result[k+1]=result[k]; // Swap
					result[k]=decomp;
					if(k) lastClass=GetCanonicalClass(result[--k]);
					else lastClass=0; // Start of string reached
				}
			}
			result_len++;
			i++;
		}
		if(result_len>=MAX_INT_LABEL_LEN-1) return false;

		pLabel++;
        cp=*pLabel;
	}

	result[result_len]=0;

	// Copy result to destination

	CopyMemory(label,result,(result_len+1)<<2); // UTF-32

	*len=result_len;

	return true;
}

int CIDNA::GetRecursiveDecomposition(bool compatability,DWORD cp,DWORD *buffer)
{
	const WORD* decomp;
	DWORD singleton;
	DWORD plane;
	int maxlen;
	int index;
	int len;
	int total_len;

	if(compatability) // Compatibility Decomposition
	{
		index=GetCompatMapping(cp);
		if(index!=-1)
		{
			decomp=UCCompat[index].To;
			plane=(UCCompat[index].Plane&0xF0)<<12;
			maxlen=4;
		}
		else
		{
			index=GetCompatMappingLarge(cp);
			if(index!=-1)
			{
				decomp=UCCompatLarge[index].To;
				plane=(UCCompatLarge[index].Plane&0xF0)<<12;
				maxlen=19;
			}
		}
	}
	else index=-1;

	if(index==-1) // Canonical Decomposition
	{
		if(SBase<=cp && cp<=SEnd) // Decompose Hangul Syllable algorithmically
		{
			WORD SIndex,L,V,T;

			SIndex=(WORD)(cp-SBase);
			L=LBase+SIndex/NCount;
			V=VBase+(SIndex%NCount)/TCount;
			T=TBase+SIndex%TCount;

			*buffer++=L;
			*buffer++=V;
			if(T==TBase) return 2;
			else
			{
				*buffer=T;
				return 3;
			}
		}
		else
		{
			index=GetDecompMapping(cp);
			if(index!=-1)
			{
				decomp=UCDecomp[index].To;
				plane=(UCDecomp[index].Plane&0xF0)<<12;
				maxlen=2;
			}
			else
			{
				singleton=GetSingletonMapping(cp);
				if(singleton) return GetRecursiveDecomposition(compatability,singleton,buffer);
			}
		}
	}

	if(index!=-1) // Recursively decompose
	{
		total_len=0;
		index=0;
		while(index<maxlen && *decomp)
		{
			len=GetRecursiveDecomposition(compatability,(*decomp)|plane,buffer);
			decomp++;
			buffer+=len;
			index++;
			total_len+=len;
		}
	}
	else // No decomposition
	{
		*buffer=cp;
		total_len=1;
	}
	return total_len;
}

bool CIDNA::Compose(DWORD *label,size_t *len)
{
	UINT starterPos, starterCp, compPos, decompPos;
	BYTE cpClass,lastClass;
	DWORD cp;
	DWORD composite;
	DWORD LIndex,VIndex;

	starterPos=0;
	starterCp=*label;
	compPos=1;
	lastClass=GetCanonicalClass(starterCp);
	// if (lastClass != 0) lastClass = 256; // Fix for strings staring with a combining mark

	// Loop on the decomposed characters, combining where possible
	decompPos=compPos;
	while(decompPos<*len)
	{
		cp=label[decompPos];
		cpClass=GetCanonicalClass(cp);
		composite=GetComposite(starterCp,cp);
		if(composite && (lastClass < cpClass || lastClass==0))
		{
			// Replace starter character with composite character
			label[starterPos]=composite;
			// we know that we will only be replacing non-supplementaries by non-supplementaries
			// so we don't have to adjust the decompPos
			starterCp=composite;
		}
		else if(LBase<=starterCp && starterCp<=LEnd && VBase<=cp && cp<=VEnd && starterPos==compPos-1)
		{
			// Hangul Syllable Composition <L,V>

			LIndex=starterCp-LBase;
			VIndex=cp-VBase;

			starterCp=(LIndex*VCount+VIndex)*TCount+SBase;
			label[starterPos]=starterCp;
		}
		else if(SBase<=starterCp && starterCp<=SEnd && TBase<cp && cp<=TEnd && ((starterCp-SBase)&TCount)==0 && starterPos==compPos-1)
		{
			// Hangul Syllable Composition <LV,T> http://unicode.org/reports/tr15/

			starterCp+=cp-TBase;
			label[starterPos]=starterCp;
		}
		else
		{
			if(cpClass==0)
			{
				starterPos=compPos;
				starterCp=cp;
			}
			lastClass=cpClass;
			label[compPos]=cp;
			compPos++;
		}
		decompPos++;
	}

	label[compPos]=0;
	*len=compPos;

	return true;
}

bool CIDNA::Nameprep(TCHAR *utf16,DWORD *codepoints,size_t *len)
{
	bool result;
	TCHAR *p;
	DWORD cp;
	const UnicodeMapping *map;
	int plane;
	size_t i,k,n;
	int idx;
	bool hasLCat,hasR_ALCat;

	result=false;

	// Case folding
	i=0;
	p=utf16;
	while(*p && i<MAX_INT_LABEL_LEN-MAPPINGSIZE-1) // 5: one char can be mapped to max 4 codepoints + terminating null
	{
		cp=DecodeUTF16(&p);
		if(!IsIgnoredChar(cp)) // Mapped to nothing?
		{
			idx=GetCaseMapping(cp);
			if(idx==-1)	codepoints[i++]=cp;
			else
			{
				map=UCMapping+idx;
				plane=(map->Plane&0xF0)<<12;
				k=0;
				while(k<MAPPINGSIZE && map->To[k])
				{
					codepoints[i++]=map->To[k]|plane;
					k++;
				}
			}
		}
		p++;
	}
	codepoints[i]=0;

	if(i)
	{
		n=i;
		// Normalize
		if(Normalize(NFKC,codepoints,&n))
		{
			// Check for prohibited characters and bidi requirements
			i=0;
			hasLCat=false;
			hasR_ALCat=false;
			while(i<n)
			{
				cp=codepoints[i];
				if(IsProhibitedChar(cp)) break;
				if(IsBidiCatL(cp)) hasLCat=true;
				else if(IsBidiCatR_AL(cp)) hasR_ALCat=true;
				i++;
			}
			if(i==n)
			{
				if(hasR_ALCat==false || (hasLCat==false && IsBidiCatR_AL(codepoints[0]) && IsBidiCatR_AL(codepoints[n-1])))
				{
					*len=n;
					result=true;
				}
			}
		} // End Normalize successful
	} // End i>0

	return result;
}